@@ -27,6 +27,7 @@
 
 #include "libgretl.h"
 #include "var.h"
+#include "system.h"
 #include "gretl_restrict.h"
 #include "gretl_func.h"
 #include "modelspec.h"
@@ -162,14 +163,12 @@ int model_test_start (int test_ci, int model_id, PRN *prn)
 	    pprintf(prn, _("Can't do this: there is no model %d\n"), model_id);
 	    err = 1;
 	}
-    }    
-    else if (!command_ok_for_model(test_ci, 
-				   model_ci_from_modelspec(modelspec, m))) {
+    } else if (!command_ok_for_model(test_ci, 
+				     model_ci_from_modelspec(modelspec, m))) {
 	pputs(prn, _("Sorry, command not available for this estimator"));
 	pputc(prn, '\n');
 	err = 1;
-    }
-    else if (model_sample_issue(NULL, modelspec, m, datainfo)) {
+    } else if (model_sample_issue(NULL, modelspec, m, datainfo)) {
 	pputs(prn, _("Can't do: the current data set is different from "
 		     "the one on which\nthe reference model was estimated\n"));
 	err = 1;
@@ -279,10 +278,11 @@ static int clear_data (void)
 
     if (Z != NULL) {
 	free_Z(Z, datainfo); 
+	Z = NULL;
     }
+
     clear_datainfo(datainfo, CLEAR_FULL);
 
-    Z = NULL;
     data_status = 0;
 
     clear_model(models[0]);
@@ -291,8 +291,12 @@ static int clear_data (void)
     free_modelspec(modelspec);
     modelspec = NULL;
 
+    gretl_equation_systems_cleanup();
+
     reset_model_count();
 
+    gretl_cmd_destroy_context(&cmd);
+
     return err;
 }
 
@@ -481,18 +485,14 @@ int main (int argc, char *argv[])
 	if (err == GRETL_NATIVE_DATA) {
 	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, 
 				 DATA_NONE, prn);
-	} 
-	else if (err == GRETL_XML_DATA) {
+	} else if (err == GRETL_XML_DATA) {
 	    err = get_xmldata(&Z, &datainfo, paths.datfile, &paths, 
 			      DATA_NONE, prn, 0);
-	} 
-	else if (err == GRETL_CSV_DATA) {
+	} else if (err == GRETL_CSV_DATA) {
 	    err = import_csv(&Z, &datainfo, paths.datfile, &paths, prn);
-	} 
-	else if (err == GRETL_BOX_DATA) {
+	} else if (err == GRETL_BOX_DATA) {
 	    err = import_box(&Z, &datainfo, paths.datfile, prn);
-	} 
-	else if (err == GRETL_SCRIPT) { /* maybe it's a script file? */
+	} else if (err == GRETL_SCRIPT) { /* maybe it's a script file? */
 	    runit = 1;
 	    strcpy(runfile, paths.datfile); 
 	    clear(paths.datfile, MAXLEN);
@@ -562,7 +562,7 @@ int main (int argc, char *argv[])
     }
 
     /* main command loop */
-    while (strcmp(cmd.cmd, "quit")) {
+    while (strcmp(cmd.word, "quit")) {
 	char linecopy[MAXLEN];
 
 	if (err && batch && errfatal) {
@@ -646,13 +646,11 @@ static int handle_user_defined_function (char *line, int *fncall)
     int ufunc = gretl_is_user_function(line);
     int err = 0;
 
-    /* allow for nested function calls */
     if (ufunc && gretl_compiling_function()) {
-	return 0;
-    }
-
-    /* an actual function call */
-    else if (ufunc) {
+	/* allow for nested function calls */
+	;
+    } else if (ufunc) {
+	/* an actual function call */
 	err = gretl_function_start_exec(line);
 	*fncall = 1;
     } 
@@ -710,8 +708,8 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	       (runit)? NULL : cmdprn);
     }
 
-    /* tell libgretl if we should pause between screens */
-    gretl_set_text_pause(!batch);
+    /* tell libgretl if we're in batch mode */
+    gretl_set_batch_mode(batch);
 
     /* if in batch mode, echo comments from input */
     if (batch && cmd.ci == CMD_COMMENT && !echo_off) {
@@ -769,17 +767,41 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     switch (cmd.ci) {
 
-    case ADF: case COINT: case COINT2: case KPSS:
-    case CORR:
-    case CRITERIA: case CRITICAL: case DATA:
-    case DIFF: case LDIFF: case LAGS: case LOGS:
-    case MULTIPLY: case SQUARE: case RHODIFF:
-    case GRAPH: case PLOT: case LABEL:
-    case INFO: case LABELS: case VARLIST:
+    case ADF: 
+    case COINT2: 
+    case COINT: 
+    case CORR: 
+    case CRITERIA: 
+    case CRITICAL: 
+    case DATA:
+    case DIFF: 
+    case ESTIMATE:
+    case GRAPH: 
+    case HURST:
+    case INFO: 
+    case KPSS:
+    case LABEL:
+    case LABELS: 
+    case LAGS: 
+    case LDIFF: 
+    case LOGS:
+    case MAHAL:
+    case MEANTEST: 
+    case MULTIPLY: 
+    case OUTFILE: 
+    case PCA:
+    case PLOT: 
     case PRINT: 
-    case SIM: case SUMMARY:
-    case MEANTEST: case VARTEST: case STORE:
-    case RUNS: case SPEARMAN: case OUTFILE: case PCA:
+    case RHODIFF:
+    case RMPLOT: 
+    case RUNS: 
+    case SIM: 
+    case SPEARMAN:
+    case SQUARE: 
+    case STORE:
+    case SUMMARY: 
+    case VARLIST:
+    case VARTEST: 
 	err = simple_commands(&cmd, line, &Z, datainfo, prn);
 	if (err) errmsg(err, prn);
 	break;
@@ -848,7 +870,6 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 			     datainfo, cmd.opt, prn);
 	if ((err = (models[0])->errcode)) { 
 	    errmsg(err, prn); 
-	    break;
 	}
 	break;
 
@@ -887,40 +908,28 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}	
 	break;
 
-    case GARCH:
-	clear_model(models[0]);
-	*models[0] = garch(cmd.list, &Z, datainfo, cmd.opt, prn);
-	if ((err = (models[0])->errcode)) { 
-	    errmsg(err, prn); 
-	} else {	
-	    printmodel(models[0], datainfo, cmd.opt, prn);
-	}	
-	break;
-
-    case CHOW:
-        if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = chow_test(line, models[0], &Z, datainfo, prn, NULL);
-	if (err) errmsg(err, prn);
-	break;
-
     case COEFFSUM:
-        if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = sum_test(cmd.list, models[0], &Z, datainfo, prn);
-	if (err) errmsg(err, prn);
-	break;
-
     case CUSUM:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = cusum_test(models[0], &Z, datainfo, prn, NULL);
-	if (err) errmsg(err, prn);
+    case RESET:
+    case CHOW:
+    case VIF:
+        if ((err = model_test_start(cmd.ci, 0, prn))) break;
+	if (cmd.ci == COEFFSUM) {
+	    err = sum_test(cmd.list, models[0], &Z, datainfo, prn);
+	} else if (cmd.ci == CUSUM) {
+	    err = cusum_test(models[0], &Z, datainfo, prn, NULL);
+	} else if (cmd.ci == RESET) {
+	    err = reset_test(models[0], &Z, datainfo, prn, NULL);
+	} else if (cmd.ci == CHOW) {
+	    err = chow_test(line, models[0], &Z, datainfo, prn, NULL);
+	} else {
+	    err = vif_test(models[0], &Z, datainfo, prn);
+	}
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
-    case RESET:
-        if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = reset_test(models[0], &Z, datainfo, prn, NULL);
-	if (err) errmsg(err, prn);
-	break;
-	
     case CORC:
     case HILU:
     case PWE:
@@ -934,25 +943,17 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	*models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, rho);
 	if ((err = (models[0])->errcode)) {
 	    errmsg(err, prn);
-	    break;
+	} else {
+	    printmodel(models[0], datainfo, cmd.opt, prn); 
 	}
-	printmodel(models[0], datainfo, cmd.opt, prn); 
-	break;
-
-    case LAD:
-	clear_model(models[0]);
-	*models[0] = lad(cmd.list, &Z, datainfo);
-	if ((err = (models[0])->errcode)) {
-	    errmsg(err, prn);
-	    break;
-	}
-	printmodel(models[0], datainfo, cmd.opt, prn);
 	break;
 
     case CORRGM:
 	order = atoi(cmd.param);
 	err = corrgram(cmd.list[1], order, &Z, datainfo, batch, prn);
-	if (err) pputs(prn, _("Failed to generate correlogram\n"));
+	if (err) {
+	    pputs(prn, _("Failed to generate correlogram\n"));
+	}
 	break;
 
     case DELEET:
@@ -986,29 +987,31 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case END:
 	if (!strcmp(cmd.param, "system")) {
 	    err = gretl_equation_system_finalize(sys, &Z, datainfo, prn);
-	    if (err) errmsg(err, prn);
+	    if (err) {
+		errmsg(err, prn);
+	    }
 	    sys = NULL;
-	} 
-	else if (!strcmp(cmd.param, "nls")) {
+	} else if (!strcmp(cmd.param, "nls")) {
 	    clear_model(models[0]);
 	    *models[0] = nls(&Z, datainfo, prn);
 	    if ((err = (models[0])->errcode)) {
 		errmsg(err, prn);
-		break;
+	    } else {
+		do_nls = 1;
+		printmodel(models[0], datainfo, cmd.opt, prn);
 	    }
-	    do_nls = 1;
-	    printmodel(models[0], datainfo, cmd.opt, prn);
-	}
-	else if (!strcmp(cmd.param, "restrict")) {
+	} else if (!strcmp(cmd.param, "restrict")) {
 	    err = gretl_restriction_set_finalize(rset, prn);
-	    if (err) errmsg(err, prn);
+	    if (err) {
+		errmsg(err, prn);
+	    }
 	    rset = NULL;
-	}  
-	else {
+	} else {
 	    err = 1;
 	}
 	break;
 
+    case BREAK:
     case ENDLOOP:
 	pputs(prn, _("You can't end a loop here, "
 		     "you haven't started one\n"));
@@ -1048,16 +1051,18 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    err *= -1;
 	    pputs(prn, _("Error retrieving fitted values\n"));
 	    errmsg(err, prn);
-	    break;
+	} else {
+	    err = 0;
+	    varlist(datainfo, prn);
 	}
-	err = 0;
-	varlist(datainfo, prn);
 	break;
 
     case FCASTERR:
 	if ((err = model_test_start(cmd.ci, 0, prn))) break;
 	err = fcast_with_errs(line, models[0], &Z, datainfo, prn, cmd.opt); 
-	if (err) errmsg(err, prn);
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
     case FIT:
@@ -1151,25 +1156,10 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case HAUSMAN:
 	err = model_test_start(cmd.ci, 0, prn);
 	if (!err) {
-	    err = hausman_test(models[0], &Z, datainfo, prn);
+	    err = hausman_test(models[0], &Z, datainfo, cmd.opt, prn);
 	}
 	break;
 
-    case HCCM:
-    case HSK:
-	clear_model(models[0]);
-	if (cmd.ci == HCCM) {
-	    *models[0] = hccm_func(cmd.list, &Z, datainfo);
-	} else {
-	    *models[0] = hsk_func(cmd.list, &Z, datainfo);
-	}
-	if ((err = (models[0])->errcode)) {
-	    errmsg(err, prn);
-	    break;
-	}
-	printmodel(models[0], datainfo, cmd.opt, prn);
-	break;
-
     case HELP:
 	help(cmd.param, paths.helpfile, prn);
 	break;
@@ -1256,22 +1246,14 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case LEVERAGE:
 	if ((err = model_test_start(cmd.ci, 0, prn))) break;	
-	err = leverage_test(models[0], &Z, datainfo, prn, cmd.opt);
+	err = leverage_test(models[0], &Z, datainfo, cmd.opt, prn);
 	if (err > 1) {
 	    errmsg(err, prn);
-	} else if (cmd.opt) {
+	} else if (cmd.opt & OPT_S) {
 	    varlist(datainfo, prn);
 	}
 	break;
 
-    case VIF:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = vif_test(models[0], &Z, datainfo, prn);
-	if (err) {
-	    errmsg(err, prn);
-	}
-	break;
-
     case LMTEST:
 	if ((err = model_test_start(cmd.ci, 0, prn))) break;
 	/* non-linearity (squares) */
@@ -1302,33 +1284,62 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if ((cmd.opt & OPT_W) || !cmd.opt) {
 	    err = whites_test(models[0], &Z, datainfo, prn, NULL);
 	    if (err) errmsg(err, prn);
-	    /* need to take more action in case of err? */
+	}
+	/* groupwise heteroskedasticity */
+	if (cmd.opt & OPT_P) {
+	    err = groupwise_hetero_test(models[0], &Z, datainfo, prn);
+	    if (err) errmsg(err, prn);
 	}
 	break;
 
+    case GARCH:
+    case HSK:
+    case LAD:
     case LOGISTIC:
     case LOGIT:
+    case POISSON:
     case PROBIT:
     case TOBIT:
+    case TSLS:
 	clear_model(models[0]);
 	if (cmd.ci == LOGIT || cmd.ci == PROBIT) {
 	    *models[0] = logit_probit(cmd.list, &Z, datainfo, cmd.ci);
+	} else if (cmd.ci == HSK) {
+	    *models[0] = hsk_func(cmd.list, &Z, datainfo);
 	} else if (cmd.ci == LOGISTIC) {
 	    *models[0] = logistic_model(cmd.list, &Z, datainfo, cmd.param);
-	} else {
+	} else if (cmd.ci == TOBIT) {
 	    *models[0] = tobit_model(cmd.list, &Z, datainfo,
 				     (cmd.opt & OPT_V)? prn : NULL);
+	} else if (cmd.ci == POISSON) {
+	    *models[0] = poisson_model(cmd.list, &Z, datainfo,
+				       (cmd.opt & OPT_V)? prn : NULL);
+	} else if (cmd.ci == TSLS) {
+	    *models[0] = tsls_func(cmd.list, atoi(cmd.param), 
+				   &Z, datainfo, cmd.opt);
+	} else if (cmd.ci == LAD) {
+	    *models[0] = lad(cmd.list, &Z, datainfo);
+	} else if (cmd.ci == GARCH) {
+	    *models[0] = garch(cmd.list, &Z, datainfo, cmd.opt, prn);
+	} else {
+	    /* can't happen */
+	    err = 1;
+	    break;
 	}
 	if ((err = (models[0])->errcode)) {
 	    errmsg(err, prn);
-	    break;
+	} else {
+	    printmodel(models[0], datainfo, cmd.opt, prn);
 	}
-	printmodel(models[0], datainfo, cmd.opt, prn);
 	break;
 
     case NLS:
 	err = nls_parse_line(line, (const double **) Z, datainfo);
-	if (err) errmsg(err, prn);
+	if (err) {
+	    errmsg(err, prn);
+	} else {
+	    gretl_cmd_set_context(&cmd, NLS);
+	}
 	break;
 
     case NULLDATA:
@@ -1357,15 +1368,19 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case OLS:
     case WLS:
+    case HCCM:
     case POOLED:
 	clear_model(models[0]);
-	*models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, 0.0);
+	if (cmd.ci == POOLED) {
+	    *models[0] = pooled(cmd.list, &Z, datainfo, cmd.opt, prn);
+	} else {
+	    *models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, 0.0);
+	}
 	if ((err = (models[0])->errcode)) {
 	    errmsg(err, prn);
-	    clear_model(models[0]);
-	    break;
+	} else {
+	    printmodel(models[0], datainfo, cmd.opt, prn);
 	}
-	printmodel(models[0], datainfo, cmd.opt, prn);
 	break;
 
 #ifdef ENABLE_GMP
@@ -1384,7 +1399,9 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case PERGM:
 	err = periodogram(cmd.list[1], &Z, datainfo, batch, cmd.opt, prn);
-	if (err) pputs(prn, _("Failed to generate periodogram\n"));
+	if (err) {
+	    pputs(prn, _("Failed to generate periodogram\n"));
+	}
 	break;
 
     case PRINTF:
@@ -1392,9 +1409,11 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case PVALUE:
-	if (batch || runit || (sscanf(line, "%s %s", s1, s2) == 2))
+	if (batch || runit || (sscanf(line, "%s %s", s1, s2) == 2)) {
 	    batch_pvalue(line, (const double **) Z, datainfo, prn);
-	else interact_pvalue();
+	} else {
+	    interact_pvalue();
+	}
 	break;
 
     case QUIT:
@@ -1409,7 +1428,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    }
 	    fb = stdin;
 	    runfile_open = 0;
-	    strcpy(cmd.cmd, "endrun"); /* overwrite "quit" */
+	    strcpy(cmd.word, "endrun"); /* overwrite "quit" */
 	    break;
 	}
 	printf(_("commands saved as %s\n"), cmdfile);
@@ -1464,13 +1483,16 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case SET:
 	err = parse_set_line(line, &echo_off, prn);
-	if (err) errmsg(err, prn);
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
     case SETOBS:
 	err = set_obs(line, datainfo, cmd.opt);
-	if (err) errmsg(err, prn);
-	else {
+	if (err) {
+	    errmsg(err, prn);
+	} else {
 	    if (datainfo->n > 0) {
 		print_smpl(datainfo, 0, prn);
 	    } else {
@@ -1505,7 +1527,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 		   !strcmp(line, "smpl --full")) {
 	    err = restore_full_sample(&Z, &datainfo, OPT_C);
 	} else { 
-	    err = set_sample(line, datainfo);
+	    err = set_sample(line, (const double **) Z, datainfo);
 	}
 
 	if (err) {
@@ -1516,13 +1538,18 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case RESTRICT:
-	/* joint hypothesis test on model */
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
+	/* joint hypothesis test on model or system */
 	if (rset == NULL) {
+	    if (*cmd.param == '\0') {
+		/* if param is non-blank, we're restricting a named system */
+		if ((err = model_test_start(cmd.ci, 0, prn))) break;
+	    }
 	    rset = restriction_set_start(line, models[0], datainfo);
 	    if (rset == NULL) {
 		err = 1;
 		errmsg(err, prn);
+	    } else {
+		gretl_cmd_set_context(&cmd, RESTRICT);
 	    }
 	} else {
 	    err = restriction_set_parse_line(rset, line);
@@ -1540,13 +1567,15 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    if (sys == NULL) {
 		err = 1;
 		errmsg(err, prn);
+	    } else {
+		gretl_cmd_set_context(&cmd, SYSTEM);
 	    }
 	} else {
 	    err = system_parse_line(sys, line, datainfo);
 	    if (err) {
 		errmsg(err, prn);
 		sys = NULL;
-	    }	
+	    } 
 	}
 	break;
 
@@ -1556,21 +1585,13 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 			       prn);
 	break;
 
-    case TSLS:
-	clear_model(models[0]);
-	*models[0] = tsls_func(cmd.list, atoi(cmd.param), 
-			       &Z, datainfo, cmd.opt);
-	if ((err = (models[0])->errcode)) {
-	    errmsg((models[0])->errcode, prn);
-	    break;
-	}
-	printmodel(models[0], datainfo, cmd.opt, prn);
-	break;
-
     case VAR:
 	order = atoi(cmd.param);
 	err = simple_var(order, cmd.list, &Z, datainfo, !batch, 
 			 cmd.opt, prn);
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
     case VARDUP:
@@ -1579,7 +1600,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     default:
 	pprintf(prn, _("Sorry, the %s command is not yet implemented "
-		       "in gretlcli\n"), cmd.cmd);
+		       "in gretlcli\n"), cmd.word);
 	err = 1;
 	break;
     }
@@ -1588,7 +1609,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	gretl_function_error();
     }
 
-    if (!err && (is_model_cmd(cmd.cmd) || do_nls || do_arch)
+    if (!err && (is_model_cmd(cmd.word) || do_nls || do_arch)
 	&& !is_quiet_model_test(cmd.ci, cmd.opt)) { 
 
 	attach_subsample_to_model(models[0], datainfo);
