@@ -40,7 +40,6 @@
 #include "../pixmaps/gretl.xpm"  /* program icon for X */
 
 /* functions private to gretl.c */
-static void clip_init (GtkWidget *w);
 static GtkWidget *make_main_window (int gui_get_data);
 
 static void set_up_main_menu (void);
@@ -98,7 +97,6 @@ double **Z;                 /* data set */
 MODEL **models;             /* gretl models structs */
 
 int plot_count, data_status, orig_vars;
-gchar *clipboard_buf; /* for copying models as LaTeX */
 float gui_scale;
 
 /* defaults for some options */
@@ -204,8 +202,8 @@ GtkItemFactoryEntry data_items[] = {
       RATS_DB, NULL },
     { N_("/File/Browse databases/sep1"), NULL, NULL, 0, "<Separator>" },
     { N_("/File/Browse databases/on database _server"), NULL, display_files, 
-
       REMOTE_DB, NULL },
+
     /* File, Create dataset */
     { N_("/File/_Create data set"), NULL, NULL, 0, "<Branch>" },
     { N_("/File/Create data set/time-series"), NULL, NULL, 0, "<Branch>" },
@@ -220,7 +218,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Create data set/time-series/high frequency/weekly"), 
       NULL, newdata_callback, 52, NULL }, 
     { N_("/File/Create data set/time-series/high frequency/daily (5-day week)"), 
-      NULL, newdata_callback, 5, NULL }, 
+      NULL, newdata_callback, 5, NULL },
+    { N_("/File/Create data set/time-series/high frequency/daily (6-day week)"), 
+      NULL, newdata_callback, 6, NULL },
     { N_("/File/Create data set/time-series/high frequency/daily (7-day week)"), 
       NULL, newdata_callback, 7, NULL }, 
     { N_("/File/Create data set/time-series/high frequency/hourly"), 
@@ -323,6 +323,7 @@ GtkItemFactoryEntry data_items[] = {
       CORR_SELECTED, NULL },
 
     { N_("/Data/_Principal components"), NULL, do_menu_op, PCA, NULL },
+    { N_("/Data/_Mahalanobis distances"), NULL, do_menu_op, MAHAL, NULL },
 
     { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>" },
@@ -363,9 +364,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/_Sample"), NULL, NULL, 0, "<Branch>" },
     { N_("/Sample/_Set range..."), NULL, sample_range_dialog, SMPL, NULL },
     { N_("/Sample/_Restore full range"), NULL, restore_sample_callback, 1, NULL },
-    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>" },    
-    { N_("/Sample/Set _frequency, startobs..."), NULL, gretl_callback, 
-      SETOBS, NULL },
+    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>" },
+    { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL },
     { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>" },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
@@ -383,9 +383,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/_Add case markers..."), NULL, gretl_callback, MARKERS, NULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL },
     { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>" },
-    { N_("/Sample/_Interpret as time series..."), NULL, time_series_dialog, 0, NULL },
-    { N_("/Sample/Interpret as _panel..."), NULL, gui_set_panel_structure, 0, NULL },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL },
+    { N_("/Sample/Transpose data..."), NULL, gui_transpose_data, 0, NULL },
 
     /* Variable menu */
     { N_("/_Variable"), NULL, NULL, 0, "<Branch>" },
@@ -401,6 +400,7 @@ GtkItemFactoryEntry data_items[] = {
       NORMAL, NULL },
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
       GAMMA, NULL },
+    { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL }, 
     { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL },
@@ -417,6 +417,7 @@ GtkItemFactoryEntry data_items[] = {
 #ifdef HAVE_TRAMO  
     { N_("/Variable/TRAMO analysis"), NULL, do_tramo_x12a, TRAMO, NULL }, 
 #endif
+    { N_("/Variable/Hurst exponent"), NULL, do_hurst, 0, NULL }, 
     { N_("/Variable/Runs test"), NULL, do_menu_op, RUNS, NULL }, 
     { N_("/Variable/sep2"), NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/Set missing value code..."), NULL, gretl_callback, 
@@ -428,7 +429,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Ordinary Least Squares..."), NULL, model_callback, OLS, NULL },
     { N_("/Model/_Weighted Least Squares..."), NULL, model_callback, WLS, NULL },
     { N_("/Model/sep1"),  NULL, NULL, 0, "<Separator>" },
+#if 0
     { N_("/Model/HCC_M..."), NULL, model_callback, HCCM, NULL },
+#endif
     { N_("/Model/H_eteroskedasticity corrected..."), NULL, model_callback, 
       HSK, NULL },
     { N_("/Model/sep2"),  NULL, NULL, 0, "<Separator>" },
@@ -440,6 +443,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Probit..."), NULL, model_callback, PROBIT, NULL },
     { N_("/Model/To_bit..."), NULL, model_callback, TOBIT, NULL },
     { "/Model/sep5",  NULL, NULL, 0, "<Separator>" },
+    { N_("/Model/Poi_sson..."), NULL, model_callback, POISSON, NULL },
     { N_("/Model/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL },
     { N_("/Model/Least _Absolute Deviation..."), NULL, model_callback, LAD, NULL },
     { N_("/Model/_Rank correlation..."), NULL, gretl_callback, SPEARMAN, NULL },
@@ -553,6 +557,20 @@ void nls_init (void)
     textdomain (PACKAGE);
 }
 
+static void root_check (void)
+{
+    if (getuid() == 0) {
+	int resp;
+
+	resp = yes_no_dialog ("gretl", _("You seem to be running gretl " 
+			      "as root.  Do you really want to do this?"), 
+			      0);
+	if (resp == GRETL_NO) {
+	    exit(EXIT_FAILURE);
+	}
+    }
+}
+
 static void force_english (void)
 {
     setlocale (LC_ALL, "C");
@@ -588,6 +606,7 @@ int main (int argc, char *argv[])
 #endif
 
     set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
+    root_check();
     set_rcfile();
     init_fileptrs();
 
@@ -750,9 +769,6 @@ int main (int argc, char *argv[])
     /* Let a first-time user set the working dir */
     first_time_set_user_dir(); 
 
-    /* enable special copying to clipboard */
-    clip_init(mdata->w);
-
     add_files_to_menus();
 
     session_menu_state(FALSE);
@@ -881,6 +897,8 @@ static void check_varmenu_state (GtkCList *list, gint i, gint j,
 	     (selcount > 1));
 	flip(mdata->ifac, "/Data/Principal components", 
 	     (selcount > 1));
+	flip(mdata->ifac, "/Data/Mahalanobis distances", 
+	     (selcount > 1));
     }
 }
 
@@ -1301,84 +1319,17 @@ drag_data_received  (GtkWidget *widget,
     }	
 }
 
-/* ........................................................... */
-
-static gint 
-special_selection_get (GtkWidget *widget,
-		       GtkSelectionData *selection_data,
-		       guint info,
-		       guint time)
-{
-    gchar *str;
-    gint length;
-
-    str = clipboard_buf;
-    if (str == NULL) return TRUE;
-    length = strlen(str);
-  
-    if (info == TARGET_STRING) {
-	gtk_selection_data_set (selection_data,
-				GDK_SELECTION_TYPE_STRING,
-				8 * sizeof(gchar), 
-				(guchar *) str, 
-				length);
-    } else if (info == TARGET_TEXT || info == TARGET_COMPOUND_TEXT) {
-	guchar *text;
-	gchar c;
-	GdkAtom encoding;
-	gint format;
-	gint new_length;
-
-	c = str[length];
-	str[length] = '\0';
-	gdk_string_to_compound_text(str, &encoding, &format, 
-				    &text, &new_length);
-	gtk_selection_data_set(selection_data, encoding, format, 
-			       text, new_length);
-	gdk_free_compound_text(text);
-	str[length] = c;
-    }
-
-#if 0
-    g_free(str);
-    clipboard_buf = NULL;
-#endif
-
-    return TRUE;
-}
-
-/* ........................................................... */
-
-static void clip_init (GtkWidget *w)
-{
-    GdkAtom clipboard_atom = GDK_NONE;
-    GtkTargetEntry targets[] = {
-	{ "STRING", 0, TARGET_STRING },
-	{ "TEXT",   0, TARGET_TEXT }, 
-	{ "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT }
-    };
-
-    gint n_targets = sizeof(targets) / sizeof(targets[0]);
-  
-    clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
-    gtk_selection_add_targets(w, GDK_SELECTION_PRIMARY,
-			      targets, n_targets);
-    gtk_selection_add_targets(w, clipboard_atom,
-			      targets, n_targets);
-    gtk_signal_connect (GTK_OBJECT(mdata->w), "selection_get",
-			GTK_SIGNAL_FUNC(special_selection_get), NULL);    
-}
-
-/* ........................................................... */
-
 static int native_datafile (void)
 {
     int n = strlen(paths.datfile);
-    extern int olddat; /* settings.c */
     
     if (n > 4) {
-	if (!strcmp(paths.datfile + n - 4, ".gdt")) return 1;
-	if (olddat && !strcmp(paths.datfile + n - 4, ".dat")) return 1;
+	if (!strcmp(paths.datfile + n - 4, ".gdt")) {
+	    return 1;
+	}
+	if (using_olddat() && !strcmp(paths.datfile + n - 4, ".dat")) {
+	    return 1;
+	}
     }
 
     return 0;
