@@ -46,7 +46,6 @@ enum {
   ARG_VALUE
 };
 
-
 static void obs_button_class_init     (ObsButtonClass *klass);
 static void obs_button_init           (ObsButton      *obs_button);
 static void obs_button_finalize       (GtkObject          *object);
@@ -359,7 +358,7 @@ obs_button_size_request (GtkWidget      *widget,
 
     width = MIN_OBS_BUTTON_WIDTH;
     max_string_len = OBSLEN;
-    string_len = strlen(datainfo->endobs) + 1;
+    string_len = strlen(OBS_BUTTON(widget)->pdinfo->endobs) + 1;
     w = MIN (string_len, max_string_len) * cw;
     width = MAX (width, w);  
   
@@ -878,7 +877,7 @@ static void default_output (ObsButton *obs_button)
     char buf[OBSLEN];
     gpointer data;
 
-    ntodate_full(buf, (int) obs_button->adjustment->value, datainfo);
+    ntodate_full(buf, (int) obs_button->adjustment->value, obs_button->pdinfo);
 
     if (strcmp (buf, gtk_entry_get_text(GTK_ENTRY(obs_button)))) {
 	gtk_entry_set_text(GTK_ENTRY(obs_button), buf);
@@ -1021,7 +1020,7 @@ obs_button_update (ObsButton *obs_button)
 	}
     }
 
-    val = dateton(gtk_entry_get_text(GTK_ENTRY (obs_button)), datainfo);
+    val = dateton(gtk_entry_get_text(GTK_ENTRY (obs_button)), obs_button->pdinfo);
 
     if (val < obs_button->adjustment->lower)
 	val = obs_button->adjustment->lower;
@@ -1093,7 +1092,7 @@ obs_button_real_spin (ObsButton *obs_button,
  ***********************************************************/
 
 GtkWidget *
-obs_button_new (GtkAdjustment *adjustment)
+obs_button_new (GtkAdjustment *adjustment, const DATAINFO *pdinfo)
 {
     ObsButton *spin;
 
@@ -1102,6 +1101,8 @@ obs_button_new (GtkAdjustment *adjustment)
 
     spin = gtk_type_new (GTK_TYPE_OBS_BUTTON);
 
+    spin->pdinfo = pdinfo;
+
     obs_button_set_adjustment(spin, adjustment);
     gtk_adjustment_value_changed (adjustment);
 
