@@ -21,6 +21,7 @@
 
 #include "gretl.h"
 #include "selector.h"
+#include "dlgutils.h"
 
 static int default_var;
 static int *xlist;
@@ -46,7 +47,8 @@ struct _selector {
 };
 
 #define WANT_TOGGLES(c) (c == OLS || c == TOBIT || c == ARMA || \
-                         c == GARCH || c == COINT2 || c == TSLS)
+                         c == GARCH || c == COINT2 || c == TSLS || \
+                         c == VAR)
 
 void clear_selector (void)
 {
@@ -249,12 +251,17 @@ static void clear_vars (GtkWidget *w, selector *sr)
     gchar *row[2];
 
     gtk_clist_unselect_all(GTK_CLIST(sr->varlist));
-    if (sr->depvar != NULL) 
+
+    if (sr->depvar != NULL) {
 	gtk_entry_set_text(GTK_ENTRY(sr->depvar), "");
-    if (sr->code == GR_DUMMY || sr->code == GR_3D)
+    }
+
+    if (sr->code == GR_DUMMY || sr->code == GR_3D) {
 	gtk_entry_set_text(GTK_ENTRY(sr->rightvars), "");
-    else
+    } else {
 	gtk_clist_clear(GTK_CLIST(sr->rightvars));
+    }
+
     if (MODEL_CODE(sr->code)) {
 	row[0] = "0";
 	row[1] = "const";
@@ -321,7 +328,7 @@ static int add_to_cmdlist (selector *sr, const char *add)
 static void construct_cmdlist (GtkWidget *w, selector *sr)
 {
     gint i = 0, rows = 0;
-    gchar numstr[6], grvar[6];
+    gchar numstr[8], endbit[12] = {0};
 
     sr->error = 0;
 
@@ -336,7 +343,7 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
     if (sr->code == WLS) {
 	gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
 
-	if (str == NULL || !strlen(str)) {
+	if (str == NULL || *str == '\0') {
 	    errbox(_("You must select a weight variable"));
 	    sr->error = 1;
 	} else {
@@ -344,8 +351,14 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	    sprintf(numstr, "%d ", i);
 	    add_to_cmdlist(sr, numstr);
 	}
-    }
-    else if (sr->code == AR) {
+    } else if (sr->code == POISSON) {
+	gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
+
+	if (str != NULL && *str != '\0') {
+	    i = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(sr->extra)));
+	    sprintf(endbit, " ; %d", i);
+	}
+    } else if (sr->code == AR) {
 	gchar *lags;
 
 	lags = gtk_entry_get_text(GTK_ENTRY(sr->extra));
@@ -356,16 +369,14 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	    add_to_cmdlist(sr, lags);
 	    add_to_cmdlist(sr, " ; ");
 	}
-    }
-    else if (sr->code == VAR || sr->code == COINT || sr->code == COINT2) {
+    } else if (sr->code == VAR || sr->code == COINT || sr->code == COINT2) {
 	GtkAdjustment *adj;
  
 	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sr->extra));
 	i = (gint) adj->value;
 	sprintf(numstr, "%d ", i);
 	add_to_cmdlist(sr, numstr);
-    }
-    else if (sr->code == ARMA || sr->code == GARCH) {
+    } else if (sr->code == ARMA || sr->code == GARCH) {
 	GtkAdjustment *adj;
 
 	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sr->extra));
@@ -379,8 +390,7 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	add_to_cmdlist(sr, numstr);
 
 	add_to_cmdlist(sr, " ; ");
-    }
-    else if (sr->code == GR_DUMMY || sr->code == GR_3D) {
+    } else if (sr->code == GR_DUMMY || sr->code == GR_3D) {
 	gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
 
 	if (str == NULL || !strlen(str)) {
@@ -403,7 +413,7 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	} else {
 	    i = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(sr->depvar)));
 	    if (sr->code == GR_XY || sr->code == GR_IMP) {
-		sprintf(grvar, " %d", i);
+		sprintf(endbit, " %d", i);
 	    } else {
 		sprintf(numstr, "%d", i);
 		add_to_cmdlist(sr, numstr);
@@ -485,8 +495,9 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	}
     }
 
-    if (sr->code == GR_XY || sr->code == GR_IMP)
-	add_to_cmdlist(sr, grvar);
+    if (endbit[0] != '\0') {
+	add_to_cmdlist(sr, endbit);
+    }
 
     if (sr->code == SCATTERS) {
 	GtkWidget *active_item = GTK_OPTION_MENU(scatters_menu)->menu_item;
@@ -496,8 +507,9 @@ static void construct_cmdlist (GtkWidget *w, selector *sr)
 	}
     }
 
-    if (sr->error) 
+    if (sr->error) {
 	gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "clicked");
+    }
 }
 
 static void destroy_selector (GtkWidget *w, selector *sr) 
@@ -530,6 +542,8 @@ static char *est_str (int cmdnum)
 	return N_("Tobit");
     case LOGISTIC:
 	return N_("Logistic");
+    case POISSON:
+	return N_("Poisson");
     case POOLED:
 	return N_("Pooled OLS");
     case WLS:
@@ -563,6 +577,8 @@ static char *extra_string (int cmdnum)
     switch (cmdnum) {
     case WLS:
 	return N_("Weight variable");
+    case POISSON:
+	return N_("Offset variable");
     case TSLS:
 	return N_("Instruments");
     case AR:
@@ -587,18 +603,35 @@ dblclick_dialog_row (GtkCList *clist, gint row, gint column,
     }
 }
 
+static void 
+listvar_special_undo (GtkCList *clist, gint arg1, gint arg2, gpointer p)
+{
+    gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
+    gtk_clist_set_reorderable(clist, FALSE);
+}
+
 static gint
-remove_right_click (GtkWidget *widget, GdkEventButton *event, 
-		    selector *sr)
+listvar_special_click (GtkWidget *widget, GdkEventButton *event, 
+		       selector *sr)
 {
     GdkWindow *topwin;
     GdkModifierType mods;
 
     topwin = gtk_widget_get_parent_window(sr->rightvars);
     gdk_window_get_pointer(topwin, NULL, NULL, &mods); 
-    if (mods & GDK_BUTTON3_MASK) 
+
+    if (mods & GDK_BUTTON2_MASK) {
+	gtk_clist_set_selection_mode(GTK_CLIST(sr->rightvars), 
+				     GTK_SELECTION_SINGLE);
+	gtk_clist_set_reorderable(GTK_CLIST(sr->rightvars), TRUE);
+    } 
+
+    if (mods & GDK_BUTTON3_MASK) {
 	remove_from_right_callback (NULL, sr);
-    return TRUE;
+	return TRUE;
+    }
+
+    return FALSE;
 }
 
 static gint
@@ -697,10 +730,12 @@ static void build_depvar_section (selector *sr, GtkWidget *right_vbox)
 {
     GtkWidget *tmp, *depvar_hbox;
 
-    if (sr->code == VAR)
+    if (sr->code == VAR) {
 	tmp = gtk_label_new (_("First dependent variable"));
-    else
+    } else {
 	tmp = gtk_label_new (_("Dependent variable"));
+    }
+
     gtk_box_pack_start(GTK_BOX(right_vbox), tmp, FALSE, TRUE, 0);
     gtk_widget_show(tmp);
 
@@ -738,11 +773,20 @@ static void lag_order_spin (selector *sr, GtkWidget *right_vbox)
 {
     GtkWidget *tmp, *midhbox;
     GtkObject *adj;
-    gfloat order = datainfo->pd;
+    gfloat order; 
+    gfloat ordermax;
+
+    if (datainfo->n < 72) {
+	ordermax = datainfo->n / 2;
+    } else {
+	ordermax = 36;
+    }
+
+    order = (datainfo->pd > 12)? 12 : datainfo->pd;
 
     midhbox = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("lag order:"));
-    adj = gtk_adjustment_new(order, 1, 24, 1, 1, 1);
+    adj = gtk_adjustment_new(order, 1, ordermax, 1, 1, 1);
     sr->extra = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
     gtk_box_pack_start (GTK_BOX (midhbox), tmp, FALSE, FALSE, 5);
     gtk_widget_show(tmp);
@@ -863,19 +907,19 @@ static void build_mid_section (selector *sr, GtkWidget *right_vbox)
 	gtk_widget_show(tmp);
     }	
 
-    if (sr->code == WLS || sr->code == GR_DUMMY || sr->code == GR_3D) 
+    if (sr->code == WLS || sr->code == POISSON ||
+	sr->code == GR_DUMMY || sr->code == GR_3D) {
 	extra_var_box (sr, right_vbox);
-    else if (sr->code == COINT || sr->code == COINT2)
+    } else if (sr->code == COINT || sr->code == COINT2) {
 	lag_order_spin (sr, right_vbox);
-    else if (sr->code == TSLS)
+    } else if (sr->code == TSLS) {
 	auxiliary_varlist_box (sr, right_vbox);
-    else if (sr->code == AR) {
+    } else if (sr->code == AR) {
 	sr->extra = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(right_vbox), sr->extra, 
 			   FALSE, TRUE, 0);
 	gtk_widget_show(sr->extra); 
-    }
-    else if (sr->code == VAR) {
+    } else if (sr->code == VAR) {
 	lag_order_spin (sr, right_vbox);
 	tmp = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(right_vbox), tmp, FALSE, FALSE, 0);
@@ -893,7 +937,8 @@ static void build_mid_section (selector *sr, GtkWidget *right_vbox)
 
 static int screen_scalar (int i, int c)
 {
-    if ((MODEL_CODE(c) || GRAPH_CODE(c) || c == LAGS || c == DIFF || c == LDIFF)
+    if ((MODEL_CODE(c) || COINT_CODE(c) || GRAPH_CODE(c) || 
+	 c == LAGS || c == DIFF || c == LDIFF)
 	&& datainfo->vector[i] == 0)
 	return 1;
     return 0;
@@ -944,6 +989,15 @@ static void robust_callback (GtkWidget *w,  selector *sr)
     }
 }
 
+static void robust_config_button (GtkWidget *w,  GtkWidget *b)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	gtk_widget_set_sensitive(b, TRUE);
+    } else {
+	gtk_widget_set_sensitive(b, FALSE);
+    }
+}
+
 static void verbose_callback (GtkWidget *w,  selector *sr)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
@@ -989,27 +1043,59 @@ static void build_pq_spinners (selector *sr)
     gtk_widget_show(hbox);
 }
 
+static void hc_config (GtkWidget *w, gpointer p)
+{
+    options_dialog(p, 4, NULL);
+}
+
 static void 
 build_selector_switches (selector *sr) 
 {
     GtkWidget *hbox, *tmp;
 
-    if (sr->code == OLS || sr->code == GARCH || sr->code == TSLS) {
-	tmp = gtk_check_button_new_with_label(_("Robust standard errors"));
-	gtk_signal_connect(GTK_OBJECT(tmp), "toggled",
-			   GTK_SIGNAL_FUNC(robust_callback), sr);
-	hbox = gtk_hbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+    if (sr->code == OLS || sr->code == GARCH || 
+	sr->code == TSLS || sr->code == VAR) {
+	GtkWidget *b1, *b2;
+
+	tmp = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sr->dlg)->vbox),
+			   tmp, FALSE, FALSE, 0);
 	gtk_widget_show(tmp);
 
+	b1 = gtk_check_button_new_with_label(_("Robust standard errors"));
+	gtk_signal_connect(GTK_OBJECT(b1), "toggled",
+			   GTK_SIGNAL_FUNC(robust_callback), sr);
+	if (using_hc_by_default()) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1), TRUE);
+	}
+
+	hbox = gtk_hbox_new(FALSE, 5);
+
+	gtk_box_pack_start(GTK_BOX(hbox), b1, FALSE, FALSE, 0);
+	gtk_widget_show(b1);
+
+	b2 = gtk_button_new_with_label(_("configure"));
+	gtk_signal_connect(GTK_OBJECT(b2), "clicked",
+			   GTK_SIGNAL_FUNC(hc_config), sr);
+	gtk_widget_set_sensitive(b2, using_hc_by_default());
+
+	gtk_signal_connect(GTK_OBJECT(b1), "toggled",
+			   GTK_SIGNAL_FUNC(robust_config_button), b2);	
+
+	gtk_box_pack_start(GTK_BOX(hbox), b2, FALSE, FALSE, 0);
+	gtk_widget_show(b2);
+
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sr->dlg)->vbox),
 			   hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
     }
-    
+
     if (sr->code == TOBIT || sr->code == ARMA || sr->code == GARCH ||
-	sr->code == COINT2) {
-	if (sr->code == COINT2) {
+	sr->code == COINT2 || sr->code == VAR) {
+	if (sr->code == VAR) {
+	    tmp = gtk_check_button_new_with_label
+		(_("Show impulse responses"));
+	} else if (sr->code == COINT2) {
 	    tmp = gtk_check_button_new_with_label
 		(_("Show details of regressions"));
 	} else {
@@ -1101,7 +1187,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 
     selector_init(sr, cmdcode, title);
 
-    if (MODEL_CODE(cmdcode))
+    if (MODEL_CODE(cmdcode) || COINT_CODE(cmdcode))
 	strcpy(topstr, _(est_str(cmdcode)));
     else if (cmdcode == GR_XY)
 	strcpy(topstr, _("XY scatterplot"));
@@ -1162,22 +1248,26 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     gtk_box_pack_start(GTK_BOX(right_vbox), tmp, FALSE, TRUE, 0);
     gtk_widget_show(tmp);
 
-    /* for models: top right -> dependent variable */
-    if (MODEL_CODE(cmdcode)) 
+    if (MODEL_CODE(cmdcode)) {
+	/* models: top right -> dependent variable */
 	build_depvar_section(sr, right_vbox);
-    /* graphs: top right -> x-axis variable */
-    else if (cmdcode == GR_XY || cmdcode == GR_IMP || cmdcode == GR_DUMMY
-	     || cmdcode == SCATTERS || cmdcode == GR_3D)
+    } else if (cmdcode == GR_XY || cmdcode == GR_IMP || cmdcode == GR_DUMMY
+	       || cmdcode == SCATTERS || cmdcode == GR_3D) {
+	/* graphs: top right -> x-axis variable */
 	build_x_axis_section(sr, right_vbox);
+    }
 
     /* middle right: used for some estimators and factored plot */
     if (cmdcode == WLS || cmdcode == AR || cmdcode == TSLS || 
 	cmdcode == VAR || cmdcode == COINT || cmdcode == COINT2 ||
-	cmdcode == GR_DUMMY || cmdcode == GR_3D) 
+	cmdcode == POISSON || cmdcode == GR_DUMMY || cmdcode == GR_3D) {
 	build_mid_section(sr, right_vbox);
+    }
     
     /* lower right: selected (independent) variables */
-    if (MODEL_CODE(cmdcode)) {
+    if (COINT_CODE(cmdcode)) {
+	tmp = gtk_label_new(_("Variables to test"));
+    } else if (MODEL_CODE(cmdcode)) {
 	tmp = gtk_label_new(_("Independent variables"));
     } else if (cmdcode == GR_XY || cmdcode == GR_IMP) {
 	tmp = gtk_label_new(_("Y-axis variables"));
@@ -1224,31 +1314,34 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 	sr->rightvars = gtk_clist_new(2);
 	gtk_clist_clear(GTK_CLIST(sr->rightvars));
 
-	if (MODEL_CODE(cmdcode) && xlist != NULL) {
-	    for (i=1; i<=xlist[0]; i++) {
-		gchar *row[2];
-		gchar id[4];
+	if (MODEL_CODE(cmdcode)) {
+	    if (xlist != NULL) {
+		for (i=1; i<=xlist[0]; i++) {
+		    gchar *row[2];
+		    gchar id[4];
 
-		sprintf(id, "%d", xlist[i]);
-		row[0] = id;
-		row[1] = datainfo->varname[xlist[i]];
+		    sprintf(id, "%d", xlist[i]);
+		    row[0] = id;
+		    row[1] = datainfo->varname[xlist[i]];
+		    gtk_clist_append(GTK_CLIST(sr->rightvars), row);
+		}
+	    } else if (cmdcode != VAR) {
+		gchar *row[2];
+
+		row[0] = "0";
+		row[1] = "const";
 		gtk_clist_append(GTK_CLIST(sr->rightvars), row);
 	    }
-	} else if (MODEL_CODE(cmdcode) && cmdcode != COINT &&
-		   cmdcode != COINT2 && cmdcode != VAR) {
-	    gchar *row[2];
-
-	    row[0] = "0";
-	    row[1] = "const";
-	    gtk_clist_append(GTK_CLIST(sr->rightvars), row);
 	}
 
 	gtk_clist_set_column_width (GTK_CLIST(sr->rightvars), 1, 80 * gui_scale);
 	gtk_widget_set_usize (sr->rightvars, 80 * gui_scale, 120 * gui_scale);
 	gtk_clist_set_selection_mode (GTK_CLIST(sr->rightvars),
 				      GTK_SELECTION_EXTENDED);
+	gtk_signal_connect(GTK_OBJECT(sr->rightvars), "row-move",
+			   (GtkSignalFunc) listvar_special_undo, sr);
 	gtk_signal_connect(GTK_OBJECT(sr->rightvars), "button_press_event",
-			   (GtkSignalFunc) remove_right_click, sr);
+			   (GtkSignalFunc) listvar_special_click, sr);
 	gtk_widget_show(sr->rightvars); 
 	gtk_container_add(GTK_CONTAINER(scroller), sr->rightvars);
 
@@ -1639,6 +1732,14 @@ const char *selector_list (const selector *sr)
     return sr->cmdlist;
 }
 
+int selector_list_hasconst (const selector *sr)
+{
+    int hc = sr->cmdlist != NULL && 
+	strstr(sr->cmdlist, " 0") != NULL;
+
+    return hc;
+}
+
 gpointer selector_get_data (const selector *sr)
 {
     return sr->data;
