@@ -28,6 +28,7 @@
 #include "textbuf.h"
 #include "textutil.h"
 #include "boxplots.h"
+#include "dlgutils.h"
 
 #ifdef OLD_GTK
 # include <gtkextra/gtkiconfilesel.h>
@@ -444,22 +445,16 @@ void model_test_callback (gpointer data, guint action, GtkWidget *widget)
 
 void selector_callback (gpointer data, guint action, GtkWidget *widget)
 {
-    char title[36];
     windata_t *vwin = (windata_t *) data;
-    void (*okfunc)() = NULL;
 
     if (action == COINT) {
-	selection_dialog (_("gretl: cointegration test"), do_coint, action);
-	return;
-    }
+	selection_dialog(_("gretl: cointegration test"), do_coint, action);
+    } else if (action == COINT2) {
+	selection_dialog(_("gretl: cointegration test"), do_coint2, action);
+    } else if (action == GR_XY || action == GR_IMP || action == GR_DUMMY
+	       || action == SCATTERS || action == GR_3D) {
+	void (*okfunc)() = NULL;
 
-    if (action == COINT2) {
-	selection_dialog (_("gretl: cointegration test"), do_coint2, action);
-	return;
-    }
-
-    if (action == GR_XY || action == GR_IMP || action == GR_DUMMY
-	|| action == SCATTERS || action == GR_3D) {
 	switch (action) {
 	case GR_XY:
 	case GR_IMP:
@@ -477,29 +472,16 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 	default:
 	    return;
 	}
-	selection_dialog (_("gretl: define graph"), okfunc, action);
-	return;
+	selection_dialog(_("gretl: define graph"), okfunc, action);
+    } else if (action == ADD || action == OMIT) {
+	simple_selection(_("gretl: model tests"), do_add_omit, action, vwin);
+    } else if (action == COEFFSUM) {
+	simple_selection(_("gretl: model tests"), do_coeff_sum, action, vwin);
+    } else if (action == GR_PLOT) {
+	simple_selection(_("gretl: model tests"), do_graph_from_selector, action, vwin);
+    } else {
+	errbox("selector_callback: code was not recognized");
     }
-
-    if (action == ADD || action == OMIT) {
-	strcpy(title, _("gretl: model tests"));
-	simple_selection(title, do_add_omit, action, vwin);
-	return;
-    }
-
-    if (action == COEFFSUM) {
-	strcpy(title, _("gretl: model tests"));
-	simple_selection(title, do_coeff_sum, action, vwin);
-	return;
-    }
-
-    if (action == GR_PLOT) {
-	strcpy(title, _("gretl: model tests"));
-	simple_selection(title, do_graph_from_selector, action, vwin);
-	return;
-    }
-
-    errbox("selector_callback: code was not recognized");
 }
 
 /* ........................................................... */
@@ -520,12 +502,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	okfunc = do_samplebool;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
-    case SETOBS:
-	strcpy(title, _("gretl: set data frequency"));
-	strcpy(query, _("Enter integer frequency and\n"
-	       "starting observation string:"));
-	okfunc = do_setobs;
-	break;
     case SETSEED:
 	strcpy(title, _("gretl: random variables"));
 	strcpy(query, _("Enter integer seed for\n"
@@ -707,241 +683,113 @@ void add_random_callback (gpointer data, guint code, GtkWidget *widget)
 
 /* ........................................................... */
 
-static void fix_obsstr (char *str)
-{
-    char pt = get_local_decpoint();
-    char *p;
-
-    p = strchr(str, ':');
-    if (p != NULL) {
-	*p = pt;
-    }
-
-    if (pt == ',' && (p = strchr(str, '.'))) {
-	*p = pt;
-    }
-
-    if (pt == '.' && (p = strchr(str, ','))) {
-	*p = pt;
-    }
-}
-
-static void prep_spreadsheet (GtkWidget *widget, dialog_t *ddata)
+static void name_first_var (GtkWidget *widget, dialog_t *ddata) 
 {
+    DATAINFO *pdinfo = (DATAINFO *) dialog_data_get_data(ddata);
     const gchar *buf;
-    char dataspec[32];
-    char *test, stobs[OBSLEN], endobs[OBSLEN], firstvar[VNAMELEN];
-    double sd0, ed0;
 
     buf = dialog_data_get_text(ddata);
-    if (buf == NULL) return;
 
-    strncpy(dataspec, buf, 31);
+    if (buf == NULL || validate_varname(buf)) return;
 
-    /* check validity of dataspec */
-    if (sscanf(dataspec, "%8s %8s %8s", stobs, endobs, firstvar) != 3) {
-	errbox(_("Insufficient dataset information supplied"));
-	return;
-    }
-
-    /* daily data: special */
-    if (datainfo->pd == 5 || datainfo->pd == 7) {
-	int err = 0;
-	sd0 = (double) get_epoch_day(stobs); 
-	ed0 = (double) get_epoch_day(endobs);
-
-	if (sd0 < 0) {
-	    err = 1;
-	    sprintf(errtext, _("Invalid starting observation '%s'"), stobs);
-	}
-	if (!err && ed0 < 0) {
-	    err = 1;
-	    sprintf(errtext, _("Invalid ending observation '%s'"), endobs);
-	}
-	if (err) {
-	    errbox(errtext);
-	    return;
-	}
-    } else { /* not daily data */
-	fix_obsstr(stobs);
-	fix_obsstr(endobs);
-
-	sd0 = strtod(stobs, &test);
-	if (!strcmp(stobs, test) || *test != 0 || sd0 < 0) {
-	    sprintf(errtext, _("Invalid starting observation '%s'"), stobs);
-	    errbox(errtext);
-	    return;
-	}
-
-	ed0 = strtod(endobs, &test);
-	if (!strcmp(endobs, test) || *test != 0 || ed0 < 0) {
-	    sprintf(errtext, _("Invalid ending observation '%s'"), endobs);
-	    errbox(errtext);
-	    return;
-	}
-    }
-
-    if (sd0 > ed0) {
-	sprintf(errtext, _("Empty data range '%s - %s'"), stobs, endobs);
-	errbox(errtext);
-	return;
-    }
-
-    colonize_obs(stobs);
-    colonize_obs(endobs);
-
-    if (datainfo->pd == 999) { /* panel */
-	char unit[8], period[8];
-
-	/* try to infer structure from ending obs */
-	if (sscanf(endobs, "%[^:]:%s", unit, period) == 2) { 
-	    datainfo->pd = atoi(period);
-	    fprintf(stderr, I_("Setting data frequency = %d\n"), datainfo->pd);
-	} else {
-	    sprintf(errtext, _("Invalid ending observation '%s'"), endobs);
-	    errbox(errtext);
-	    return;	    
-	}
-    }    
-
-    if (datainfo->pd == 1) {
-	size_t i, n;
-	
-	n = strlen(stobs);
-	for (i=0; i<n; i++) {
-	    if (!isdigit((unsigned char) stobs[i])) {
-		sprintf(errtext, _("Invalid starting observation '%s'\n"
-				   "for data frequency 1"), stobs);
-		errbox(errtext);
-		return;
-	    }
-	}
-	n = strlen(endobs);
-	for (i=0; i<n; i++) {
-	    if (!isdigit((unsigned char) endobs[i])) {
-		sprintf(errtext, _("Invalid ending observation '%s'\n"
-				   "for data frequency 1"), endobs);
-		errbox(errtext);
-		return;
-	    }
-	}	
-    } 
-    else if (datainfo->pd != 5 && datainfo->pd != 7) { 
-	char year[8], subper[8];
-
-	if (sscanf(stobs, "%[^:]:%s", year, subper) != 2 ||
-	    strlen(year) > 4 || atoi(subper) > datainfo->pd ||
-	    (datainfo->pd < 10 && strlen(subper) != 1) ||
-	    (datainfo->pd >= 10 && strlen(subper) != 2)) {
-	    sprintf(errtext, _("Invalid starting observation '%s'\n"
-			       "for data frequency %d"), stobs, datainfo->pd);
-	    errbox(errtext);
-	    return;
-	}
-	if (sscanf(endobs, "%[^:]:%s", year, subper) != 2 ||
-	    strlen(year) > 4 || atoi(subper) > datainfo->pd ||
-	    (datainfo->pd < 10 && strlen(subper) != 1) ||
-	    (datainfo->pd >= 10 && strlen(subper) != 2)) {
-	    sprintf(errtext, _("Invalid ending observation '%s'\n"
-			       "for data frequency %d"), endobs, datainfo->pd);
-	    errbox(errtext);
-	    return;
-	}	    
-    }
+    pdinfo->varname[1][0] = 0;
+    strncat(pdinfo->varname[1], buf, VNAMELEN - 1);
 
     close_dialog(ddata);
 
+    show_spreadsheet(datainfo);
+}
+
+static void first_var_dialog (DATAINFO *pdinfo) 
+{
+    edit_dialog (_("gretl: name variable"), 
+		 _("Enter name for new variable\n"
+		   "(max. 8 characters)"),
+		 NULL, name_first_var, pdinfo, 
+		 0, 0);
+}
+
+static int prep_spreadsheet (const char *dataspec)
+{
+    char stobs[OBSLEN], endobs[OBSLEN];
+    int t1, t2;
+
+    if (sscanf(dataspec, "%10s %10s", stobs, endobs) != 2) {
+	errbox(_("Insufficient dataset information supplied"));
+	return 1;
+    }
+
+    t1 = dateton(stobs, datainfo);
+    t2 = dateton(endobs, datainfo);
+
     strcpy(datainfo->stobs, stobs);
     strcpy(datainfo->endobs, endobs);
-    datainfo->sd0 = sd0;
-    datainfo->n = -1;
-    datainfo->n = dateton(datainfo->endobs, datainfo) + 1; 
 
-    if (datainfo->n <= 0) {
-	errbox("Got zero-length data series");
-	return;
-    }
+    datainfo->sd0 = get_date_x(datainfo->pd, datainfo->stobs);
+
+    datainfo->n = t2 - t1 + 1;
+    datainfo->t1 = 0;
+    datainfo->t2 = datainfo->n - 1;
 
     datainfo->v = 2;
     start_new_Z(&Z, datainfo, 0);
     datainfo->markers = 0;
 
-    strcpy(datainfo->varname[1], firstvar);
+    first_var_dialog(datainfo);
 
-    show_spreadsheet(datainfo);
+    return 0;
 }
 
-/* ........................................................... */
+static void n_obs_callback (GtkWidget *w, dialog_t *ddata)
+{
+    const gchar *buf;
+    char obsstr[32];
+    int n;
+
+    buf = dialog_data_get_text(ddata);
+    if (buf == NULL) return;
+
+    n = atoi(buf);
+    if (n < 1 || n > 10000) {
+	errbox(_("Invalid number of observations"));
+	return;
+    }
+
+    close_dialog(ddata);
+
+    datainfo->n = n;
+    sprintf(obsstr, "1 %d", n);
+    prep_spreadsheet(obsstr);
+}
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget) 
 {
-    windata_t *wdata = NULL;
     gchar *obsstr = NULL;
 
     if (pd_code == 0) {
-	datainfo->time_series = 0;
+	datainfo->structure = CROSS_SECTION;
 	datainfo->pd = 1;
+	datainfo->sd0 = 1.0;
+	strcpy(datainfo->stobs, "1");
+	edit_dialog (_("gretl: create data set"), 
+		     _("Number of observations:"), "50",
+		     n_obs_callback, NULL, 0, 0);
+	return;
     } else {
-	datainfo->time_series = TIME_SERIES;
+	datainfo->structure = TIME_SERIES;
 	datainfo->pd = pd_code;
     }
 
-    switch (pd_code) {
-    case 0:
-	datainfo->pd = 1;
-	obsstr = g_strdup_printf("1 50 %s", _("newvar"));
-	break;
-    case 1:
-	obsstr = g_strdup_printf("1950 2001 %s", _("newvar"));
-	break;
-    case 4:
-	obsstr = g_strdup_printf("1950:1 2001:4 %s", _("newvar"));
-	break;
-    case 5:
-	obsstr = g_strdup_printf("99/01/18 01/03/31 %s", _("newvar"));
-	break;
-    case 7:
-	obsstr = g_strdup_printf("99/01/18 01/03/31 %s", _("newvar"));
-	break;
-    case 12:
-	obsstr = g_strdup_printf("1950:01 2001:12 %s", _("newvar"));
-	break;
-    case 24:
-	obsstr = g_strdup_printf("0:01 0:24 %s", _("newvar"));
-	break;
-    case 52:
-	obsstr = g_strdup_printf("1990:01 2001:52 %s", _("newvar"));
-	break;
+    compute_default_ts_info(datainfo, 1);
+
+    sample_range_dialog(&obsstr, CREATE_DATASET, NULL);
+
+    if (obsstr != NULL) {
+	prep_spreadsheet(obsstr);
+	g_free(obsstr);
     }
-
-    edit_dialog (_("gretl: create data set"), 
-		 _("Enter start and end obs for new data set\n"
-		   "and name of first var to add:"), 
-		 obsstr, 
-		 prep_spreadsheet, wdata, 
-		 0, 0);
-
-    g_free(obsstr);
 }
 
-#if 0
-void start_panel_callback (gpointer data, guint u, GtkWidget *widget) 
-{
-    windata_t *wdata = NULL;
-
-    datainfo->pd = 999;
-
-    edit_dialog (_("gretl: create panel data set"), 
-		 _("Enter starting and ending observations and\n"
-		   "the name of the first variable to add.\n"
-		   "The example below is suitable for 20 units\n"
-		   "observed over 10 periods"), 
-		 "1:01 10:20 newvar", 
-		 prep_spreadsheet, wdata, 
-		 0, 0);
-}
-#endif
-
 void do_nistcheck (gpointer p, guint v, GtkWidget *w)
 {
     void *handle;
