@@ -10,7 +10,8 @@
 #if GTK_MAJOR_VERSION == 1
 void selectrow (GtkCList *clist, gint row, gint column, 
                 GdkEventButton *event, gpointer data);
-
+void unselectrow (GtkCList *clist, gint row, gint column, 
+		  GdkEventButton *event, gpointer data);
 #else
 
 void listbox_select_row (GtkTreeSelection *selection, gpointer data);
