@@ -202,14 +202,13 @@ static int console_sample (const DATAINFO *pdinfo, int code)
 	pd = pdinfo->pd;
 	t1 = pdinfo->t1;
 	t2 = pdinfo->t2;
-	ts = pdinfo->time_series;
+	ts = pdinfo->structure;
 	sd0 = pdinfo->sd0;
-    }
-    else if (code == SAMPLE_CHECK) {
+    } else if (code == SAMPLE_CHECK) {
 	if (pdinfo->pd != pd ||
 	    pdinfo->t1 != t1 ||
 	    pdinfo->t2 != t2 ||
-	    pdinfo->time_series != ts ||
+	    pdinfo->structure != ts ||
 	    pdinfo->sd0 != sd0) {
 	    ret = 1;
 	}
@@ -281,17 +280,16 @@ static void console_exec (void)
 		  CONSOLE_EXEC, NULL);
 
     if (looprun) { 
-	if (loop_exec(loop, execline, 
-		      &Z, &datainfo, models,
-		      &echo_off, console_prn)) {
-	    return;
-	}
+	loop_exec(loop, execline, &Z, &datainfo, models,
+		  &echo_off, console_prn);
 	gretl_loop_destroy(loop);
 	loop = NULL;
 	looprun = 0;
     }    
 
-    if (console_prn->fp == NULL) redirected = 0;
+    if (console_prn->fp == NULL) {
+	redirected = 0;
+    }
 
 #ifndef OLD_GTK
     gtk_text_buffer_get_end_iter(buf, &start);
