@@ -66,7 +66,7 @@ enum db_data_actions {
 static void set_time_series (DATAINFO *pdinfo)
 {
     if (pdinfo->pd != 1 || strcmp(pdinfo->stobs, "1")) { 
-	pdinfo->time_series = TIME_SERIES;
+	pdinfo->structure = TIME_SERIES;
     }
 }
 
@@ -188,7 +188,7 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
     int err, lines[1], list[3];
     char pd[7];
 
-    if (dbdinfo->time_series == 0) { /* undated */
+    if (dbdinfo->structure == CROSS_SECTION) {
 	list[0] = 1; list[1] = 1;
 	err = boxplots(list, NULL, dbZ, dbdinfo, 0);
 	if (err) {
@@ -225,10 +225,12 @@ static void
 init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
 {
     pdinfo->pd = sinfo->pd;
+
     strcpy(pdinfo->stobs, sinfo->stobs);
     strcpy(pdinfo->endobs, sinfo->endobs);
     colonize_obs(pdinfo->stobs);
     colonize_obs(pdinfo->endobs);
+
     pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
     pdinfo->n = sinfo->nobs;
     pdinfo->v = 2;
@@ -318,13 +320,17 @@ static void add_dbdata (windata_t *dbwin, double **dbZ, SERIESINFO *sinfo)
 	strcpy(VARLABEL(datainfo, dbv), sinfo->descrip);
 	get_db_padding(sinfo, datainfo, &pad1, &pad2);
 
+	fprintf(stderr, "pad1 = %d, pad2 = %d\n", pad1, pad2);
+
 	if (pad1 > 0) {
 	    fprintf(stderr, "Padding at start, %d obs\n", pad1);
 	    for (t=0; t<pad1; t++) {
 		Z[dbv][t] = NADBL;
 	    }
 	    start = pad1;
-	} else start = 0;
+	} else {
+	    start = 0;
+	}
 
 	if (pad2 > 0) {
 	    fprintf(stderr, "Padding at end, %d obs\n", pad2);
@@ -332,7 +338,9 @@ static void add_dbdata (windata_t *dbwin, double **dbZ, SERIESINFO *sinfo)
 		Z[dbv][t] = NADBL;
 	    }
 	    stop = n - pad2;
-	} else stop = n;
+	} else {
+	    stop = n;
+	}
 
 	/* fill in actual data values */
 	fprintf(stderr, "Filling in values from %d to %d\n", start, stop - 1);
@@ -496,40 +504,20 @@ static void build_db_popup (windata_t *win, int cb)
     win->popup = gtk_menu_new();
 
     add_popup_item(_("Display"), win->popup, 
-#ifndef OLD_GTK
 		   G_CALLBACK(gui_display_series), 
-#else
-		   gui_display_series,
-#endif
 		   win);
     add_popup_item(_("Graph"), win->popup, 
-#ifndef OLD_GTK
 		   G_CALLBACK(gui_graph_series), 
-#else
-		   gui_graph_series,
-#endif
 		   win);
     add_popup_item(_("Import"), win->popup, 
-#ifndef OLD_GTK
 		   G_CALLBACK(gui_import_series), 
-#else
-		   gui_import_series,
-#endif
 		   win);
     add_popup_item(_("Find..."), win->popup, 
-#ifndef OLD_GTK
 		   G_CALLBACK(db_menu_find), 
-#else
-		   db_menu_find,
-#endif
 		   win);
     if (cb) {
 	add_popup_item(_("Codebook"), win->popup, 
-#ifndef OLD_GTK
 		       G_CALLBACK(db_view_codebook), 
-#else
-		       db_view_codebook,
-#endif
 		       win);
     }
 }
@@ -601,7 +589,7 @@ static void test_db_book (const char *fname, int *cb)
 
     strcpy(testname, fname);
     strcat(testname, ".cb");
-    fp = fopen(testname, "r");
+    fp = gretl_fopen(testname, "r");
     if (fp == NULL) {
 	*cb = 0;
     } else {
@@ -610,29 +598,6 @@ static void test_db_book (const char *fname, int *cb)
     }
 }
 
-/* ........................................................... */
-
-#ifdef OLD_GTK
-static gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
-{
-    if (key->keyval == GDK_q) { 
-	gtk_widget_destroy(vwin->w);
-    }
-    else if (key->keyval == GDK_f) {
-	GdkModifierType mods;
-
-	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
-	if (mods & GDK_CONTROL_MASK) {
-	    menu_find(vwin, 1, NULL);
-	    return TRUE;
-	}	
-    }
-    return FALSE;
-}
-#endif
-
-/* ........................................................... */
-
 static int display_db_series_list (int action, char *fname, char *buf)
 {
     GtkWidget *listbox, *closebutton;
@@ -702,12 +667,9 @@ static int display_db_series_list (int action, char *fname, char *buf)
 
     closebutton = gtk_button_new_with_label(_("Close"));
     gtk_box_pack_start (GTK_BOX (main_vbox), closebutton, FALSE, TRUE, 0);
-#ifndef OLD_GTK
     g_signal_connect (G_OBJECT(closebutton), "clicked", 
 		      G_CALLBACK(delete_widget), dbwin->w);
-#else
-    gtk_signal_connect (GTK_OBJECT(closebutton), "clicked", 
-			GTK_SIGNAL_FUNC(delete_widget), dbwin->w);
+#ifdef OLD_GTK
     gtk_signal_connect (GTK_OBJECT(dbwin->w), "key_press_event",
 			GTK_SIGNAL_FUNC(catch_listbox_key),
 			dbwin);
@@ -735,9 +697,9 @@ static int display_db_series_list (int action, char *fname, char *buf)
 static int check_serinfo (char *str, char *sername)
 {
     char pdc;
-    char stobs[11], endobs[11];
-    int n;
+    char stobs[OBSLEN], endobs[OBSLEN];
     char msg[64];
+    int n, err = 0;
 
     if (!isalpha((unsigned char) sername[0]) || 
 	sscanf(str, "%c %10s - %10s %*s = %d", 
@@ -748,9 +710,10 @@ static int check_serinfo (char *str, char *sername)
 	 pdc != 'D' && pdc != 'B')) {
 	sprintf(msg, _("Database parse error at variable '%s'"), sername);
 	errbox(msg);
-	return 1;
+	err = 1;
     }
-    return 0;
+
+    return err;
 }
 
 /* ........................................................... */
@@ -818,15 +781,10 @@ static void db_drag_connect (windata_t *dbwin)
     gtk_drag_source_set(dbwin->listbox, GDK_BUTTON1_MASK,
 			&gretl_drag_targets[GRETL_POINTER],
 			1, GDK_ACTION_COPY);
-#ifndef OLD_GTK
+
     g_signal_connect(G_OBJECT(dbwin->listbox), "drag_data_get",
 		     G_CALLBACK(db_drag_series),
 		     dbwin);
-#else
-    gtk_signal_connect(GTK_OBJECT(dbwin->listbox), "drag_data_get",
-		       GTK_SIGNAL_FUNC(db_drag_series),
-		       dbwin);
-#endif
 }
 
 /* ........................................................... */
@@ -847,7 +805,7 @@ static int populate_series_list (windata_t *dbwin)
 
     strcpy(dbidx, dbwin->fname);
     strcat(dbidx, ".idx");
-    fp = fopen(dbidx, "r");
+    fp = gretl_fopen(dbidx, "r");
     if (fp == NULL) {
 	errbox(_("Couldn't open database index file"));
 	return 1;
@@ -1038,7 +996,7 @@ static int rats_populate_series_list (windata_t *dbwin)
     FILE *fp;
     db_table *tbl;
 
-    fp = fopen(dbwin->fname, "rb");
+    fp = gretl_fopen(dbwin->fname, "rb");
     if (fp == NULL) {
 	errbox(_("Couldn't open RATS data file"));
 	return 1;
@@ -1289,11 +1247,11 @@ void open_named_db_list (char *dbname)
 
     if (has_rats_suffix(dbname)) action = RATS_SERIES;
 
-    fp = fopen(dbname, "rb");
+    fp = gretl_fopen(dbname, "rb");
 
     if (fp == NULL && action != RATS_SERIES) {
 	strcat(dbname, ".bin");
-	fp = fopen(dbname, "rb");
+	fp = gretl_fopen(dbname, "rb");
     }
 
     if (fp == NULL) {
@@ -1488,21 +1446,21 @@ static int ggz_extract (char *errbuf, char *ggzname)
         return 1;
     }
 
-    fidx = fopen(idxname, "wb");
+    fidx = gretl_fopen(idxname, "wb");
     if (fidx == NULL) {
         sprintf(errbuf, _("Couldn't open %s for writing\n"), idxname);
 	err = 1;
 	goto bailout;
     }
 
-    fbin = fopen(binname, "wb");
+    fbin = gretl_fopen(binname, "wb");
     if (fbin == NULL) {
         sprintf(errbuf, _("Couldn't open %s for writing\n"), binname);
 	err = 1;
 	goto bailout;
     }
 
-    fcbk = fopen(cbname, "wb");
+    fcbk = gretl_fopen(cbname, "wb");
     if (fcbk == NULL) {
 	sprintf(errbuf, _("Couldn't open %s for writing\n"), cbname);
 	err = 1;
@@ -1620,7 +1578,7 @@ void grab_remote_db (GtkWidget *w, gpointer data)
 
     /* test write to gzipped file */
     errno = 0;
-    fp = fopen(ggzname, "w");
+    fp = gretl_fopen(ggzname, "w");
     if (fp == NULL) {
 	if (errno == EACCES) { /* write to user dir instead */
 	    build_path(paths.userdir, dbname, ggzname, ".ggz");
@@ -1695,7 +1653,7 @@ static gchar *get_descrip (char *fname, const char *dbdir)
     build_path(dbdir, fname, tmp, NULL);
     if ((p = strrchr(tmp, '.'))) strcpy(p, ".idx");
     
-    if ((fp = fopen(tmp, "r")) == NULL) {
+    if ((fp = gretl_fopen(tmp, "r")) == NULL) {
 	g_free(line);
 	return NULL;
     }
