@@ -19,7 +19,7 @@
 
 /* datafiles.c : for gretl */
 
-#undef COLL_DEBUG
+#undef COLL_DEBUG 
 
 #include "gretl.h"
 #include "datafiles.h"
@@ -155,7 +155,7 @@ static int get_title_from_descfile (file_collection *coll)
 
     test = full_path(coll->path, coll->descfile);
 
-    fp = fopen(test, "r");
+    fp = gretl_fopen(test, "r");
 
     if (fp == NULL) return err;
 
@@ -272,14 +272,10 @@ static file_collection *collection_stack (file_collection *coll, int op)
 		ret = coll;
 	    }
 	}
-    } else if (op == STACK_POP_DATA) {
-        if (n_data_popped < n_data) {
-            ret = datacoll[n_data_popped++];
-        }
-    } else if (op == STACK_POP_PS) {
-        if (n_ps_popped < n_ps) {
-            ret = pscoll[n_ps_popped++];
-        }
+    } else if (op == STACK_POP_DATA && n_data_popped < n_data) {
+	ret = datacoll[n_data_popped++];
+    } else if (op == STACK_POP_PS && n_ps_popped < n_ps) {
+	ret = pscoll[n_ps_popped++];
     } else if (op == STACK_RESET_DATA) {
 	n_data_popped = 0;
     } else if (op == STACK_RESET_PS) {
@@ -386,8 +382,9 @@ static int test_dir_for_file_collections (const char *dname, DIR *dir)
 static int dont_go_there (const char *s)
 {
     if (!strcmp(s, "..") || strstr(s, ".inp") || strstr(s, ".gdt") || 
-	strstr(s, ".gretl") || strstr(s, ".hdr"))
+	strstr(s, ".gretl") || strstr(s, ".hdr")) {
 	return 1;
+    }
 
     return 0;
 }
@@ -401,7 +398,9 @@ static int seek_file_collections (const char *topdir)
     char *tmp = unslash(topdir);
 
     dir = opendir(tmp);
-    if (dir == NULL) return 1;
+    if (dir == NULL) {
+	return 1;
+    }
 
 #ifdef COLL_DEBUG
     fprintf(stderr, "seeking file collections in '%s'\n", topdir);
@@ -461,6 +460,7 @@ static void print_script_collections (void)
     file_collection *coll;
 
     printf("\n*** Script collections:\n");
+
     while ((coll = pop_ps_collection())) {
 	print_collection(coll);
     }
@@ -475,8 +475,12 @@ static int build_file_collections (void)
 
     if (!built) {
 	err = seek_file_collections(paths.datadir);
-	if (!err) err = seek_file_collections(paths.scriptdir);
-	if (!err) err = seek_file_collections(paths.userdir);
+	if (!err) {
+	    err = seek_file_collections(paths.scriptdir);
+	}
+	if (!err) {
+	    err = seek_file_collections(paths.userdir);
+	}
 	built = 1;
     }
 
@@ -518,8 +522,10 @@ static int read_file_descriptions (windata_t *win, gpointer p)
 
     index = full_path(coll->path, coll->descfile);
 
-    fp = fopen(index, "r");
-    if (fp == NULL) return 1;
+    fp = gretl_fopen(index, "r");
+    if (fp == NULL) {
+	return 1;
+    }
 
 #ifndef OLD_GTK
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox)));
@@ -882,25 +888,20 @@ static gint populate_remote_db_list (windata_t *win)
 
 static void build_datafiles_popup (windata_t *win)
 {
-    if (win->popup != NULL) return;
+    if (win->popup != NULL) {
+	/* already done */
+	return;
+    }
 
     win->popup = gtk_menu_new();
 
-#ifndef OLD_GTK
     add_popup_item(_("Info"), win->popup, 
 		   G_CALLBACK(display_datafile_info), 
 		   win);
+
     add_popup_item(_("Open"), win->popup, 
 		   G_CALLBACK(browser_open_data), 
 		   win);
-#else
-    add_popup_item(_("Info"), win->popup, 
-		   GTK_SIGNAL_FUNC(display_datafile_info), 
-		   win);
-    add_popup_item(_("Open"), win->popup, 
-		   GTK_SIGNAL_FUNC(browser_open_data), 
-		   win);
-#endif
 }
 
 /* ........................................................... */
@@ -916,6 +917,7 @@ int browser_busy (guint code)
 	    return 1;
 	}
     }
+
     return 0;
 }
 
@@ -938,15 +940,10 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
 
     fdata->role = code;
     fdata->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#ifndef OLD_GTK
     g_signal_connect (G_OBJECT (fdata->w), "destroy",
 		      G_CALLBACK (free_browser),
 		      fdata);
-#else
-    gtk_signal_connect (GTK_OBJECT (fdata->w), "destroy",
-			GTK_SIGNAL_FUNC (free_browser),
-			fdata);
-#endif
+
     set_browser_status(fdata, BROWSER_BUSY);
 
     switch (code) {
@@ -986,27 +983,19 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
 
     gtk_box_pack_start(GTK_BOX(main_vbox), filebox, TRUE, TRUE, 0);
 
-    /* popup menu? */
     if (code == TEXTBOOK_DATA) { 
 	file_collection *coll;
 
+	/* crate popup menu */
 	build_datafiles_popup(fdata);
 
 	while ((coll = pop_data_collection())) {
-#ifndef OLD_GTK
 	    g_signal_connect (G_OBJECT(coll->page), "button_press_event",
 			      G_CALLBACK(popup_menu_handler), 
 			      (gpointer) fdata->popup);
-#else
-	    gtk_signal_connect (GTK_OBJECT(coll->page), "button_press_event",
-				GTK_SIGNAL_FUNC(popup_menu_handler), 
-				(gpointer) fdata->popup);
-#endif
 	}
 	reset_data_stack();
-    }
-
-    if (code == REMOTE_DB) {
+    } else if (code == REMOTE_DB) {
 	GtkWidget *hbox;
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -1023,59 +1012,36 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
     openbutton = gtk_button_new_with_label 
 	((code == REMOTE_DB)? _("Get series listing") : _("Open"));
     gtk_box_pack_start (GTK_BOX (button_box), openbutton, FALSE, TRUE, 0);
-#ifndef OLD_GTK
+
     g_signal_connect(G_OBJECT(openbutton), "clicked",
 		     G_CALLBACK(browse_func), fdata);
+
     if (code != NATIVE_DB && code != RATS_DB && code != REMOTE_DB) {
        	g_signal_connect(G_OBJECT(openbutton), "clicked", 
 			 G_CALLBACK(delete_widget), fdata->w); 
     }
-#else
-    gtk_signal_connect(GTK_OBJECT(openbutton), "clicked",
-		       GTK_SIGNAL_FUNC(browse_func), fdata);
-    if (code != NATIVE_DB && code != RATS_DB && code != REMOTE_DB) 
-       	gtk_signal_connect(GTK_OBJECT(openbutton), "clicked", 
-	GTK_SIGNAL_FUNC(delete_widget), fdata->w); 
-#endif
 
     if (code == TEXTBOOK_DATA || code == REMOTE_DB) {
 	midbutton = gtk_button_new_with_label 
 	    ((code == REMOTE_DB)? _("Install") : _("Info"));
 	gtk_box_pack_start (GTK_BOX (button_box), midbutton, FALSE, TRUE, 0);
-#ifndef OLD_GTK
 	g_signal_connect(G_OBJECT(midbutton), "clicked",
 			 (code == REMOTE_DB)?
 			 G_CALLBACK(grab_remote_db) :
 			 G_CALLBACK(display_datafile_info), fdata);
-#else
-	gtk_signal_connect(GTK_OBJECT(midbutton), "clicked",
-			   (code == REMOTE_DB)?
-			   GTK_SIGNAL_FUNC(grab_remote_db) :
-			   GTK_SIGNAL_FUNC(display_datafile_info), fdata);
-#endif
     }
 
     if (code == TEXTBOOK_DATA) {
 	midbutton = gtk_button_new_with_label(_("Find"));
 	gtk_box_pack_start(GTK_BOX (button_box), midbutton, FALSE, TRUE, 0);
-#ifndef OLD_GTK
 	g_signal_connect(G_OBJECT(midbutton), "clicked",
 			 G_CALLBACK(datafile_find), fdata);
-#else
-	gtk_signal_connect(GTK_OBJECT(midbutton), "clicked",
-			   GTK_SIGNAL_FUNC(datafile_find), fdata);	
-#endif	
     }
 
     closebutton = gtk_button_new_with_label(_("Close"));
     gtk_box_pack_start (GTK_BOX (button_box), closebutton, FALSE, TRUE, 0);
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(closebutton), "clicked",
 		     G_CALLBACK(delete_widget), fdata->w);
-#else
-    gtk_signal_connect(GTK_OBJECT(closebutton), "clicked",
-		       GTK_SIGNAL_FUNC(delete_widget), fdata->w);
-#endif
 
     /* put stuff into list box(es) */
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
@@ -1085,6 +1051,7 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
     }
 
     if (err) {
+	errbox(_("Couldn't open database"));
 	gtk_widget_destroy(fdata->w);
     } else {
 	gtk_widget_show_all(fdata->w); 
@@ -1097,13 +1064,9 @@ gint populate_filelist (windata_t *fdata, gpointer p)
 {
     if (fdata->role == NATIVE_DB || fdata->role == RATS_DB) {
 	return populate_dbfilelist(fdata);
-    }
-
-    else if (fdata->role == REMOTE_DB) {
+    } else if (fdata->role == REMOTE_DB) {
 	return populate_remote_db_list(fdata);
-    }
-
-    else {
+    } else {
 	return read_file_descriptions(fdata, p);
     }
 }
@@ -1266,6 +1229,7 @@ static GtkWidget *files_window (windata_t *fdata)
 	gtk_clist_set_column_justification (GTK_CLIST (fdata->listbox), i, 
 					    GTK_JUSTIFY_LEFT);
     }
+
     gtk_box_pack_start (GTK_BOX (box), scroller, TRUE, TRUE, TRUE);
     gtk_signal_connect_after (GTK_OBJECT (fdata->listbox), "select_row", 
 			      GTK_SIGNAL_FUNC (selectrow), fdata);
@@ -1279,93 +1243,6 @@ static GtkWidget *files_window (windata_t *fdata)
 
 #endif /* old vs new gtk */
 
-/* .................................................................. */
-
-void panel_restructure_dialog (gpointer data, guint u, GtkWidget *w)
-{
-    int resp;
-    gchar *msg;
-
-    msg = g_strdup_printf(_("Do you want to restructure the current panel data set\n"
-			    "as stacked time series?"));
-
-    resp = yes_no_dialog(_("gretl: panel structure"), msg, 0);
-    g_free(msg);
-
-    if (resp == GRETL_YES) {
-	void *handle;
-	int (*switch_panel_orientation)(double **, DATAINFO *);
-
-	switch_panel_orientation = gui_get_plugin_function("switch_panel_orientation",
-							   &handle);
-	
-	if (switch_panel_orientation != NULL) {
-	    if (switch_panel_orientation(Z, datainfo)) {
-		errbox(_("Failed to change panel structure"));
-	    } else {
-		msg = g_strdup_printf(_("Panel structure changed to %s"), 
-				      _("stacked time series"));
-		infobox(msg);
-		g_free(msg);
-		data_status |= MODIFIED_DATA;
-		set_sample_label(datainfo);
-	    }
-	    close_plugin(handle);
-	}
-    }
-}
-
-/* .................................................................. */
-
-struct ts_pd {
-    int pd;
-    const char *label;
-};
-
-void time_series_dialog (gpointer data, guint u, GtkWidget *w)
-{
-    gchar *msg = NULL;
-    const char *label = NULL;
-    int i;
-    struct ts_pd ok_pd[] = {
-	{  1, N_("annual data") },
-	{  4, N_("quarterly data") },
-	{ 12, N_("monthly data") },
-	{ 52, N_("weekly data") },
-	{  5, N_("daily data") },
-	{  7, N_("daily data") },
-	{ 24, N_("hourly data") },
-	{  0, NULL }
-    };
-	
-    for (i=0; ok_pd[i].pd != 0; i++) { 
-	if (datainfo->pd == ok_pd[i].pd) {
-	    label = ok_pd[i].label;
-	    break;
-	}
-    }
-
-    if (label != NULL) {
-	int resp;
-
-	msg = g_strdup_printf(_("Do you want to register the current data set\n"
-		"as %s?"), _(label));
-	resp = yes_no_dialog(_("gretl: time series data"), msg, 0);
-	if (resp == GRETL_YES) {
-	    if (!(datainfo->time_series == TIME_SERIES)) {
-		data_status |= MODIFIED_DATA;
-	    }
-	    datainfo->time_series = TIME_SERIES;
-	    set_sample_label(datainfo);
-	}
-    } else {
-	msg = g_strdup_printf(_("The current data frequency, %d, is not "
-				"recognized\nas a valid time-series frequency"), 
-			      datainfo->pd);
-	errbox(msg);
-    }
-}
-
 static void 
 switch_file_page_callback (GtkNotebook *notebook, GtkNotebookPage *page,
 			   guint page_num, windata_t *fdata)
@@ -1373,24 +1250,19 @@ switch_file_page_callback (GtkNotebook *notebook, GtkNotebookPage *page,
     char winnum[3];
     gpointer p;
 
-#ifndef OLD_GTK
     p = g_object_get_data(G_OBJECT(notebook), "browse_ptr");
-#else
-    p = gtk_object_get_data(GTK_OBJECT(notebook), "browse_ptr");
-#endif
-    if (p == NULL) return;
-    else {
+
+    if (p == NULL) {
+	return;
+    } else {
 	GtkWidget *w = *(GtkWidget **) p;
 
 	if (w == NULL) return;
     }
 
     sprintf(winnum, "%d", (int) page_num);
-#ifndef OLD_GTK
+
     fdata->listbox = g_object_get_data(G_OBJECT(notebook), winnum);
-#else
-    fdata->listbox = gtk_object_get_data(GTK_OBJECT(notebook), winnum);
-#endif
 }
 
 /* below: Construct a set of notebook pages for either
@@ -1422,6 +1294,7 @@ static GtkWidget *files_notebook (windata_t *fdata, int code)
 	} else {
 	    coll = pop_ps_collection();
 	}
+
 	if (coll == NULL) break;
 
 	listpage = files_window(fdata);
@@ -1430,32 +1303,22 @@ static GtkWidget *files_notebook (windata_t *fdata, int code)
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), listpage, label);
 	coll->page = fdata->listbox;
 	sprintf(winnum, "%d", j);
-#ifndef OLD_GTK
 	g_object_set_data(G_OBJECT(notebook), winnum, coll->page);
 	g_object_set_data(G_OBJECT(coll->page), "coll", coll);
-#else
-	gtk_object_set_data(GTK_OBJECT(notebook), winnum, coll->page);
-	gtk_object_set_data(GTK_OBJECT(coll->page), "coll", coll);
-#endif
 	j++;
     }
     
-    if (code == TEXTBOOK_DATA) reset_data_stack();
-    else reset_ps_stack();
+    if (code == TEXTBOOK_DATA) {
+	reset_data_stack();
+    } else {
+	reset_ps_stack();
+    }
 
-#ifndef OLD_GTK
     g_object_set_data(G_OBJECT(GTK_NOTEBOOK(notebook)), "browse_ptr",
 		      get_browser_ptr(fdata->role));
     g_signal_connect(G_OBJECT(GTK_NOTEBOOK(notebook)), "switch-page",
 		     G_CALLBACK(switch_file_page_callback),
 		     fdata);
-#else
-    gtk_object_set_data(GTK_OBJECT(GTK_NOTEBOOK(notebook)), "browse_ptr",
-			get_browser_ptr(fdata->role));
-    gtk_signal_connect(GTK_OBJECT(GTK_NOTEBOOK(notebook)), "switch-page",
-		       GTK_SIGNAL_FUNC(switch_file_page_callback),
-		       fdata);
-#endif
 
     gtk_widget_show(notebook);
 
@@ -1472,8 +1335,13 @@ static int populate_notebook_filelists (windata_t *win,
 {
     file_collection *coll;
     const char *title;
+    int gotcol = 0;
     int j;
 
+    if (win == NULL) {
+	return 1;
+    }
+
     while (1) {
 	if (code == TEXTBOOK_DATA) {
 	    coll = pop_data_collection();
@@ -1481,14 +1349,25 @@ static int populate_notebook_filelists (windata_t *win,
 	    coll = pop_ps_collection();
 	}
 
-	if (coll == NULL) break;
+	if (coll != NULL) {
+	    gotcol = 1;
+	} else {
+	    break;
+	}
 
 	win->listbox = coll->page;
 	populate_filelist(win, coll);
     }
 
-    if (code == TEXTBOOK_DATA) reset_data_stack();
-    else reset_ps_stack();
+    if (!gotcol) {
+	return 1;
+    }
+
+    if (code == TEXTBOOK_DATA) {
+	reset_data_stack();
+    } else {
+	reset_ps_stack();
+    }
 
     j = 0;
 
@@ -1499,7 +1378,9 @@ static int populate_notebook_filelists (windata_t *win,
 		if (!strcmp(coll->title, title)) break;
 		j++;
 	    }
-	} else coll = pop_data_collection();
+	} else {
+	    coll = pop_data_collection();
+	}
     } else {
 	title = get_scriptpage();
 	if (*title != '\0') {
@@ -1507,19 +1388,22 @@ static int populate_notebook_filelists (windata_t *win,
 		if (!strcmp(coll->title, title)) break;
 		j++;
 	    }
-	} else coll = pop_ps_collection();
+	} else {
+	    coll = pop_ps_collection();
+	}
     }
 
     win->listbox = coll->page;
 
-    if (code == TEXTBOOK_DATA) reset_data_stack();
-    else reset_ps_stack();
+    if (code == TEXTBOOK_DATA) {
+	reset_data_stack();
+    } else {
+	reset_ps_stack();
+    }
 
-#ifndef OLD_GTK
     gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), j);
-#else
-    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), j);
-#endif
+
+    gtk_widget_grab_focus(win->listbox);
 
     return 0;
 } 
