@@ -72,8 +72,6 @@
                           i == EXPORT_CSV || \
                           i == EXPORT_DAT)
 
-extern int olddat; /* settings.c */
-
 static char remember_dir[MAXLEN];
 
 struct extmap {
@@ -176,7 +174,7 @@ static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
 
-    if (olddat && IS_DAT_ACTION(action)) { 
+    if (using_olddat() && IS_DAT_ACTION(action)) { 
 	return ".dat";
     }
 
@@ -213,13 +211,16 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
     if (isdir(fname)) return 1;
 
     /* don't mess with the name of a previously existing file */
-    if ((fp = fopen(fname, "r")) && fgetc(fp) != EOF) {
+    fp = gretl_fopen(fname, "r");
+    if (fp != NULL && fgetc(fp) != EOF) {
 	fclose(fp);
 	return 0;
     }    
 
     /* don't mess with a filename that already has an extension */
-    if (dotpos(fname) != strlen(fname)) return 0;
+    if (dotpos(fname) != strlen(fname)) {
+	return 0;
+    }
     
     /* otherwise add an appropriate extension */
     ext = get_ext(action, data);
@@ -277,7 +278,8 @@ static void save_editable_content (int action, const char *fname,
 	return;
     }
 
-    if ((fp = fopen(fname, "w")) == NULL) {
+    fp = gretl_fopen(fname, "w");
+    if (fp == NULL) {
 	errbox(_("Couldn't open file for writing"));
 	g_free(buf);
 	return;
@@ -317,7 +319,9 @@ static void set_startdir (char *startdir)
     }
 
 #ifndef G_OS_WIN32
-    if (startdir[strlen(startdir) - 1] != '/') strcat(startdir, "/");
+    if (startdir[strlen(startdir) - 1] != '/') {
+	strcat(startdir, "/");
+    }
 #endif
 }
 
@@ -372,7 +376,7 @@ static char *suggested_savename (const char *fname)
     sfx = strrchr(s, '.');
 
     if (sfx != NULL && (strlen(sfx) == 4 || !strcmp(sfx, ".gnumeric"))) {
-	const char *test = (olddat)? ".dat" : ".gdt";
+	const char *test = (using_olddat())? ".dat" : ".gdt";
 
 	if (strcmp(test, sfx)) {
 	    strcpy(sfx, test);
@@ -425,25 +429,38 @@ static char *suggested_exportname (const char *fname, int action)
     return s;
 }
 
+static void remember_this_dir (const char *fname)
+{
+    int spos = slashpos(fname);
+    
+    if (spos > 0 && spos < sizeof remember_dir) {
+	*remember_dir = '\0';
+	strncat(remember_dir, fname, spos);
+    }
+}
+
 static void
 file_selector_process_result (const char *in_fname, int action, gpointer data)
 {
     char fname[FILENAME_MAX];
-    FILE *fp;
 
     *fname = 0;
     strncat(fname, in_fname, FILENAME_MAX - 1);
 
-    /* do some elementary checking */
     if (action < END_OPEN) {
-	if ((fp = fopen(fname, "r")) == NULL) {
-	    errbox(_("Couldn't open the specified file"));
+	FILE *fp = gretl_fopen(fname, "r");
+
+	if (fp == NULL) {
+	    sprintf(errtext, _("Couldn't open %s"), fname);
+	    errbox(errtext);
 	    return;
-	} else fclose(fp);
+	} else {
+	    fclose(fp);
+	}
     } 
 
     if (action != SET_PATH) {
-	strncpy(remember_dir, fname, slashpos(fname));
+	remember_this_dir(fname);
     }
 
     if (OPEN_DATA_ACTION(action)) {
@@ -611,7 +628,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	N_("gretl data files (*.dat)"), "*.dat"
     };
 
-    if (olddat && IS_DAT_ACTION(action)) {
+    if (using_olddat() && IS_DAT_ACTION(action)) {
 	return olddat_filter;
     }
 
@@ -829,8 +846,12 @@ void file_selector (const char *msg, int action, gpointer data)
 
     set_startdir(startdir);
 
+    /* FIXME: parent window below should not always be mdata->w,
+       in particular when action == SAVE_THIS_GRAPH
+    */
+
     if (action > END_OPEN && action != SET_PATH) {
-	filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(mdata->w), 
+	filesel = gtk_file_chooser_dialog_new(msg, NULL, /* GTK_WINDOW(mdata->w), */
 					      GTK_FILE_CHOOSER_ACTION_SAVE,
 					      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -876,8 +897,8 @@ void file_selector (const char *msg, int action, gpointer data)
 	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 					  strvar);
 	} else {
-	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-					  "/usr/bin");
+	    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), 
+						"/usr/bin");
 	}	    
     }	
 
