@@ -36,6 +36,7 @@
 
 #ifndef G_OS_WIN32
 # include <unistd.h>
+# include <sys/types.h>
 # include "../pixmaps/gretl.xpm"  /* program icon for X */
 #else
 # include <windows.h>
@@ -43,6 +44,7 @@
 #endif
 
 /* #define WINDEBUG 1 */
+
 #ifdef WINDEBUG
 FILE *dbg;
 #endif
@@ -50,9 +52,6 @@ FILE *dbg;
 /* functions private to gretl.c */
 static void sort_varlist (gpointer p, guint col, GtkWidget *w);
 static GtkWidget *make_main_window (int gui_get_data);
-#ifndef G_OS_WIN32
-static void clip_init (GtkWidget *w);
-#endif
 
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
@@ -107,7 +106,6 @@ double **Z;                 /* data set */
 MODEL **models;             /* gretl models structs */
 
 int plot_count, data_status, orig_vars;
-gchar *clipboard_buf; /* for copying models as LaTeX */
 float gui_scale;
 
 /* defaults for some options */
@@ -180,6 +178,22 @@ static void osx_help (void)
 }
 #endif 
 
+#ifndef G_OS_WIN32
+static void root_check (void)
+{
+    if (getuid() == 0) {
+	int resp;
+
+	resp = yes_no_dialog ("gretl", _("You seem to be running gretl " 
+			      "as root.  Do you really want to do this?"), 
+			      0);
+	if (resp == GRETL_NO) {
+	    exit(EXIT_FAILURE);
+	}
+    }
+}
+#endif
+
 extern void find_var (gpointer p, guint u, GtkWidget *w); /* gui_utils.c */
 
 static void varinfo_callback (gpointer p, guint u, GtkWidget *w)
@@ -268,6 +282,8 @@ GtkItemFactoryEntry data_items[] = {
       NULL, newdata_callback, 52, NULL, GNULL }, 
     { N_("/File/Create data set/time-series/high frequency/daily (5-day week)"), 
       NULL, newdata_callback, 5, NULL, GNULL }, 
+    { N_("/File/Create data set/time-series/high frequency/daily (6-day week)"), 
+      NULL, newdata_callback, 6, NULL, GNULL }, 
     { N_("/File/Create data set/time-series/high frequency/daily (7-day week)"), 
       NULL, newdata_callback, 7, NULL, GNULL }, 
     { N_("/File/Create data set/time-series/high frequency/hourly"), 
@@ -381,6 +397,7 @@ GtkItemFactoryEntry data_items[] = {
       CORR_SELECTED, NULL, GNULL },
 
     { N_("/Data/_Principal components"), NULL, do_menu_op, PCA, NULL, GNULL },
+    { N_("/Data/_Mahalanobis distances"), NULL, do_menu_op, MAHAL, NULL, GNULL },
 
     { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>", NULL },
@@ -422,9 +439,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/_Sample"), NULL, NULL, 0, "<Branch>", NULL },
     { N_("/Sample/_Set range..."), NULL, sample_range_dialog, SMPL, NULL, GNULL },
     { N_("/Sample/_Restore full range"), NULL, restore_sample_callback, 1, NULL, GNULL },
-    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>", NULL },    
-    { N_("/Sample/Set _frequency, startobs..."), NULL, gretl_callback, 
-      SETOBS, NULL, GNULL },
+    { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>", NULL }, 
+    { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL, GNULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL, GNULL },
     { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>", NULL },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
@@ -442,13 +458,12 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/_Add case markers..."), NULL, gretl_callback, MARKERS, NULL, GNULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL, GNULL },
     { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>", NULL },
-    { N_("/Sample/_Interpret as time series..."), NULL, time_series_dialog, 0, NULL, GNULL },
-    { N_("/Sample/Interpret as _panel..."), NULL, gui_set_panel_structure, 0, NULL, GNULL },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL, GNULL },
+    { N_("/Sample/Transpose data..."), NULL, gui_transpose_data, 0, NULL, GNULL },
 
     /* Variable menu */
     { N_("/_Variable"), NULL, NULL, 0, "<Branch>", NULL },
-    { N_("/Variable/Find..."), NULL, find_var, 0, NULL, GNULL },
+    { N_("/Variable/Find..."), NULL, find_var, 0, "<StockItem>", GTK_STOCK_FIND },
     { N_("/Variable/_Display values"), NULL, display_var, 0, NULL, GNULL },
     { N_("/Variable/_Summary statistics"), NULL, do_menu_op, 
       VAR_SUMMARY, NULL, GNULL },
@@ -460,6 +475,7 @@ GtkItemFactoryEntry data_items[] = {
       NORMAL, NULL, GNULL },
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
       GAMMA, NULL, GNULL },
+    { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL, GNULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL, GNULL }, 
     { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL, GNULL },
@@ -476,6 +492,7 @@ GtkItemFactoryEntry data_items[] = {
 #ifdef HAVE_TRAMO
     { N_("/Variable/TRAMO analysis"), NULL, do_tramo_x12a, TRAMO, NULL, GNULL },
 #endif
+    { N_("/Variable/Hurst exponent"), NULL, do_hurst, 0, NULL, GNULL }, 
     { N_("/Variable/Runs test"), NULL, do_menu_op, RUNS, NULL, GNULL }, 
     { N_("/Variable/sep2"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Variable/_Edit attributes"), NULL, varinfo_callback, 0, NULL, GNULL },
@@ -489,7 +506,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Ordinary Least Squares..."), NULL, model_callback, OLS, NULL, GNULL },
     { N_("/Model/_Weighted Least Squares..."), NULL, model_callback, WLS, NULL, GNULL },
     { N_("/Model/sep1"),  NULL, NULL, 0, "<Separator>", GNULL },
+#if 0
     { N_("/Model/HCC_M..."), NULL, model_callback, HCCM, NULL, GNULL },
+#endif
     { N_("/Model/H_eteroskedasticity corrected..."), NULL, model_callback, 
       HSK, NULL, GNULL },
     { N_("/Model/sep2"),  NULL, NULL, 0, "<Separator>", NULL },
@@ -501,6 +520,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Probit..."), NULL, model_callback, PROBIT, NULL, GNULL },
     { N_("/Model/To_bit..."), NULL, model_callback, TOBIT, NULL, GNULL },
     { "/Model/sep5",  NULL, NULL, 0, "<Separator>", NULL },
+    { N_("/Model/Poi_sson..."), NULL, model_callback, POISSON, NULL, GNULL },
     { N_("/Model/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL, GNULL },
     { N_("/Model/Least _Absolute Deviation..."), NULL, model_callback, LAD, NULL, GNULL },
     { N_("/Model/_Rank correlation..."), NULL, gretl_callback, SPEARMAN, NULL, GNULL },
@@ -728,6 +748,7 @@ int main (int argc, char *argv[])
 #ifdef G_OS_WIN32
     gretl_win32_init(argv[0]);
 #else 
+    root_check();
     set_rcfile(); /* also calls read_rc() */
 #endif/* G_OS_WIN32 */
 
@@ -902,9 +923,6 @@ int main (int argc, char *argv[])
 #ifndef G_OS_WIN32
     /* Let a first-time user set the working dir */
     first_time_set_user_dir(); 
-
-    /* enable special copying to clipboard */
-    clip_init(mdata->w);
 #endif
 
     add_files_to_menus();
@@ -985,6 +1003,8 @@ static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
 	     (selcount > 1));
 	flip(mdata->ifac, "/Data/Principal components", 
 	     (selcount > 1));
+	flip(mdata->ifac, "/Data/Mahalanobis distances", 
+	     (selcount > 1));
     }
 }
 
@@ -1020,7 +1040,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 		do_graph_var(mdata->active_var);
 	    } 
 	}
-    }
+    } 
 
     return FALSE;
 }
@@ -1451,90 +1471,17 @@ drag_data_received  (GtkWidget *widget,
     }	
 }
 
-/* ........................................................... */
-
-#ifndef G_OS_WIN32
-
-static void gretl_clipboard_get (GtkClipboard *clip,
-				 GtkSelectionData *selection_data,
-				 guint info,
-				 gpointer p)
-{
-    gchar *str;
-    gint length;
-
-    str = clipboard_buf; /* global */
-    if (str == NULL) return;
-    length = strlen(str);
-
-    if (info == TARGET_STRING) {
-	gtk_selection_data_set (selection_data,
-				GDK_SELECTION_TYPE_STRING,
-				8 * sizeof(gchar), 
-				(guchar *) str, 
-				length);
-    } else if (info == TARGET_TEXT || info == TARGET_COMPOUND_TEXT) {
-	guchar *text;
-	gchar c;
-	GdkAtom seltype;
-	gint format;
-	gint new_length;
-
-	c = str[length];
-	str[length] = '\0';
-	gdk_string_to_compound_text(str, &seltype, &format, 
-				    &text, &new_length);
-	gtk_selection_data_set(selection_data, seltype, format, 
-			       text, new_length);
-	gdk_free_compound_text(text);
-	str[length] = c;
-    }
-}
-
-/* ........................................................... */
-
-static void gretl_clipboard_clear (GtkClipboard *clip, gpointer p)
-{
-    free(clipboard_buf);
-    clipboard_buf = NULL;
-}
-
-/* ........................................................... */
-
-static void clip_init (GtkWidget *w)
-{
-    GtkClipboard *clip;
-    GtkTargetEntry targets[] = {
-	{ "STRING", 0, TARGET_STRING },
-	{ "TEXT",   0, TARGET_TEXT }, 
-	{ "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT }
-    };
-
-    gint n_targets = sizeof targets / sizeof targets[0];
-
-    clip = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-
-    if (!gtk_clipboard_set_with_owner(clip,
-				      targets, n_targets,
-				      gretl_clipboard_get,
-				      gretl_clipboard_clear,
-				      G_OBJECT(w))) {
-	fprintf(stderr, "Failed to initialize clipboard\n");
-    }
-}
-
-#endif /* G_OS_WIN32 */
-
-/* ........................................................... */
-
 static int native_datafile (void)
 {
     int n = strlen(paths.datfile);
-    extern int olddat; /* settings.c */
     
     if (n > 4) {
-	if (!strcmp(paths.datfile + n - 4, ".gdt")) return 1;
-	if (olddat && !strcmp(paths.datfile + n - 4, ".dat")) return 1;
+	if (!strcmp(paths.datfile + n - 4, ".gdt")) {
+	    return 1;
+	}
+	if (using_olddat() && !strcmp(paths.datfile + n - 4, ".dat")) {
+	    return 1;
+	}
     }
 
     return 0;
