@@ -60,6 +60,10 @@ enum extra_cmds {
     LMTEST_SQUARES,
     LMTEST_WHITE,
     LMTEST_BG,
+    LMTEST_GROUPWISE,
+    KERNEL_DENSITY,
+    CREATE_DATASET,
+    HCCME,
     GUI_CMD_MAX
 };
 
@@ -133,7 +137,9 @@ enum stat_codes {
     TR2,
     DF,
     SIGMA,
-    LNL
+    LNL,
+    AIC,
+    BIC
 };
 
 enum exec_codes {
@@ -147,7 +153,8 @@ enum exec_codes {
 enum clipstuff {
     TARGET_STRING,
     TARGET_TEXT,
-    TARGET_COMPOUND_TEXT
+    TARGET_COMPOUND_TEXT,
+    TARGET_RTF
 };
 
 enum copy_variants {
