@@ -17,10 +17,12 @@
  *
  */
 
-/* #define CHILD_DEBUG */
+#undef CHILD_DEBUG
 
 #include "gretl.h"
 #include "gretlwin32.h"
+#include "textutil.h"
+
 #include <dirent.h>
 
 #define HUSH_RUNTIME_WARNINGS
@@ -84,7 +86,7 @@ void startR (const char *Rcommand)
     }
 
     build_path(paths.userdir, "gretl.Rprofile", Rprofile, NULL);
-    fp = fopen(Rprofile, "w");
+    fp = gretl_fopen(Rprofile, "w");
     if (fp == NULL) {
 	errbox(_("Couldn't write R startup file"));
 	return;
@@ -117,7 +119,7 @@ void startR (const char *Rcommand)
 	FILE *fq;
 
 	build_path(paths.userdir, "Rtmp", Rtmp, NULL);
-	fq = fopen(Rtmp, "w");
+	fq = gretl_fopen(Rtmp, "w");
 	fprintf(fq, "gretldata <- read.table(\"%s\")\n", 
 		slash_convert(Rdata, FROM_BACKSLASH));
 	fprintf(fq, "attach(gretldata)\n");
@@ -223,7 +225,9 @@ void win_help (void)
 	found = 1;
     }
 
-    if (!found) errbox(_("Couldn't access help file"));
+    if (!found) {
+	errbox(_("Couldn't access help file"));
+    }
 }
 
 #ifdef HUSH_RUNTIME_WARNINGS
@@ -310,7 +314,9 @@ static void try_to_get_windows_font (void)
 	g_object_unref(G_OBJECT(pc));
 	gtk_widget_destroy(w);
 
-	if (match) set_app_font(fontspec);
+	if (match) {
+	    set_app_font(fontspec);
+	}
 	g_free(fontspec);
     }
 }
@@ -323,8 +329,8 @@ void set_up_windows_look (void)
 	int needslash = (paths.gretldir[n-1] != SLASH);
 	gchar *wimprc;
 
-	wimprc = g_strdup_printf("%s%setc\\gtk-2.0\\gtkrc.wimp", paths.gretldir,
-				 (needslash)? "\\" : "");
+	wimprc = g_strdup_printf("%s%setc\\gtk-2.0\\gtkrc.wimp", 
+				 paths.gretldir, (needslash)? "\\" : "");
 	gtk_rc_parse(wimprc);
 	g_free(wimprc);
     } else {
@@ -437,3 +443,104 @@ void win32_make_user_dirs (void)
     win_mkdir(dirname);
 }
 
+static int win_copy_buf (char *buf, int format, size_t buflen)
+{
+    HGLOBAL winclip;
+    LPTSTR ptr;
+    char *winbuf;
+    unsigned clip_format;
+    size_t len;
+    gchar *tr = NULL;
+
+    if (buf == NULL) {
+	errbox(_("Copy buffer was empty"));
+	return 0;
+    }
+
+    if (!OpenClipboard(NULL)) {
+	errbox(_("Cannot open the clipboard"));
+	return 1;
+    }
+
+    EmptyClipboard();
+
+    if (doing_nls() && (format == COPY_TEXT || format == COPY_TEXT_AS_RTF)) { 
+	tr = my_locale_from_utf8(buf);
+	if (tr == NULL) {
+	    CloseClipboard();
+	    return 1;
+	}
+	winbuf = dosify_buffer(tr, format);
+    } else {
+	winbuf = dosify_buffer(buf, format);
+    }
+
+    if (winbuf == NULL) {
+	CloseClipboard();
+	return 1;
+    }
+
+    if (buflen == 0) {
+	len = strlen(winbuf) + 1; 
+    } else {
+	len = buflen;
+    }
+        
+    winclip = GlobalAlloc(GMEM_MOVEABLE, len * sizeof(TCHAR));        
+
+    ptr = GlobalLock(winclip);
+    memcpy(ptr, winbuf, len);
+    GlobalUnlock(winclip); 
+
+    if (format == COPY_RTF || format == COPY_TEXT_AS_RTF) { 
+	clip_format = RegisterClipboardFormat("Rich Text Format");
+    } else if (format == COPY_CSV) {
+	clip_format = RegisterClipboardFormat("CSV");
+    } else {
+	clip_format = CF_TEXT;
+    }
+
+    SetClipboardData(clip_format, winclip);
+
+    CloseClipboard();
+
+    if (tr != NULL) {
+	free(tr);
+    }
+
+    free(winbuf);
+
+    return 0;
+}
+
+int prn_to_clipboard (PRN *prn, int copycode)
+{
+    return win_copy_buf(prn->buf, copycode, 0);
+}
+
+int win_buf_to_clipboard (const char *buf)
+{
+    HGLOBAL winclip;
+    LPTSTR ptr;
+    size_t len;
+
+    if (!OpenClipboard(NULL)) {
+	errbox(_("Cannot open the clipboard"));
+	return 1;
+    }
+
+    EmptyClipboard();
+
+    len = strlen(buf);
+    winclip = GlobalAlloc(GMEM_MOVEABLE, (len + 1) * sizeof(TCHAR));        
+
+    ptr = GlobalLock(winclip);
+    memcpy(ptr, buf, len + 1);
+    GlobalUnlock(winclip); 
+
+    SetClipboardData(CF_TEXT, winclip);
+
+    CloseClipboard();
+
+    return 0;
+}
