@@ -35,6 +35,7 @@
 #include "cmdstack.h"
 #include "filelists.h"
 #include "menustate.h"
+#include "dlgutils.h"
 
 #ifdef G_OS_WIN32
 # include <windows.h>
@@ -90,6 +91,7 @@ static gint check_model_menu (GtkWidget *w, GdkEventButton *eb,
 			      gpointer data);
 static void buf_edit_save (GtkWidget *widget, gpointer data);
 static void model_copy_callback (gpointer p, guint u, GtkWidget *w);
+static void panel_heteroskedasticity_menu (windata_t *vwin);
 
 #ifndef OLD_GTK
 static int maybe_recode_file (const char *fname);
@@ -305,13 +307,13 @@ int copyfile (const char *src, const char *dest)
 
     if (!strcmp(src, dest)) return 1;
    
-    if ((srcfd = fopen(src, "rb")) == NULL) {
+    if ((srcfd = gretl_fopen(src, "rb")) == NULL) {
 	sprintf(errtext, _("Couldn't open %s"), src);
 	errbox(errtext);
 	return 1; 
     }
 
-    if ((destfd = fopen(dest, "wb")) == NULL) {
+    if ((destfd = gretl_fopen(dest, "wb")) == NULL) {
 	sprintf(errtext, _("Couldn't write to %s"), dest);
 	errbox(errtext);
 	fclose(srcfd);
@@ -599,8 +601,7 @@ static gint catch_edit_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	    } else {
 		file_viewer_save(NULL, vwin);
 	    }
-	} 
-	else if (gdk_keyval_to_upper(key->keyval) == GDK_Q) {
+	} else if (gdk_keyval_to_upper(key->keyval) == GDK_Q) {
 	    if (vwin->role == EDIT_SCRIPT && CONTENT_IS_CHANGED(vwin)) {
 		gint resp;
 
@@ -634,8 +635,9 @@ static int set_or_get_audio_stop (int set, int val)
 
 static int should_stop_talking (void)
 {
-    while (gtk_events_pending())
+    while (gtk_events_pending()) {
 	gtk_main_iteration();
+    }
 
     return set_or_get_audio_stop(0, 0);
 }
@@ -683,11 +685,9 @@ static gint catch_viewer_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 
     if (key->keyval == GDK_q) { 
         gtk_widget_destroy(w);
-    }
-    else if (key->keyval == GDK_s && Z != NULL && vwin->role == VIEW_MODEL) {
+    } else if (key->keyval == GDK_s && Z != NULL && vwin->role == VIEW_MODEL) {
 	remember_model(vwin, 1, NULL);
-    }
-    else if (key->keyval == GDK_w) {
+    } else if (key->keyval == GDK_w) {
 	GdkModifierType mods;
 
 	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
@@ -699,8 +699,7 @@ static gint catch_viewer_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
     else if (key->keyval == GDK_a) {
 	audio_render_window(vwin, AUDIO_TEXT);
-    }
-    else if (key->keyval == GDK_x) {
+    } else if (key->keyval == GDK_x) {
 	audio_render_window(NULL, AUDIO_TEXT);
     }
 #endif
@@ -715,6 +714,40 @@ static gint catch_viewer_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	}	
     }
 #endif
+
+    /* Ctrl-F for find */
+    if (key->keyval == GDK_f) {
+	GdkModifierType mods;
+
+	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
+	if (mods & GDK_CONTROL_MASK) {
+	    text_find_callback(NULL, vwin);
+	    return TRUE;
+	}
+    }
+
+    return FALSE;
+}
+
+/* ........................................................... */
+
+gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
+{
+    if (key->keyval == GDK_q) { 
+	gtk_widget_destroy(vwin->w);
+    } else if (key->keyval == GDK_f) {
+	GdkModifierType mods;
+
+	if (vwin == mdata && !data_status) {
+	    return TRUE;
+	}
+
+	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
+	if (mods & GDK_CONTROL_MASK) {
+	    menu_find(vwin, 1, NULL);
+	    return TRUE;
+	}	
+    }
     return FALSE;
 }
 
@@ -813,7 +846,7 @@ int get_worksheet_data (char *fname, int datatype, int append,
     FILE *fp;
     int (*sheet_get_data)(const char*, double ***, DATAINFO *, PRN *);
     
-    fp = fopen(fname, "r");
+    fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	sprintf(errtext, _("Couldn't open %s"), fname);
 	errbox(errtext);
@@ -825,12 +858,10 @@ int get_worksheet_data (char *fname, int datatype, int append,
     if (datatype == GRETL_GNUMERIC) {
 	sheet_get_data = gui_get_plugin_function("wbook_get_data",
 						 &handle);
-    }
-    else if (datatype == GRETL_EXCEL) {
+    } else if (datatype == GRETL_EXCEL) {
 	sheet_get_data = gui_get_plugin_function("excel_get_data",
 						 &handle);
-    }
-    else {
+    } else {
 	errbox(_("Unrecognized data type"));
 	return 1;
     }
@@ -871,7 +902,9 @@ int get_worksheet_data (char *fname, int datatype, int append,
     } else {
 	data_status |= IMPORT_DATA;
 	strcpy(paths.datfile, fname);
-	if (mdata != NULL) register_data(fname, NULL, 1);
+	if (mdata != NULL) {
+	    register_data(fname, NULL, 1);
+	}
     }
 
     return 0;
@@ -1021,13 +1054,14 @@ void verify_open_session (gpointer userdata)
 
 void save_session (char *fname) 
 {
-    int spos;
     char msg[MAXLEN], savedir[MAXLEN], fname2[MAXLEN];
     char session_base[MAXLEN];
+    int spos;
     FILE *fp;
     PRN *prn;
 
     *savedir = '\0';
+
     spos = slashpos(fname);
     if (spos) {
 	safecpy(savedir, fname, spos);
@@ -1037,14 +1071,20 @@ void save_session (char *fname)
     dump_command_stack("stderr", 0);
 #endif
 
+    /* append ".gretl" to session filename? */
+    if (haschar('.', fname) < 0) {
+	strcat(fname, ".gretl");
+    }
+
     /* save commands, by dumping the command stack */
-    if (haschar('.', fname) < 0) strcat(fname, ".gretl");
-    if (dump_command_stack(fname, 1)) return;
+    if (dump_command_stack(fname, 1)) {
+	return;
+    }
 
     get_base(session_base, fname, '.');
 
     /* get ready to save "session" */
-    fp = fopen(fname, "a");
+    fp = gretl_fopen(fname, "a");
     if (fp == NULL) {
 	sprintf(errtext, _("Couldn't open session file %s"), fname);
 	errbox(errtext);
@@ -1071,10 +1111,14 @@ void save_session (char *fname)
 	return;
     }
 
+    /* preamble */
     gui_logo(prn->fp);
     session_time(prn->fp);
     pprintf(prn, _("Output from %s\n"), fname);
+
+    /* actual commands output */
     execute_script(fname, NULL, prn, SAVE_SESSION_EXEC); 
+
     gretl_print_destroy(prn);
 
     /* output may need re-encoding, UTF-8 to locale? */
@@ -1162,7 +1206,7 @@ static void file_viewer_save (GtkWidget *widget, windata_t *vwin)
 	FILE *fp;
 	gchar *text;
 
-	if ((fp = fopen(vwin->fname, "w")) == NULL) {
+	if ((fp = gretl_fopen(vwin->fname, "w")) == NULL) {
 	    errbox(_("Can't open file for writing"));
 	    return;
 	} else {
@@ -1253,6 +1297,8 @@ void free_windata (GtkWidget *w, gpointer data)
 	    gretl_var_free_unnamed(vwin->data);
 	else if (vwin->role == LEVERAGE) 
 	    gretl_matrix_free(vwin->data);
+	else if (vwin->role == MAHAL)
+	    free(vwin->data); /* string */
 
 	if (vwin->dialog)
 	    winstack_remove(vwin->dialog);
@@ -1338,9 +1384,10 @@ static void add_data_callback (GtkWidget *w, windata_t *vwin)
 
     if (vwin->role == PCA) {
 	add_pca_data(vwin);
-    }
-    else if (vwin->role == LEVERAGE) {
+    } else if (vwin->role == LEVERAGE) {
 	add_leverage_data(vwin);
+    } else if (vwin->role == MAHAL) {
+	add_mahalanobis_data(vwin);
     }
 
     if (datainfo->v > oldv) {
@@ -1453,7 +1500,9 @@ static void make_viewbar (windata_t *vwin, int text_out)
     int save_as_ok = (vwin->role != EDIT_HEADER && 
 		      vwin->role != EDIT_NOTES);
 
-    int help_ok = (vwin->role == LEVERAGE || vwin->role == COINT2);
+    int help_ok = (vwin->role == LEVERAGE || 
+		   vwin->role == COINT2 ||
+		   vwin->role == MAHAL);
 
 #ifndef OLD_GTK
     if (vwin->role == VIEW_MODELTABLE) tex_icon_init();
@@ -1509,8 +1558,8 @@ static void make_viewbar (windata_t *vwin, int text_out)
 	    continue;
 	}
 
-	if (vwin->role != PCA && vwin->role != LEVERAGE &&
-	    viewbar_items[i].flag == ADD_ITEM) {
+	if (vwin->role != PCA && vwin->role != LEVERAGE && 
+	    vwin->role != MAHAL && viewbar_items[i].flag == ADD_ITEM) {
 	    continue;
 	}
 
@@ -1804,14 +1853,12 @@ windata_t *view_buffer (PRN *prn, int hsize, int vsize,
     view_buffer_insert_text(vwin, prn);
     gretl_print_destroy(prn);
 
-#ifndef OLD_GTK    
     g_signal_connect(G_OBJECT(vwin->dialog), "key_press_event", 
 		     G_CALLBACK(catch_viewer_key), vwin);
+
+#ifndef OLD_GTK    
     g_signal_connect (G_OBJECT(vwin->w), "button_press_event", 
 		      G_CALLBACK(catch_button_3), vwin->w);
-#else
-    gtk_signal_connect(GTK_OBJECT(vwin->dialog), "key_press_event", 
-		       GTK_SIGNAL_FUNC(catch_viewer_key), vwin);
 #endif
 
     gtk_widget_show(vwin->vbox);
@@ -1858,7 +1905,7 @@ windata_t *view_file (const char *filename, int editable, int del_file,
 			role == VIEW_LOG);
 
     /* first check that we can open the specified file */
-    fp = fopen(filename, "r");
+    fp = gretl_fopen(filename, "r");
     if (fp == NULL) {
 	sprintf(errtext, _("Can't open %s for reading"), filename);
 	errbox(errtext);
@@ -2246,7 +2293,7 @@ static void auto_save_script (windata_t *vwin)
 	unsaved = 1;
     }
 
-    if ((fp = fopen(vwin->fname, "w")) == NULL) {
+    if ((fp = gretl_fopen(vwin->fname, "w")) == NULL) {
 	sprintf(msg, _("Couldn't write to %s"), vwin->fname);
 	errbox(msg); 
 	return;
@@ -2411,14 +2458,14 @@ static void adjust_model_menu_state (windata_t *vwin, const MODEL *pmod)
     } 
 
     /* disallow saving an already-saved model */
-    if (pmod->name) model_save_state(vwin->ifac, FALSE);
+    if (pmod->name != NULL) {
+	model_save_state(vwin->ifac, FALSE);
+    }
 
     if (pmod->ci == LAD) {
 	fcast_menu_off(vwin->ifac);
 	vcv_menu_off(vwin->ifac);
-    }
-
-    else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
+    } else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
 	fcast_menu_off(vwin->ifac);
 	confint_menu_off(vwin->ifac);
 	if (pmod->ci == ARMA && arma_by_x12a(pmod)) {
@@ -2428,6 +2475,7 @@ static void adjust_model_menu_state (windata_t *vwin, const MODEL *pmod)
 
     if (dataset_is_panel(datainfo)) {
 	arch_menu_off(vwin->ifac);
+	panel_heteroskedasticity_menu(vwin);
     }
 }
 
@@ -2504,6 +2552,13 @@ static GtkItemFactoryEntry lnl_data_item = {
     model_stat_callback, LNL, NULL, GNULL 
 };
 
+static GtkItemFactoryEntry criteria_items[] = {
+    { N_("/Model data/Add to data set/Akaike Information Criterion"), NULL, 
+      model_stat_callback, AIC, NULL, GNULL },
+    { N_("/Model data/Add to data set/Bayesian Information Criterion"), NULL, 
+      model_stat_callback, BIC, NULL, GNULL }
+};
+
 static GtkItemFactoryEntry garch_data_item = {
     N_("/Model data/Add to data set/predicted error variance"), NULL, 
     fit_resid_callback, GENR_H, NULL, GNULL 
@@ -2547,6 +2602,13 @@ static GtkItemFactoryEntry lnl_data_item = {
     model_stat_callback, LNL, NULL
 };
 
+static GtkItemFactoryEntry criteria_items[] = {
+    { N_("/Model data/Add to data set/Akaike Information Criterion"), NULL, 
+      model_stat_callback, AIC, NULL },
+    { N_("/Model data/Add to data set/Bayesian Information Criterion"), NULL, 
+      model_stat_callback, BIC, NULL }
+};
+
 static GtkItemFactoryEntry garch_data_item = {
     N_("/Model data/Add to data set/predicted error variance"), NULL, 
     fit_resid_callback, GENR_H, NULL
@@ -2580,8 +2642,7 @@ static void add_model_dataset_items (windata_t *vwin)
 	}
     }
 
-    if (pmod->ci != TOBIT && pmod->ci != LAD && pmod->ci != GARCH &&
-	!(LIMDEP(pmod->ci))) {
+    if (!ML_ESTIMATOR(pmod->ci) && pmod->ci != LAD && !na(pmod->rsq)) {
 	n = sizeof r_squared_items / sizeof r_squared_items[0];
 	for (i=0; i<n; i++) {
 	    gtk_item_factory_create_item(vwin->ifac, &r_squared_items[i], 
@@ -2589,10 +2650,15 @@ static void add_model_dataset_items (windata_t *vwin)
 	}
     }
 
-    if (LIMDEP(pmod->ci) || pmod->ci == ARMA || pmod->ci == GARCH) {
+    if (ML_ESTIMATOR(pmod->ci)) {
 	gtk_item_factory_create_item(vwin->ifac, &lnl_data_item, vwin, 1);
     }
 
+    n = sizeof criteria_items / sizeof criteria_items[0];
+    for (i=0; i<n; i++) {
+	gtk_item_factory_create_item(vwin->ifac, &criteria_items[i], vwin, 1);
+    }
+
     if (pmod->ci == GARCH) {
 	gtk_item_factory_create_item(vwin->ifac, &garch_data_item, vwin, 1);
     }
@@ -2775,6 +2841,28 @@ static void x12_output_callback (gpointer p, guint v, GtkWidget *w)
     }
 }
 
+static void panel_heteroskedasticity_menu (windata_t *vwin)
+{
+    GtkItemFactoryEntry hitem;
+
+    gtk_item_factory_delete_item(vwin->ifac, "/Tests/heteroskedasticity");
+
+    hitem.accelerator = NULL;
+    hitem.item_type = NULL;
+
+    hitem.callback = do_lmtest;
+    hitem.callback_action = LMTEST_WHITE;
+    hitem.path = g_strdup(_("/Tests/heteroskedasticity (White's test)"));
+    gtk_item_factory_create_item(vwin->ifac, &hitem, vwin, 1);
+    g_free(hitem.path);
+
+    hitem.callback = do_lmtest;
+    hitem.callback_action =  LMTEST_GROUPWISE;
+    hitem.path = g_strdup(_("/Tests/heteroskedasticity (groupwise)"));
+    gtk_item_factory_create_item(vwin->ifac, &hitem, vwin, 1);
+    g_free(hitem.path);
+}
+
 /* ........................................................... */
 
 static void add_x12_output_menu_item (windata_t *vwin)
@@ -3085,7 +3173,7 @@ static int seven_bit_file (const char *fname)
     char line[256];
     int ascii = 1;
     
-    fp = fopen(fname, "r");
+    fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	return 1;
     }
@@ -3120,14 +3208,14 @@ static int maybe_recode_file (const char *fname)
 	gchar *trbuf;
 	int err = 0;
 
-	fin = fopen(fname, "r");
+	fin = gretl_fopen(fname, "r");
 	if (fin == NULL) {
 	    return 1;
 	}
 
 	sprintf(trname, "%s.tr", fname);
 
-	fout = fopen(trname, "w");
+	fout = gretl_fopen(trname, "w");
 	if (fout == NULL) {
 	    fclose(fin);
 	    return 1;
@@ -3373,3 +3461,22 @@ void startR (const char *Rcommand)
 }
 
 #endif /* ! G_OS_WIN32 */
+
+#ifdef OLD_GTK /* for forwards compatibility */
+
+static gint entry_activate (GtkWidget *w, GdkEventKey *key, gpointer p)
+{
+    GtkWidget *top = gtk_widget_get_toplevel(w);
+
+    gtk_window_activate_default(GTK_WINDOW(top));
+
+    return FALSE;
+}
+
+void gtk_entry_set_activates_default (GtkEntry *entry, gboolean setting)
+{
+    gtk_signal_connect(GTK_OBJECT(entry), "activate", 
+		       GTK_SIGNAL_FUNC(entry_activate), NULL);
+}
+
+#endif /* old GTK */
