@@ -22,12 +22,17 @@
 
 #ifdef OLD_GTK
 
+/* various forward-compatibility bodges for GTK+ 1.2 */
+
 enum {
     GTK_STOCK_OK,
     GTK_STOCK_CANCEL,
     GTK_STOCK_CLOSE,
     GTK_STOCK_APPLY,
-    GTK_STOCK_HELP
+    GTK_STOCK_HELP,
+    GTK_STOCK_GO_FORWARD,
+    GTK_STOCK_GO_BACK,
+    GTK_STOCK_FIND
 };
 
 # define G_OBJECT(o)                    GTK_OBJECT(o)
@@ -36,6 +41,9 @@ enum {
 # define G_CALLBACK(f)                  GTK_SIGNAL_FUNC(f)
 # define g_signal_connect(o,s,f,p)      gtk_signal_connect(o,s,f,p)
 # define gtk_radio_button_get_group(b)  gtk_radio_button_group(b)
+# define gtk_notebook_set_current_page(n, p) gtk_notebook_set_page(n, p)
+
+void gtk_entry_set_activates_default (GtkEntry *entry, gboolean setting);
 
 #else
 
@@ -109,6 +117,9 @@ void file_view_set_editable (windata_t *vwin);
 
 int validate_varname (const char *varname);
 
+gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, 
+			windata_t *vwin);
+
 gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
 			 gpointer data);
 
