@@ -23,7 +23,6 @@
 #define GUIPRINT_H
 
 #ifdef G_OS_WIN32
-int win_copy_buf (char *buf, int format, size_t len);
 int winprint_graph (char *emfname);
 #endif
 
@@ -31,10 +30,6 @@ int winprint_graph (char *emfname);
 void winprint (char *fullbuf, char *selbuf);
 #endif
 
-#ifndef OLD_GTK
-char *dosify_buffer (const char *buf, int format);
-#endif
-
 void texprint_summary (GRETLSUMMARY *summ,
 		       const DATAINFO *pdinfo,
 		       PRN *prn);
