@@ -9,10 +9,10 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
     sprintf(temp, "%sgpttmp.XXXXXX", paths.userdir);
     if (mktemp(temp) == NULL) return NULL;
 
-    ftmp = fopen(temp, "w");
+    ftmp = gretl_fopen(temp, "w");
     if (ftmp == NULL) return NULL;
 
-    fsrc = fopen(fname, "r");
+    fsrc = gretl_fopen(fname, "r");
     if (fsrc == NULL) {
 	fclose(ftmp);
 	remove(temp);
@@ -113,7 +113,7 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 	pprintf(prn, "%s\\par\n\n", tmp);
 	pputs(prn, "{" VAR_SUMM_ROW "\\intbl ");
     } else {
-	strcpy(tmp, I_("(missing values denoted by -999 will be skipped)"));
+	strcpy(tmp, I_("(missing values will be skipped)"));
 	pprintf(prn, "%s\\par\n\n", tmp);
 	pprintf(prn, "{" SUMM_ROW
 		"\\intbl \\qc %s\\cell", I_("Variable"));
@@ -558,7 +558,7 @@ void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     pputs(prn, I_("Model estimation range:"));
     pprintf(prn, " %s--%s", date1, date2);
 
-    pprintf(prn, " ($n$ = %d)\\\\\n", fr->nobs); 
+    pprintf(prn, " ($n$ = %d)\\\\\n", fr->real_nobs); 
 
     pprintf(prn, I_("Standard error of residuals = %g"), fr->sigma);
     pputs(prn, "\n\\end{raggedright}\n");
@@ -578,7 +578,7 @@ void rtf_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
 
     pputs(prn, "{\\rtf1\\par\n\\qc ");
     pputs(prn, I_("Model estimation range:")); 
-    pprintf(prn, " %s - %s (n = %d)\\par\n", date1, date2, fr->nobs);
+    pprintf(prn, " %s - %s (n = %d)\\par\n", date1, date2, fr->real_nobs);
 
     sprintf(tmp, I_("Standard error of residuals = %g"), 
 	    fr->sigma);
