@@ -20,9 +20,12 @@
 /* helpfiles.c for gretl */
 
 #include "gretl.h"
-#ifndef OLD_GTK
-#include "treeutils.h"
 #include "textbuf.h"
+
+#ifdef OLD_GTK
+# include "dlgutils.h"
+#else
+# include "treeutils.h"
 #endif
 
 #undef HDEBUG
@@ -50,8 +53,6 @@ static struct help_head_t **cli_heads, **gui_heads;
 static windata_t *helpwin (int script, int english);
 
 /* searching stuff */
-static int look_for_string (const char *haystack, const char *needle, 
-			    int start);
 static void find_in_text (GtkWidget *widget, gpointer data);
 static void find_in_listbox (GtkWidget *widget, gpointer data);
 static void find_string_dialog (void (*findfunc)(), gpointer data);
@@ -102,26 +103,28 @@ struct gui_help_item {
 };
 
 static struct gui_help_item gui_help_items[] = {
-    { 0,          "nothing" },
-    { GR_PLOT,    "graphing" },
-    { GR_XY,      "graphing" },
-    { GR_DUMMY,   "factorized" },
-    { BXPLOT,     "boxplots" },
-    { GR_BOX,     "boxplots" },
-    { GR_NBOX,    "boxplots" },
-    { GR_3D,      "3-D" },
-    { ONLINE,     "online" },
-    { MARKERS,    "markers" },
-    { EXPORT,     "export" },
-    { SMPLBOOL,   "sampling" },
-    { SMPLDUM,    "sampling" },
-    { COMPACT,    "compact" },
-    { VSETMISS,   "missing" },
-    { GSETMISS,   "missing" },
-    { GUI_HELP,   "dialog" },
-    { MODELTABLE, "modeltab" },
-    { GRAPHPAGE , "graphpag" },
-    { SETSEED,    "seed" },
+    { 0,              "nothing" },
+    { GR_PLOT,        "graphing" },
+    { GR_XY,          "graphing" },
+    { GR_DUMMY,       "factorized" },
+    { BXPLOT,         "boxplots" },
+    { GR_BOX,         "boxplots" },
+    { GR_NBOX,        "boxplots" },
+    { GR_3D,          "3-D" },
+    { ONLINE,         "online" },
+    { MARKERS,        "markers" },
+    { EXPORT,         "export" },
+    { SMPLBOOL,       "sampling" },
+    { SMPLDUM,        "sampling" },
+    { COMPACT,        "compact" },
+    { VSETMISS,       "missing" },
+    { GSETMISS,       "missing" },
+    { GUI_HELP,       "dialog" },
+    { MODELTABLE,     "modeltab" },
+    { GRAPHPAGE ,     "graphpag" },
+    { SETSEED,        "seed" },
+    { KERNEL_DENSITY, "density" },
+    { HCCME,          "hccme" },
     { -1,          NULL },
 };
 
@@ -220,7 +223,7 @@ static void set_english_help_file (int script)
 	    english_gui_helpfile = tmp;
 	}
 
-	fp = fopen(tmp, "r");
+	fp = gretl_fopen(tmp, "r");
 	if (fp != NULL) {
 	    char test[128];
 	    int len = 0;
@@ -547,7 +550,7 @@ static int real_helpfile_init (int cli)
     }
 #endif
 
-    fp = fopen(helpfile, "r");
+    fp = gretl_fopen(helpfile, "r");
     if (fp == NULL) {
 	fprintf(stderr, I_("help file %s is not accessible\n"), helpfile);
 	return -1;
@@ -845,10 +848,10 @@ static void real_do_help (guint pos, int cli)
 	    } else {
 		w = gui_help_view = hwin->w;
 	    }
+	    g_signal_connect(G_OBJECT(w), "destroy",
+			     G_CALLBACK(gtk_widget_destroyed),
+			     (cli)? &script_help_view : &gui_help_view);
 	}
-	g_signal_connect(G_OBJECT(w), "destroy",
-			 G_CALLBACK(gtk_widget_destroyed),
-			 (cli)? &script_help_view : &gui_help_view);
     } else {
 	gdk_window_show(w->parent->window);
 	gdk_window_raise(w->parent->window);
@@ -1014,8 +1017,43 @@ static gint close_find_dialog (GtkWidget *widget, gpointer data)
     return FALSE;
 }
 
+/* .................................................................. */
+
+static int look_for_string (const char *haystack, const char *needle, 
+			    int start)
+{
+    int hlen = strlen(haystack);
+    int nlen = strlen(needle);
+    int pos;
+
+    for (pos = start; pos < hlen; pos++) {
+        if (strncmp(&haystack[pos], needle, nlen) == 0) { 
+             return pos;
+	}
+    }
+
+    return -1;
+}
+
+/* .................................................................. */
+
 #ifdef OLD_GTK
 
+static int is_all_lower (const char *s)
+{
+    int ret = 1;
+
+    while (*s) {
+	if (!islower(*s)) {
+	    ret = 0;
+	    break;
+	}
+	s++;
+    }
+
+    return ret;
+}
+
 static void find_in_help (GtkWidget *widget, gpointer data)
 {
     int found = 0, i, linecount = 0;
@@ -1046,6 +1084,10 @@ static void find_in_help (GtkWidget *widget, gpointer data)
     needle = gtk_editable_get_chars(GTK_EDITABLE (find_entry), 0, -1);
     found = GTK_EDITABLE(vwin->w)->selection_end_pos;
 
+    if (is_all_lower(needle)) {
+	lower(haystack);
+    }
+
     found = look_for_string(haystack, needle, found);
 
     if (found >= 0) {
@@ -1056,12 +1098,15 @@ static void find_in_help (GtkWidget *widget, gpointer data)
 	gtk_text_thaw(GTK_TEXT(vwin->w));
         gtk_editable_select_region (GTK_EDITABLE(vwin->w), 
 				    found, found + strlen(needle));
-	for (i=0; i<found; i++) 
+	for (i=0; i<found; i++) {
 	    if (haystack[i] == '\n') linecount++;
+	}
 	gtk_adjustment_set_value(GTK_TEXT(vwin->w)->vadj, 
 				 (gfloat) (linecount - 2) *
 				 GTK_TEXT(vwin->w)->vadj->upper / help_length);
-    } else infobox(_("String was not found."));
+    } else {
+	infobox(_("String was not found."));
+    }
 
     g_free(haystack);
 }
@@ -1074,13 +1119,17 @@ static void find_in_text (GtkWidget *widget, gpointer data)
 	(windata_t *) gtk_object_get_data(GTK_OBJECT(data), "windat");
 
     haystack = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0,
-	gtk_text_get_length(GTK_TEXT(vwin->w)));
+				      gtk_text_get_length(GTK_TEXT(vwin->w)));
 
     if (needle) g_free(needle);
 
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
     found = GTK_EDITABLE(vwin->w)->selection_end_pos;
 
+    if (is_all_lower(needle)) {
+	lower(haystack);
+    }
+
     found = look_for_string(haystack, needle, found);
 
     if (found >= 0) {
@@ -1148,7 +1197,9 @@ static gboolean real_find_in_text (GtkTextView *view, const gchar* str,
 	}
     }
 
-    if (found && wrapped) infobox(_("Search wrapped"));
+    if (found && wrapped) {
+	infobox(_("Search wrapped"));
+    }
 
     return found;
 }
@@ -1163,7 +1214,9 @@ static void find_in_text (GtkWidget *widget, gpointer data)
 
     found = real_find_in_text(GTK_TEXT_VIEW(vwin->w), needle, TRUE);
 
-    if (!found) infobox(_("String was not found."));
+    if (!found) {
+	infobox(_("String was not found."));
+    }
 }
 
 
@@ -1196,14 +1249,16 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
     lower(needle);
 
 #ifndef OLD_GTK
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW(win->listbox));
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox));
 
     /* try searching downward from the current line plus one */
     pstr = g_strdup_printf("%d", win->active_var);
     gtk_tree_model_get_iter_from_string (model, &iter, pstr);
     g_free(pstr);
     iterhere = iter;
-    if (!gtk_tree_model_iter_next(model, &iter)) iter = iterhere;
+    if (!gtk_tree_model_iter_next(model, &iter)) {
+	iter = iterhere;
+    }
 
  search_wrap:
 
@@ -1215,8 +1270,13 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 	lower(haystack);
 	found = look_for_string(haystack, needle, 0);
 	if (found >= 0) break;
-	/* then column 0 */
-	gtk_tree_model_get (model, &iter, 0, &tmp, -1);
+	if (win == mdata) {
+	    /* then column 2 */
+	    gtk_tree_model_get(model, &iter, 2, &tmp, -1);
+	} else {
+	    /* then column 0 */
+	    gtk_tree_model_get(model, &iter, 0, &tmp, -1);
+	}
 	strcpy(haystack, tmp);
 	g_free(tmp);
 	lower(haystack);
@@ -1263,8 +1323,13 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 	lower(haystack);
 	found = look_for_string(haystack, needle, 0);
 	if (found >= 0) break;
-	/* try column 0? */
-	gtk_clist_get_text(GTK_CLIST(win->listbox), i, 0, &tmp);
+	if (win == mdata) {
+	    /* try column 2? */
+	    gtk_clist_get_text(GTK_CLIST(win->listbox), i, 2, &tmp);
+	} else {
+	    /* try column 0? */
+	    gtk_clist_get_text(GTK_CLIST(win->listbox), i, 0, &tmp);
+	}
 	strcpy(haystack, tmp);
 	lower(haystack);
 	found = look_for_string(haystack, needle, 0);
@@ -1281,34 +1346,20 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 
     if (found >= 0) {
 	if (wrapped) infobox(_("Search wrapped"));
+	gtk_clist_freeze(GTK_CLIST(win->listbox));
 	gtk_clist_moveto(GTK_CLIST(win->listbox), i, 0, 0, .1);
+	gtk_clist_unselect_all(GTK_CLIST(win->listbox));
 	gtk_clist_select_row(GTK_CLIST(win->listbox), i, 0);
+	GTK_CLIST(win->listbox)->focus_row = i;
+	gtk_clist_thaw(GTK_CLIST(win->listbox));
 	win->active_var = i;
 	clist_start_row = i + 1;
     } else {
-	gtk_clist_select_row(GTK_CLIST(win->listbox), 0, 0);
-	win->active_var = 0;
 	infobox(_("String was not found."));
     }
 #endif /* OLD_GTK */
 }
 
-/* .................................................................. */
-
-static int look_for_string (const char *haystack, const char *needle, 
-			    int start)
-{
-    int pos;
-    int hlen = strlen(haystack);
-    int nlen = strlen(needle);
-
-    for (pos = start; pos < hlen; pos++) {
-        if (strncmp(&haystack[pos], needle, nlen) == 0) 
-             return pos;
-    }
-    return -1;
-}
-
 /* .................................................................. */
  
 static void cancel_find (GtkWidget *widget, gpointer data)
@@ -1372,33 +1423,19 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 #endif
 
     if (find_window != NULL) {
-#ifndef OLD_GTK
 	g_object_set_data(G_OBJECT(find_window), "windat", mydat);
-#else
-	gtk_object_set_data(GTK_OBJECT(find_window), "windat", mydat);
-#endif
 	parent_find(find_window, mydat);
 	gdk_window_raise(find_window->window);
 	return;
     }
 
     find_window = gtk_dialog_new();
-#ifndef OLD_GTK
     g_object_set_data(G_OBJECT(find_window), "windat", mydat);
-#else
-    gtk_object_set_data(GTK_OBJECT(find_window), "windat", mydat);
-#endif
     parent_find(find_window, mydat);
 
-#ifndef OLD_GTK
     g_signal_connect (G_OBJECT (find_window), "destroy",
 		      G_CALLBACK (close_find_dialog),
 		      find_window);
-#else
-    gtk_signal_connect (GTK_OBJECT (find_window), "destroy",
-	                GTK_SIGNAL_FUNC (close_find_dialog),
-	                find_window);
-#endif
 
     gtk_window_set_title (GTK_WINDOW (find_window), _("gretl: find"));
     gtk_container_set_border_width (GTK_CONTAINER (find_window), 5);
@@ -1413,16 +1450,11 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 	gtk_editable_select_region (GTK_EDITABLE (find_entry), 0, 
 				    strlen (needle));
     }
-#ifndef OLD_GTK
-    g_signal_connect(G_OBJECT (find_entry), "activate", 
-		     G_CALLBACK (findfunc), find_window);
-#else
-    gtk_signal_connect(GTK_OBJECT (find_entry), "activate", 
-			GTK_SIGNAL_FUNC(findfunc), find_window);
 
-#endif
+    g_signal_connect(G_OBJECT(find_entry), "activate", 
+		     G_CALLBACK(findfunc), find_window);
 
-    gtk_widget_show (find_entry);
+    gtk_widget_show(find_entry);
 
     gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX(hbox), find_entry, TRUE, TRUE, 0);
@@ -1437,31 +1469,22 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
     gtk_window_set_position(GTK_WINDOW (find_window), GTK_WIN_POS_MOUSE);
 
     /* find button -- make this the default */
-    button = gtk_button_new_with_label (_("Find next"));
+    button = standard_button(GTK_STOCK_FIND);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (find_window)->action_area), 
 		       button, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT (button), "clicked",
 		     G_CALLBACK (findfunc), find_window);
-#else
-    gtk_signal_connect(GTK_OBJECT (button), "clicked",
-		       GTK_SIGNAL_FUNC (findfunc), find_window);
-#endif
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
     /* cancel button */
-    button = gtk_button_new_with_label (_("Cancel"));
+    button = standard_button(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (find_window)->action_area), 
 		       button, TRUE, TRUE, FALSE);
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT (button), "clicked",
 		     G_CALLBACK (cancel_find), find_window);
-#else
-    gtk_signal_connect(GTK_OBJECT (button), "clicked",
-		       GTK_SIGNAL_FUNC (cancel_find), find_window);
-#endif
     gtk_widget_show(button);
 
     gtk_widget_grab_focus(find_entry);
@@ -1472,7 +1495,17 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 
 void text_find_callback (GtkWidget *w, gpointer data)
 {
+#ifdef OLD_GTK
+    windata_t *vwin = (windata_t *) data;
+
+    if (help_role(vwin->role)) {
+	find_string_dialog(find_in_help, data);
+    } else {
+	find_string_dialog(find_in_text, data);
+    }
+#else
     find_string_dialog(find_in_text, data);
+#endif
 }
 
 /* ........................................................... */
