@@ -79,8 +79,6 @@ int bool_subsample (gretlopt opt);
 
 void do_samplebool (GtkWidget *widget, dialog_t *ddata);
 
-void do_setobs (GtkWidget *widget, dialog_t *ddata);
-
 void drop_all_missing (gpointer data, guint opt, GtkWidget *w);
 
 void count_missing (void);
@@ -93,7 +91,7 @@ int dataset_is_restricted (void);
 
 int maybe_restore_full_data (int action);
 
-void gui_set_panel_structure (gpointer data, guint u, GtkWidget *w);
+void gui_transpose_data (gpointer p, guint u, GtkWidget *w);
 
 /* model-related functions */
 
@@ -129,6 +127,8 @@ void do_model (GtkWidget *widget, gpointer p);
 
 void do_arma (int v, int ar, int ma, gretlopt opts);
 
+void do_kernel (gpointer data, guint u, GtkWidget *w);
+
 void do_vif (gpointer data, guint u, GtkWidget *w);
 
 void do_leverage (gpointer data, guint u, GtkWidget *w);
@@ -173,6 +173,8 @@ void do_tramo_x12a (gpointer data, guint opt, GtkWidget *widget);
 
 void do_range_mean (gpointer data, guint opt, GtkWidget *widget);
 
+void do_hurst (gpointer data, guint opt, GtkWidget *widget);
+
 void do_outcovmx (gpointer data, guint action, GtkWidget *widget);
 
 void add_dummies (gpointer data, guint action, GtkWidget *widget);
@@ -253,5 +255,7 @@ int verify_and_record_command (char *line);
 
 int latex_compile (char *texshort);
 
+void add_mahalanobis_data (windata_t *vwin);
+
 #endif /* LIBRARY_H */
 
