@@ -98,7 +98,7 @@ static GtkItemFactoryEntry time_series_model_items[] = {
 
 #endif
 
-#define DATASET_COMPACTABLE(d) (d->time_series == TIME_SERIES && \
+#define DATASET_COMPACTABLE(d) (d->structure == TIME_SERIES && \
                                 (d->pd == 4 || d->pd == 12 || \
                                  d->pd == 5 || d->pd == 7))
 
@@ -121,6 +121,7 @@ void time_series_menu_state (gboolean s)
 #ifdef HAVE_TRAMO
 	flip(mdata->ifac, "/Variable/TRAMO analysis", s);
 #endif
+	flip(mdata->ifac, "/Variable/Hurst exponent", s);
 	/* Model menu */
 	flip(mdata->ifac, "/Model/Time series", s);
 	/* Sample menu */
@@ -219,6 +220,8 @@ static gint var_popup_click (GtkWidget *widget, gpointer data)
 	unit_root_test(NULL, KPSS, NULL);
     else if (!strcmp(item, _("Runs test"))) 
 	do_menu_op(NULL, RUNS, NULL);
+    else if (!strcmp(item, _("Hurst exponent"))) 
+	do_hurst(NULL, 0, NULL);
     else if (!strcmp(item, _("Edit attributes")))  
 	varinfo_dialog(mdata->active_var, 1);
     else if (!strcmp(item, _("Delete"))) 
@@ -247,9 +250,9 @@ GtkWidget *build_var_popup (void)
 	N_("Dickey-Fuller test"),
 	N_("KPSS test"),
 	N_("Runs test"),
+	N_("Hurst exponent"),
 	N_("Edit attributes"),
 	N_("Delete"),
-	N_("Simulate..."),
 	N_("Define new variable...")
     };
 
@@ -260,11 +263,8 @@ GtkWidget *build_var_popup (void)
     var_menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (i == 2 && !dataset_is_time_series(datainfo) &&
-	    datainfo->time_series != STACKED_TIME_SERIES) {
-	    continue;
-	}
-	if (!dataset_is_time_series(datainfo) && i >= 6 && i <= 11) {
+	if (!dataset_is_time_series(datainfo) && 
+	    (i == 2 || (i >= 6 && i <= 11))) {
 	    continue;
 	}
 	var_item = gtk_menu_item_new_with_label(_(var_items[i]));
@@ -295,6 +295,8 @@ static gint selection_popup_click (GtkWidget *widget, gpointer data)
 	do_menu_op(NULL, CORR_SELECTED, NULL);
     else if (!strcmp(item, _("Principal components"))) 
 	do_menu_op(NULL, PCA, NULL);
+    else if (!strcmp(item, _("Mahalanobis distances"))) 
+	do_menu_op(NULL, MAHAL, NULL);
     else if (!strcmp(item, _("Time series plot"))) 
 	plot_from_selection(NULL, GR_PLOT, NULL);
     else if (!strcmp(item, _("XY scatterplot"))) 
@@ -315,9 +317,10 @@ GtkWidget *build_selection_popup (void)
 	N_("Display values"),
 	N_("Descriptive statistics"),
 	N_("Correlation matrix"),
-	N_("Principal components"),
 	N_("Time series plot"),
 	N_("XY scatterplot"),
+	N_("Principal components"),
+	N_("Mahalanobis distances"),
 	N_("Copy to clipboard"),
 	N_("Delete"),
     };
@@ -329,7 +332,7 @@ GtkWidget *build_selection_popup (void)
     sel_menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (!dataset_is_time_series(datainfo) && i == 4) {
+	if (!dataset_is_time_series(datainfo) && i == 3) {
 	    continue;
 	}
 	item = gtk_menu_item_new_with_label(_(items[i]));
@@ -367,7 +370,9 @@ void set_sample_label (DATAINFO *pdinfo)
     ntodate(stobs, 0, pdinfo);
     ntodate(endobs, pdinfo->n - 1, pdinfo);
 
-    if (dataset_is_time_series(pdinfo)) {
+    if (custom_time_series(pdinfo)) {
+	strcpy(pdstr, _("Time series"));
+    } else if (dataset_is_time_series(pdinfo)) {
 	switch (pdinfo->pd) {
 	case 1:
 	    strcpy(pdstr, _("Annual")); break;
@@ -397,14 +402,11 @@ void set_sample_label (DATAINFO *pdinfo)
     ts_or_panel_menu_state(dataset_is_time_series(pdinfo) ||
 			   dataset_is_panel(pdinfo));
 
-    flip(mdata->ifac, "/Sample/Interpret as time series...", 
-	 !(dataset_is_time_series(pdinfo)));
-
-    flip(mdata->ifac, "/Sample/Interpret as panel...", 
-	 !(pdinfo->pd == 1));
-
     flip(mdata->ifac, "/Sample/Restructure panel...", 
-	 pdinfo->time_series == STACKED_CROSS_SECTION);
+	 dataset_is_panel(pdinfo));
+
+    flip(mdata->ifac, "/Sample/Transpose data...", 
+	 !dataset_is_panel(pdinfo));
 
     sprintf(labeltxt, _("%s: Full range %s - %s"), 
 	    pdstr, stobs, endobs);
