@@ -73,7 +73,6 @@ static int match_object_command (const char *s, char sort)
 
 static void print_model_stat (MODEL *pmod, const char *param, PRN *prn)
 {
-    /* FIXME NatLangS */
     if (!strcmp(param, "ess")) {
 	pprintf(prn, _("%s: ess = %.8g\n"), pmod->name, pmod->ess);
     }
@@ -154,7 +153,7 @@ static void get_word_and_command (const char *s, char *word,
     }
 
 #if 0
-    fprintf(stderr, "s='%s', len=%d\n", s, len);
+    fprintf(stderr, "remaining s ='%s', len=%d\n", s, len);
 #endif
 
     /* is an object command embedded? */
@@ -164,11 +163,19 @@ static void get_word_and_command (const char *s, char *word,
 	len -= (len - d);
     }
 
-    if (len > MAXSAVENAME - 1) len = MAXSAVENAME - 1;
+    if (len == 0) {
+	return;
+    }
+
+    if (len > MAXSAVENAME - 1) {
+	len = MAXSAVENAME - 1;
+    }
+
     strncat(word, p, len);
 
-    if (word[strlen(word) - 1] == '"') 
-	word[strlen(word) - 1] = 0;
+    if (word[len - 1] == '"') {
+	word[len - 1] = 0;
+    }
 
 #if 0
     fprintf(stderr, "word='%s', cmd='%s'\n", word, cmd);
@@ -179,7 +186,7 @@ static int parse_object_request (const char *line,
 				 char *objname, char *param,
 				 void **pptr, PRN *prn)
 {
-    char word[MAXSAVENAME];
+    char word[MAXSAVENAME] = {0};
     char sort = 0;
     int action;
 
@@ -219,22 +226,28 @@ int maybe_save_model (const CMD *cmd, MODEL **ppmod,
 {
     int err;
 
-    if ((*ppmod)->errcode) return 1;
-    if (*cmd->savename == 0) return 0;
+    if ((*ppmod)->errcode) {
+	return 1;
+    }
+
+    if (*cmd->savename == 0) {
+	return 0;
+    }
 
     (*ppmod)->name = g_strdup(cmd->savename);
+
     err = try_add_model_to_session(*ppmod);
 
     if (!err) {
 	MODEL *mnew = malloc(sizeof *mnew);
 
 	if (mnew != NULL) {
-	    /* FIXME: is this OK? */
 	    copy_model(mnew, *ppmod, pdinfo);
 	    *ppmod = mnew;
 	    pprintf(prn, _("%s saved\n"), cmd->savename);
+	} else {
+	    err = E_ALLOC;
 	}
-	else err = E_ALLOC;
     }
 
     return err;
@@ -330,9 +343,10 @@ int saved_object_action (const char *line,
 			 const DATAINFO *pdinfo,
 			 PRN *prn)
 {
+    char savename[MAXSAVENAME] = {0};
+    char param[9] = {0};
+    void *ptr = NULL;
     int code;
-    char savename[MAXSAVENAME], param[9];
-    void *ptr;
 
     if (*line == '!' || *line == '#') { 
 	/* shell command or comment */
