@@ -22,6 +22,7 @@
 #include "gretl.h"
 #include "selector.h"
 #include "treeutils.h"
+#include "dlgutils.h"
 
 enum {
     SR_VARLIST,
@@ -50,7 +51,8 @@ struct _selector {
 };
 
 #define WANT_TOGGLES(c) (c == OLS || c == TOBIT || c == ARMA || \
-                         c == GARCH || c == COINT2 || c == TSLS)
+                         c == GARCH || c == COINT2 || c == TSLS || \
+                         c == VAR)
 
 static int default_var;
 static int *xlist;
@@ -60,8 +62,8 @@ static GtkWidget *scatters_menu;
 
 static gint dblclick_varlist_row (GtkWidget *w, GdkEventButton *event, 
 				  selector *sr); 
-static gint remove_right_click (GtkWidget *widget, GdkEventButton *event, 
-				gpointer data);
+static gint listvar_special_click (GtkWidget *widget, GdkEventButton *event, 
+				   gpointer data);
 static gint add_right_click (GtkWidget *widget, GdkEventButton *event, 
 			     selector *sr);
 
@@ -113,6 +115,7 @@ static GtkWidget *var_list_box_new (GtkBox *box, selector *sr, int which)
 						       1, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);	
     gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
+    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(view), FALSE);
 
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
     gtk_tree_selection_set_mode (select, GTK_SELECTION_EXTENDED);
@@ -133,15 +136,14 @@ static GtkWidget *var_list_box_new (GtkBox *box, selector *sr, int which)
 	g_signal_connect (G_OBJECT(view), "button_press_event",
 			  G_CALLBACK(dblclick_varlist_row),
 			  sr);
-    }
-    else if (which == SR_RIGHTVARS || which == SR_AUXVARS) { 
+    } else if (which == SR_RIGHTVARS || which == SR_AUXVARS) { 
 	/* lists of selected items */
 	g_signal_connect (G_OBJECT(view), "button_press_event",
 			  G_CALLBACK(set_active_var),
 			  NULL);
 	
 	g_signal_connect (G_OBJECT(view), "button_press_event",
-			  G_CALLBACK(remove_right_click),
+			  G_CALLBACK(listvar_special_click),
 			  view);
     } 
 
@@ -165,8 +167,10 @@ static GtkWidget *var_list_box_new (GtkBox *box, selector *sr, int which)
 void clear_selector (void)
 {
     default_var = 0;
+
     free(xlist);
     xlist = NULL;
+
     free(auxlist);
     auxlist = NULL;
 }
@@ -270,8 +274,11 @@ static void real_add_generic (GtkTreeModel *model, GtkTreeIter *iter,
 
     gtk_tree_model_get (model, iter, 0, &vnum, 1, &vname, -1);
 
-    if (which == SR_AUXVARS) list = sr->auxvars;
-    else list = sr->rightvars;
+    if (which == SR_AUXVARS) {
+	list = sr->auxvars;
+    } else {
+	list = sr->rightvars;
+    }
 
     if (!GTK_IS_TREE_VIEW(list)) return;
 
@@ -392,18 +399,26 @@ static gint dblclick_varlist_row (GtkWidget *w, GdkEventButton *event,
     return FALSE;
 }
 
-static gint remove_right_click (GtkWidget *widget, GdkEventButton *event, 
-				gpointer data)
+static gint listvar_special_click (GtkWidget *widget, GdkEventButton *event, 
+				   gpointer data)
 {
     GdkWindow *topwin;
     GdkModifierType mods;
 
     topwin = gtk_widget_get_parent_window(GTK_WIDGET(data));
     gdk_window_get_pointer(topwin, NULL, NULL, &mods); 
+
+    if (mods & GDK_BUTTON2_MASK) {
+	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(data), TRUE);
+    } else {
+	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(data), FALSE);
+    }
+
     if (mods & GDK_BUTTON3_MASK) {
 	remove_from_right_callback (NULL, data);
 	return TRUE;
-    }
+    } 
+
     return FALSE;
 }
 
@@ -505,35 +520,48 @@ static void reverse_list (char *list)
     free(tmp);
 }
 
+enum cmdlist_codes {
+    ADD_NOW,
+    ADD_AT_END
+};
+
 static int add_to_cmdlist (selector *sr, const char *add)
 {
     int n = strlen(sr->cmdlist);
     char *cmdlist = NULL;
+    int err = 0;
 
     if (n % MAXLEN > MAXLEN - 32) {
 	int blocks = 2 + n / MAXLEN;
 
 	cmdlist = realloc(sr->cmdlist, blocks * MAXLEN);
-	if (cmdlist == NULL) return 1;
-	else sr->cmdlist = cmdlist;
+	if (cmdlist == NULL) {
+	    err = 1;
+	} else {
+	    sr->cmdlist = cmdlist;
+	}
     }
 
-    strcat(sr->cmdlist, add);
+    if (!err) {
+	strcat(sr->cmdlist, add);
+    }
 
-    return 0;
+    return err;
 }
 
 static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 {
     gint i = 0, rows = 0;
-    gchar numstr[6], grvar[6];
+    gchar numstr[8], endbit[12] = {0};
     GtkTreeModel *model;
     GtkTreeIter iter;
 
     sr->error = 0;
 
     sr->cmdlist = mymalloc(MAXLEN); 
-    if (sr->cmdlist == NULL) return FALSE;
+    if (sr->cmdlist == NULL) {
+	return FALSE;
+    }
 
     sr->cmdlist[0] = 0;
 
@@ -545,7 +573,7 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
     if (sr->code == WLS) {
 	const gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
 
-	if (str == NULL || !strlen(str)) {
+	if (str == NULL || *str == '\0') {
 	    errbox(_("You must select a weight variable"));
 	    sr->error = 1;
 	} else {
@@ -553,8 +581,14 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	    sprintf(numstr, "%d ", i);
 	    add_to_cmdlist(sr, numstr);
 	}
-    }
-    else if (sr->code == AR) {
+    } else if (sr->code == POISSON) {
+	const gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
+
+	if (str != NULL && *str != '\0') {
+	    i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(sr->extra), "data"));
+	    sprintf(endbit, " ; %d", i);
+	}
+    } else if (sr->code == AR) {
 	const gchar *lags;
 
 	lags = gtk_entry_get_text(GTK_ENTRY(sr->extra));
@@ -565,16 +599,14 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	    add_to_cmdlist(sr, lags);
 	    add_to_cmdlist(sr, " ; ");
 	}
-    }
-    else if (sr->code == VAR || sr->code == COINT || sr->code == COINT2) {
+    } else if (sr->code == VAR || sr->code == COINT || sr->code == COINT2) {
 	GtkAdjustment *adj;
  
 	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sr->extra));
 	i = (gint) adj->value;
 	sprintf(numstr, "%d ", i);
 	add_to_cmdlist(sr, numstr);
-    }
-    else if (sr->code == ARMA || sr->code == GARCH) {
+    } else if (sr->code == ARMA || sr->code == GARCH) {
 	GtkAdjustment *adj;
 
 	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sr->extra));
@@ -587,8 +619,7 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	sprintf(numstr, "%d ", i);
 	add_to_cmdlist(sr, numstr);
 	add_to_cmdlist(sr, " ; ");
-    }
-    else if (sr->code == GR_DUMMY || sr->code == GR_3D) {
+    } else if (sr->code == GR_DUMMY || sr->code == GR_3D) {
 	const gchar *str = gtk_entry_get_text(GTK_ENTRY(sr->extra));
 
 	if (str == NULL || !strlen(str)) {
@@ -611,7 +642,7 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	} else {
 	    i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(sr->depvar), "data"));
 	    if (sr->code == GR_XY || sr->code == GR_IMP) {
-		sprintf(grvar, " %d", i);
+		sprintf(endbit, " %d", i);
 	    } else {
 		sprintf(numstr, "%d", i);
 		add_to_cmdlist(sr, numstr);
@@ -652,10 +683,14 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 
     if (MODEL_CODE(sr->code)) {
 	if (rows > 0) { 
-	    xlist = realloc(xlist, (rows + 1) * sizeof *xlist);
-	    if (xlist != NULL) {
-		xlist[0] = rows;
+	    int *rlist;
+
+	    rlist = myrealloc(xlist, (rows + 1) * sizeof *rlist);
+	    if (rlist == NULL) {
+		return FALSE;
 	    }
+	    xlist = rlist;
+	    xlist[0] = rows;
 	}
     }
 
@@ -705,8 +740,8 @@ static gboolean construct_cmdlist (GtkWidget *w, selector *sr)
 	}
     }
 
-    if (sr->code == GR_XY || sr->code == GR_IMP) {
-	add_to_cmdlist(sr, grvar);
+    if (endbit[0] != '\0') {
+	add_to_cmdlist(sr, endbit);
     }
 
     if (sr->code == SCATTERS && 
@@ -777,6 +812,8 @@ static char *est_str (int cmdnum)
 	return N_("Tobit");
     case LOGISTIC:
 	return N_("Logistic");
+    case POISSON:
+	return N_("Poisson");
     case POOLED:
 	return N_("Pooled OLS");
     case WLS:
@@ -810,6 +847,8 @@ static char *extra_string (int cmdnum)
     switch (cmdnum) {
     case WLS:
 	return N_("Weight variable");
+    case POISSON:
+	return N_("Offset variable");
     case TSLS:
 	return N_("Instruments");
     case AR:
@@ -974,11 +1013,20 @@ static void lag_order_spin (selector *sr, GtkWidget *right_vbox)
 {
     GtkWidget *tmp, *midhbox;
     GtkObject *adj;
-    gfloat order = datainfo->pd;
+    gdouble order; 
+    gdouble ordermax;
+
+    if (datainfo->n < 72) {
+	ordermax = datainfo->n / 2;
+    } else {
+	ordermax = 36;
+    }
+
+    order = (datainfo->pd > 12)? 12 : datainfo->pd;
 
     midhbox = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("lag order:"));
-    adj = gtk_adjustment_new(order, 1, 24, 1, 1, 1);
+    adj = gtk_adjustment_new(order, 1, ordermax, 1, 1, 1);
     sr->extra = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
     gtk_box_pack_start (GTK_BOX (midhbox), tmp, FALSE, FALSE, 5);
     gtk_widget_show(tmp);
@@ -1075,22 +1123,19 @@ static void build_mid_section (selector *sr, GtkWidget *right_vbox)
 	gtk_widget_show(tmp);
     }	
 
-    if (sr->code == WLS || sr->code == GR_DUMMY || sr->code == GR_3D) { 
-	extra_var_box (sr, right_vbox);
-    }
-    else if (sr->code == COINT || sr->code == COINT2) {
-	lag_order_spin (sr, right_vbox);
-    }
-    else if (sr->code == TSLS) {
-	auxiliary_varlist_box (sr, right_vbox);
-    }
-    else if (sr->code == AR) {
+    if (sr->code == WLS || sr->code == POISSON ||
+	sr->code == GR_DUMMY || sr->code == GR_3D) { 
+	extra_var_box(sr, right_vbox);
+    } else if (sr->code == COINT || sr->code == COINT2) {
+	lag_order_spin(sr, right_vbox);
+    } else if (sr->code == TSLS) {
+	auxiliary_varlist_box(sr, right_vbox);
+    } else if (sr->code == AR) {
 	sr->extra = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(right_vbox), sr->extra, 
 			   FALSE, TRUE, 0);
 	gtk_widget_show(sr->extra); 
-    }
-    else if (sr->code == VAR) {
+    } else if (sr->code == VAR) {
 	lag_order_spin (sr, right_vbox);
 	tmp = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(right_vbox), tmp, FALSE, FALSE, 0);
@@ -1108,21 +1153,31 @@ static void build_mid_section (selector *sr, GtkWidget *right_vbox)
 
 static int screen_scalar (int i, int c)
 {
-    if ((MODEL_CODE(c) || GRAPH_CODE(c) || c == LAGS || c == DIFF || c == LDIFF)
-	&& datainfo->vector[i] == 0)
+    if ((MODEL_CODE(c) || COINT_CODE(c) || GRAPH_CODE(c) || 
+	 c == LAGS || c == DIFF || c == LDIFF)
+	&& datainfo->vector[i] == 0) {
 	return 1;
-    return 0;
+    } else {
+	return 0;
+    }
 }
 
 static void selector_init (selector *sr, guint code, const char *title)
 {
     GtkWidget *base, *hsep;
     int dlgheight = 320;
+    
+    if (MODEL_CODE(code) && datainfo->v > 10) {
+	dlgheight = 400;
+    } else if (code == WLS || code == POISSON || code == AR) {
+	dlgheight = 350;
+    } else if (code == TSLS) {
+	dlgheight = 400;
+    }
 
-    if (MODEL_CODE(code) && datainfo->v > 10) dlgheight = 400;
-    else if (code == WLS || code == AR) dlgheight = 350;
-    else if (code == TSLS) dlgheight = 400;
-    else if (code == VAR) dlgheight = 420;
+    if (code == VAR) {
+	dlgheight = 450;
+    }
 
     if (WANT_TOGGLES(code)) {
 	dlgheight += 40;
@@ -1190,6 +1245,15 @@ static void robust_callback (GtkWidget *w,  selector *sr)
     }
 }
 
+static void robust_config_button (GtkWidget *w,  GtkWidget *b)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	gtk_widget_set_sensitive(b, TRUE);
+    } else {
+	gtk_widget_set_sensitive(b, FALSE);
+    }
+}
+
 static void verbose_callback (GtkWidget *w,  selector *sr)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
@@ -1234,26 +1298,58 @@ static void build_pq_spinners (selector *sr)
     gtk_widget_show(hbox);
 }
 
+static void hc_config (GtkWidget *w, gpointer p)
+{
+    options_dialog(p, 4, NULL);
+}
+
 static void 
 build_selector_switches (selector *sr) 
 {
     GtkWidget *hbox, *tmp;
 
-    if (sr->code == OLS || sr->code == GARCH || sr->code == TSLS) {
-	tmp = gtk_check_button_new_with_label(_("Robust standard errors"));
-	g_signal_connect(G_OBJECT(tmp), "toggled",
-			 G_CALLBACK(robust_callback), sr);
-	hbox = gtk_hbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+    if (sr->code == OLS || sr->code == GARCH || 
+	sr->code == TSLS || sr->code == VAR) {
+	GtkWidget *b1, *b2;
+
+	tmp = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(sr->vbox),
+			   tmp, FALSE, FALSE, 0);
 	gtk_widget_show(tmp);
 
+	b1 = gtk_check_button_new_with_label(_("Robust standard errors"));
+	g_signal_connect(G_OBJECT(b1), "toggled",
+			 G_CALLBACK(robust_callback), sr);
+	if (using_hc_by_default()) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1), TRUE);
+	}
+
+	hbox = gtk_hbox_new(FALSE, 5);
+
+	gtk_box_pack_start(GTK_BOX(hbox), b1, FALSE, FALSE, 0);
+	gtk_widget_show(b1);
+
+	b2 = gtk_button_new_with_label(_("configure"));
+	g_signal_connect(G_OBJECT(b2), "clicked",
+			 G_CALLBACK(hc_config), sr);
+	gtk_widget_set_sensitive(b2, using_hc_by_default());
+
+	g_signal_connect(G_OBJECT(b1), "toggled",
+			 G_CALLBACK(robust_config_button), b2);	
+
+	gtk_box_pack_start(GTK_BOX(hbox), b2, FALSE, FALSE, 0);
+	gtk_widget_show(b2);
+
 	gtk_box_pack_start(GTK_BOX(sr->vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
     }
 
     if (sr->code == TOBIT || sr->code == ARMA || sr->code == GARCH ||
-	sr->code == COINT2) {
-	if (sr->code == COINT2) {
+	sr->code == COINT2 || sr->code == VAR) {
+	if (sr->code == VAR) {
+	    tmp = gtk_check_button_new_with_label
+		(_("Show impulse responses"));
+	} else if (sr->code == COINT2) {
 	    tmp = gtk_check_button_new_with_label
 		(_("Show details of regressions"));
 	} else {
@@ -1262,6 +1358,7 @@ build_selector_switches (selector *sr)
 	}
 	g_signal_connect(G_OBJECT(tmp), "toggled",
 			 G_CALLBACK(verbose_callback), sr);
+
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
 	gtk_widget_show(tmp);
@@ -1337,7 +1434,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 
     selector_init(sr, cmdcode, title);
 
-    if (MODEL_CODE(cmdcode))
+    if (MODEL_CODE(cmdcode) || COINT_CODE(cmdcode))
 	topstr = _(est_str(cmdcode));
     else if (cmdcode == GR_XY)
 	topstr = _("XY scatterplot");
@@ -1380,21 +1477,19 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     gtk_box_pack_start(GTK_BOX(right_vbox), tmp, FALSE, FALSE, 0);
     gtk_widget_show(tmp);
 
-    /* for models: top right -> dependent variable */
     if (MODEL_CODE(cmdcode)) { 
+	/* models: top right -> dependent variable */
 	build_depvar_section(sr, right_vbox);
-    }
-
-    /* graphs: top right -> x-axis variable */
-    else if (cmdcode == GR_XY || cmdcode == GR_IMP || cmdcode == GR_DUMMY
-	     || cmdcode == SCATTERS || cmdcode == GR_3D) {
+    } else if (cmdcode == GR_XY || cmdcode == GR_IMP || cmdcode == GR_DUMMY
+	       || cmdcode == SCATTERS || cmdcode == GR_3D) {
+	/* graphs: top right -> x-axis variable */
 	build_x_axis_section(sr, right_vbox);
     }
 
     /* middle right: used for some estimators and factored plot */
     if (cmdcode == WLS || cmdcode == AR || cmdcode == TSLS || 
 	cmdcode == VAR || cmdcode == COINT || cmdcode == COINT2 || 
-	cmdcode == GR_DUMMY || cmdcode == GR_3D) {
+	cmdcode == POISSON || cmdcode == GR_DUMMY || cmdcode == GR_3D) {
 	build_mid_section(sr, right_vbox);
     }
     
@@ -1409,13 +1504,13 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 	GtkWidget *remove;
 	GtkWidget *indepvar_hbox;
 
-	if (MODEL_CODE(cmdcode)) {
+	if (COINT_CODE(cmdcode)) {
+	    tmp = gtk_label_new(_("Variables to test"));
+	} else if (MODEL_CODE(cmdcode)) {
 	    tmp = gtk_label_new(_("Independent variables"));
-	}
-	else if (cmdcode == GR_XY || cmdcode == GR_IMP) {
+	} else if (cmdcode == GR_XY || cmdcode == GR_IMP) {
 	    tmp = gtk_label_new(_("Y-axis variables"));
-	}
-	else if (cmdcode == SCATTERS) {
+	} else if (cmdcode == SCATTERS) {
 	    scatters_label = tmp = gtk_label_new(_("X-axis variables"));
 	}
     
@@ -1447,16 +1542,24 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 	gtk_list_store_clear (store);
 	gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
 
-	if (MODEL_CODE(cmdcode) && xlist != NULL) {
-	    for (i=1; i<=xlist[0]; i++) {
+	if (MODEL_CODE(cmdcode)) {
+	    if (xlist != NULL) {
+		/* we have a saved list of regressors */
+		for (i=1; i<=xlist[0]; i++) {
+		    gtk_list_store_append(store, &iter);
+		    gtk_list_store_set(store, &iter, 
+				       0, xlist[i], 
+				       1, datainfo->varname[xlist[i]], 
+				       -1);
+		}
+	    } else if (cmdcode != VAR) {
+		/* stick the constant in by default */
 		gtk_list_store_append(store, &iter);
-		gtk_list_store_set(store, &iter, 0, xlist[i], 
-				   1, datainfo->varname[xlist[i]], -1);
+		gtk_list_store_set(store, &iter, 
+				   0, 0, 
+				   1, "const", 
+				   -1);
 	    }
-	} else if (MODEL_CODE(cmdcode) && cmdcode != COINT &&
-		   cmdcode != COINT2 && cmdcode != VAR) {
-	    gtk_list_store_append(store, &iter);
-	    gtk_list_store_set(store, &iter, 0, 0, 1, "const", -1);
 	}
 
 	/* hook remove button to listing */
@@ -1488,7 +1591,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     }
 
     /* buttons: OK, Clear, Cancel, Help */
-    build_selector_buttons (sr, okfunc);
+    build_selector_buttons(sr, okfunc);
 
     gtk_widget_show(sr->dlg);
 }
@@ -1841,6 +1944,14 @@ const char *selector_list (const selector *sr)
     return sr->cmdlist;
 }
 
+int selector_list_hasconst (const selector *sr)
+{
+    int hc = sr->cmdlist != NULL && 
+	strstr(sr->cmdlist, " 0") != NULL;
+
+    return hc;
+}
+
 gpointer selector_get_data (const selector *sr)
 {
     return sr->data;
