@@ -24,17 +24,18 @@
 #define MODEL_CODE(c) (c == OLS || c == CORC || c == HILU || c == WLS || \
                        c == POOLED || c == HCCM || c == HSK || c == ARMA || \
                        c == TSLS || c == LOGIT || c == PROBIT || c == GARCH || \
-                       c == AR || c == VAR || c == COINT || c == COINT2 || \
-                       c == MPOLS || c == LAD || c == LOGISTIC || c == TOBIT || \
-                       c == PWE)
+                       c == AR || c == VAR || c == MPOLS || c == LAD || \
+                       c == LOGISTIC || c == TOBIT || c == PWE || c == POISSON)
 #else
 #define MODEL_CODE(c) (c == OLS || c == CORC || c == HILU || c == WLS || \
                        c == POOLED || c == HCCM || c == HSK || c == ARMA || \
                        c == TSLS || c == LOGIT || c == PROBIT || c == GARCH || \
-                       c == AR || c == VAR || c == COINT || c == COINT2 || \
-                       c == LAD || c == LOGISTIC || c == TOBIT || c == PWE)
+                       c == AR || c == VAR || c == LAD || c == LOGISTIC || \
+                       c == TOBIT || c == PWE || c == POISSON)
 #endif
 
+#define COINT_CODE(c) (c == COINT || c == COINT2)
+
 #define ADDVAR_CODE(c) (c == LOGS || c == LAGS || c == SQUARE || \
                         c == DIFF || c == LDIFF)
 
@@ -59,6 +60,8 @@ int selector_code (const selector *sr);
 
 const char *selector_list (const selector *sr);
 
+int selector_list_hasconst (const selector *sr);
+
 gpointer selector_get_data (const selector *sr);
 
 gretlopt selector_get_opts (const selector *sr);
