@@ -20,6 +20,8 @@
 #include "gretl.h"
 #include "treeutils.h"
 #include "ssheet.h"
+#include "dlgutils.h"
+
 #include <errno.h>
 #include <ctype.h>
 #include <float.h>
@@ -39,7 +41,7 @@ typedef struct {
     GtkCellRenderer *datacell;
     gchar location[20];
     gint datacols, datarows;
-    gint padcols, totcols;
+    gint totcols;
     gint n_scalars;
     guint cid;
     guint point;
@@ -53,7 +55,8 @@ enum {
 static void sheet_add_var_callback (gpointer data, guint code, GtkWidget *w);
 static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w);
 static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet);
-static void set_up_sheet_column (GtkTreeViewColumn *column, gint width);
+static void set_up_sheet_column (GtkTreeViewColumn *column, gint width, 
+				 gboolean expand);
 static gint get_data_col_width (void);
 static int add_data_column (spreadsheet *sheet);
 static void create_sheet_cell_renderers (spreadsheet *sheet);
@@ -84,6 +87,7 @@ static char *single_underscores (char *targ, const char *src)
 	}
 	src++;
     }
+
     *p = '\0';
 
     return targ;
@@ -159,9 +163,8 @@ static void move_to_next_cell (spreadsheet *sheet, GtkTreePath *path,
 	    set_locator_label(sheet, newpath, column);
 	    gtk_tree_path_free(newpath);
 	}
-    } 
-    /* ...or try the next column */
-    else {
+    } else {
+	/* ...or try the next column */
 	gpointer p = g_object_get_data(G_OBJECT(column), "colnum");
 
 	if (p != NULL && (colnum = GPOINTER_TO_INT(p)) < sheet->datacols) {
@@ -182,12 +185,25 @@ static void move_to_next_cell (spreadsheet *sheet, GtkTreePath *path,
 
 static gint sheet_cell_edited (GtkCellRendererText *cell,
 			       const gchar *path_string,
-			       const gchar *new_text,
+			       const gchar *user_text,
 			       spreadsheet *sheet)
 {
-    if (check_atof(new_text)) {
-	errbox(get_gretl_errmsg());
+    const gchar *new_text = NULL;
+    int err = 0;
+
+    if (!strcmp(user_text, "na") || !strcmp(user_text, "NA")) {
+	/* allow conversion to "missing" */
+	new_text = "";
     } else {
+	err = check_atof(user_text);
+	if (err) {
+	    errbox(get_gretl_errmsg());
+	} else {
+	    new_text = user_text;
+	}
+    }
+
+    if (!err && new_text != NULL) {
 	GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
 	GtkTreeModel *model = gtk_tree_view_get_model(view);
 	GtkTreeViewColumn *column;
@@ -198,11 +214,11 @@ static gint sheet_cell_edited (GtkCellRendererText *cell,
 
 	gtk_tree_view_get_cursor(view, NULL, &column);
 	colnum = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(column), "colnum"));
-	path = gtk_tree_path_new_from_string (path_string);
+	path = gtk_tree_path_new_from_string(path_string);
 	gtk_tree_model_get_iter(model, &iter, path);
 	gtk_tree_model_get(model, &iter, colnum, &old_text, -1);
 
-	if (strcmp(old_text, new_text)) {
+	if (old_text != NULL && strcmp(old_text, new_text)) {
 	    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
 			       colnum, new_text, -1);
 	    sheet_modified = 1;
@@ -223,33 +239,25 @@ static int real_add_new_var (spreadsheet *sheet, const char *varname)
 {
     GtkTreeViewColumn *column;
     GList *collist = NULL, *clstart = NULL;
-    gint i, oldcols, cols;
+    gint i, cols;
     char tmp[16];
 
-    oldcols = sheet->totcols;
-
 #ifdef SSDEBUG
-    fprintf(stderr, "real_add_new_var: sheet->totcols=%d\n", oldcols);
+    fprintf(stderr, "real_add_new_var: sheet->totcols=%d\n", 
+	    sheet->totcols);
 #endif
 
     if (add_data_column(sheet)) return 1;
 
 #ifdef SSDEBUG
-    fprintf(stderr, "real_add_new_var: now: sheet->totcols=%d, oldcols=%d\n", 
-	    sheet->totcols, oldcols);
+    fprintf(stderr, "real_add_new_var: now: sheet->totcols=%d\n", 
+	    sheet->totcols);
 #endif
 
-    if (sheet->totcols == oldcols) {
-	/* remove a padding column from the tree view */
-	column = gtk_tree_view_get_column(GTK_TREE_VIEW(sheet->view), 
-					  sheet->datacols);
-	gtk_tree_view_remove_column(GTK_TREE_VIEW(sheet->view), column);
-    }
-
     column = gtk_tree_view_column_new();
     double_underscores(tmp, varname);
     gtk_tree_view_column_set_title(column, tmp);
-    set_up_sheet_column(column, get_data_col_width());
+    set_up_sheet_column(column, get_data_col_width(), TRUE);
 
     cols = gtk_tree_view_insert_column(GTK_TREE_VIEW(sheet->view), 
 				       column, sheet->datacols);
@@ -259,20 +267,18 @@ static int real_add_new_var (spreadsheet *sheet, const char *varname)
     create_sheet_cell_renderers(sheet);
 
     i = 0;
-    while (collist != NULL && i < sheet->totcols - 2) {
+    while (collist != NULL && i < sheet->totcols) {
 	GtkCellRenderer *renderer;
-	gint editcol, idx;
+	gint idx;
 
 	column = GTK_TREE_VIEW_COLUMN(collist->data);
 	gtk_tree_view_column_clear(column);
 
 	if (i > 0 && i <= sheet->datacols) {
 	    renderer = sheet->datacell;
-	    editcol = sheet->totcols - 1;
 	    idx = i;
 	} else {
 	    renderer = sheet->dumbcell;
-	    editcol = sheet->totcols - 2;
 	    idx = 0;
 	}
 
@@ -280,7 +286,6 @@ static int real_add_new_var (spreadsheet *sheet, const char *varname)
 	gtk_tree_view_column_set_attributes (column, 
 					     renderer,
 					     "text", i,
-					     "editable", editcol,
 					     NULL);
 	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(idx));
 
@@ -310,22 +315,21 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
 
     if (sheet->point == SHEET_AT_END) {
 	gtk_list_store_append(store, &iter);
-    } 
-    else if (sheet->point == SHEET_AT_POINT) {
+    } else if (sheet->point == SHEET_AT_POINT) {
 	GtkTreePath *path;
-	GtkTreeViewColumn *column;
 
-	gtk_tree_view_get_cursor(view, &path, &column);
-	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
+	gtk_tree_view_get_cursor(view, &path, NULL);
 	pointpath = gtk_tree_path_get_indices(path)[0];
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
 	gtk_list_store_insert(store, &iter, pointpath);
 	gtk_tree_path_free(path);
-    } 
-    else return;
+    } else {
+	return;
+    }
 
     sheet->datarows += 1;
 
-    if (datainfo->markers) {
+    if (datainfo->markers && obsname != NULL) {
 	gtk_list_store_set(store, &iter, 0, obsname, -1);
     } else if (sheet->point == SHEET_AT_END) {
 	get_full_obs_string(rowlabel, sheet->datarows - 1, datainfo);
@@ -336,15 +340,6 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
 	gtk_list_store_set(store, &iter, i, "", -1);
     }
 
-    for (i=1; i<=sheet->padcols; i++) {
-	gtk_list_store_set(store, &iter, sheet->datacols + i, "", -1);
-    }
-
-    gtk_list_store_set(store, &iter, 
-		       sheet->totcols - 2, FALSE, 
-		       sheet->totcols - 1, TRUE,
-		       -1);
-
     if (sheet->point == SHEET_AT_POINT && !datainfo->markers) {
 	for (i=pointpath; i<sheet->datarows; i++) {
 	    get_full_obs_string(rowlabel, i, datainfo);
@@ -365,13 +360,22 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
 	column = gtk_tree_view_get_column(view, 1);
 	gtk_tree_view_set_cursor(view, path, column, FALSE);
 	adj = 
-	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(gtk_widget_get_ancestor
-								    (GTK_WIDGET(view),
-								     GTK_TYPE_BIN)));
+	    gtk_viewport_get_vadjustment(GTK_VIEWPORT(gtk_widget_get_ancestor
+						      (GTK_WIDGET(view),
+						       GTK_TYPE_BIN)));
 	adjval = gtk_adjustment_get_value(adj);
 	gtk_adjustment_set_value (adj, adjval + 30); /* why is this hack needed? */
 	gtk_tree_path_free(path);
 	g_free(pathstr);
+    } else {
+	gchar *pathstr = g_strdup_printf("%d", pointpath);
+	GtkTreePath *path = gtk_tree_path_new_from_string(pathstr);
+	GtkTreeIter insiter;
+
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &insiter, path);
+	gtk_list_store_set(store, &insiter, 1, "", -1);
+	gtk_tree_path_free(path);
+	g_free(pathstr);
     }
 
     sheet_modified = 1;
@@ -447,44 +451,24 @@ static int add_data_column (spreadsheet *sheet)
     GtkListStore *old_store, *new_store;
     GtkTreeIter old_iter, new_iter;
     gint i, row, newcol;
-    int totcols = sheet->totcols;
-    int padcols = sheet->padcols;
 
     /* This is relatively complex because, so far as I can tell, you can't
        append or insert additional columns in a GtkListStore: we have to
        create a whole new liststore and copy the old info across.
     */
 
-#ifdef SSDEBUG
-    fprintf(stderr, "add_data_column: totcols=%d, padcols=%d\n", 
-	    totcols, padcols);
-#endif    
-
-    if (padcols > 0) padcols--;
-    else totcols++;
-
     /* make an expanded column types list */
-    types = mymalloc(totcols * sizeof *types);
+    types = mymalloc((sheet->totcols + 1) * sizeof *types);
     if (types == NULL) {
 	return 1;
     }
 
     sheet->datacols += 1;
-    sheet->totcols = totcols;
-    sheet->padcols = padcols;
-
-#ifdef SSDEBUG
-    fprintf(stderr, "add_data_column: now sheet->totcols=%d, sheet->padcols=%d,"
-	    " sheet->datacols=%d\n", sheet->totcols, sheet->padcols, sheet->datacols);
-    if (sheet->totcols < sheet->datacols + 3) {
-	fprintf(stderr, "PROBLEM: sheet->totcols < sheet->datacols + 3\n");
-    }
-#endif 
+    sheet->totcols += 1;
 
     /* configure the types */
     for (i=0; i<sheet->totcols; i++) {
-	if (i >= sheet->totcols - 2) types[i] = G_TYPE_BOOLEAN;
-	else types[i] = G_TYPE_STRING;
+	types[i] = G_TYPE_STRING;
     }
 
     newcol = sheet->datacols;
@@ -511,16 +495,10 @@ static int add_data_column (spreadsheet *sheet)
 		gtk_list_store_set(new_store, &new_iter, col, str, -1);
 		g_free(str);
 	    }
-	    else if (col >= newcol && col <= newcol + sheet->padcols) {
-		/* new data values (and any padding) set blank */
+	    else if (col >= newcol && col <= newcol) {
+		/* new data values: set blank */
 		gtk_list_store_set(new_store, &new_iter, col, "", -1);
 	    }
-	    else {
-		/* editable flags at end of row */
-		gboolean editable = (col == sheet->totcols - 1);
-
-		gtk_list_store_set(new_store, &new_iter, col, editable, -1);
-	    }
 	}
 	gtk_tree_model_iter_next(GTK_TREE_MODEL(old_store), &old_iter);
     }    
@@ -551,7 +529,7 @@ static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w)
     if (datainfo->markers) {
 	new_case_dialog(sheet);
     } else {
-	real_add_new_obs(sheet, "");
+	real_add_new_obs(sheet, NULL);
     }
 }
 
@@ -647,8 +625,9 @@ static gboolean update_cell_position (GtkTreeView *view, spreadsheet *sheet)
 
 static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet)
 {
-    gint i, t, n = datainfo->n, oldv = datainfo->v; 
-    gint orig_cols, newvars, newobs, missobs = 0;
+    int n = datainfo->n, oldv = datainfo->v;
+    int i, t;
+    int orig_cols, newvars, newobs, missobs = 0;
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreeIter iter;
     GtkTreeViewColumn *column;
@@ -686,7 +665,9 @@ static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet)
 
     colnum = 0;
     for (i=1; i<datainfo->v; i++) {
-	if (datainfo->vector[i] == 0) continue;
+	if (datainfo->vector[i] == 0) {
+	    continue;
+	}
 	colnum++;
 	gtk_tree_model_get_iter_first(model, &iter);	
 	for (t=0; t<n; t++) {
@@ -796,24 +777,6 @@ static int add_data_to_sheet (spreadsheet *sheet, int new)
 	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
     }
 
-    /* insert padding cols if needed */
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    for (t=0; t<n; t++) {
-	for (i=0; i<sheet->padcols; i++) {
-	    gtk_list_store_set(store, &iter, i + datainfo->v, "", -1);
-	}
-	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
-    }
-
-    /* set the editable flags */
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    for (t=0; t<n; t++) {
-	gtk_list_store_set(store, &iter, 
-			   sheet->totcols - 2, FALSE,
-			   sheet->totcols - 1, TRUE, -1);
-	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
-    }
-
     return 0;
 }
 
@@ -858,12 +821,16 @@ static gint get_data_col_width (void)
 
 /* ........................................................... */
 
-static void set_up_sheet_column (GtkTreeViewColumn *column, gint width)
+static void 
+set_up_sheet_column (GtkTreeViewColumn *column, gint width, gboolean expand)
 {
     gtk_tree_view_column_set_alignment(column, 0.5); /* header centered */
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_min_width(column, width);
+#if (GTK_MINOR_VERSION >= 4)
+    gtk_tree_view_column_set_expand(column, expand);
+#endif
 }
 
 /* ........................................................... */
@@ -873,14 +840,16 @@ static void create_sheet_cell_renderers (spreadsheet *sheet)
     GtkCellRenderer *r;
 
     r = gtk_cell_renderer_text_new();
-    g_object_set(r, "ypad", 1, NULL);
-    g_object_set(r, "xalign", 1.0, NULL);
-    g_object_set(r, "background", "gray", NULL);
+    g_object_set(r, "ypad", 1, 
+		 "xalign", 1.0,
+		 "background", "gray",
+		 "editable", FALSE, NULL);
     sheet->dumbcell = r;
 
     r = gtk_cell_renderer_text_new();
-    g_object_set(r, "ypad", 1, NULL);
-    g_object_set(r, "xalign", 1.0, NULL);
+    g_object_set(r, "ypad", 1, 
+		 "xalign", 1.0, 
+		 "editable", TRUE, NULL);
     g_signal_connect (G_OBJECT (r), "edited",
 		      G_CALLBACK (sheet_cell_edited), sheet);
     sheet->datacell = r;
@@ -953,16 +922,43 @@ static gboolean update_selected (GtkTreeSelection *selection, spreadsheet *sheet
 
 #else
 
-/* Below: prevent cursor movement outside of the data area */
+static void manufacture_keystroke (GtkWidget *widget, guint uval)
+{
+    GdkKeymapKey *keys;
+    gint n_keys;
+
+    if (gdk_keymap_get_entries_for_keyval(NULL, uval, &keys, &n_keys)) {
+	guint16 hardware_keycode;
+	GdkEvent *event;
+
+	hardware_keycode = keys[0].keycode;
+	g_free(keys);
+
+	event = gdk_event_new(GDK_KEY_PRESS);
+	event->key.window = g_object_ref(widget->window);
+	event->key.hardware_keycode = hardware_keycode;
+
+	event->key.keyval = gdk_unicode_to_keyval(uval);
+	event->key.length = 1;
+
+	event->key.send_event = FALSE;
+	event->key.time = GDK_CURRENT_TIME;   
+
+	gtk_main_do_event(event);
+	gdk_event_free(event);
+    }
+}
 
 static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key, 
 				   spreadsheet *sheet)
 {
     if (key->keyval == GDK_Tab) {
-	/* FIXME: translate this to Down or Right? */
-	return TRUE;
+	/* FIXME */
+	;
     }
 
+    /* prevent cursor movement outside of the data area */
+
     if (key->keyval == GDK_Right || key->keyval == GDK_Left) {
 	GtkTreeViewColumn *column;
 	gpointer p;
@@ -979,6 +975,23 @@ static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key,
 		return TRUE;
 	    }
 	}
+    } 
+
+    /* numeric key: start editing */
+
+    else if ((key->keyval >= GDK_0 && key->keyval <= GDK_9) ||
+	key->keyval == GDK_minus || key->keyval == GDK_period) {
+	GtkTreePath *path = NULL;
+	GtkTreeViewColumn *column;
+
+	gtk_tree_view_get_cursor(GTK_TREE_VIEW(view), &path, &column);
+
+	if (path != NULL && column != NULL) {
+	    gtk_tree_view_set_cursor(GTK_TREE_VIEW(view), path, column, 
+				     TRUE);
+	    gtk_tree_path_free(path);
+	    manufacture_keystroke(view, key->keyval);
+	}
     }
 
     return FALSE;
@@ -1078,22 +1091,15 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
 	}
     }
 
-    if (sheet->datacols < 6) {
-	sheet->padcols = 6 - sheet->datacols;
-    } else {
-	sheet->padcols = 0;
-    }
-
     /* obs, data, padding, boolean cols */
-    sheet->totcols = 1 + sheet->datacols + sheet->padcols + 2;
+    sheet->totcols = 1 + sheet->datacols;
 
     types = mymalloc(sheet->totcols * sizeof *types);
     if (types == NULL) return NULL;
 
     /* configure the types */
     for (i=0; i<sheet->totcols; i++) {
-	if (i >= sheet->totcols - 2) types[i] = G_TYPE_BOOLEAN;
-	else types[i] = G_TYPE_STRING;
+	types[i] = G_TYPE_STRING;
     }
 
     store = gtk_list_store_newv (sheet->totcols, types);
@@ -1112,11 +1118,9 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
     column = gtk_tree_view_column_new_with_attributes (NULL,
 						       sheet->dumbcell,
 						       "text", 0, 
-						       "editable", 
-						       sheet->totcols - 2,
 						       NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
-    set_up_sheet_column(column, width);
+    set_up_sheet_column(column, width, FALSE);
     g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(0));
 
     /* construct the data columns */
@@ -1132,28 +1136,12 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
 							   sheet->datacell,
 							   "text", 
 							   colnum, 
-							   "editable", 
-							   sheet->totcols - 1,
 							   NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
-	set_up_sheet_column(column, width);
+	set_up_sheet_column(column, width, TRUE);
 	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(i));
     }
 
-    /* add some padding columns if needed */
-    for (i=0; i<sheet->padcols; i++) {
-	column = gtk_tree_view_column_new_with_attributes (NULL,
-							   sheet->dumbcell,
-							   "text", 
-							   i + sheet->datacols + 1,
-							   "editable", 
-							   sheet->totcols - 2,
-							   NULL);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
-	set_up_sheet_column(column, width);
-	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(0));
-    }
-
     /* set the selection property on the tree view */
     select = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
 #ifdef OLD_SELECTION
@@ -1203,7 +1191,7 @@ static spreadsheet *sheet_new (void)
     sheet->dumbcell = NULL;
     sheet->datacell = NULL;
     sheet->datacols = sheet->datarows = 0;
-    sheet->padcols = sheet->totcols = 0;
+    sheet->totcols = 0;
     sheet->n_scalars = 0;
     sheet->cid = 0;
 
@@ -1259,11 +1247,19 @@ void show_spreadsheet (DATAINFO *pdinfo)
     static spreadsheet *sheet;    
     GtkWidget *tmp, *button_box;
     GtkWidget *scroller, *main_vbox;
+    GtkWidget *hbox, *padbox;
     GtkWidget *status_box, *mbar;
     GtkItemFactory *ifac;
     int sheetwidth;
     int err = 0;
 
+#ifdef G_OS_WIN32
+    if (pdinfo == NULL && datainfo->n > 1600) {
+	errbox(_("Sorry, can't edit more than 1600 rows"));
+	return;
+    }    
+#endif
+
     if (sheet != NULL) {
 	gdk_window_raise(sheet->win->window);
 	return;
@@ -1326,9 +1322,19 @@ void show_spreadsheet (DATAINFO *pdinfo)
     gtk_box_pack_start(GTK_BOX(main_vbox), scroller, TRUE, TRUE, 0);
     gtk_widget_show(scroller);
 
+    hbox = gtk_hbox_new(FALSE, 1);
+    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller), hbox);
+    gtk_widget_show(hbox);
+
     sheet->view = data_sheet_new(sheet, datainfo->n, datainfo->v);
-    gtk_container_add(GTK_CONTAINER(scroller), sheet->view);
-    gtk_widget_show(sheet->view);
+    gtk_container_add(GTK_CONTAINER(hbox), sheet->view);
+    gtk_widget_show(sheet->view); 
+
+    if (datainfo->v < 7) {
+	padbox = gtk_vbox_new(FALSE, 1);
+	gtk_container_add(GTK_CONTAINER(hbox), padbox);
+	gtk_widget_show(padbox);
+    }
 
     /* apply and close buttons */
     button_box = gtk_hbox_new (FALSE, 5);
