@@ -23,7 +23,7 @@
 #include "textbuf.h"
 #include "guiprint.h"
 #include "model_table.h"
-
+#include "clipboard.h"
 
 /* find-and-replace related materials */
 
@@ -35,8 +35,6 @@ struct search_replace {
     gchar *r_text;
 };
 
-/* .................................................................. */
-
 static void replace_string_callback (GtkWidget *widget, 
 				     struct search_replace *s)
 {
@@ -47,8 +45,6 @@ static void replace_string_callback (GtkWidget *widget,
     gtk_widget_destroy(s->w);
 }
 
-/* .................................................................. */
-
 static void trash_replace (GtkWidget *widget, 
 			   struct search_replace *s)
 {
@@ -57,8 +53,6 @@ static void trash_replace (GtkWidget *widget,
     gtk_widget_destroy(s->w);
 }
 
-/* .................................................................. */
-
 static void replace_string_dialog (struct search_replace *s)
 {
     GtkWidget *label, *button, *hbox;
@@ -85,15 +79,10 @@ static void replace_string_dialog (struct search_replace *s)
     label = gtk_label_new(_("Replace with:"));
     gtk_widget_show (label);
     s->r_entry = gtk_entry_new();
-#ifdef OLD_GTK
-    gtk_signal_connect(GTK_OBJECT (s->r_entry), 
-			"activate", 
-		       GTK_SIGNAL_FUNC (replace_string_callback), s);
-#else
     g_signal_connect(G_OBJECT (s->r_entry), 
 		     "activate", 
 		     G_CALLBACK (replace_string_callback), s);
-#endif
+
     gtk_widget_show (s->r_entry);
     gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX(hbox), s->r_entry, TRUE, TRUE, 0);
@@ -106,26 +95,16 @@ static void replace_string_dialog (struct search_replace *s)
 			    (GTK_DIALOG (s->w)->action_area), TRUE);
     gtk_window_set_position(GTK_WINDOW (s->w), GTK_WIN_POS_MOUSE);
 
-#ifdef OLD_GTK
-    gtk_signal_connect(GTK_OBJECT(s->w), "destroy",
-		       gtk_main_quit, NULL);
-#else
     g_signal_connect(G_OBJECT(s->w), "destroy",
 		     gtk_main_quit, NULL);
-#endif
 
     /* replace button -- make this the default */
     button = gtk_button_new_with_label (_("Replace all"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (s->w)->action_area), 
 		       button, TRUE, TRUE, FALSE);
-#ifdef OLD_GTK
-    gtk_signal_connect(GTK_OBJECT (button), "clicked",
-		       GTK_SIGNAL_FUNC (replace_string_callback), s);
-#else
     g_signal_connect(G_OBJECT (button), "clicked",
 		     G_CALLBACK (replace_string_callback), s);
-#endif
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
@@ -134,13 +113,8 @@ static void replace_string_dialog (struct search_replace *s)
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (s->w)->action_area), 
 		       button, TRUE, TRUE, FALSE);
-#ifdef OLD_GTK
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(trash_replace), s);
-#else
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(trash_replace), s);
-#endif
 
     gtk_widget_show(button);
 
@@ -152,8 +126,6 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_main();
 }
 
-/* ........................................................... */
-
 #ifdef OLD_GTK
 
 void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
@@ -280,7 +252,7 @@ void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
     g_free(buf);
 }
 
-#else
+#else /* not gtk-1.2 */
 
 void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
 {
@@ -412,87 +384,7 @@ void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
     g_free(buf);
 }
 
-#endif /* !OLD_GTK */
-
-/* clipboard-related materials */
-
-#if defined(G_OS_WIN32)
-
-int prn_to_clipboard (PRN *prn, int copycode)
-{
-    return win_copy_buf(prn->buf, copycode, 0);
-}
-
-#elif defined(ENABLE_NLS) && !defined(OLD_GTK)
-
-int prn_to_clipboard (PRN *prn, int copycode)
-{
-    if (prn->buf == NULL) return 0;
-
-    if (clipboard_buf) g_free(clipboard_buf);
-    clipboard_buf = NULL;
-
-    if (copycode == COPY_TEXT || copycode == COPY_TEXT_AS_RTF) { 
-	/* need to convert from utf8 */
-	gchar *trbuf = my_locale_from_utf8(prn->buf);
-	
-	if (trbuf != NULL) {
-	    size_t len = strlen(trbuf);
-
-	    if (copycode == COPY_TEXT_AS_RTF) {
-		clipboard_buf = dosify_buffer(trbuf, copycode);
-	    } else {
-		clipboard_buf = mymalloc(len + 1);
-	    }
-	    if (clipboard_buf == NULL) {
-		g_free(trbuf);
-		return 1;
-	    }
-	    if (copycode != COPY_TEXT_AS_RTF) {
-		memcpy(clipboard_buf, trbuf, len + 1);
-	    }
-	    g_free(trbuf);
-	}
-    } else { /* copying TeX, RTF or CSV */
-	size_t len = strlen(prn->buf);
-
-	fprintf(stderr, "Copying to clipboard, %d bytes\n", (int) len);
-	clipboard_buf = mymalloc(len + 1);
-	if (clipboard_buf == NULL) {
-	    return 1;
-	}
-	memcpy(clipboard_buf, prn->buf, len + 1);
-    }
-
-    gtk_selection_owner_set(mdata->w,
-			    GDK_SELECTION_PRIMARY, 
-			    GDK_CURRENT_TIME);
-    return 0;
-}
-
-#else /* plain GTK, no NLS */
-
-int prn_to_clipboard (PRN *prn, int copycode)
-{
-    size_t len;
-    
-    if (prn->buf == NULL) return 0;
-    len = strlen(prn->buf);
-    if (len == 0) return 0;
-
-    if (clipboard_buf) g_free(clipboard_buf);
-    clipboard_buf = mymalloc(len + 1);
-    if (clipboard_buf == NULL) return 1;
-
-    memcpy(clipboard_buf, prn->buf, len + 1);
-
-    gtk_selection_owner_set(mdata->w,
-			    GDK_SELECTION_PRIMARY,
-			    GDK_CURRENT_TIME);
-    return 0;
-}
-
-#endif /* switch for prn_to_clipboard */
+#endif /* old versus new gtk */
 
 /* copying text from gretl windows */
 
@@ -738,7 +630,7 @@ void text_copy (gpointer data, guint how, GtkWidget *w)
     }
 }
 
-/* printing from gretl windows */
+/* native printing from gretl windows */
 
 #if defined(G_OS_WIN32) || defined (USE_GNOME)
 
@@ -795,3 +687,69 @@ void system_print_buf (const gchar *buf, FILE *fp)
 	putc('\n', fp);
     }
 }
+
+/* convert a buffer to DOS/Windows text format, optionally
+   adding minimal RTF formatting */
+
+char *dosify_buffer (const char *buf, int format)
+{
+    int extra = 0, nlines = 0;
+    char *targ, *q;
+    const char *p;
+    const char *rtf_preamble = "{\\rtf1\n"
+	"{\\fonttbl{\\f0\\fnil\\fprq1\\fcharset1 Courier New;}}\n"
+	"\\f0\\fs18\n";
+    int rtf_add_bytes = strlen(rtf_preamble) + 4;
+
+    if (buf == NULL || *buf == '\0') {
+	return NULL;
+    }
+
+    p = buf;
+    while (*p) {
+	if (*p++ == '\n') nlines++;
+    }
+    extra = nlines + 1;
+
+    if (format == COPY_TEXT_AS_RTF) {
+	extra *= 5;
+	extra += rtf_add_bytes;
+    }
+
+    targ = malloc(strlen(buf) + extra);
+    if (targ == NULL) {
+	return NULL;
+    }
+
+    if (format == COPY_TEXT_AS_RTF) {
+	strcpy(targ, rtf_preamble);
+	q = targ + strlen(targ);
+    } else {
+	q = targ;
+    }
+
+    p = buf;
+    while (*p) {
+	if (*p == '\n') {
+	    if (format == COPY_TEXT_AS_RTF) {
+		*q++ = '\\';
+		*q++ = 'p';
+		*q++ = 'a';
+		*q++ = 'r';
+	    }
+	    *q++ = '\r';
+	    *q++ = '\n';
+	} else {
+	    *q++ = *p;
+	}
+	p++;
+    } 
+    *q = 0;
+
+    if (format == COPY_TEXT_AS_RTF) {
+	strcat(q, "}\n");
+    }
+
+    return targ;
+}
+
