@@ -47,6 +47,7 @@
 
 #ifdef WIN32
 # include <winsock.h>
+# include <glib/gstdio.h>
 #else
 # include <sys/socket.h>
 # include <netdb.h>
@@ -54,6 +55,12 @@
 # include <arpa/inet.h>
 #endif /* WIN32 */
 
+#if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
+# ifdef WIN32
+#  define USE_G_FOPEN
+# endif
+#endif
+
 #include "webget.h"
 
 #ifdef UPDATER
@@ -855,7 +862,11 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 #endif   
 
     if (u->saveopt == SAVE_TO_FILE) { 
+#ifdef USE_G_FOPEN
+	fp = g_fopen(u->localfile, "wb");
+#else
 	fp = fopen(u->localfile, "wb");
+#endif
 	if (fp == NULL) {
 	    close(sock);
 	    free(all_headers);
@@ -1595,8 +1606,11 @@ static time_t get_time_from_stamp_file (const char *fname)
         "Oct", "Nov", "Dec"
     };
 
-
+#ifdef USE_G_FOPEN
+    fp = g_fopen(fname, "r");
+#else
     fp = fopen(fname, "r");
+#endif
     if (fp == NULL) return (time_t) 0;
 
     if (fscanf(fp, "%3s %3s %d %d:%d:%d %*s %d", 
