@@ -27,18 +27,19 @@
 struct _model_info {
 
     /* members that may be set by caller of bhhh_max, via accessor
-       functions: */
+       functions */
 
     int k;              /* number of parameters */
-    int p, q, r;        /* for use with ARMA: AR, MA orders and number of
-                           other regressors.  Otherwise unused. */
     int t1, t2;         /* starting and ending point of sample */
     int n_series;       /* number of additional series needed in the
                            likelihood and/or score calculations */
     double tol;         /* tolerance for convergence */
     unsigned char opts; /* options from among bhhh_opts */
 
-    /* members set within bhhh_max: */
+    void *extra_info;   /* pointer to additional info which may be
+			   used in likelihood callback */
+
+    /* members set within bhhh_max */
 
     int n;            /* length of series */
     int iters;        /* number of iterations taken */
@@ -192,24 +193,6 @@ int model_info_get_iters (const model_info *model)
     return model->iters;
 }
 
-/**
- * model_info_get_pqr:
- * @model: model info pointer.
- * @p: pointer to receive the AR order of @model.
- * @q: pointer to receive the MA order of @model.
- * @r: pointer to receive the number of regular regressors in 
- * ARMA(X) @model.
- *
- */  
- 
-void model_info_get_pqr (const model_info *model, 
-			 int *p, int *q, int *r)
-{
-    *p = model->p;
-    *q = model->q;
-    *r = model->r;
-}
-
 /**
  * model_info_get_series:
  * @model: model info pointer.
@@ -241,8 +224,9 @@ double model_info_get_ll (const model_info *model)
  * @do_score:
  *
  * Sets the log-likelihood for @model.  If @do_score is non-zero,
- * sets the primary ll value, otherwise sets the secondary value.
- * FIXME: explain this.
+ * sets the primary ll value, otherwise sets the secondary value,
+ * which is used for comparison (to see if we have succeeded in
+ * increasing the likelihood by following the estimated gradient).
  */ 
 
 void model_info_set_ll (model_info *model, double ll, int do_score)
@@ -274,7 +258,6 @@ void model_info_set_opts (model_info *model, unsigned char opts)
  * @tol: tolerance for convergence of estimates.
  *
  * Sets the convergence tolerance for @model. 
- * FIXME: explain this.
  */ 
 
 void model_info_set_tol (model_info *model, double tol)
@@ -282,43 +265,24 @@ void model_info_set_tol (model_info *model, double tol)
     model->tol = tol;
 }
 
-/**
- * model_info_set_pqr:
- * @model: model info pointer.
- * @p: AR order for @model.
- * @q: MA order for @model.
- * @r: Number of regular regressors in ARMA(X) @model.
- *
- * Sets the specified parameters for @model.
- */ 
-
-void model_info_set_pqr (model_info *model, int p, int q, int r)
-{
-    model->p = p;
-    model->q = q;
-    model->r = r;
-    model->k = p + q + r + 1;
-}
-
 /**
  * model_info_set_k:
  * @model: model info pointer.
- * @k: number of regressors in (non-ARMA) model.
+ * @k: number of regressors in model.
  *
- * Sets the number of regressors.
+ * Sets the total number of regressors.
  */ 
 
 void model_info_set_k (model_info *model, int k)
 {
     model->k = k;
-    model->p = model->q = model->r = 0;
 }
 
 /**
  * model_info_get_k:
  * @model: model info pointer.
  *
- * Returns: the number of regressors in (non-ARMA) @model.
+ * Returns: the number of regressors in @model.
  */ 
 
 int model_info_get_k (model_info *model)
@@ -331,7 +295,8 @@ int model_info_get_k (model_info *model)
  * @model: model info pointer.
  * @n: number of data series.
  *
- * Sets the number of auxiliary data series needed for @model.
+ * Sets the number of auxiliary data series needed for the
+ * OPG regression associated with @model.
  */ 
 
 void model_info_set_n_series (model_info *model, int n)
@@ -355,8 +320,22 @@ void model_info_set_t1_t2 (model_info *model, int t1, int t2)
     model->n = t2 + 1;
 }
 
-/* Below: construct the regression list for the OPG regression, with
-   the appropriate indices into the temporary artificial dataset.
+/* set extra information pointer on basic model_info */
+
+void model_info_set_extra_info (model_info *model, void *extra)
+{
+    model->extra_info = extra;
+}
+
+/* retrieve extra information pointer from model_info */
+
+void *model_info_get_extra_info (model_info *model)
+{
+    return model->extra_info;
+}
+
+/* Construct the regression list for the OPG regression, with the
+   appropriate indices into the temporary artificial dataset.
 */
 
 static int *make_opg_list (int k)
@@ -382,12 +361,10 @@ static int *make_opg_list (int k)
 
 static int model_info_init (model_info *model, const double *init_coeff)
 {
-    int i, t, k, err = 0;
+    int i, t, err = 0;
     int n_series = model->n_series;
 
-    k = model->k;  
-
-    model->theta = malloc(k * sizeof *model->theta);
+    model->theta = malloc(model->k * sizeof *model->theta);
 
     if (model->theta == NULL) {
 	model_info_free(model);
@@ -416,7 +393,7 @@ static int model_info_init (model_info *model, const double *init_coeff)
     }
 
     /* initialize parameters */
-    for (i=0; i<k; i++) {
+    for (i=0; i<model->k; i++) {
 	model->theta[i] = init_coeff[i];
     }
 
@@ -451,7 +428,8 @@ static int bhhh_iter_info (int iter, double *theta, int m, double ll,
 {
     int i;
 
-    pprintf(prn, "\n*** %s %d: theta, ll ***\n", ("iteration"), iter);
+    pprintf(prn, "\n*** %s %d: theta, ll ***\n", _("iteration"), iter);
+
     for (i=0; i<m; i++) {
 	if (i && i % 5 == 0) pputc(prn, '\n');
 	if (na(theta[i]) || isnan(theta[i])) {
@@ -460,6 +438,7 @@ static int bhhh_iter_info (int iter, double *theta, int m, double ll,
 	}
 	pprintf(prn, "%#12.5g ", theta[i]);
     }
+
     pprintf(prn, "\n    %s = %g, ll = %g\n", _("step length"),
 	    steplength, ll);
 
@@ -470,7 +449,7 @@ static int bhhh_iter_info (int iter, double *theta, int m, double ll,
  * bhhh_max:
  * @loglik: pointer to function for calculating log-likelihood and
  * score matrix.
- * @X: data set (used by @loglik).
+ * @X: original data set (passed on for use by @loglik).
  * @init_coeff: starting values for coefficients.
  * @model: model info struct, with some initialization carried out by
  * the caller.
@@ -507,7 +486,9 @@ int bhhh_max (LL_FUNC loglik,
     int i, t, err, k;
 
     err = model_info_init(model, init_coeff);
-    if (err) return E_ALLOC;
+    if (err) {
+	return E_ALLOC;
+    }
 
     k = model->k;
     blist = make_opg_list(k);
@@ -539,6 +520,9 @@ int bhhh_max (LL_FUNC loglik,
 
     /* zero the dataset */
     for (i=1; i<=k; i++) {
+#ifdef BHHH_DEBUG
+	sprintf(tinfo->varname[i], "tZ%d", i);
+#endif
 	for (t=0; t<model->n; t++) {
 	    tZ[i][t] = 0.0;
 	}
@@ -560,6 +544,10 @@ int bhhh_max (LL_FUNC loglik,
 	    break;
 	}
 
+#ifdef BHHH_DEBUG
+	pprintf(prn, "Top of loop: ll = %g\n", model->ll);
+#endif
+
 	/* BHHH via OPG regression */
 	*bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A, 0.0);
 	if (bmod->errcode) {
@@ -568,9 +556,9 @@ int bhhh_max (LL_FUNC loglik,
 	}
 
 #ifdef BHHH_DEBUG
-	pprintf(prn, "Dataset for OPG regression\n");
+	pprintf(prn, "Dataset for OPG regression:\n");
 	printdata(blist, (const double **) tZ, tinfo, OPT_O, prn);
-	pprintf(prn, "OLS model, OPG regression\n");
+	pprintf(prn, "OLS model, OPG regression:\n");
 	printmodel(bmod, tinfo, 0, prn);
 #endif
 
@@ -581,9 +569,13 @@ int bhhh_max (LL_FUNC loglik,
 	
 	clear_model(bmod);
 
-	/* see if we've gone up... (0 = "don't compute score") */
+	/* see if we've gone up... (0 means "don't compute score") */
 	err = loglik(ctemp, X, tZ, model, 0); 
 
+#ifdef BHHH_DEBUG
+	pprintf(prn, "bhhh loop: initial ll2 = %g\n", model->ll2);
+#endif
+
 	while (model->ll2 < model->ll || err) { 
 	    /* ... if not, halve the steplength */
 	    stepsize *= 0.5;
@@ -596,6 +588,9 @@ int bhhh_max (LL_FUNC loglik,
 		ctemp[i] = model->theta[i] + delta[i];
 	    }
 	    err = loglik(ctemp, X, tZ, model, 0);
+#ifdef BHHH_DEBUG
+	    pprintf(prn, "bhhh loop: modified ll2 = %g\n", model->ll2);
+#endif
 	}
 
 	if (err) break;
@@ -606,7 +601,9 @@ int bhhh_max (LL_FUNC loglik,
 	}	
 
 	/* double the steplength? */
-	if (stepsize < 4.0) stepsize *= 2.0;
+	if (stepsize < 4.0) {
+	    stepsize *= 2.0;
+	}
 
 	/* print interation info, if wanted */
 	bhhh_iter_info(iters, model->theta, k, model->ll, stepsize, prn);
