@@ -39,17 +39,54 @@ struct COMPARE {
     int robust;    /* = 1 when robust vcv is in use, else 0 */
 };
 
+static int 
+add_diffvars_to_test (GRETLTEST *test, const int *list, 
+		      const DATAINFO *pdinfo)
+{
+    int i, len = 0;
+    int err = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	len += strlen(pdinfo->varname[list[i]]) + 1;
+    }
+
+    test->param = malloc(len);
+
+    if (test->param == NULL) {
+	err = 1;
+    } else {
+	*test->param = '\0';
+	for (i=1; i<=list[0]; i++) {
+	    strcat(test->param, pdinfo->varname[list[i]]);
+	    if (i < list[0]) {
+		strcat(test->param, " ");
+	    }
+	}
+    }
+
+    return err;
+}
+
 static void
 gretl_print_compare (const struct COMPARE *cmp, const int *diffvars, 
-		     const DATAINFO *pdinfo, PRN *prn,
+		     MODEL *new, const DATAINFO *pdinfo, PRN *prn,
 		     gretlopt opt)
 {
+    GRETLTEST test;
+    GRETLTEST *ptest = NULL;
+    double pval = NADBL;
     int i;
 
     if (cmp->ci == LAD) {
 	return;
     }
 
+    if (opt & OPT_S) {
+	gretl_test_init(&test, (cmp->cmd == OMIT)? 
+			GRETL_TEST_OMIT : GRETL_TEST_ADD);
+	ptest = &test;
+    }
+
     if (!(opt & OPT_Q)) {
 	pprintf(prn, _("Comparison of Model %d and Model %d:\n"), 
 		cmp->m1, cmp->m2);
@@ -63,12 +100,21 @@ gretl_print_compare (const struct COMPARE *cmp, const int *diffvars,
 	    pprintf(prn, "    %s\n", pdinfo->varname[diffvars[i]]);	
 	}
 	if (!na(cmp->F)) {
+	    pval = fdist(cmp->F, cmp->dfn, cmp->dfd);
 	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		    (cmp->robust)? _("Robust F") : "F",
 		    cmp->dfn, cmp->dfd, cmp->F);
-	    pprintf(prn, _("with p-value = %g\n"), 
-		    fdist(cmp->F, cmp->dfn, cmp->dfd));	    
+	    pprintf(prn, _("with p-value = %g\n"), pval);
+	    record_test_result(cmp->F, pval, (cmp->cmd == OMIT)? "omit" : "add");
 	} 
+
+	if (ptest != NULL) {
+	    ptest->teststat = GRETL_STAT_F;
+	    ptest->dfn = cmp->dfn;
+	    ptest->dfd = cmp->dfd;
+	    ptest->value = cmp->F;
+	    ptest->pvalue = pval;
+	}
     } else if (LIMDEP(cmp->ci) && cmp->dfn > 0 && diffvars[0] > 0) {
         pputs(prn, _("\n  Null hypothesis: the regression parameters are "
 		     "zero for the variables\n\n"));
@@ -77,14 +123,30 @@ gretl_print_compare (const struct COMPARE *cmp, const int *diffvars,
         }
 	pprintf(prn, "\n  %s: %s(%d) = %g, ",  _("Test statistic"),
 		_("Chi-square"), cmp->dfn, cmp->chisq);
-	pprintf(prn, _("with p-value = %g\n\n"), 
-		chisq(cmp->chisq, cmp->dfn));
-	return;
+	pval = chisq(cmp->chisq, cmp->dfn);
+	pprintf(prn, _("with p-value = %g\n\n"), pval);
+	record_test_result(cmp->chisq, pval, (cmp->cmd == OMIT)? "omit" : "add");
+
+	if (ptest != NULL) {
+	    ptest->teststat = GRETL_STAT_LR;
+	    ptest->dfn = cmp->dfn;
+	    ptest->value = cmp->chisq;
+	    ptest->pvalue = pval;
+	}	
     } 
 
+    if (ptest != NULL) {
+	int err;
+
+	err = add_diffvars_to_test(ptest, diffvars, pdinfo);
+	if (!err) {
+	    add_test_to_model(new, ptest);
+	}
+    }
+
     if (!(opt & OPT_Q)) {
-	pprintf(prn, _("  Of the 8 model selection statistics, %d "), 
-		cmp->score);
+	pprintf(prn, _("  Of the %d model selection statistics, %d "), 
+		C_MAX, cmp->score);
 	if (cmp->score == 1) {
 	    pputs(prn, _("has improved.\n"));
 	} else {
@@ -135,7 +197,7 @@ add_or_omit_compare (const MODEL *pmodA, const MODEL *pmodB, int add)
 	cmp.F = ((rmod->ess - umod->ess) / umod->ess) * cmp.dfd / cmp.dfn;
     }
 
-    for (i=0; i<8; i++) { 
+    for (i=0; i<C_MAX; i++) { 
 	if (na(pmodB->criterion[i]) || na(pmodA->criterion[i])) {
 	    continue;
 	}
@@ -147,7 +209,7 @@ add_or_omit_compare (const MODEL *pmodA, const MODEL *pmodB, int add)
     return cmp;
 }
 
-/* reconstitute full varlist for WLS and AR models */
+/* reconstitute full varlist for WLS, POISSON and AR models */
 
 static int *
 full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
@@ -155,18 +217,23 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
     int i, len, pos = 0;
     int *flist = NULL;
 
-    if (pmod->ci != WLS && pmod->ci != AR) 
+    if (pmod->ci != WLS && pmod->ci != POISSON && pmod->ci != AR) {
 	return NULL;
+    }
 
     if (pmod->ci == WLS) { 
 	len = inlist[0] + 2;
+    } else if (pmod->ci == POISSON) {
+	len = inlist[0] + 3;
     } else {
 	pos = pmod->arinfo->arlist[0] + 1;
 	len = pos + inlist[0] + 2;
     }
 
     flist = malloc(len * sizeof *flist);
-    if (flist == NULL) return NULL;
+    if (flist == NULL) {
+	return NULL;
+    }
 
     if (pmod->ci == WLS) { 
 	flist[0] = len - 1;
@@ -174,8 +241,16 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
 	for (i=1; i<=inlist[0]; i++) {
 	    flist[i+1] = inlist[i];
 	}
-    }
-    else if (pmod->ci == AR) {
+    } else if (pmod->ci == POISSON) {
+	int offvar = gretl_model_get_int(pmod, "offset_var");
+
+	flist[0] = len - 1;
+	for (i=1; i<=inlist[0]; i++) {
+	    flist[i] = inlist[i];
+	}
+	flist[flist[0] - 1] = LISTSEP;
+	flist[flist[0]] = offvar;
+    } else if (pmod->ci == AR) {
 	flist[0] = len - 2;
 	for (i=1; i<pos; i++) {
 	    flist[i] = pmod->arinfo->arlist[i];
@@ -220,7 +295,8 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	/* panel model with per-unit weights */
 	lsqopt |= OPT_W;
 	repci = POOLED;
-    } else if (orig->ci == WLS || orig->ci == AR) {
+    } else if (orig->ci == WLS || orig->ci == AR || 
+	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
 	int *full_list = full_model_list(orig, list, &pos);
 
 	free(list);
@@ -251,6 +327,9 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
     case LAD:
 	rep = lad(list, pZ, pdinfo);
 	break;
+    case POISSON:
+	rep = poisson_model(list, pZ, pdinfo, NULL);
+	break;
     case LOGISTIC: 
 	{
 	    char lmaxstr[32];
@@ -266,6 +345,9 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	if (gretl_model_get_int(orig, "robust")) {
 	    lsqopt |= OPT_R;
 	}
+	if (gretl_model_get_int(orig, "hc_version") == 4) {
+	    repci = HCCM;
+	}
 	rep = lsq(list, pZ, pdinfo, repci, lsqopt, rho);
 	break;
     }
@@ -312,25 +394,25 @@ real_nonlinearity_test (const MODEL *pmod, int *list,
     } else {
 	double trsq = aux.rsq * aux.nobs;
 	int df = list[0] - pmod->list[0];
+	double pval = chisq(trsq, df);
 
 	aux.aux = aux_code;
 	printmodel(&aux, pdinfo, opt, prn);
 
 	pprintf(prn, "\n%s: TR^2 = %g,\n", _("Test statistic"), trsq);
 	pprintf(prn, _("with p-value = prob(Chi-square(%d) > %g) = %g\n\n"), 
-		df, trsq, chisq(trsq, df));
+		df, trsq, pval);
 
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, (aux_code == AUX_SQ)?
-		   N_("Non-linearity test (squares)") :
-		   N_("Non-linearity test (logs)"));
-	    strcpy(test->h_0, N_("relationship is linear"));
-	    test->teststat = GRETL_TEST_TR2;
+	    gretl_test_init(test, (aux_code == AUX_SQ)?
+			    GRETL_TEST_SQUARES : GRETL_TEST_LOGS);
+	    test->teststat = GRETL_STAT_TR2;
 	    test->dfn = df;
 	    test->value = trsq;
 	    test->pvalue = chisq(trsq, df);
 	}
+
+	record_test_result(trsq, pval, "non-linearity");
     } 
 
     clear_model(&aux);
@@ -433,8 +515,13 @@ int add_test (int *addvars, MODEL *orig, MODEL *new,
 {
     int *tmplist = NULL;
     const int orig_nvar = pdinfo->v; 
+    int save_test = 0;
     int err = 0;
 
+    if (orig == NULL || orig->list == NULL) {
+	return 1;
+    }
+
     if (!command_ok_for_model(ADD, orig->ci)) {
 	return E_NOTIMP;
     }
@@ -455,6 +542,11 @@ int add_test (int *addvars, MODEL *orig, MODEL *new,
        was estimated */
     exchange_smpl(orig, pdinfo);
 
+    if (opt & OPT_S) {
+	save_test = 1;
+	opt &= ~OPT_S;
+    }
+
     /* run augmented regression, matching the original
        estimation method */
     *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, opt, prn);
@@ -483,7 +575,11 @@ int add_test (int *addvars, MODEL *orig, MODEL *new,
 		cmp.F = robust_omit_F(addvars, new);
 	    }
 
-	    gretl_print_compare(&cmp, addvars, pdinfo, prn, opt);
+	    if (save_test) {
+		opt |= OPT_S;
+	    }
+
+	    gretl_print_compare(&cmp, addvars, new, pdinfo, prn, opt);
 	}
     }
 
@@ -631,10 +727,16 @@ int omit_test (int *omitvars, MODEL *orig, MODEL *new,
 {
     int *tmplist = NULL;
     int maxlag = 0, t1 = pdinfo->t1;
+    int save_test = 0;
     int err = 0;
 
-    if (!command_ok_for_model(OMIT, orig->ci))
+    if (orig == NULL || orig->list == NULL) {
+	return 1;
+    }
+
+    if (!command_ok_for_model(OMIT, orig->ci)) {
 	return E_NOTIMP;
+    }
 
     /* check that vars to omit have not been redefined */
     if ((err = list_members_replaced(orig->list, pdinfo, orig->ID))) {
@@ -642,7 +744,13 @@ int omit_test (int *omitvars, MODEL *orig, MODEL *new,
     }
 
     /* create list for test model */
-    tmplist = gretl_list_omit(orig->list, omitvars, &err);
+    if (omitvars == NULL) {
+	/* special: just drop the last variable */
+	tmplist = gretl_list_omit_last(orig->list, &err);
+    } else {
+	tmplist = gretl_list_omit(orig->list, omitvars, &err);
+    }
+
     if (tmplist == NULL) {
 	return err;
     }
@@ -663,6 +771,11 @@ int omit_test (int *omitvars, MODEL *orig, MODEL *new,
 	pdinfo->t1 -= 1;
     }
 
+    if (opt & OPT_S) {
+	save_test = 1;
+	opt &= ~OPT_S;
+    }
+
     *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, opt, prn);
 
     if (new->errcode) {
@@ -679,18 +792,22 @@ int omit_test (int *omitvars, MODEL *orig, MODEL *new,
 	    printmodel(new, pdinfo, opt, prn); 
 	}	
 
-	if (new->nobs == orig->nobs) {
+	if (new->nobs == orig->nobs && omitvars != NULL) {
 	    struct COMPARE cmp;
 
 	    cmp = add_or_omit_compare(orig, new, 0);
 
 	    gretl_list_diff(omitvars, orig->list, new->list);
 
-	    if (gretl_model_get_int(orig, "robust") || orig->ci == HCCM) {
+	    if (gretl_model_get_int(orig, "robust")) {
 		cmp.F = robust_omit_F(omitvars, orig);
 	    }
 
-	    gretl_print_compare(&cmp, omitvars, pdinfo, prn, opt); 
+	    if (save_test) {
+		opt |= OPT_S;
+	    }
+
+	    gretl_print_compare(&cmp, omitvars, new, pdinfo, prn, opt); 
 	}
 
 	if (orig->ci == LOGIT || orig->ci == PROBIT) {
@@ -752,13 +869,6 @@ static int ljung_box (int varno, int order, const double **Z,
     return 0;
 }
 
-void gretl_test_init (GRETLTEST *test)
-{
-    test->type[0] = 0;
-    test->h_0[0] = 0;
-    test->param[0] = 0;
-}
-
 /**
  * reset_test:
  * @pmod: pointer to model to be tested.
@@ -785,8 +895,9 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     gretl_model_init(&aux);
 
-    if (pmod->ncoeff + 2 >= pdinfo->t2 - pdinfo->t1)
+    if (pmod->ncoeff + 2 >= pdinfo->t2 - pdinfo->t1) {
 	return E_DF;
+    }
 
     newlist = malloc((pmod->list[0] + 3) * sizeof *newlist);
     if (newlist == NULL) {
@@ -824,24 +935,27 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     } 
 
     if (!err) {
+	double pval;
+
 	aux.aux = AUX_RESET;
 	printmodel(&aux, pdinfo, OPT_NONE, prn);
 	RF = ((pmod->ess - aux.ess) / 2) / (aux.ess / aux.dfd);
+	pval = fdist(RF, 2, aux.dfd);
 
 	pprintf(prn, "\n%s: F = %f,\n", _("Test statistic"), RF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
-		2, aux.dfd, RF, fdist(RF, 2, aux.dfd));
+		2, aux.dfd, RF, pval);
 
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, N_("RESET test for specification"));
-	    strcpy(test->h_0, N_("specification is adequate"));
-	    test->teststat = GRETL_TEST_RESET;
+	    gretl_test_init(test, GRETL_TEST_RESET);
+	    test->teststat = GRETL_STAT_RESET;
 	    test->dfn = 2;
 	    test->dfd = aux.dfd;
 	    test->value = RF;
 	    test->pvalue = fdist(RF, test->dfn, test->dfd);
 	}
+
+	record_test_result(RF, pval, "RESET");
     }
 
     free(newlist);
@@ -1084,7 +1198,9 @@ int autocorr_test (MODEL *pmod, int order,
 	    (aux.nobs - pmod->ncoeff - order)/order; 
 
 	pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
+
 	pval = fdist(LMF, order, aux.nobs - pmod->ncoeff - order);
+
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
 		order, aux.nobs - pmod->ncoeff - order, LMF, pval);
 
@@ -1101,16 +1217,16 @@ int autocorr_test (MODEL *pmod, int order,
 	}
 
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, N_("LM test for autocorrelation up to order %s"));
-	    strcpy(test->h_0, N_("no autocorrelation"));
-	    sprintf(test->param, "%d", order);
-	    test->teststat = GRETL_TEST_LMF;
+	    gretl_test_init(test, GRETL_TEST_AUTOCORR);
+	    test->order = order;
+	    test->teststat = GRETL_STAT_LMF;
 	    test->dfn = order;
 	    test->dfd = aux.nobs - pmod->ncoeff - order;
 	    test->value = LMF;
-	    test->pvalue = fdist(LMF, test->dfn, test->dfd);
+	    test->pvalue = pval;
 	}
+
+	record_test_result(LMF, pval, "autocorrelation");
     }
 
     free(newlist);
@@ -1225,27 +1341,28 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	    err = chow_mod.errcode;
 	    errmsg(err, prn);
 	} else {
+	    double pval;
+
 	    chow_mod.aux = AUX_CHOW;
 	    printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
 	    F = (pmod->ess - chow_mod.ess) * chow_mod.dfd / 
 		(chow_mod.ess * newvars);
+	    pval = fdist(F, newvars, chow_mod.dfd);
 	    pprintf(prn, _("\nChow test for structural break at observation %s:\n"
 		    "  F(%d, %d) = %f with p-value %f\n\n"), chowdate,
-		    newvars, chow_mod.dfd, F, 
-		    fdist(F, newvars, chow_mod.dfd)); 
+		    newvars, chow_mod.dfd, F, pval);
 
 	    if (test != NULL) {
-		gretl_test_init(test);
-		strcpy(test->type, N_("Chow test for structural break at "
-			"observation %s"));
-		strcpy(test->param, chowdate);
-		strcpy(test->h_0, N_("no structural break"));
-		test->teststat = GRETL_TEST_F;
+		gretl_test_init(test, GRETL_TEST_CHOW);
+		test->param = gretl_strdup(chowdate);
+		test->teststat = GRETL_STAT_F;
 		test->dfn = newvars;
 		test->dfd = chow_mod.dfd;
 		test->value = F;
-		test->pvalue = fdist(F, newvars, chow_mod.dfd);
+		test->pvalue = pval;
 	    }
+
+	    record_test_result(F, pval, "Chow");
 	}
 	clear_model(&chow_mod);
     }
@@ -1370,10 +1487,13 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn,
     }
 
     if (!err) {
+	double pval = NADBL;
+
 	wbar /= T - K;
 	pprintf(prn, "\n%s\n\n",
 		_("CUSUM test for stability of parameters"));
 	pprintf(prn, _("mean of scaled residuals = %g\n"), wbar);
+
 	sigma = 0;
 	for (j=0; j<n_est; j++) {
 	    xx = (cresid[j] - wbar);
@@ -1391,7 +1511,9 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn,
     
 	for (j=0; j<n_est; j++) {
 	    W[j] = 0.0;
-	    for (i=0; i<=j; i++) W[j] += cresid[i];
+	    for (i=0; i<=j; i++) {
+		W[j] += cresid[i];
+	    }
 	    W[j] /= sigma;
 	    t = pmod->t1 + K + j;
 	    ntodate(cumdate, t, pdinfo);
@@ -1399,20 +1521,22 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn,
 	    pprintf(prn, " %s %9.3f %s\n", cumdate, W[j],
 		    (fabs(W[j]) > xx + (j+1)*yy)? "*" : "");
 	}
+
 	hct = (sqrt((double) (T-K)) * wbar) / sigma;
+	pval = tprob(hct, T-K-1);
 	pprintf(prn, _("\nHarvey-Collier t(%d) = %g with p-value %.4g\n\n"), 
-		T-K-1, hct, tprob(hct, T-K-1));
+		T-K-1, hct, pval);
 
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, N_("CUSUM test for parameter stability"));
-	    strcpy(test->h_0, N_("no change in parameters"));
-	    test->teststat = GRETL_TEST_HARVEY_COLLIER;
+	    gretl_test_init(test, GRETL_TEST_CUSUM);
+	    test->teststat = GRETL_STAT_HARVEY_COLLIER;
 	    test->dfn = T-K-1;
 	    test->value = hct;
-	    test->pvalue = tprob(hct, T-K-1);
+	    test->pvalue = pval;
 	}
 
+	record_test_result(hct, pval, "Harvey-Collier");
+
 #ifdef ENABLE_NLS
         setlocale(LC_NUMERIC, "C");
 #endif
@@ -1460,6 +1584,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn,
  * @pmod: pointer to model to be tested.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
+ * @opt: option flags.
  * @prn: gretl printing struct.
  *
  * Tests the given pooled model for fixed and random effects.
@@ -1469,7 +1594,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn,
  */
 
 int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
-		  PRN *prn) 
+		  gretlopt opt, PRN *prn) 
 {
     if (pmod->ci != POOLED) {
 	pputs(prn, _("This test is only relevant for pooled models\n"));
@@ -1488,13 +1613,14 @@ int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     } else {
 	void *handle;
-	void (*panel_diagnostics)(MODEL *, double ***, DATAINFO *, PRN *);
+	void (*panel_diagnostics)(MODEL *, double ***, DATAINFO *, 
+				  gretlopt, PRN *);
 
 	panel_diagnostics = get_plugin_function("panel_diagnostics", &handle);
 	if (panel_diagnostics == NULL) {
 	    return 1;
 	}
-	(*panel_diagnostics) (pmod, pZ, pdinfo, prn);
+	(*panel_diagnostics) (pmod, pZ, pdinfo, opt, prn);
 	close_plugin(handle);
     }
     return 0;
@@ -1505,7 +1631,7 @@ int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
  * @m: matrix containing leverage values.
- * @opt: option flag: combination of SAVE_LEVERAGE, SAVE_INFLUENCE,
+ * @flags: option flags: combination of SAVE_LEVERAGE, SAVE_INFLUENCE,
  * and SAVE_DFFITS.
  *
  * Adds to the working dataset one or more series calculated by
@@ -1516,14 +1642,14 @@ int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
  */
 
 int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
-				    gretl_matrix *m, unsigned char opt)
+				    gretl_matrix *m, unsigned char flags)
 {
     int t1, t2;
     int addvars = 0;
 
-    if (opt & SAVE_LEVERAGE) addvars++;
-    if (opt & SAVE_INFLUENCE) addvars++;
-    if (opt & SAVE_DFFITS) addvars++;
+    if (flags & SAVE_LEVERAGE) addvars++;
+    if (flags & SAVE_INFLUENCE) addvars++;
+    if (flags & SAVE_DFFITS) addvars++;
 
     if (dataset_add_vars(addvars, pZ, pdinfo)) {
 	strcpy(gretl_errmsg, _("Out of memory adding series"));
@@ -1534,7 +1660,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
     t2 = t1 + gretl_matrix_rows(m);
 
     /* add leverage? */
-    if (opt & SAVE_LEVERAGE) {
+    if (flags & SAVE_LEVERAGE) {
 	int t, v = pdinfo->v - addvars;
 	int j = 0;
 
@@ -1551,7 +1677,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
     }
 
     /* add influence? */
-    if (opt & SAVE_INFLUENCE) {
+    if (flags & SAVE_INFLUENCE) {
 	int t, v = pdinfo->v - (addvars - 1);
 	int j = 0;
 
@@ -1568,7 +1694,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
     }
 
     /* add DFFITS? */
-    if (opt & SAVE_DFFITS) {
+    if (flags & SAVE_DFFITS) {
 	int t, v = pdinfo->v - (addvars - 2);
 	int j = 0;
 
@@ -1602,8 +1728,8 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
  * @pmod: pointer to model to be tested.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
+ * @opt: if OPT_S, add calculated series to data set.
  * @prn: gretl printing struct.
- * @oflag: if non-zero, add calculated series to data set.
  *
  * Tests the data used in the given model for points with
  * high leverage and influence on the estimates
@@ -1613,7 +1739,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
  */
 
 int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
-		   PRN *prn, gretlopt oflag)
+		   gretlopt opt, PRN *prn)
 {
     void *handle;
     gretl_matrix *(*model_leverage) (const MODEL *, double ***, 
@@ -1632,7 +1758,7 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     if (m == NULL) {
 	err = 1;
     } else {
-	if (oflag) {
+	if (opt & OPT_S) {
 	    err = add_leverage_values_to_dataset(pZ, pdinfo, m, 
 						 SAVE_LEVERAGE |
 						 SAVE_INFLUENCE| 
@@ -1912,15 +2038,16 @@ int sum_test (const int *sumvars, MODEL *pmod,
 	    pprintf(prn, "\n   %s = %g\n", _("Sum of coefficients"), 
 		    summod.coeff[testcoeff - 2]);
 	    if (!na(summod.sderr[testcoeff - 2])) {
-		double tval;
+		double tval, pval;
 
 		pprintf(prn, "   %s = %g\n", _("Standard error"),
 			summod.sderr[testcoeff - 2]);
 		tval = summod.coeff[testcoeff - 2] / 
 		    summod.sderr[testcoeff - 2];
+		pval = coeff_pval(&summod, tval, summod.dfd);
 		pprintf(prn, "   t(%d) = %g ", summod.dfd, tval);
-		pprintf(prn, _("with p-value = %g\n"), 
-			tprob(tval, summod.dfd));
+		pprintf(prn, _("with p-value = %g\n"), pval);
+		record_test_result(tval, pval, "sum");
 	    }
 	}
     }
