@@ -47,6 +47,9 @@ enum aux_codes {
     AUX_OMIT,  /* aux. regression for omitting variables */
     AUX_RESET, /* aux. regression for Ramsey's RESET */
     AUX_SYS,   /* single equation from multivariate system */
+    AUX_VAR,   /* single equation from VAR */
+    AUX_JOHANSEN,  /* Johansen cointegration test */
+    AUX_GROUPWISE, /* testing for groupwise heteroskedasticity */
     AUX_AUX    /* auxiliary regression not otherwise specified */
 };
 
@@ -83,7 +86,7 @@ int cusum_test (MODEL *pmod,
 
 int hausman_test (MODEL *pmod, 
 		  double ***pZ, DATAINFO *pdinfo, 
-		  PRN *prn);
+		  gretlopt opt, PRN *prn);
 
 int vif_test (MODEL *pmod, 
 	      double ***pZ, DATAINFO *pdinfo, 
@@ -91,10 +94,10 @@ int vif_test (MODEL *pmod,
 
 int leverage_test (MODEL *pmod, 
 		   double ***pZ, DATAINFO *pdinfo, 
-		   PRN *prn, gretlopt oflag);
+		   gretlopt opt, PRN *prn);
 
 int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
-				    gretl_matrix *m, unsigned char opt);
+				    gretl_matrix *m, unsigned char flags);
 
 int mp_ols (const int *list, const char *pos,
 	    double ***pZ, DATAINFO *pdinfo, 
