@@ -23,7 +23,7 @@
 
 #include "libgretl.h"
 
-/* #define DB_DEBUG */
+#undef DB_DEBUG
 
 #define RECNUM long
 #define NAMELENGTH 16
@@ -103,7 +103,7 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
 	strcat(dbbin, ".bin");
     }
 
-    fp = fopen(dbbin, "rb");
+    fp = gretl_fopen(dbbin, "rb");
     if (fp == NULL) return 1;
     
     fseek(fp, (long) sinfo->offset, SEEK_SET);
@@ -163,7 +163,7 @@ static int get_native_series_obs (SERIESINFO *sinfo,
 				  const char *stobs,
 				  const char *endobs)
 {
-    if (strchr(stobs, '/')) { /* daily data */
+    if (strchr(stobs, '/')) { /* calendar data */
 	const char *q = stobs;
 	const char *p = strchr(stobs, '/');
 
@@ -206,7 +206,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	strcat(dbidx, ".idx");
     }
 
-    fp = fopen(dbidx, "r");
+    fp = gretl_fopen(dbidx, "r");
     if (fp == NULL) {
 	strcpy(gretl_errmsg, _("Couldn't open database index file"));
 	return DB_NOT_FOUND;
@@ -711,7 +711,7 @@ int get_rats_data_by_series_number (const char *fname,
     long first_data;
     int ret = DB_OK;
 
-    fp = fopen(fname, "rb");
+    fp = gretl_fopen(fname, "rb");
     if (fp == NULL) return DB_NOT_FOUND;
     
     offset = get_rats_series_offset_by_number(fp, series_number);
@@ -772,7 +772,7 @@ get_rats_series_info_by_name (const char *series_name,
     int offset;
     int ret = DB_OK;
 
-    fp = fopen(db_name, "rb");
+    fp = gretl_fopen(db_name, "rb");
     if (fp == NULL) return DB_NOT_FOUND;
     
     offset = get_rats_series_offset_by_name(fp, series_name, sinfo);
@@ -798,7 +798,7 @@ get_rats_data_by_offset (const char *fname,
     FILE *fp;
     int ret = DB_OK;
 
-    fp = fopen(fname, "rb");
+    fp = gretl_fopen(fname, "rb");
     if (fp == NULL) return DB_NOT_FOUND;
 
     if (get_rats_series(sinfo->offset, sinfo, fp, Z)) {
@@ -902,6 +902,7 @@ double *compact_db_series (const double *src, SERIESINFO *sinfo,
     fprintf(stderr, "startskip = %d\n", skip);
     fprintf(stderr, "endskip = %d\n", endskip);
     fprintf(stderr, "goodobs = %d\n", goodobs);
+    fprintf(stderr, "compfac = %d\n", compfac);
     fprintf(stderr, "starting date = %s\n", sinfo->stobs);
 #endif
 
@@ -923,7 +924,7 @@ int set_db_name (const char *fname, int filetype, const PATHS *ppaths,
     *db_name = 0;
     strncat(db_name, fname, MAXLEN - 1);
 
-    fp = fopen(db_name, "rb");
+    fp = gretl_fopen(db_name, "rb");
 
     if (fp == NULL) {
 	/* try looking a bit more */
@@ -935,7 +936,7 @@ int set_db_name (const char *fname, int filetype, const PATHS *ppaths,
 		 strstr(db_name, ppaths->ratsbase) == NULL) {
 	    build_path(ppaths->ratsbase, fname, db_name, NULL);
 	}
-	fp = fopen(db_name, "rb");
+	fp = gretl_fopen(db_name, "rb");
     }
 	    
     if (fp == NULL) {
@@ -963,13 +964,16 @@ int db_set_sample (const char *line, DATAINFO *pdinfo)
 
     if (strcmp(start, ";")) {
 	t1 = dateton(start, pdinfo);
-	if (t1 < 0 || strlen(gretl_errmsg)) {
+	if (t1 < 0 || *gretl_errmsg != '\0') {
 	    return 1;
 	}
     }
 
     t2 = dateton(stop, pdinfo);
-    if (strlen(gretl_errmsg)) return 1;
+
+    if (*gretl_errmsg != '\0') {
+	return 1;
+    }
 
     if (t1 > t2) {
 	sprintf(gretl_errmsg, _("Invalid null sample"));
@@ -1085,6 +1089,11 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
     double **dbZ;
     int err = 0;
 
+#ifdef DB_DEBUG
+    fprintf(stderr, "db_get_series: line='%s', pZ=%p, pdinfo=%p\n", 
+	    line, (void *) pZ, (void *) pdinfo);
+#endif
+
     if (*db_name == 0) {
 	strcpy(gretl_errmsg, _("No database has been opened"));
 	return 1;
@@ -1108,9 +1117,9 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 
 	/* find the series information in the database */
 	if (db_type == GRETL_RATS_DB) {
-	    err = get_rats_series_info_by_name (series, &sinfo);
+	    err = get_rats_series_info_by_name(series, &sinfo);
 	} else {	
-	    err = get_native_series_info (series, &sinfo);
+	    err = get_native_series_info(series, &sinfo);
 	} 
 
 	if (err) {
@@ -1125,9 +1134,9 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	if (db_type == GRETL_RATS_DB) {
-	    err = get_rats_data_by_offset (db_name, &sinfo, dbZ);
+	    err = get_rats_data_by_offset(db_name, &sinfo, dbZ);
 	} else {
-	    get_native_db_data (db_name, &sinfo, dbZ);
+	    get_native_db_data(db_name, &sinfo, dbZ);
 	}
 
 	if (!err) {
@@ -1783,8 +1792,8 @@ static int dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 
 static int get_daily_skip (const DATAINFO *pdinfo, int t)
 {
-    int dd = daily_obs_number(pdinfo->S[t], pdinfo) -
-	daily_obs_number(pdinfo->S[t-1], pdinfo);
+    int dd = calendar_obs_number(pdinfo->S[t], pdinfo) -
+	calendar_obs_number(pdinfo->S[t-1], pdinfo);
 
     return dd - 1;
 }
