@@ -1013,7 +1013,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 	    fprintf(fq, "set x2label '%s'\n", I_("quarters"));
 	} else if (pdinfo->pd == 12) {
 	    fprintf(fq, "set x2label '%s'\n", I_("months"));
-	} else if (pdinfo->pd == 1 && pdinfo->time_series == TIME_SERIES) {
+	} else if (pdinfo->pd == 1 && pdinfo->structure == TIME_SERIES) {
 	    fprintf(fq, "set x2label '%s'\n", I_("years"));
 	} else {
 	    fprintf(fq, "set x2label '%s'\n", I_("periods"));
@@ -1378,7 +1378,7 @@ static GRETLSUMMARY *summary_new (const int *list)
 /**
  * summary:
  * @list: list of variables to process.
- * @pZ: pointer to data matrix.
+ * @Z: data matrix.
  * @pdinfo: information on the data set.
  * @prn: gretl printing struct.
  *
@@ -1474,7 +1474,7 @@ void free_corrmat (CORRMAT *corrmat)
 /**
  * corrlist:
  * @list: list of variables to process, by ID number.
- * @pZ: pointer to data matrix.
+ * @Z: data matrix.
  * @pdinfo: data information struct.
  *
  * Computes pairwise correlation coefficients for the variables
@@ -1600,7 +1600,7 @@ int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo,
  * @list: gives the ID numbers of the variables to compare.
  * @Z: data matrix.
  * @pdinfo: data information struct.
- * @vardiff: if non-zero, assume population variances are different.
+ * @opt: if OPT_O, assume population variances are different.
  * @prn: gretl printing struct.
  *
  * Carries out test of the null hypothesis that the means of two
@@ -1610,16 +1610,26 @@ int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo,
  */
 
 int means_test (const int *list, const double **Z, const DATAINFO *pdinfo, 
-		gretlopt vardiff, PRN *prn)
+		gretlopt opt, PRN *prn)
 {
     double m1, m2, s1, s2, skew, kurt, se, mdiff, t, pval;
+    double v1, v2;
     double *x = NULL, *y = NULL;
+    int vardiff = (opt & OPT_O);
     int df, n1, n2, n = pdinfo->n;
 
     if (list[0] < 2) return E_ARGS;
 
-    if ((x = malloc(n * sizeof *x)) == NULL) return E_ALLOC;
-    if ((y = malloc(n * sizeof *y)) == NULL) return E_ALLOC;
+    x = malloc(n * sizeof *x);
+    if (x == NULL) {
+	return E_ALLOC;
+    }
+
+    y = malloc(n * sizeof *y);
+    if (y == NULL) {
+	free(x);
+	return E_ALLOC;
+    }    
 
     n1 = ztox(list[1], x, Z, pdinfo);
     n2 = ztox(list[2], y, Z, pdinfo);
@@ -1629,23 +1639,29 @@ int means_test (const int *list, const double **Z, const DATAINFO *pdinfo,
 	free(x); free(y);
 	return 1;
     }
+
     if (n1 == 1 || n2 == 1) {
 	pputs(prn, _("Sample range has only one observation."));
 	free(x); free(y);
 	return 1;
     }
+
     df = n1 + n2 - 2;
 
     moments(0, n1-1, x, &m1, &s1, &skew, &kurt, 1);
     moments(0, n2-1, y, &m2, &s2, &skew, &kurt, 1);
     mdiff = m1 - m2;
 
+    v1 = s1 * s1;
+    v2 = s2 * s2;
+
     if (vardiff) {
-	se = sqrt((s1 * s1 / n1) + (s2 * s2 / n2));
+	se = sqrt((v1 / n1) + (v2 / n2));
     } else {
+	/* form pooled estimate of variance */
 	double sp2;
 
-	sp2 = ((n1-1) * s1 * s1 + (n2-1) * s2 * s2) / df;
+	sp2 = ((n1-1) * v1 + (n2-1) * v2) / df;
 	se = sqrt(sp2 / n1 + sp2 / n2);
     }
 
@@ -1667,6 +1683,8 @@ int means_test (const int *list, const double **Z, const DATAINFO *pdinfo,
     if (pval > .10)
 	pputs(prn, _("   The difference is not statistically significant.\n\n"));
 
+    record_test_result(t, pval, "difference of means");
+
     free(x);
     free(y);
 
@@ -1689,7 +1707,8 @@ int means_test (const int *list, const double **Z, const DATAINFO *pdinfo,
 int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	       PRN *prn)
 {
-    double m, skew, kurt, s1, s2, var1, var2, F;
+    double m, skew, kurt, s1, s2, var1, var2;
+    double F, pval;
     double *x = NULL, *y = NULL;
     int dfn, dfd, n1, n2, n = pdinfo->n;
 
@@ -1726,17 +1745,203 @@ int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo,
 	dfd = n1 - 1;
     }
 
+    pval = fdist(F, dfn, dfd);
+
     pputs(prn, _("\nEquality of variances test\n\n"));
     pprintf(prn, _("   Ratio of sample variances = %g\n"), F);
     pprintf(prn, "   %s: %s\n", _("Null hypothesis"), 
 	    _("The two population variances are equal"));
     pprintf(prn, "   %s: F(%d,%d) = %g\n", _("Test statistic"), dfn, dfd, F);
-    pprintf(prn, _("   p-value (two-tailed) = %g\n\n"), fdist(F, dfn, dfd));
+    pprintf(prn, _("   p-value (two-tailed) = %g\n\n"), pval);
     if (fdist(F, dfn, dfd) > .10)
 	pputs(prn, _("   The difference is not statistically significant.\n\n"));
 
+    record_test_result(F, pval, "difference of variances");
+
     free(x);
     free(y);
 
     return 0;
 }
+
+static int n_string_digits (int k)
+{
+    int n = 1;
+
+    while ((k = k / 10)) {
+	n++;
+    }
+
+    return n;
+}
+
+char *unique_savename (char *vname, DATAINFO *pdinfo, int vmax)
+{
+    int suf = 0;
+    int i;
+
+    for (i=1; i<vmax; i++) {
+	char base[VNAMELEN];
+	char *test = pdinfo->varname[i];
+	int k = 0;
+
+	if (!strcmp(test, vname)) {
+	    k = 1;
+	} else if (sscanf(test, "%[^0-9]%d", base, &k) == 2) {
+	    if (!strcmp(base, vname)) {
+		k++;
+	    } else {
+		k = 0;
+	    }
+	}
+	if (k > suf) {
+	    suf = k;
+	}
+    }
+	
+    if (suf > 0) {
+	char tmp[VNAMELEN];
+	int n = n_string_digits(suf);
+
+	sprintf(tmp, "%.*s", 8 - n, vname);
+	sprintf(vname, "%s%d", tmp, suf);
+    } 
+
+    return vname;
+}
+
+static int mdist_saver (double ***pZ, DATAINFO *pdinfo)
+{
+    int sv = 0;
+    int err;
+
+    err = dataset_add_vars(1, pZ, pdinfo);
+
+    if (!err) {
+	int t;
+
+	sv = pdinfo->v - 1;
+	for (t=0; t<pdinfo->n; t++) {
+	    (*pZ)[sv][t] = NADBL;
+	}
+
+	strcpy(pdinfo->varname[sv], "mdist");
+	unique_savename(pdinfo->varname[sv], pdinfo, sv);
+
+	strcpy(VARLABEL(pdinfo, sv), "Mahalanobis distances");	
+    }
+		
+    return sv;
+}
+
+int mahalanobis_distance (const int *list, double ***pZ,
+			  DATAINFO *pdinfo, gretlopt opt, 
+			  PRN *prn)
+{
+    gretl_matrix *S = NULL;
+    gretl_vector *means = NULL;
+    gretl_vector *xdiff;
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+    int n, err = 0;
+
+    adjust_t1t2(NULL, list, &pdinfo->t1, &pdinfo->t2, 
+		(const double **) *pZ, NULL);
+
+    n = pdinfo->t2 - pdinfo->t1 + 1;
+    if (n < 2) {
+	pdinfo->t1 = orig_t1;
+	pdinfo->t2 = orig_t2;
+	return E_DATA;
+    }
+
+    xdiff = gretl_column_vector_alloc(list[0]);
+    if (xdiff == NULL) {
+	pdinfo->t1 = orig_t1;
+	pdinfo->t2 = orig_t2;
+	return E_ALLOC;
+    }
+
+    S = gretl_covariance_matrix_from_varlist(list, 
+					     (const double **) *pZ, 
+					     pdinfo, 
+					     &means,
+					     &err);
+
+    if (!err) {
+	if (opt & OPT_V) {
+	    gretl_matrix_print(S, _("Covariance matrix"), prn);
+	}
+	err = gretl_invert_symmetric_matrix(S);
+	if (err) {
+	    fprintf(stderr, "error inverting covariance matrix\n");
+	} else if (opt & OPT_V) {
+	    gretl_matrix_print(S, _("Inverse of covariance matrix"), prn);
+	}
+    }
+
+    if (!err) {
+	int k = gretl_vector_get_length(means);
+	char obs_string[OBSLEN];
+	int savevar = 0;
+	double m;
+	int i, t;
+
+	if (opt & OPT_S) {
+	    /* save the results to a data series */
+	    savevar = mdist_saver(pZ, pdinfo);
+	}
+
+	pprintf(prn, "%s\n", _("Mahalanobis distances from the centroid"));
+	pprintf(prn, "%s\n", _("using the variables:"));
+	for (i=1; i<=list[0]; i++) {
+	    pprintf(prn, " %s\n", pdinfo->varname[list[i]]);
+	}
+	pputc(prn, '\n');
+
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+
+	    /* write vector of deviations from centroid for
+	       observation t */
+	    for (i=0; i<k; i++) {
+		int vi = list[i+1];
+		double xbar = gretl_vector_get(means, i);
+
+		gretl_vector_set(xdiff, i, (*pZ)[vi][t] - xbar);
+	    }
+
+	    m = gretl_scalar_b_prime_X_b(xdiff, S, &err);
+
+	    get_obs_string(obs_string, t, pdinfo);
+	    pprintf(prn, "%8s ", obs_string);
+
+	    if (err) {
+		pprintf(prn, "NA\n");
+	    } else {
+		m = sqrt(m);
+		pprintf(prn, "%9.6f\n", m);
+		if (savevar > 0) {
+		    (*pZ)[savevar][t] = m;
+		}
+	    }
+	}
+
+	if (savevar > 0) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Distances saved as '%s'"), 
+		    pdinfo->varname[savevar]);
+	    pputc(prn, '\n');
+	}
+
+	pputc(prn, '\n');
+    }
+
+    gretl_matrix_free(xdiff);
+    gretl_matrix_free(means);
+    gretl_matrix_free(S);
+
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
+    return err;
+}
