@@ -89,7 +89,7 @@ int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo,
 
 int means_test (const int *list, const double **Z, 
 		const DATAINFO *pdinfo, 
-		gretlopt vardiff, PRN *prn);
+		gretlopt opt, PRN *prn);
 
 int vars_test (const int *list, const double **Z, 
 	       const DATAINFO *pdinfo, PRN *prn);
@@ -105,3 +105,9 @@ int moments (int t1, int t2, const double *zx,
 
 double doornik_chisq (double skew, double kurt, int n);
 
+int mahalanobis_distance (const int *list, double ***pZ,
+			  DATAINFO *pdinfo, gretlopt opt,
+			  PRN *prn);
+
+char *unique_savename (char *vname, DATAINFO *pdinfo, int vmax);
+
