@@ -62,13 +62,12 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv);
 static double dwstat (int order, MODEL *pmod, double **Z);
 static double rhohat (int order, int t1, int t2, const double *uhat);
 static int hatvar (MODEL *pmod, int n, double **Z);
-static void omitzero (MODEL *pmod, const DATAINFO *pdinfo, double **Z);
-static void tsls_omitzero (int *list, double **Z, int t1, int t2);
+static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo);
+static void tsls_omitzero (int *list, const double **Z, int t1, int t2);
 static int depvar_zero (int t1, int t2, int yno, int nwt, 
 			const double **Z);
-static int lagdepvar (const int *list, const DATAINFO *pdinfo, 
-		      double ***pZ);
-static MODEL pooled_wls (int *list, double ***pZ, DATAINFO *pdinfo);
+static int lagdepvar (const int *list, const double **Z, const DATAINFO *pdinfo); 
+static int jackknife_vcv (MODEL *pmod, const double **Z);
 /* end private protos */
 
 
@@ -195,6 +194,166 @@ static int get_model_df (MODEL *pmod)
     return 0;
 }
 
+#define LDDEBUG 0
+
+static int
+transcribe_ld_vcv (MODEL *targ, MODEL *src)
+{
+    int nv = targ->ncoeff;
+    int nxpx = (nv * nv + nv) / 2;
+    int i, j;
+
+    if (makevcv(src)) {
+	return 1;
+    }
+
+    if (targ->vcv == NULL) {
+	targ->vcv = malloc(nxpx * sizeof *targ->vcv);
+	if (targ->vcv == NULL) {
+	    return 1;
+	}
+    }
+
+    for (i=0; i<nv; i++) {
+	for (j=i; j<nv; j++) {
+	    targ->vcv[ijton(i, j, nv)] = 
+		src->vcv[ijton(i, j, src->ncoeff)];
+	}
+    }
+
+    return 0;
+}
+
+/* Calculate consistent standard errors (and VCV matrix) when doing
+   AR(1) estimation of a model with lagged dependent variable.  See
+   Ramanathan, Introductory Econometrics, 5e, p. 450.
+*/
+
+static int 
+ldepvar_std_errors (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
+{
+    MODEL emod;
+    const double *x;
+
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+
+    double rho = gretl_model_get_double(pmod, "rho_in");
+    int origv = pdinfo->v;
+    int vnew = pmod->list[0] + 1 - pmod->ifc;
+
+    int *list;
+    int vi, vm;
+    int i, t;
+    int err = 0;
+
+#if LDDEBUG
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
+    printlist(pmod->list, "pmod->list");
+    printf("vnew = %d\n", vnew);
+    printf("rho = %g\n", rho);
+#endif
+
+    list = malloc((vnew + 1 + pmod->ifc) * sizeof *list);
+    if (list == NULL) {
+	pmod->errcode = E_ALLOC;
+	return 1;
+    }
+
+    err = dataset_add_vars(vnew, pZ, pdinfo);
+    if (err) {
+	free(list);
+	pmod->errcode = E_ALLOC;
+	return 1;
+    }
+
+    list[0] = vnew + pmod->ifc;
+
+    vi = origv;
+
+    /* dependent var is residual from original model */
+    for (t=0; t<pdinfo->n; t++) {
+	(*pZ)[vi][t] = pmod->uhat[t];
+    }    
+    strcpy(pdinfo->varname[vi], "eps");
+    list[1] = vi++;
+
+    /* indep vars are rho-differenced vars from original model */
+    for (i=2; i<=pmod->list[0]; i++) {
+	vm = pmod->list[i];
+	if (vm == 0) {
+	    list[i] = 0;
+	    continue;
+	}
+	sprintf(pdinfo->varname[vi], "%.6s_r", pdinfo->varname[vm]);
+	x = (*pZ)[vm];
+	for (t=0; t<pdinfo->n; t++) {
+	    if (t == 0 || na(x[t]) || na(x[t-1])) {
+		(*pZ)[vi][t] = NADBL;
+	    } else {
+		(*pZ)[vi][t] = x[t] - rho * x[t-1];
+	    }
+	}
+	list[i] = vi++;
+    }
+
+    /* last indep var is lagged u-hat */
+    for (t=0; t<pdinfo->n; t++) {
+	if (t == 0) {
+	    (*pZ)[vi][t] = NADBL;
+	} else { 
+	    (*pZ)[vi][t] = (*pZ)[pmod->list[1]][t-1];
+	}
+	if (na((*pZ)[vi][t])) {
+	    continue;
+	}
+	for (i=0; i<pmod->ncoeff; i++) {
+	    x = (*pZ)[pmod->list[i+2]];
+	    if (na(x[t-1])) {
+		(*pZ)[vi][t] = NADBL;
+		break;
+	    } else {
+		(*pZ)[vi][t] -= pmod->coeff[i] * x[t-1];
+	    }
+	}
+    }
+
+    list[list[0]] = vi;
+    strcpy(pdinfo->varname[vi], "uhat_1");
+
+    pdinfo->t1 = pmod->t1;
+    pdinfo->t2 = pmod->t2;
+
+    emod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+    if (emod.errcode) {
+	err = emod.errcode;
+    } else {
+#if LDDEBUG
+	printmodel(&emod, pdinfo, OPT_NONE, prn);
+	gretl_print_destroy(prn);
+#endif
+	for (i=0; i<pmod->ncoeff; i++) {
+	    pmod->sderr[i] = emod.sderr[i];
+	}
+
+	err = transcribe_ld_vcv(pmod, &emod);
+    }
+    
+    clear_model(&emod);
+    
+    free(list);
+    dataset_drop_vars(vnew, pZ, pdinfo);
+
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
+    if (err) {
+	pmod->errcode = err;
+    }
+
+    return err;
+}
+
 /* special computation of statistics for autoregressive models */
 
 static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
@@ -295,8 +454,6 @@ static void fix_wls_values (MODEL *pmod, double **Z)
 	    }
 	}
     } else {
-	double x;
-
 	pmod->ess_wt = pmod->ess;
 	pmod->sigma_wt = pmod->sigma;
 	pmod->ess = 0.0;
@@ -309,8 +466,8 @@ static void fix_wls_values (MODEL *pmod, double **Z)
 		pmod->nobs -= 1;
 	    } else {
 		pmod->yhat[t] /= Z[pmod->nwt][t];
-		x = pmod->uhat[t] /= Z[pmod->nwt][t];
-		pmod->ess += x * x;
+		pmod->uhat[t] /= Z[pmod->nwt][t];
+		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
 	    }
 	}
 	pmod->sigma = sqrt(pmod->ess / pmod->dfd);
@@ -396,29 +553,162 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
 	/* reject missing obs within adjusted sample */
 	missv = adjust_t1t2(pmod, pmod->list, &pmod->t1, &pmod->t2,
 			    Z, misst);
-    } else if (pmod->ci == POOLED) {
-	/* compensate for missing obs only if they preserve a
-	   balanced panel */
-	missv = adjust_t1t2(pmod, pmod->list, &pmod->t1, &pmod->t2,
-			    Z, NULL);
-	if (pmod->missmask != NULL) {
-	    if (!model_mask_leaves_balanced_panel(pmod, pdinfo)) {
-		free(pmod->missmask);
-		pmod->missmask = NULL;
-		missv = adjust_t1t2(pmod, pmod->list, 
-				    &pmod->t1, &pmod->t2,
-				    Z, misst);
-	    }
-	}
     } else {
 	/* we'll try to compensate for missing obs */
 	missv = adjust_t1t2(pmod, pmod->list, &pmod->t1, &pmod->t2,
 			    Z, NULL);
+	if (pmod->ci == POOLED && pmod->missmask != NULL) {
+	    if (!model_mask_leaves_balanced_panel(pmod, pdinfo)) {
+		gretl_model_set_int(pmod, "unbalanced", 1);
+	    }
+	}
     }
 
+#if SMPL_DEBUG
+    if (1) {
+	char t1s[OBSLEN], t2s[OBSLEN];
+
+	ntodate(t1s, pmod->t1, pdinfo);
+	ntodate(t2s, pmod->t2, pdinfo);
+	fprintf(stderr, "*** after adjustment, t1=%d (%s), t2=%d (%s)\n", 
+		pmod->t1, t1s, pmod->t2, t2s);
+    }
+#endif
+
     return missv;
 }
 
+static int const_pos (const int *list)
+{
+    int i;
+
+    for (i=2; i<=list[0]; i++) {
+        if (list[i] == 0) {
+	    return i;
+	}
+    }
+
+    return 0;
+}
+
+static void 
+lagged_depvar_check (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
+{
+    int ldv = lagdepvar(pmod->list, Z, pdinfo);
+
+    if (ldv) {
+	gretl_model_set_int(pmod, "ldepvar", ldv);
+    } else if (gretl_model_get_int(pmod, "ldepvar")) {
+	gretl_model_set_int(pmod, "ldepvar", 0);
+    }
+}
+
+#define COLL_DEBUG 0
+
+int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
+{
+    MODEL cmod;
+    int targ, ml0 = pmod->list[0];
+    int *list;
+    int err = E_SINGULAR;
+    int i, ret = 0;
+
+    if (ml0 < 3) {
+	/* shouldn't happen */
+	return 0;
+    }
+
+    /* can't handle compound lists */
+    for (i=1; i<=ml0; i++) {
+	if (pmod->list[i] == LISTSEP) {
+	    return 0;
+	}
+    }
+
+    list = malloc(ml0 * sizeof *list);
+    if (list == NULL) {
+	return 0;
+    }
+
+#if COLL_DEBUG
+    fprintf(stderr, "\n*** redundant_var called (trim = %d) ***\n", trim);
+    printlist(pmod->list, "original model list");
+#endif
+
+    while (err == E_SINGULAR && ml0 > 3) {
+
+	list[0] = ml0 - 1;
+
+	for (targ=ml0; targ>2; targ--) {
+	    double ess = 1.0, rsq = 0.0;
+	    int j = 2;
+
+	    list[1] = pmod->list[targ];
+
+	    for (i=2; i<=ml0; i++) {
+		if (i != targ) {
+		    list[j++] = pmod->list[i];
+		}
+	    }
+
+#if COLL_DEBUG
+	    fprintf(stderr, "target list position = %d\n", targ);
+	    printlist(list, "temp list for redundancy check");
+#endif
+
+	    cmod = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_Z, 0.0);
+
+	    err = cmod.errcode;
+	    if (err == 0) {
+		ess = cmod.ess;
+		rsq = cmod.rsq;
+	    } 
+
+	    clear_model(&cmod);
+
+	    if (err && err != E_SINGULAR) {
+		break;
+	    } else if (ess == 0.0 || rsq == 1.0) {
+		ret = 1;
+		break;
+	    }
+	}
+
+	if (ret) break;
+
+	ml0--;
+    }
+
+    if (ret == 1) {
+	static char msg[ERRLEN];
+	int v = pmod->list[targ];
+
+	/* remove var from list and reduce number of coeffs */
+	gretl_list_delete_at_pos(pmod->list, targ);
+	pmod->ncoeff -= 1;
+
+	/* compose a message */
+	if (trim == 0) {
+	    strcpy(msg, _("Omitted due to exact collinearity:"));
+	}
+	if (pdinfo->varname[v][0] != 0) {
+	    strcat(msg, " ");
+	    strcat(msg, pdinfo->varname[v]);
+	}
+
+	strcpy(gretl_msg, msg);
+
+	/* if there's a lagged dep var, it may have moved */
+	if (gretl_model_get_int(pmod, "ldepvar")) {
+	    lagged_depvar_check(pmod, (const double **) *pZ, pdinfo);
+	}
+    }
+
+    free(list);
+
+    return ret;
+}
+
 /**
  * lsq:
  * @list: dependent variable plus list of regressors.
@@ -426,14 +716,16 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
  * @pdinfo: information on the data set.
  * @ci: command index (see gretl_commands.h)
  * @opts: option flags: 
- *   if & OPT_R compute robust standard errors;
- *   if & OPT_C force use of Cholesky decomp;
- *   if & OPT_A treat as auxiliary regression (don't bother checking
+ *   OPT_R compute robust standard errors;
+ *   OPT_C force use of Cholesky decomp;
+ *   OPT_A treat as auxiliary regression (don't bother checking
  *     for presence of lagged dependent var, don't augment model count);
- *   if & OPT_P use Prais-Winsten for first obs.
- *   if & OPT_N don't use degrees of freedom correction for standard
+ *   OPT_P use Prais-Winsten for first obs.
+ *   OPT_N don't use degrees of freedom correction for standard
  *      error of regression
- *   if & OPT_M reject missing observations within sample range
+ *   OPT_M reject missing observations within sample range
+ *   OPT_Z (internal use) suppress the automatic elimination of 
+ *      perfectly collinear variables
  * @rho: coefficient for rho-differencing the data (0.0 for no
  * differencing)
  *
@@ -449,7 +741,7 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
     int l0, yno, i;
     int effobs = 0;
     int missv = 0, misst = 0;
-    int ldepvar = 0;
+    int jackknife = 0;
     int use_qr = get_use_qr();
     int pwe = (ci == PWE || (opts & OPT_P));
     double *xpy;
@@ -466,11 +758,7 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (ci == HSK) {
 	return hsk_func(list, pZ, pdinfo);
-    } else if (ci == HCCM) {
-	return hccm_func(list, pZ, pdinfo);
-    } else if (ci == POOLED && (opts & OPT_W)) {
-	return pooled_wls(list, pZ, pdinfo);
-    }
+    } 
 
     gretl_model_init(&mdl);
     gretl_model_smpl_init(&mdl, pdinfo);
@@ -560,10 +848,10 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* drop any vars that are all zero and repack the list */
-    omitzero(&mdl, pdinfo, *pZ);
+    omitzero(&mdl, (const double **) *pZ, pdinfo);
 
     /* if regressor list contains a constant, place it first */
-    i = gretl_hasconst(mdl.list);
+    i = const_pos(mdl.list);
     mdl.ifc = (i > 1);
     if (i > 2) {
 	rearrange_list(mdl.list);
@@ -572,10 +860,7 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
     /* Check for presence of lagged dependent variable? 
        (Don't bother if this is an auxiliary regression.) */
     if (!(opts & OPT_A)) {
-	ldepvar = lagdepvar(mdl.list, pdinfo, pZ);
-	if (ldepvar) {
-	    gretl_model_set_int(&mdl, "ldepvar", ldepvar);
-	}
+	lagged_depvar_check(&mdl, (const double **) *pZ, pdinfo);
     }
 
     /* AR1: advance the starting observation by one? */
@@ -608,17 +893,44 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
 	opts |= OPT_T;
     }
 
-    if ((opts & OPT_R) || (use_qr && !(opts & OPT_C))) { 
+    if (mdl.ci == HCCM || ((opts & OPT_R) && get_hc_version() == 4)) {
+	jackknife = 1;
+    }
+
+    if (((opts & OPT_R) && !jackknife) || (use_qr && !(opts & OPT_C))) { 
 	mdl.rho = rho;
-	gretl_qr_regress(&mdl, (const double **) *pZ, pdinfo->n, opts);
+	gretl_qr_regress(&mdl, pZ, pdinfo, opts);
     } else {
-	int l = l0 - 1;
-	int nxpx = l * (l + 1) / 2;
+	int trim = 0;
+	int l, nxpx;
+
+    trim_var:
+
+	if (trim) {
+	    l0 = mdl.list[0];
+	    free(mdl.xpx);
+	    free(mdl.coeff);
+	    free(mdl.sderr);
+	    mdl.errcode = 0;
+	}
+ 
+	l = l0 - 1;
+	nxpx = l * (l + 1) / 2;
 
 	xpy = malloc((l0 + 1) * sizeof *xpy);
 	mdl.xpx = malloc(nxpx * sizeof *mdl.xpx);
 	mdl.coeff = malloc(mdl.ncoeff * sizeof *mdl.coeff);
-	if (xpy == NULL || mdl.xpx == NULL || mdl.coeff == NULL) {
+	mdl.sderr = malloc(mdl.ncoeff * sizeof *mdl.sderr);
+
+	if (mdl.yhat == NULL) {
+	    mdl.yhat = malloc(pdinfo->n * sizeof *mdl.yhat);
+	} 
+	if (mdl.uhat == NULL) {
+	    mdl.uhat = malloc(pdinfo->n * sizeof *mdl.uhat);
+	}
+
+	if (xpy == NULL || mdl.xpx == NULL || mdl.coeff == NULL ||
+	    mdl.sderr == NULL || mdl.yhat == NULL || mdl.uhat == NULL) {
 	    mdl.errcode = E_ALLOC;
 	    return mdl;
 	}
@@ -646,6 +958,11 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
 
 	regress(&mdl, xpy, *pZ, pdinfo->n, rho);
 	free(xpy);
+
+	if (mdl.errcode == E_SINGULAR && !(opts & OPT_Z) &&
+	    redundant_var(&mdl, pZ, pdinfo, trim++)) {
+	    goto trim_var;
+	}
     }
 
     if (mdl.errcode) {
@@ -660,6 +977,11 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
 	if (compute_ar_stats(&mdl, (const double **) *pZ, rho)) { 
 	    goto lsq_abort;
 	}
+	if (gretl_model_get_int(&mdl, "ldepvar")) {
+	    if (ldepvar_std_errors(&mdl, pZ, pdinfo)) {
+		goto lsq_abort;
+	    }
+	}
     }
 
     /* weighted least squares: fix fitted values, ESS, sigma */
@@ -682,11 +1004,18 @@ MODEL lsq (int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* Generate model selection statistics */
-    gretl_aic_etc(&mdl);
+    calculate_criteria(mdl.criterion, 
+		       (mdl.ci == WLS)? mdl.ess_wt : mdl.ess, 
+		       mdl.nobs, mdl.ncoeff);
+
+    /* hccm command or HC3a */
+    if (jackknife) {
+	mdl.errcode = jackknife_vcv(&mdl, (const double **) *pZ);
+    }
 
  lsq_abort:
 
-    /* If we resuffled any missing observations, put them
+    /* If we reshuffled any missing observations, put them
        back in their right places now */
     if (gretl_model_get_int(&mdl, "daily_repack")) {
 	undo_daily_repack(&mdl, *pZ, pdinfo);
@@ -902,6 +1231,10 @@ static void compute_r_squared (MODEL *pmod, double *y)
 		1 - ((1 - pmod->rsq) * (pmod->nobs - 1) / pmod->dfd);
 	} 
     }
+
+    if (pmod->rsq < 0.0) {
+	pmod->rsq = 0.0;
+    }
 }
 
 /*
@@ -927,15 +1260,6 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
     double *diag = NULL;
     int i, err = 0;
 
-    pmod->sderr = malloc(pmod->ncoeff * sizeof *pmod->sderr);
-    pmod->yhat = malloc(n * sizeof *pmod->yhat);
-    pmod->uhat = malloc(n * sizeof *pmod->uhat);
-
-    if (pmod->sderr == NULL || pmod->yhat == NULL || pmod->uhat == NULL) {
-        pmod->errcode = E_ALLOC;
-        return;
-    }
-
     for (i=0; i<n; i++) {
 	pmod->yhat[i] = pmod->yhat[i] = NADBL;
     }    
@@ -1073,7 +1397,9 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
         d2 = xpx[kk] - d;
 	test = d2 / xpx[kk];
         if (test < TINY) {
+#if 0
 	    fprintf(stderr, "cholbeta: test = %g\n", test);
+#endif
 	    if (rss != NULL) *rss = -1.0;
 	    return E_SINGULAR;
         }
@@ -1771,6 +2097,22 @@ static int get_pos (const int *list)
 
 /* .......................................................... */
 
+void tsls_free_data (const MODEL *pmod)
+{
+    const char *endog = gretl_model_get_data(pmod, "endog");
+    double **X = gretl_model_get_data(pmod, "tslsX");
+    int i, m = 0;
+
+    if (endog != NULL && X != NULL) {
+	for (i=0; i<pmod->ncoeff; i++) {
+	    if (endog[i]) m++;
+	}
+	for (i=0; i<m; i++) {
+	    free(X[i]);
+	}
+    }
+}
+
 static int tsls_save_data (MODEL *pmod, const int *list, 
 			   double **Z, DATAINFO *pdinfo)
 {
@@ -1779,13 +2121,38 @@ static int tsls_save_data (MODEL *pmod, const int *list,
     int addvars = list[0];
     int i, j, k, pos, m, err = 0;
     size_t esize, Xsize = list[0] * sizeof *X;
+    size_t es_old, xs_old;
+    int recycle_X = 0;
+    int recycle_e = 0;
 
     pos = get_pos(pmod->list);
     m = pos - 2;
     esize = m * sizeof *endog;
 
-    X = malloc(Xsize);
-    endog = malloc(esize);
+    /* re-use old pointers if applicable */
+
+    X = gretl_model_get_data_and_size(pmod, "tslsX", &xs_old);
+    if (X == NULL) {
+	X = malloc(Xsize);
+    } else if (Xsize != xs_old) {
+	tsls_free_data(pmod);
+	gretl_model_destroy_data_item(pmod, "tslsX");
+	free(X);
+	X = malloc(Xsize);
+    } else {
+	recycle_X = 1;
+    }
+
+    endog = gretl_model_get_data_and_size(pmod, "endog", &es_old);
+    if (endog == NULL) {
+	endog = malloc(esize);
+    } else if (esize != es_old) {
+	gretl_model_destroy_data_item(pmod, "endog");
+	free(endog);
+	endog = malloc(esize);
+    } else {
+	recycle_e = 1;
+    }
 
     if (X == NULL || endog == NULL) {
 	free(X);
@@ -1811,17 +2178,21 @@ static int tsls_save_data (MODEL *pmod, const int *list,
     }
 
     /* now attach X and endog to the model */
-    gretl_model_set_data(pmod, "tslsX", X, Xsize);
-    gretl_model_set_data(pmod, "endog", endog, esize);
+    if (!recycle_X) {
+	gretl_model_set_data(pmod, "tslsX", X, Xsize);
+    }
+    if (!recycle_e) {
+	gretl_model_set_data(pmod, "endog", endog, esize);
+    }
 
     return err;
 }
 
-const double *tsls_get_Xi (const MODEL *pmod, const double **Z, int i)
+double *tsls_get_Xi (const MODEL *pmod, double **Z, int i)
 {
     const char *endog;
     double **X;
-    const double *ret;
+    double *ret;
 
     endog = gretl_model_get_data(pmod, "endog");
     X = gretl_model_get_data(pmod, "tslsX");
@@ -1842,22 +2213,6 @@ const double *tsls_get_Xi (const MODEL *pmod, const double **Z, int i)
     return ret;
 }
 
-void tsls_free_data (const MODEL *pmod)
-{
-    const char *endog = gretl_model_get_data(pmod, "endog");
-    double **X = gretl_model_get_data(pmod, "tslsX");
-    int i, m = 0;
-
-    if (endog != NULL && X != NULL) {
-	for (i=0; i<pmod->ncoeff; i++) {
-	    if (endog[i]) m++;
-	}
-	for (i=0; i<m; i++) {
-	    free(X[i]);
-	}
-    }
-}
-
 /*
   tsls_make_replist: determines which variables in reglist, when
   checked against the predetermined and exogenous vars in instlist,
@@ -1909,7 +2264,7 @@ tsls_make_replist (const int *reglist, int *instlist, int *replist)
  * @pdinfo: information on the data set.
  * @opt: may contain OPT_R for robust VCV, OPT_S to save second-
  * stage regressors (OPT_S is used in context of three-stage least 
- * squares).
+ * squares), OPT_N (no df correction).
  *
  * Estimate the model given in @list by means of Two-Stage Least
  * Squares.
@@ -1980,9 +2335,9 @@ MODEL tsls_func (int *list, int pos_in, double ***pZ, DATAINFO *pdinfo,
     /* drop any vars that are all zero, and reshuffle the constant
        into first position among the independent vars 
     */
-    tsls_omitzero(reglist, *pZ, pdinfo->t1, pdinfo->t2);
+    tsls_omitzero(reglist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
     rearrange_list(reglist);
-    tsls_omitzero(instlist, *pZ, pdinfo->t1, pdinfo->t2);
+    tsls_omitzero(instlist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
 
     /* initial composition of second-stage regression list (will be
        modified below) 
@@ -2069,20 +2424,24 @@ MODEL tsls_func (int *list, int pos_in, double ***pZ, DATAINFO *pdinfo,
 
     tsls.ess = 0.0;
     for (t=tsls.t1; t<=tsls.t2; t++) {
-	double xx = 0.0;
+	double yh = 0.0;
 
 	if (model_missing(&tsls, t)) {
 	    continue;
 	}
 	for (i=0; i<tsls.ncoeff; i++) {
-	    xx += tsls.coeff[i] * (*pZ)[reglist[i+2]][t];
+	    yh += tsls.coeff[i] * (*pZ)[reglist[i+2]][t];
 	}
-	tsls.yhat[t] = xx; 
-	tsls.uhat[t] = (*pZ)[tsls.list[1]][t] - xx;
+	tsls.yhat[t] = yh; 
+	tsls.uhat[t] = (*pZ)[tsls.list[1]][t] - yh;
 	tsls.ess += tsls.uhat[t] * tsls.uhat[t];
     }
 
-    tsls.sigma = (tsls.ess >= 0.0) ? sqrt(tsls.ess / tsls.dfd) : 0.0;
+    if (tsls.ess <= 0.0) {
+	tsls.sigma = 0.0;
+    } else {
+	tsls.sigma = sqrt(tsls.ess / ((opt & OPT_N)? tsls.nobs : tsls.dfd));
+    }
 
     /* computation of covariance matrix of parameter estimates */
 
@@ -2128,7 +2487,7 @@ MODEL tsls_func (int *list, int pos_in, double ***pZ, DATAINFO *pdinfo,
     tsls.adjrsq = 
 	1.0 - ((1.0 - tsls.rsq) * (tsls.nobs - 1.0) / tsls.dfd);
     tsls.fstt = tsls.rsq * tsls.dfd / (tsls.dfn * (1.0 - tsls.rsq));
-    gretl_aic_etc(&tsls);
+    ls_aic_bic(&tsls);
 
     if (tsls.missmask == NULL) {
 	/* no missing obs within sample range */
@@ -2191,18 +2550,28 @@ MODEL tsls_func (int *list, int pos_in, double ***pZ, DATAINFO *pdinfo,
 }
 
 /* Given an original regression list, augment it by adding the squares
-   or logs of the original independent variables.  Generate these
-   variables if need be.  Return the augmented list, or NULL on
-   failure.
+   (and cross-products, if wanted) or logs of the original independent
+   variables.  Generate these variables if need be.  Return the
+   augmented list, or NULL on failure.
 */
 
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo)
 {
     int *list;
+    int listlen;
     int i, k;
 
-    list = malloc((2 * orig[0]) * sizeof *list);
+    if (aux == AUX_WHITE) {
+	int trv = orig[0] - 1 - gretl_hasconst(orig);
+	int nt = (trv * trv + trv) / 2;
+
+	listlen = orig[0] + nt + 1;
+    } else {
+	listlen = 2 * orig[0];
+    }
+
+    list = malloc(listlen * sizeof *list);
     if (list == NULL) {
 	return NULL;
     }
@@ -2212,7 +2581,7 @@ int *augment_regression_list (const int *orig, int aux,
 	list[i] = orig[i];
     }
 
-    /* add squares or logs of independent vars */
+    /* add squares, cross-products or logs of independent vars */
     k = list[0];
     for (i=2; i<=orig[0]; i++) {
 	int vnew, vi = orig[i];
@@ -2220,15 +2589,28 @@ int *augment_regression_list (const int *orig, int aux,
 	if (vi == 0) {
 	    continue;
 	}
-	if (aux == AUX_SQ) {
+
+	if (aux == AUX_SQ || aux == AUX_WHITE) {
 	    vnew = xpxgenr(vi, vi, pZ, pdinfo);
-	} else {
+	    if (vnew > 0) list[++k] = vnew;
+	    if (aux == AUX_WHITE) {
+		int j, vj;
+
+		for (j=i+1; j<=orig[0]; j++) {
+		    vj = orig[j];
+		    if (vj == 0) {
+			continue;
+		    }
+		    vnew = xpxgenr(vi, vj, pZ, pdinfo);
+		    if (vnew > 0) list[++k] = vnew;
+		}
+	    }
+	} else if (aux == AUX_LOG) {
 	    vnew = loggenr(vi, pZ, pdinfo);
-	}
-	if (vnew > 0) {
-	    list[++k] = vnew;
+	    if (vnew > 0) list[++k] = vnew;
 	}
     }
+
     list[0] = k;
 
     return list;
@@ -2372,14 +2754,14 @@ MODEL hsk_func (int *list, double ***pZ, DATAINFO *pdinfo)
     return hsk;
 }
 
-static double **allocate_hccm_p (int lo, int n)
+static double **allocate_hccm_p (int k, int n)
 {
+    double **p = malloc(k * sizeof *p);
     int i;
-    double **p = malloc(lo * sizeof *p);
 
     if (p == NULL) return NULL;
 
-    for (i=0; i<lo; i++) {
+    for (i=0; i<k; i++) {
 	p[i] = malloc(n * sizeof **p);
 	if (p[i] == NULL) {
 	    int j;
@@ -2389,6 +2771,7 @@ static double **allocate_hccm_p (int lo, int n)
 	    }
 	    free(p);
 	    p = NULL;
+	    break;
 	}
     }
 
@@ -2407,83 +2790,60 @@ static void free_hccm_p (double **p, int m)
     }
 }
 
-/**
- * hccm_func:
- * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- *
- * Estimate the model given in @list using OLS, compute
- * heteroskedasticity-consistent covariance matrix using the
- * McKinnon-White procedure, and report standard errors using this
- * matrix.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
-
-MODEL hccm_func (int *list, double ***pZ, DATAINFO *pdinfo)
+static int jackknife_vcv (MODEL *pmod, const double **Z)
 {
-    int nobs, nc, i, j, k, t, t1, t2, tp;
-    double *st = NULL, *uhat1 = NULL, **p = NULL;
+    double *st = NULL, *ustar = NULL;
+    double **p = NULL;
+    int nobs, tp, nc = 0;
+    int i, j, k, t;
+    int t1, t2;
     double xx;
-    MODEL hccm;
+    int err = 0;
 
     *gretl_errmsg = '\0';
 
-    gretl_model_init(&hccm);
+    t1 = pmod->t1;
+    t2 = pmod->t2;
+    nobs = pmod->nobs;
+    nc = pmod->ncoeff;
 
-    nc = list[0] - 1;
-    rearrange_list(list);
+    st = malloc(nc * sizeof *st);
+    ustar = malloc(nobs * sizeof *ustar);
+    p = allocate_hccm_p(nc, nobs);
 
-    /* run a regular OLS */
-    hccm = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
-    if (hccm.errcode) {
+    if (st == NULL || p == NULL || ustar == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }  
+
+    if (pmod->vcv != NULL) {
+	free(pmod->vcv);
+	pmod->vcv = NULL;
+    }
+
+    pmod->ci = HCCM;
+
+    if (makevcv(pmod)) {
+	err = E_ALLOC;
 	goto bailout;
     }
 
-    t1 = hccm.t1;
-    t2 = hccm.t2;
-    nobs = hccm.nobs;
+    /* form elements of (X'X)^{-1}X' */
 
-    hccm.ci = HCCM;
-
-    /* now try allocating memory */
-    st = malloc(list[0] * sizeof *st);
-    uhat1 = malloc(nobs * sizeof *uhat1);
-    p = allocate_hccm_p(list[0], nobs);
-
-    if (st == NULL || p == NULL || uhat1 == NULL) {
-	hccm.errcode = E_ALLOC;
-	goto bailout;
-    }    
-
-    if (get_use_qr()) {
-	/* vcv is already computed in this case */
-	int nt = (nc * nc + nc) / 2; 
-	double s2 = hccm.sigma * hccm.sigma;
-
-	for (i=0; i<nt; i++) {
-	    hccm.vcv[i] /= s2;
-	}
-    } else if (makevcv(&hccm)) {
-	hccm.errcode = E_ALLOC;
-	goto bailout;
-    } 
-
-    for (i=1; i<=nc; i++) {
+    for (i=0; i<nc; i++) {
 	tp = 0;
 	for (t=t1; t<=t2; t++) {
-	    if (model_missing(&hccm, t)) {
+	    if (model_missing(pmod, t)) {
 		continue;
 	    }
 	    xx = 0.0;
-	    for (j=1; j<=nc; j++) {
+	    for (j=0; j<nc; j++) {
 		if (i <= j) {
-		    k = ijton(i-1, j-1, nc);
+		    k = ijton(i, j, nc);
 		} else {
-		    k = ijton(j-1, i-1, nc);
+		    k = ijton(j, i, nc);
 		}
-		xx += hccm.vcv[k] * (*pZ)[list[j+1]][t];
+		xx += pmod->vcv[k] * Z[pmod->list[j+2]][t];
 	    }
 	    p[i][tp++] = xx;
 	}
@@ -2491,63 +2851,73 @@ MODEL hccm_func (int *list, double ***pZ, DATAINFO *pdinfo)
 
     tp = 0;
     for (t=t1; t<=t2; t++) {
-	if (model_missing(&hccm, t)) {
+	if (model_missing(pmod, t)) {
 	    continue;
 	}	
 	xx = 0.0;
-	for (i=1; i<=nc; i++) {
-	    xx += (*pZ)[list[i+1]][t] * p[i][tp];
+	for (i=0; i<nc; i++) {
+	    xx += Z[pmod->list[i+2]][t] * p[i][tp];
 	}
 	if (floateq(xx, 1.0)) {
 	    xx = 0.0;
 	}
-	uhat1[tp++] = hccm.uhat[t] / (1 - xx);
+	ustar[tp++] = pmod->uhat[t] / (1.0 - xx);
     }
 
-    for (i=1; i<=nc; i++) {
+    for (i=0; i<nc; i++) {
 	xx = 0.0;
 	for (t=0; t<nobs; t++) {
-	    xx += p[i][t] * uhat1[t];
+	    xx += p[i][t] * ustar[t];
 	}
 	st[i] = xx;
     }
 
     for (t=0; t<nobs; t++) {
-	for (i=1; i<=nc; i++) {
-	    p[i][t] *= uhat1[t];
+	for (i=0; i<nc; i++) {
+	    p[i][t] *= ustar[t];
 	}
     }
 
+    /* MacKinnon and White, 1985, equation (13) */
+
     k = 0;
-    for (i=1; i<=nc; i++) {
-	for (j=i; j<=nc; j++) {
+    for (i=0; i<nc; i++) {
+	for (j=i; j<nc; j++) {
 	    xx = 0.0;
 	    for (t=0; t<nobs; t++) {
 		xx += p[i][t] * p[j][t];
 	    }
-	    xx = xx * (nobs - 1) / nobs -
-		(nobs - 1) * st[i] * st[j] / (nobs * nobs);
+	    xx -= st[i] * st[j] / nobs;
+	    /* MacKinnon and White: "It is tempting to omit the factor
+	       (n - 1)/n from HC3" (1985, p. 309).  Here we leave it in
+	       place, as in their simulations.
+	    */
+	    xx *= (nobs - 1.0) / nobs;
 	    if (i == j) {
-		hccm.sderr[i-1] = sqrt(xx);
+		pmod->sderr[i] = sqrt(xx);
 	    }
-	    hccm.vcv[k++] = xx;
+	    pmod->vcv[k++] = xx;
 	}
     }
 
     /* substitute robust F stat */
-    if (hccm.dfd > 0 && hccm.dfn > 1) {
-	hccm.fstt = robust_omit_F(NULL, &hccm);
+    if (pmod->dfd > 0 && pmod->dfn > 1) {
+	pmod->fstt = robust_omit_F(NULL, pmod);
     }
 
+    gretl_model_set_int(pmod, "robust", 1);
+    gretl_model_set_int(pmod, "hc", 1);
+    gretl_model_set_int(pmod, "hc_version", 4);
+
  bailout:
 
+    pmod->ci = OLS;
+
     free(st);
-    free(uhat1);
-    free_hccm_p(p, list[0]);
+    free(ustar);
+    free_hccm_p(p, nc);
 
-    set_model_id(&hccm);
-
-    return hccm;
+    return err;
 }
 
 /**
@@ -2568,7 +2938,7 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		 PRN *prn, GRETLTEST *test)
 {
     int lo, ncoeff, yno, t;
-    int shrink, v = pdinfo->v;
+    int v = pdinfo->v;
     int *list = NULL;
     double zz;
     MODEL white;
@@ -2607,9 +2977,9 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	/* build aux regression list, adding the squares of the original
-	   independent vars */
-	list = augment_regression_list(pmod->list, AUX_SQ, pZ, pdinfo);
+	/* build aux regression list, adding squares and
+	   cross-products of the original independent vars */
+	list = augment_regression_list(pmod->list, AUX_WHITE, pZ, pdinfo);
 	if (list == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -2624,26 +2994,28 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
+	double TR2, pval;
+
 	white.aux = AUX_WHITE;
 	printmodel(&white, pdinfo, OPT_NONE, prn);
 
+	TR2 = white.rsq * white.nobs;
+	pval = chisq(TR2, white.ncoeff - 1);
+
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, N_("White's test for heteroskedasticity"));
-	    strcpy(test->h_0, N_("heteroskedasticity not present"));
-	    test->teststat = GRETL_TEST_TR2;
+	    gretl_test_init(test, GRETL_TEST_WHITES);
+	    test->teststat = GRETL_STAT_TR2;
 	    test->dfn = white.ncoeff - 1;
-	    test->value = white.rsq * white.nobs;
-	    test->pvalue = chisq(test->value, test->dfn);
+	    test->value = TR2;
+	    test->pvalue = pval;
 	}
+
+	record_test_result(TR2, pval, "White's");
     }
 
     clear_model(&white);
 
-    shrink = pdinfo->v - v;
-    if (shrink > 0) {
-	dataset_drop_vars(shrink, pZ, pdinfo);
-    }
+    dataset_drop_vars(pdinfo->v - v, pZ, pdinfo);
 
     free(list);
 
@@ -2718,7 +3090,7 @@ MODEL ar_func (int *list, int pos, double ***pZ,
     rholist[0] = arlist[0] + 1;
     maxlag = ar_list_max(arlist);
 
-    if (gretl_hasconst(reglist)) {
+    if (const_pos(reglist) > 2) {
 	rearrange_list(reglist);
     }
 
@@ -2836,8 +3208,7 @@ MODEL ar_func (int *list, int pos, double ***pZ,
     for (i=0; i<=reglist[0]; i++) {
 	ar.list[i] = reglist[i];
     }
-    i = gretl_hasconst(reglist);
-    if (i > 1) {
+    if (gretl_hasconst(reglist)) {
 	ar.ifc = 1;
     }
     if (ar.ifc) {
@@ -2874,7 +3245,7 @@ MODEL ar_func (int *list, int pos, double ***pZ,
 	tss += ((*pZ)[ryno][t] - xx) * ((*pZ)[ryno][t] - xx);
     }
     ar.fstt = ar.dfd * (tss - ar.ess) / (ar.dfn * ar.ess);
-    gretl_aic_etc(&ar);
+    ls_aic_bic(&ar);
     ar.dw = dwstat(maxlag, &ar, *pZ);
     ar.rho = rhohat(maxlag, ar.t1, ar.t2, ar.uhat);
 
@@ -2911,9 +3282,9 @@ MODEL ar_func (int *list, int pos, double ***pZ,
 /* From 2 to end of list, omits variables with all zero observations
    and re-packs the rest of them */
 
-static void omitzero (MODEL *pmod, const DATAINFO *pdinfo, double **Z)
+static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 {
-    int v, lv, offset, drop = 0;
+    int v, lv, offset, dropmsg = 0;
     double xx = 0.0;
     char vnamebit[20];
 
@@ -2925,11 +3296,9 @@ static void omitzero (MODEL *pmod, const DATAINFO *pdinfo, double **Z)
 	    list_exclude(v, pmod->list);
 	    if (pdinfo->varname[lv][0] != 0) {
 		sprintf(vnamebit, "%s ", pdinfo->varname[lv]);
-	    } else {
-		sprintf(vnamebit, "%s %d ", _("variable"), lv);
+		strcat(gretl_msg, vnamebit);
+		dropmsg = 1;
 	    }
-	    strcat(gretl_msg, vnamebit);
-	    drop = 1;
 	}
     }
 
@@ -2948,19 +3317,19 @@ static void omitzero (MODEL *pmod, const DATAINFO *pdinfo, double **Z)
 	    }
 	    if (wtzero) {
 		list_exclude(v, pmod->list);
-		sprintf(vnamebit, _("weighted %s "), pdinfo->varname[lv]);
+		sprintf(vnamebit, "%s ", pdinfo->varname[lv]);
 		strcat(gretl_msg, vnamebit);
-		drop = 1;
+		dropmsg = 1;
 	    }
 	}
     }
 
-    if (drop) {
+    if (dropmsg) {
 	strcat(gretl_msg, _("omitted because all obs are zero."));
     }
 }
 
-static void tsls_omitzero (int *list, double **Z, int t1, int t2)
+static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
 {
     int i, v;
 
@@ -3003,8 +3372,8 @@ static int depvar_zero (int t1, int t2, int yno, int nwt,
    this lagged var in the list; otherwise return 0
 */
 
-static int lagdepvar (const int *list, const DATAINFO *pdinfo, 
-		      double ***pZ)
+static int 
+lagdepvar (const int *list, const double **Z, const DATAINFO *pdinfo) 
 {
     int i, t;
     char depvar[VNAMELEN], othervar[VNAMELEN];
@@ -3013,7 +3382,9 @@ static int lagdepvar (const int *list, const DATAINFO *pdinfo,
     strcpy(depvar, pdinfo->varname[list[1]]);
 
     for (i=2; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) break;
+	if (list[i] == LISTSEP) {
+	    break;
+	}
 	strcpy(othervar, pdinfo->varname[list[i]]);
 	p = strrchr(othervar, '_');
 	if (p != NULL && isdigit(*(p + 1))) {
@@ -3025,7 +3396,7 @@ static int lagdepvar (const int *list, const DATAINFO *pdinfo,
 
 		/* strong candidate for lagged depvar, but make sure */
 		for (t=pdinfo->t1+1; t<=pdinfo->t2; t++) {
-		    if ((*pZ)[list[1]][t-1] != (*pZ)[list[i]][t]) {
+		    if (Z[list[1]][t-1] != Z[list[i]][t]) {
 			gotlag = 0;
 			break;
 		    }
@@ -3061,6 +3432,8 @@ MODEL arch (int order, int *list, double ***pZ, DATAINFO *pdinfo,
 {
     MODEL archmod;
     int *wlist = NULL, *arlist = NULL;
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int oldv = pdinfo->v;
     int i, t, nwt, nv, n = pdinfo->n;
     double LM, xx;
     int err = 0;
@@ -3070,7 +3443,7 @@ MODEL arch (int order, int *list, double ***pZ, DATAINFO *pdinfo,
     gretl_model_init(&archmod);
 
     /* assess the lag order */
-    if (order < 1) {
+    if (order < 1 || order > T - list[0]) {
 	archmod.errcode = E_UNSPEC;
 	sprintf(gretl_errmsg, _("Invalid lag order for arch (%d)"), order);
 	err = 1;
@@ -3128,23 +3501,25 @@ MODEL arch (int order, int *list, double ***pZ, DATAINFO *pdinfo,
 	/* print results */
 	archmod.aux = AUX_ARCH;
 	archmod.order = order;
+
 	printmodel(&archmod, pdinfo, OPT_NONE, prn);
+
 	pprintf(prn, _("No of obs. = %d, unadjusted R^2 = %f\n"),
 		archmod.nobs, archmod.rsq);
 	LM = archmod.nobs * archmod.rsq;
 	xx = chisq(LM, order);
 
 	if (test != NULL) {
-	    gretl_test_init(test);
-	    strcpy(test->type, N_("Test for ARCH of order %s"));
-	    sprintf(test->param, "%d", order);
-	    strcpy(test->h_0, N_("no ARCH effect is present"));
-	    test->teststat = GRETL_TEST_TR2;
+	    gretl_test_init(test, GRETL_TEST_ARCH);
+	    test->order = order;
+	    test->teststat = GRETL_STAT_TR2;
 	    test->dfn = order;
 	    test->value = LM;
 	    test->pvalue = xx;
 	}
 
+	record_test_result(LM, xx, "ARCH");
+
 	pprintf(prn, _("LM test statistic (%f) is distributed as Chi-square "
 		"(%d)\nArea to the right of LM = %f  "), LM, order, xx);
 	if (xx > 0.1) 
@@ -3188,7 +3563,7 @@ MODEL arch (int order, int *list, double ***pZ, DATAINFO *pdinfo,
     if (arlist != NULL) free(arlist);
     if (wlist != NULL) free(wlist);
 
-    dataset_drop_vars(order + 1, pZ, pdinfo); 
+    dataset_drop_vars(pdinfo->v - oldv, pZ, pdinfo); 
 
     return archmod;
 }
@@ -3214,6 +3589,7 @@ MODEL lad (int *list, double ***pZ, DATAINFO *pdinfo)
     /* run an initial OLS to "set the model up" and check for errors.
        the lad_driver function will overwrite the coefficients etc.
     */
+
     lad_model = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
 
     if (lad_model.errcode) {
@@ -3294,7 +3670,7 @@ MODEL arma_x12 (int *list, const double **Z, DATAINFO *pdinfo, PRN *prn,
     void *handle;
     MODEL (*arma_x12_model) (int *, const double **, DATAINFO *, PRN *, 
 			     const char *, const char *, int);
-    int gui = gretl_using_gui();
+    int gui = gretl_in_gui_mode();
 
     *gretl_errmsg = '\0';
 
@@ -3392,6 +3768,70 @@ MODEL tobit_model (int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return tmod;
 }
 
+static int get_offset_var (int *list)
+{
+    int l0 = list[0];
+    int ret = 0;
+
+    if (list[l0 - 1] == LISTSEP) {
+	ret = list[l0];
+	list[0] -= 2;
+    }
+
+    return ret;
+}
+
+/**
+ * poisson_model:
+ * @list: dependent variable plus list of regressors.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @prn: printing struct for iteration info (or NULL is this is not
+ * wanted).
+ *
+ * Estimate the Poisson regression model given in @list using ML.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL poisson_model (int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    MODEL pmodel;
+    void *handle;
+    int offvar;
+    int (* poisson_estimate) (MODEL *, int, double ***, DATAINFO *, PRN *);
+
+    *gretl_errmsg = '\0';
+
+    offvar = get_offset_var(list);
+
+    /* run an initial OLS to "set the model up" and check for errors.
+       the poisson_estimate_driver function will overwrite the
+       coefficients etc.
+    */
+
+    pmodel = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+
+    if (pmodel.errcode) {
+        return pmodel;
+    }
+
+    poisson_estimate = get_plugin_function("poisson_estimate", &handle);
+
+    if (poisson_estimate == NULL) {
+	pmodel.errcode = E_FOPEN;
+	return pmodel;
+    }
+
+    (*poisson_estimate) (&pmodel, offvar, pZ, pdinfo, prn);
+
+    close_plugin(handle);
+
+    set_model_id(&pmodel);
+
+    return pmodel;
+}
+
 /**
  * garch:
  * @list: dependent variable plus arch and garch orders
@@ -3439,25 +3879,57 @@ MODEL garch (int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
     return gmod;
 } 
 
-static MODEL pooled_wls (int *list, double ***pZ, DATAINFO *pdinfo)
+MODEL pooled (int *list, double ***pZ, DATAINFO *pdinfo,
+	      gretlopt opt, PRN *prn)
 {
     MODEL wmod;
-    void *handle;
-    MODEL (*panel_wls_by_unit) (int *, double ***, DATAINFO *);
 
     *gretl_errmsg = '\0';
 
-    panel_wls_by_unit = get_plugin_function("panel_wls_by_unit", &handle);
+    if (opt & OPT_W) {
+	void *handle;
+	MODEL (*panel_wls_by_unit) (int *, double ***, DATAINFO *,
+				    gretlopt, PRN *);
 
-    if (panel_wls_by_unit == NULL) {
-	gretl_model_init(&wmod);
-	wmod.errcode = E_FOPEN;
-	return wmod;
+	panel_wls_by_unit = get_plugin_function("panel_wls_by_unit", &handle);
+
+	if (panel_wls_by_unit == NULL) {
+	    gretl_model_init(&wmod);
+	    wmod.errcode = E_FOPEN;
+	    return wmod;
+	}
+
+	wmod = (*panel_wls_by_unit) (list, pZ, pdinfo, opt, prn);
+
+	close_plugin(handle);
+    } else {
+	wmod = lsq(list, pZ, pdinfo, POOLED, opt, 0.0);
     }
 
-    wmod = (*panel_wls_by_unit) (list, pZ, pdinfo);
-
-    close_plugin(handle);
-
     return wmod;
 }
+
+int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+			   PRN *prn)
+{
+    MODEL wmod;
+    int err;
+
+    if (!dataset_is_panel(pdinfo)) {
+	strcpy(gretl_errmsg, _("This test is only available for panel data"));
+	return 1;
+    }
+
+    wmod = pooled(pmod->list, pZ, pdinfo, OPT_W | OPT_T | OPT_A, prn);
+    err = wmod.errcode;
+
+    if (!err) {
+	gretl_model_set_auxiliary(&wmod, AUX_GROUPWISE);
+	printmodel(&wmod, pdinfo, OPT_NONE, prn);
+    }
+
+    clear_model(&wmod);
+
+    return err;
+}
+
