@@ -20,13 +20,13 @@
 #ifndef GENERATE_H
 #define GENERATE_H
 
-#define INDEXNUM 996
-
 int generate (double ***pZ, DATAINFO *pdinfo, 
 	      const char *line, MODEL *pmod); 
 
 int genrtime (double ***pZ, DATAINFO *pdinfo, int tm);
 
+int genrunit (double ***pZ, DATAINFO *pdinfo);
+
 int dummy (double ***pZ, DATAINFO *pdinfo);
 
 int paneldum (double ***pZ, DATAINFO *pdinfo); 
