@@ -94,6 +94,8 @@ enum transformations {
     T_CNORM,
     T_RESAMPLE,
     T_HPFILT,
+    T_BKFILT,
+    T_VARNUM,
 #ifdef HAVE_MPFR
     T_MLOG,
 #endif
