@@ -64,6 +64,7 @@ struct gnuplot_info {
     int npoints;
     int miss;
     int oddman;
+    int toomany;
     double xrange;
     double *yvar1;
     double *yvar2;
@@ -71,7 +72,7 @@ struct gnuplot_info {
     
 #ifdef GLIB2
 
-/* #define SPAWN_DEBUG */
+#undef SPAWN_DEBUG
 
 int gnuplot_test_command (const char *cmd)
 {
@@ -202,7 +203,7 @@ const char *get_timevar_name (DATAINFO *pdinfo)
 	return "months";
     } else if (pdinfo->pd == 24) {
 	return "hrs";
-    } else if (dated_daily_data(pdinfo)) {
+    } else if (calendar_data(pdinfo)) {
 	return "decdate";
     } else {
 	return "time";
@@ -303,7 +304,8 @@ int gnuplot_has_specified_colors (void)
     static int err = -1; 
 
     if (err == -1) {
-	/* try the old-style command: if it fails, we have the new driver */
+	/* try the old-style command: 
+	   if it fails, we have the new driver, we hope */
 	err = gnuplot_test_command("set term png color");
     }
     return err;
@@ -453,9 +455,13 @@ const char *get_gretl_emf_term_line (int plottype, int color)
 
 int gnuplot_init (int plottype, FILE **fpp)
 {
-    int gui = gretl_using_gui();
+    int gui = gretl_in_gui_mode();
     char plotfile[MAXLEN];
 
+    if (gretl_looping()) {
+	return E_OK;
+    }
+
     /* 'gnuplot_path' is file-scope static var */
     if (*gnuplot_path == 0) {
 	strcpy(gnuplot_path, gretl_gnuplot_path());
@@ -464,7 +470,7 @@ int gnuplot_init (int plottype, FILE **fpp)
     if (gui) {
 	sprintf(plotfile, "%sgpttmp.XXXXXX", gretl_user_dir());
 	if (mktemp(plotfile) == NULL) {
-	    return 1;
+	    return E_FOPEN;
 	}
     } else {
 	sprintf(plotfile, "%sgpttmp.plt", gretl_user_dir());
@@ -472,9 +478,9 @@ int gnuplot_init (int plottype, FILE **fpp)
 
     set_gretl_plotfile(plotfile);
 
-    *fpp = fopen(plotfile, "w");
+    *fpp = gretl_fopen(plotfile, "w");
     if (*fpp == NULL) {
-	return 1;
+	return E_FOPEN;
     }
 
     if (gui) {
@@ -498,14 +504,19 @@ int gnuplot_make_graph (void)
     int err = 0;
     char plotcmd[MAXLEN];
 
-# ifdef WIN32
+#ifdef WIN32
     sprintf(plotcmd, "\"%s\" \"%s\"", gretl_gnuplot_path(), gretl_plotfile());
     err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-# else
+#else
     sprintf(plotcmd, "%s%s \"%s\"", gretl_gnuplot_path(), 
-	    (gretl_using_gui())? "" : " -persist", gretl_plotfile());
+	    (gretl_in_gui_mode())? "" : " -persist", gretl_plotfile());
     err = gretl_spawn(plotcmd);  
-# endif /* WIN32 */
+#endif
+
+#if 0
+    fprintf(stderr, "gnuplot_make_graph:\n"
+	    " plotcmd='%s', err = %d\n", plotcmd, err);
+#endif
 
     return err;
 }
@@ -626,7 +637,10 @@ get_gnuplot_output_file (FILE **fpp, unsigned char flags,
     *fpp = NULL;
 
     if ((flags & GP_FILE) && *plotfile != '\0') {
-	*fpp = fopen(plotfile, "w");
+	*fpp = gretl_fopen(plotfile, "w");
+	if (*fpp == NULL) {
+	    err = E_FOPEN;
+	}
     } else if ((flags & GP_BATCH) && plot_count != NULL) {  
 	char fname[MAXLEN];
 
@@ -635,13 +649,12 @@ get_gnuplot_output_file (FILE **fpp, unsigned char flags,
 	    sprintf(fname, "%sgpttmp%02d.plt", gretl_user_dir(), *plot_count);
 	}
 	set_gretl_plotfile(fname);
-	*fpp = fopen(fname, "w");
+	*fpp = gretl_fopen(fname, "w");
+	if (*fpp == NULL) {
+	    err = E_FOPEN;
+	}
     } else {
-	gnuplot_init(code, fpp);
-    }
-
-    if (*fpp == NULL) {
-	err = E_FOPEN;
+	err = gnuplot_init(code, fpp);
     }
 
     return err;
@@ -700,7 +713,7 @@ print_x_range (struct gnuplot_info *gpinfo, const double *x, FILE *fp)
 	gpinfo->xrange = xmax - xmin;
 	xmin -= gpinfo->xrange * .025;
 	xmax += gpinfo->xrange * .025;
-	fprintf(fp, "set xrange [%.8g:%.8g]\n", xmin, xmax);
+	fprintf(fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
 	gpinfo->xrange = xmax - xmin;
     }
 }
@@ -716,11 +729,13 @@ check_for_yscale (struct gnuplot_info *gpinfo, const int *list,
     double ratio;
     int i, j, oddcount;
 
-    /* find minima, maxima of the vars */
+    /* find minima, maxima of the y-axis vars */
     for (i=1; i<list[0]; i++) {
 	gretl_minmax(gpinfo->t1, gpinfo->t2, Z[list[i]], &ymin[i], &ymax[i]);
     }
 
+    gpinfo->yscale = 0;
+
     for (i=1; i<list[0]; i++) {
 	oddcount = 0;
 	for (j=1; j<list[0]; j++) {
@@ -837,6 +852,11 @@ gp_info_init (struct gnuplot_info *gpinfo, unsigned char flags,
     gpinfo->miss = 0;
     gpinfo->oddman = 0;
     gpinfo->xrange = 0.0;
+    gpinfo->toomany = 0;
+
+    if (lo > MAX_PLOT_LINES + 1) {
+	gpinfo->toomany = 1;
+    }
 
     if (lo > 2 && lo < 7 && !(flags & GP_RESIDS) && !(flags & GP_FA)
 	&& !(flags & GP_DUMMY)) {
@@ -961,7 +981,11 @@ int gnuplot (int *list, const int *lines, const char *literal,
 
     /* special tics for short time series plots */
     if (gpinfo.ts_plot) {
-	fprintf(fp, "# timeseries %d\n", pdinfo->pd);
+	if (gpinfo.toomany) {
+	    fprintf(fp, "# multiple timeseries %d\n", pdinfo->pd);
+	} else {
+	    fprintf(fp, "# timeseries %d\n", pdinfo->pd);
+	}
 	if (pdinfo->pd == 4 && (gpinfo.t2 - gpinfo.t1) / 4 < 8) {
 	    fputs("set xtics nomirror 0,1\n", fp); 
 	    fputs("set mxtics 4\n", fp);
@@ -970,6 +994,8 @@ int gnuplot (int *list, const int *lines, const char *literal,
 	    fputs("set xtics nomirror 0,1\n", fp); 
 	    fputs("set mxtics 12\n", fp);
 	}
+    } else if (gpinfo.toomany) {
+	fputs("# multiple data series\n", fp);
     }
 
     fprintf(fp, "set xlabel '%s'\n", xlabel);
@@ -1009,6 +1035,10 @@ int gnuplot (int *list, const int *lines, const char *literal,
 	fprintf(fp, "set ylabel '%s'\n", series_name(pdinfo, list[2]));
     } 
 
+    if (gpinfo.toomany) {
+	strcpy(keystr, "set key outside\n");
+    }
+
     fputs(keystr, fp);
 
 #ifdef ENABLE_NLS
@@ -1149,18 +1179,21 @@ int multi_scatters (const int *list, int pos, double ***pZ,
 
     if (yvar) {
 	plotlist[0] = list[0] - pos;
-	for (i=1; i<=plotlist[0]; i++)
-	   plotlist[i] = list[i+pos]; 
+	for (i=1; i<=plotlist[0]; i++) {
+	   plotlist[i] = list[i + pos]; 
+	}
     } else {
 	plotlist[0] = pos - 1;
-	for (i=1; i<pos; i++)
+	for (i=1; i<pos; i++) {
 	   plotlist[i] = list[i]; 
+	}
     }
 
     /* max 6 plots */
     if (plotlist[0] > 6) {
 	plotlist[0] = 6;
     }
+
     nplots = plotlist[0];
 
     if (get_gnuplot_output_file(&fp, flags, plot_count, PLOT_MULTI_SCATTER)) {
@@ -1173,6 +1206,10 @@ int multi_scatters (const int *list, int pos, double ***pZ,
     fputs("set nokey\n", fp);
     fputs("set noxtics\nset noytics\n", fp);
 
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+
     for (i=0; i<nplots; i++) {  
 	if (nplots <= 4) {
 	    fputs("set size 0.45,0.5\n", fp);
@@ -1191,42 +1228,47 @@ int multi_scatters (const int *list, int pos, double ***pZ,
 	    else if (i == 4) fputs("0.32,0.0\n", fp);
 	    else if (i == 5) fputs("0.64,0.0\n", fp);
 	}
+
 	fprintf(fp, "set xlabel '%s'\n",
 		(yvar)? pdinfo->varname[plotlist[i+1]] :
 		pdinfo->varname[xvar]);
+
 	fprintf(fp, "set ylabel '%s'\n", 
 		(yvar)? pdinfo->varname[yvar] :
 		pdinfo->varname[plotlist[i+1]]);
+
 	fputs("plot '-' using 1:2\n", fp);
 
-#ifdef ENABLE_NLS
-	setlocale(LC_NUMERIC, "C");
-#endif
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    double xx;
 	    int m;
 
 	    m = (yvar)? plotlist[i+1] : xvar;
 	    xx = (*pZ)[m][t];
+
 	    if (na(xx)) {
 		fputs("? ", fp);
 	    } else {
 		fprintf(fp, "%.8g ", xx);
 	    }
+
 	    m = (yvar)? yvar : plotlist[i+1];
 	    xx = (*pZ)[m][t];
+
 	    if (na(xx)) {
 		fputs("?\n", fp);
 	    } else {
 		fprintf(fp, "%.8g\n", xx);
 	    }
 	}
+
 	fputs("e\n", fp);
-#ifdef ENABLE_NLS
-	setlocale(LC_NUMERIC, "");
-#endif
     } 
 
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
+
     fputs("set nomultiplot\n", fp);
 
     fclose(fp);
@@ -1246,7 +1288,7 @@ static int get_3d_output_file (FILE **fpp)
     int err = 0;
 
     sprintf(fname, "%sgpttmp.plt", gretl_user_dir());
-    *fpp = fopen(fname, "w");
+    *fpp = gretl_fopen(fname, "w");
     if (*fpp == NULL) {
 	err = E_FOPEN;
     } else {
@@ -1385,6 +1427,44 @@ int gnuplot_3d (int *list, const char *literal,
     return 0;
 }
 
+static void print_freq_test_label (char *s, const char *format, 
+				   double v, double pv)
+{
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
+    sprintf(s, format, v, pv);
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+}
+
+static void print_freq_dist_label (char *s, int dist, double x, double y)
+{
+    int dcomma = 0;
+#ifdef ENABLE_NLS
+    char test[8];
+
+    setlocale(LC_NUMERIC, "");
+    sprintf(test, "%g", 0.5);
+    if (strchr(test, ',')) {
+	dcomma = 1;
+    }
+#endif
+
+    if (dist == NORMAL) {
+	sprintf(s, "N(%.5g%c%.5g)", x, 
+		((dcomma)? ' ' : ','), y);
+    } else if (dist == GAMMA) {
+	sprintf(s, "gamma(%.5g%c%.5g)", x, 
+		((dcomma)? ' ' : ','), y);
+    }
+
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+}
+
 /**
  * plot_freq:
  * @freq: frequency distribution struct.
@@ -1402,15 +1482,21 @@ int plot_freq (FREQDIST *freq, int dist)
     FILE *fp = NULL;
     int i, K = freq->numbins;
     char withstr[16] = {0};
+    char label[80] = {0};
     double plotmin = 0.0, plotmax = 0.0;
     double barwidth, barskip;
     int plottype = PLOT_FREQ_SIMPLE;
     int use_boxes = 1;
+    int err;
 
     if (K == 0) {
 	return 1;
     }
 
+    if ((err = gnuplot_init(plottype, &fp))) {
+	return err;
+    }    
+
     barwidth = freq->endpt[K-1] - freq->endpt[K-2];
     barskip = 0.005 * (freq->endpt[K] - freq->endpt[0]);
 
@@ -1424,10 +1510,6 @@ int plot_freq (FREQDIST *freq, int dist)
 	plottype = PLOT_FREQ_GAMMA;
     }
 
-    if (gnuplot_init(plottype, &fp)) {
-	return E_FOPEN;
-    }
-
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
 #endif
@@ -1435,7 +1517,6 @@ int plot_freq (FREQDIST *freq, int dist)
     fputs("# frequency plot ", fp);
 
     if (dist) {
-	char testlbl[80];
 	double propn;
 
 	/* find the endpts that straddle the mean... */
@@ -1471,10 +1552,10 @@ int plot_freq (FREQDIST *freq, int dist)
 		fprintf(fp, "set label '%s:' at graph .03, graph .97%s\n",
 			I_("Test statistic for normality"),
 			label_front());
-		sprintf(testlbl, I_("Chi-squared(2) = %.3f, pvalue %.5f"), 
-			freq->test, chisq(freq->test, 2));
+		print_freq_test_label(label, I_("Chi-squared(2) = %.3f pvalue = %.5f"), 
+				      freq->test, chisq(freq->test, 2));
 		fprintf(fp, "set label '%s' at graph .03, graph .93%s\n", 
-			testlbl, label_front());
+			label, label_front());
 	    }	
 	} else if (dist == GAMMA) {
 	    double xx, height, var = freq->sdx * freq->sdx;
@@ -1501,10 +1582,10 @@ int plot_freq (FREQDIST *freq, int dist)
 		fprintf(fp, "set label '%s:' at graph .03, graph .97%s\n",
 			I_("Test statistic for gamma"),
 			label_front());
-		sprintf(testlbl, I_("z = %.3f, pvalue %.5f"), 
-			freq->test, 2.0 * normal(fabs(freq->test)));
+		print_freq_test_label(label, I_("z = %.3f pvalue = %.5f"), 
+				      freq->test, 2.0 * normal(fabs(freq->test)));
 		fprintf(fp, "set label '%s' at graph .03, graph .93%s\n", 
-			testlbl, label_front());
+			label, label_front());
 	    }	
 	}
 
@@ -1516,7 +1597,7 @@ int plot_freq (FREQDIST *freq, int dist)
 	    plotmax = freq->midpt[K-1];
 	}
 
-	fprintf(fp, "set xrange [%.8g:%.8g]\n", plotmin, plotmax);
+	fprintf(fp, "set xrange [%.7g:%.7g]\n", plotmin, plotmax);
 	fputs("set key right top\n", fp);
     } else { 
 	/* plain frequency plot */
@@ -1525,7 +1606,7 @@ int plot_freq (FREQDIST *freq, int dist)
 	lambda = 1.0 / freq->n;
 	plotmin = freq->midpt[0] - barwidth;
 	plotmax = freq->midpt[K-1] + barwidth;
-	fprintf(fp, "set xrange [%.8g:%.8g]\n", plotmin, plotmax);
+	fprintf(fp, "set xrange [%.7g:%.7g]\n", plotmin, plotmax);
 	fputs("set nokey\n", fp);
 	fprintf(fp, "set xlabel '%s %s'\n", 
 		I_("Frequency distribution for"), freq->varname);	
@@ -1551,17 +1632,19 @@ int plot_freq (FREQDIST *freq, int dist)
     if (!dist) {
 	fprintf(fp, "plot '-' using 1:($2) %s\n", withstr);
     } else if (dist == NORMAL) {
+	print_freq_dist_label(label, dist, freq->xbar, freq->sdx);
 	fputs("plot \\\n", fp);
 	fprintf(fp, "'-' using 1:($2) title '%s' %s , \\\n"
 		"(1/(sqrt(2*pi)*sigma)*exp(-(x-mu)**2/(2*sigma**2))) "
-		"title 'N(%.5g,%.5g)' w lines\n",
-		freq->varname, withstr, freq->xbar, freq->sdx);
+		"title '%s' w lines\n",
+		freq->varname, withstr, label);
     } else if (dist == GAMMA) {
+	print_freq_dist_label(label, dist, alpha, beta);
 	fputs("plot \\\n", fp);
 	fprintf(fp, "'-' using 1:($2) title '%s' %s ,\\\n"
 		"x**(alpha-1.0)*exp(-x/beta)/(exp(lgamma(alpha))*(beta**alpha)) "
-		"title 'gamma(%.5g,%.5g)' w lines\n",
-		freq->varname, withstr, alpha, beta); 
+		"title '%s' w lines\n",
+		freq->varname, withstr, label); 
     }
 
     for (i=0; i<K; i++) { 
@@ -1590,10 +1673,10 @@ int plot_fcast_errs (int n, const double *obs,
 {
     FILE *fp = NULL;
     double xmin, xmax, xrange;
-    int t;
+    int t, err;
 
-    if (gnuplot_init(PLOT_FORECAST, &fp)) {
-	return E_FOPEN;
+    if ((err = gnuplot_init(PLOT_FORECAST, &fp))) {
+	return err;
     }
 
     fputs("# forecasts with 95 pc conf. interval\n", fp);
@@ -1602,7 +1685,7 @@ int plot_fcast_errs (int n, const double *obs,
     xrange = xmax - xmin;
     xmin -= xrange * .025;
     xmax += xrange * .025;
-    fprintf(fp, "set xrange [%.8g:%.8g]\n", xmin, xmax);
+    fprintf(fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
     fputs("set missing \"?\"\n", fp);
     if (!time_series) {
 	fputs("set xtics 1\n", fp);
@@ -1659,14 +1742,15 @@ int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
     double *h, *obs;
     double sd2;
     int t, pv;
+    int err;
 
     h = gretl_model_get_data(pmod, "garch_h");
     if (h == NULL) {
 	return E_DATA;
     }
 
-    if (gnuplot_init(PLOT_FORECAST, &fp)) {
-	return E_FOPEN;
+    if ((err = gnuplot_init(PLOT_FORECAST, &fp))) {
+	return err;
     }
 
     pv = plotvar(pZ, pdinfo, get_timevar_name(pdinfo));
@@ -1728,11 +1812,11 @@ void free_plotspec (GPT_SPEC *spec)
 	}
     }
 
-    if (spec->labels != NULL) {
-	for (i=0; i<spec->nlabels; i++) {
-	    free(spec->labels[i]);
+    if (spec->markers != NULL) {
+	for (i=0; i<spec->nmarkers; i++) {
+	    free(spec->markers[i]);
 	}
-	free(spec->labels);
+	free(spec->markers);
     }
 
     free(spec);
@@ -1775,8 +1859,9 @@ int get_termstr (const GPT_SPEC *spec, char *termstr)
 
 static char *escape_quotes (const char *s)
 {
-    if (strchr(s, '"') == NULL) return NULL;
-    else {
+    if (strchr(s, '"') == NULL) {
+	return NULL;
+    } else {
 	int qcount = 0;
 	char *ret, *r;
 	const char *p = s;
@@ -1787,7 +1872,9 @@ static char *escape_quotes (const char *s)
 	}
 
 	ret = malloc(strlen(s) + 1 + qcount);
-	if (ret == NULL) return NULL;
+	if (ret == NULL) {
+	    return NULL;
+	}
 
 	r = ret;
 	while (*s) {
@@ -1799,8 +1886,8 @@ static char *escape_quotes (const char *s)
 	    }
 	    s++;
 	}
-
 	*r = 0;
+
 	return ret;
     }
 }
@@ -1824,11 +1911,46 @@ gp_string (FILE *fp, const char *fmt, const char *s, int png)
 #endif
 }
 
+const char *gp_justification_string (int j)
+{
+    if (j == GP_JUST_LEFT) return "left";
+    else if (j == GP_JUST_CENTER) return "center";
+    else if (j == GP_JUST_RIGHT) return "right";
+    else return "left";
+}
+
+static void 
+print_plot_labelspec (const GPT_LABEL *lbl, int png, FILE *fp)
+{
+    char *label = escape_quotes(lbl->text);
+
+    gp_string(fp, "set label \"%s\" ", (label != NULL)? 
+	      label : lbl->text, png);
+
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+
+    fprintf(fp, "at %g,%g %s%s\n", 
+	    lbl->pos[0], lbl->pos[1],
+	    gp_justification_string(lbl->just),
+	    label_front());
+
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
+
+    if (label != NULL) {
+	free(label);
+    }
+}
+
 int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 {
     int i, k, t, datlines;
     int plotn, nlines = spec->nlines;
     int png = get_png_output(spec);
+    int any_y2 = 0;
     int miss = 0;
     double xx;
 
@@ -1840,29 +1962,22 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 	    gp_string(fp, "set title '%s'\n", spec->titles[0], png);
 	}
     }
+
     if (!string_is_blank(spec->titles[1])) {
 	gp_string(fp, "set xlabel '%s'\n", spec->titles[1], png);
     }
+
     if (!string_is_blank(spec->titles[2])) {
 	gp_string(fp, "set ylabel '%s'\n", spec->titles[2], png);
     }
+
     if ((spec->flags & GPTSPEC_Y2AXIS) && !string_is_blank(spec->titles[3])) {
 	gp_string(fp, "set y2label '%s'\n", spec->titles[3], png);
     }
 
     for (i=0; i<MAX_PLOT_LABELS; i++) {
-	if (!string_is_blank(spec->text_labels[i].text)) {
-	    char *label = escape_quotes(spec->text_labels[i].text);
-
-	    gp_string(fp, "set label \"%s\" ", (label != NULL)? 
-		      label : spec->text_labels[i].text, png);
-	    fprintf(fp, "at %s %s%s\n", 
-		    spec->text_labels[i].pos,
-		    spec->text_labels[i].just,
-		    label_front());
-	    if (label != NULL) {
-		free(label);
-	    }
+	if (!string_is_blank(spec->labels[i].text)) {
+	    print_plot_labelspec(&(spec->labels[i]), png, fp);
 	}
     }
 
@@ -1875,13 +1990,26 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 	fprintf(fp, "set key %s\n", spec->keyspec);
     }
 
-    k = (spec->flags & GPTSPEC_Y2AXIS)? 3: 2;
+    k = (spec->flags & GPTSPEC_Y2AXIS)? 3 : 2;
+
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+
     for (i=0; i<k; i++) {
-	fprintf(fp, "set %srange [%s:%s]\n",
-		(i==0)? "x" : (i==1)? "y" : "y2",
+	if (na(spec->range[i][0]) || na(spec->range[i][1]) ||
+	    spec->range[i][0] == spec->range[i][1]) {
+	    continue;
+	}
+	fprintf(fp, "set %srange [%.7g:%.7g]\n",
+		(i == 0)? "x" : (i == 1)? "y" : "y2",
 		spec->range[i][0], spec->range[i][1]);
     }
 
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
+
     /* customized xtics? */
     if (!string_is_blank(spec->xtics)) {
 	fprintf(fp, "set xtics %s\n", spec->xtics);
@@ -1911,16 +2039,13 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 
     if (spec->code == PLOT_FORECAST) {
 	fputs("# forecasts with 95 pc conf. interval\n", fp);
-    }
-    else if (spec->code == PLOT_CORRELOGRAM) {
+    } else if (spec->code == PLOT_CORRELOGRAM) {
 	fputs("# correlogram\n", fp); 
-    }
-    else if (spec->code == PLOT_FREQ_SIMPLE) {
+    } else if (spec->code == PLOT_FREQ_SIMPLE) {
 	fputs("# frequency plot (simple)\n", fp); 
-    }
-    else if (spec->code == PLOT_FREQ_NORMAL || 
-	     spec->code == PLOT_FREQ_GAMMA ||
-	     spec->code == PLOT_PERIODOGRAM) { 
+    } else if (spec->code == PLOT_FREQ_NORMAL || 
+	       spec->code == PLOT_FREQ_GAMMA ||
+	       spec->code == PLOT_PERIODOGRAM) { 
 	if (spec->code == PLOT_FREQ_NORMAL) {
 	    fputs("# frequency plot (against normal)\n", fp); 
 	} else if (spec->code == PLOT_FREQ_GAMMA) {
@@ -1950,6 +2075,13 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 
     datlines = nlines; /* ? */ 
 
+    for (i=0; i<nlines; i++) {
+	if ((spec->flags & GPTSPEC_Y2AXIS) && spec->lines[i].yaxis != 1) {
+	    any_y2 = 1;
+	    break;
+	}
+    }
+
     for (i=0; i<nlines; i++) {
 	if (strcmp(spec->lines[i].scale, "NA")) {
 	    fprintf(fp, "'-' using 1:($2*%s) ", 
@@ -1958,11 +2090,25 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 	    fprintf(fp, "%s ", spec->lines[i].formula); 
 	    datlines--;
 	}
-	if (spec->lines[i].yaxis != 1) {
+
+	if ((spec->flags & GPTSPEC_Y2AXIS) && spec->lines[i].yaxis != 1) {
 	    fprintf(fp, "axes x1y%d ", spec->lines[i].yaxis);
 	}
-	gp_string(fp, "title '%s' ", spec->lines[i].title, png);
+
+	gp_string(fp, "title '%s", spec->lines[i].title, png);
+
+	if (any_y2) {
+	    if (spec->lines[i].yaxis == 1) {
+		fprintf(fp, " (%s)' ", I_("left"));
+	    } else {
+		fprintf(fp, " (%s)' ", I_("right"));
+	    }
+	} else {
+	    fputs("' ", fp);
+	}
+
 	fprintf(fp, "w %s", spec->lines[i].style);
+
 	if (i == nlines - 1) {
 	    fputc('\n', fp);
 	} else {
@@ -1974,6 +2120,7 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
 #endif
+
     miss = 0;
     plotn = spec->t2 - spec->t1 + 1;
     for (i=1; i<=datlines; i++) {  
@@ -1992,13 +2139,14 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
 	    } else {
 		fprintf(fp, "%.8g", xx);
 	    }
-	    if (spec->labels != NULL && i == 1) {
-		fprintf(fp, " # %s", spec->labels[t]);
+	    if (spec->markers != NULL && i == 1) {
+		fprintf(fp, " # %s", spec->markers[t]);
 	    }
 	    fputc('\n', fp);
 	}
 	fputs("e\n", fp);
     }
+
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "");
 #endif
@@ -2021,12 +2169,12 @@ int go_gnuplot (GPT_SPEC *spec, char *fname)
     if (dump) {  
 	/* dump of gnuplot commands to named file */
 	if (fname == NULL) return 1;  /* impossible */
-	fp = fopen(fname, "w");
+	fp = gretl_fopen(fname, "w");
 	if (fp == NULL) return 1;
     } else {     
 	/* output to gnuplot, for screen or other "term" */
 	if (spec->fp == NULL) {
-	    fp = fopen(gretl_plotfile(), "w");
+	    fp = gretl_fopen(gretl_plotfile(), "w");
 	}
 	if (fp == NULL) return 1;
 
@@ -2085,26 +2233,50 @@ int go_gnuplot (GPT_SPEC *spec, char *fname)
 
 /* ........................................................... */
 
-int rmplot (const int *list, double **Z, DATAINFO *pdinfo, PRN *prn)
+int 
+rmplot (const int *list, const double **Z, DATAINFO *pdinfo, PRN *prn)
 {
     int err;
     void *handle;
-    int (*range_mean_graph) (int, double **, const DATAINFO *, PRN *);
+    int (*range_mean_graph) (int, const double **, const DATAINFO *, PRN *);
 
     range_mean_graph = get_plugin_function("range_mean_graph", &handle);
     if (range_mean_graph == NULL) {
         return 1;
     }
 
-    err = range_mean_graph (list[1], Z, pdinfo, prn);
+    err = range_mean_graph(list[1], Z, pdinfo, prn);
 
     close_plugin(handle);
 
-    if (!err) {
-        return gnuplot_make_graph();
-    } else {
-	return err;
+    if (!err && !gretl_in_batch_mode() && !gretl_looping()) {
+        err = gnuplot_make_graph();
     }
+
+    return err;
+}
+
+int 
+hurstplot (const int *list, const double **Z, DATAINFO *pdinfo, PRN *prn)
+{
+    int err;
+    void *handle;
+    int (*hurst_exponent) (int, const double **, const DATAINFO *, PRN *);
+
+    hurst_exponent = get_plugin_function("hurst_exponent", &handle);
+    if (hurst_exponent == NULL) {
+        return 1;
+    }
+
+    err = hurst_exponent(list[1], Z, pdinfo, prn);
+
+    close_plugin(handle);
+
+    if (!err && !gretl_in_batch_mode() && !gretl_looping()) {
+        err = gnuplot_make_graph();
+    } 
+
+    return err;
 }
 
 /* ........................................................... */
@@ -2118,18 +2290,26 @@ gretl_var_plot_impulse_response (GRETL_VAR *var,
     int vtarg, vshock;
     double *resp;
     char title[128];
-    int t;
+    int t, err;
 
     if (periods == 0) {
-	if (pdinfo->pd == 4) periods = 24;
-	else if (pdinfo->pd == 12) periods = 36;
-	else periods = 12;
+	if (pdinfo->pd == 4) {
+	    periods = 24;
+	} else if (pdinfo->pd == 12) {
+	    periods = 36;
+	} else {
+	    periods = 12;
+	}
+    }
+
+    if ((err = gnuplot_init(PLOT_REGULAR, &fp))) {
+	return err;
     }
 
     resp = gretl_var_get_impulse_responses(var, targ, shock, periods);
-    if (resp == NULL) return E_ALLOC;
-
-    if (gnuplot_init(PLOT_REGULAR, &fp)) return E_FOPEN;
+    if (resp == NULL) {
+	return E_ALLOC;
+    }
 
     vtarg = gretl_var_get_variable_number(var, targ);
     vshock = gretl_var_get_variable_number(var, shock);
@@ -2192,8 +2372,7 @@ const char *get_gnuplot_pallette (int i, int ptype)
 		   ptype == PLOT_FREQ_NORMAL || 
 		   ptype == PLOT_FREQ_GAMMA)) {
 	return gnuplot_pallette[3];
-    }
-    else if (i >= 0 && i < 3) {
+    } else if (i >= 0 && i < 3) {
 	return gnuplot_pallette[i];
     } else {
 	return "";
