@@ -38,6 +38,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { AR,       "ar" },       
     { ARCH,     "arch" },
     { ARMA,     "arma" },
+    { BREAK,    "break" },
     { BXPLOT,   "boxplot" },
     { CHOW,     "chow" },     
     { COEFFSUM, "coeffsum" },
@@ -58,6 +59,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { ENDLOOP,  "endloop" },
     { EQNPRINT, "eqnprint" },
     { EQUATION, "equation" },
+    { ESTIMATE, "estimate" },
     { FCAST,    "fcast" }, 
     { FCASTERR, "fcasterr" },
     { FIT,      "fit" }, 
@@ -72,6 +74,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { HELP,     "help" },    
     { HILU,     "hilu" },    
     { HSK,      "hsk" }, 
+    { HURST,    "hurst" },
     { IF,       "if" },
     { IMPORT,   "import" },
     { INFO,     "info" }, 
@@ -87,6 +90,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { LOGIT,    "logit" },
     { LOGS,     "logs" },
     { LOOP,     "loop" },
+    { MAHAL,    "mahal" },
     { MEANTEST, "meantest" },
     { MODELTAB, "modeltab" },
     { MPOLS,    "mpols" },
@@ -102,6 +106,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { PCA,      "pca" },
     { PERGM,    "pergm" },
     { PLOT,     "plot" },    
+    { POISSON,  "poisson" },
     { POOLED,   "pooled" },
     { PRINT,    "print" }, 
     { PRINTF,   "printf" },
