@@ -29,6 +29,7 @@ enum gretl_cmd_codes {
     AR,      
     ARCH,
     ARMA,
+    BREAK,
     BXPLOT,
     CHOW,
     COEFFSUM,
@@ -49,6 +50,7 @@ enum gretl_cmd_codes {
     ENDLOOP,
     EQNPRINT, 
     EQUATION,
+    ESTIMATE,
     FCAST,   
     FCASTERR, 
     FIT,     
@@ -63,6 +65,7 @@ enum gretl_cmd_codes {
     HELP,    
     HILU,    
     HSK,
+    HURST,
     IF,
     IMPORT,
     INFO,
@@ -78,6 +81,7 @@ enum gretl_cmd_codes {
     LOGIT,
     LOGS,
     LOOP,
+    MAHAL,
     MEANTEST,
     MODELTAB,
     MPOLS,
@@ -93,6 +97,7 @@ enum gretl_cmd_codes {
     PCA,
     PERGM,
     PLOT,    
+    POISSON,
     POOLED,
     PRINT, 
     PRINTF,
@@ -177,8 +182,6 @@ enum gretl_cmd_codes {
                         c == VIF || \
                         c == VARTEST)
 
-#define LIMDEP(c) (c == LOGIT || c == PROBIT || c == TOBIT)
-
 int gretl_command_number (const char *s);
 
 const char *gretl_command_word (int i);
