@@ -88,6 +88,7 @@ const char *gretl_error_messages[] = {
     N_("The convergence criterion was not met"),                 /* E_NOCONV */
     N_("The operation was canceled"),                            /* E_CANCEL */
     N_("Missing values encountered"),                            /* E_MISSDATA */
+    NULL,                                                        /* E_OK */
     NULL                                                         /* E_NAN */
 };
 
@@ -195,3 +196,8 @@ void gretl_errmsg_set (const char *str)
 	strncat(gretl_errmsg, str, ERRLEN - 1);
     }
 }
+
+void gretl_errmsg_clear (void)
+{
+    *gretl_errmsg = '\0';
+}
