@@ -191,6 +191,20 @@ gretl_matrix *gretl_matrix_copy (const gretl_matrix *m)
     return gretl_matrix_copy_mod(m, GRETL_MOD_NONE);
 }
 
+/**
+ * gretl_matrix_copy_transpose:
+ * @m: source matrix to be copied.
+ *
+ * Returns: an allocated copy of the tranpose of @m, or %NULL on failure.  
+ * 
+ */
+
+gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m)
+{
+    return gretl_matrix_copy_mod(m, GRETL_MOD_TRANSPOSE);
+}
+
+
 /**
  * gretl_matrix_free:
  * @m: matrix to be freed.
@@ -700,22 +714,61 @@ void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn)
 }
 
 /**
- * gretl_LU_determinant:
+ * gretl_vcv_log_determinant:
  * @a: gretl_matrix.
  *
- * Compute the determinant of the square matrix @a using the LU
- * factorization.  Matrix @a is not preserved: it is overwritten
- * by the factorization.
+ * Compute the log determinant of the symmetric positive-definite
+ * matrix @a using Cholesky decomposition.  Matrix @a is not 
+ * preserved: it is overwritten by the decomposition.
  * 
- * Returns: the determinant, or #NABDL on failure.
+ * Returns: the log determinant, or #NABDL on failure.
  * 
  */
 
-double gretl_LU_determinant (gretl_matrix *a)
+double gretl_vcv_log_determinant (gretl_matrix *a)
+{
+    char uplo = 'U';
+    integer info;
+    integer n = a->rows;
+    double det;
+    int i;
+
+    if (a->rows != a->cols) {
+	fputs("gretl_vcv_log_determinant: matrix must be square\n", stderr);
+	return NADBL;
+    }
+
+    dpotrf_(&uplo, &n, a->val, &n, &info);
+
+    if (info != 0) {
+	if (info > 0) {
+	    fputs("gretl_vcv_log_determinant: matrix not positive definite\n", 
+		  stderr);
+	} else {
+	    fputs("gretl_vcv_log_determinant: illegal argument to dpotrf\n", 
+		  stderr);
+	}
+	return NADBL;
+    }
+
+    det = 1.0;
+    for (i=0; i<n; i++) {
+	det *= a->val[mdx(a, i, i)] * a->val[mdx(a, i, i)];
+    }
+
+    return log(det);
+}
+
+/* calculate determinant using LU factorization.
+   if logdet != 0, return the log of the determinant.
+   if logdet != 0 and absval != 0, return the log of the
+   absolute value of the determinant.  
+*/   
+
+static double gretl_LU_determinant (gretl_matrix *a, int logdet, int absval)
 {
     integer info;
-    integer m = a->rows;
-    integer n = a->cols;
+    integer n = a->rows;
     integer *ipiv;
     double det;
     int i;
@@ -728,7 +781,7 @@ double gretl_LU_determinant (gretl_matrix *a)
     ipiv = malloc(n * sizeof *ipiv);
     if (ipiv == NULL) return NADBL;
 
-    dgetrf_(&m, &n, a->val, &n, ipiv, &info);
+    dgetrf_(&n, &n, a->val, &n, ipiv, &info);
 
     if (info != 0) {
 	fprintf(stderr, "gretl_LU_determinant: dgetrf gave info = %d\n", 
@@ -737,12 +790,46 @@ double gretl_LU_determinant (gretl_matrix *a)
 	return NADBL;
     }
 
-    det = 1.0;
-    for (i=0; i<n; i++) {
-	if (ipiv[i] != i + 1) {
-	    det = -det;
+    if (logdet) {
+	int negcount = 0;
+
+	/* not sure if this is worth the bother: but do we get a bit
+	   more numerical accuracy in some cases by adding logs,
+	   rather than by multiplying terms then taking the log of the
+	   product? 
+	*/
+
+	det = 0.0;
+	for (i=0; i<n; i++) {
+	    double aii = a->val[mdx(a, i, i)];
+
+	    if (aii == 0.0) {
+		fputs("gretl_matrix_log_determinant: determinant = 0\n", stderr);
+		det = NADBL;
+		break;
+	    }
+
+	    if (ipiv[i] != i + 1) {
+		aii = -aii;
+	    }
+	    if (aii < 0) {
+		aii = -aii;
+		negcount++;
+	    }
+	    det += log(aii);
+	}
+	if (!absval && negcount % 2) {
+	    fputs("gretl_matrix_log_determinant: determinant is < 0\n", stderr);
+	    det = NADBL;
+	}
+    } else {
+	det = 1.0;
+	for (i=0; i<n; i++) {
+	    if (ipiv[i] != i + 1) {
+		det = -det;
+	    }
+	    det *= a->val[mdx(a, i, i)];
 	}
-	det *= a->val[mdx(a, i, i)];
     }
 
     free(ipiv);
@@ -750,6 +837,57 @@ double gretl_LU_determinant (gretl_matrix *a)
     return det;
 }
 
+/**
+ * gretl_matrix_determinant:
+ * @a: gretl_matrix.
+ *
+ * Compute the determinant of the square matrix @a using the LU
+ * factorization.  Matrix @a is not preserved: it is overwritten
+ * by the factorization.
+ * 
+ * Returns: the determinant, or #NABDL on failure.
+ * 
+ */
+
+double gretl_matrix_determinant (gretl_matrix *a)
+{
+    return gretl_LU_determinant(a, 0, 0);
+}
+
+/**
+ * gretl_matrix_log_determinant:
+ * @a: gretl_matrix.
+ *
+ * Compute the log of the determinant of the square matrix @a using LU
+ * factorization.  Matrix @a is not preserved: it is overwritten
+ * by the factorization.
+ * 
+ * Returns: the determinant, or #NABDL on failure.
+ * 
+ */
+
+double gretl_matrix_log_determinant (gretl_matrix *a)
+{
+    return gretl_LU_determinant(a, 1, 0);
+}
+
+/**
+ * gretl_matrix_log_abs_determinant:
+ * @a: gretl_matrix.
+ *
+ * Compute the log of the absolute value of the determinant of the 
+ * square matrix @a using LU factorization.  Matrix @a is not 
+ * preserved: it is overwritten by the factorization.
+ * 
+ * Returns: the determinant, or #NABDL on failure.
+ * 
+ */
+
+double gretl_matrix_log_abs_determinant (gretl_matrix *a)
+{
+    return gretl_LU_determinant(a, 1, 1);
+}
+
 /**
  * gretl_LU_solve:
  * @a: gretl_matrix.
@@ -823,6 +961,29 @@ gretl_matrix *gretl_matrix_from_2d_array (const double **X,
     return m;
 }
 
+static int 
+matrix_multiply_self_transpose (const gretl_matrix *a,
+				gretl_matrix *c)
+{
+    register int i, j, k;
+    int nc = a->cols;
+    int nr = a->rows;
+    double targ;
+
+    for (i=0; i<nc; i++) {
+	for (j=i; j<nc; j++) {
+	    targ = 0.0;
+	    for (k=0; k<nr; k++) {
+		targ += a->val[mdxtr(a,i,k)] * a->val[mdx(a,k,j)];
+	    }
+	    c->val[mdx(c,i,j)] = targ;
+	    c->val[mdx(c,j,i)] = targ;
+	}
+    } 
+
+    return GRETL_MATRIX_OK;
+}
+
 /**
  * gretl_matrix_multiply_mod:
  * @a: left-hand matrix.
@@ -859,6 +1020,10 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
 	return GRETL_MATRIX_ERR;
     }
 
+    if (a == b && atr && !btr) {
+	return matrix_multiply_self_transpose(a, c);
+    }
+
     lrows = (atr)? a->cols : a->rows;
     lcols = (atr)? a->rows : a->cols;
     rrows = (btr)? b->cols : b->rows;
@@ -909,24 +1074,31 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
 
 double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
 				 const gretl_matrix *b, int bflag,
-				 int *err)
+				 int *errp)
 {
-    gretl_matrix *c;
+    gretl_matrix *c = NULL;
     double ret = NADBL;
+    int err = 0;
 
     c = gretl_matrix_alloc(1, 1);
     if (c == NULL) {
-	*err = GRETL_MATRIX_ERR;
-	return ret;
+	err = E_ALLOC;
     }
 
-    *err = gretl_matrix_multiply_mod(a, aflag, b, bflag, c);
-    if (*err == GRETL_MATRIX_OK) {
+    if (!err) {
+	err = gretl_matrix_multiply_mod(a, aflag, b, bflag, c);
+    }
+
+    if (!err) {
 	ret = c->val[0];
     }
 	
     gretl_matrix_free(c);
 
+    if (errp != NULL) {
+	*errp = err;
+    }
+
     return ret;
 }
 
@@ -1163,6 +1335,36 @@ gretl_symmetric_matrix_expand (gretl_matrix *m, char uplo)
     return 0;
 }
 
+/**
+ * gretl_invert_diagonal_matrix:
+ * @a: matrix to invert.
+ * 
+ * Computes the inverse of a diagonal matrix.
+ * On exit @a is overwritten with the inverse. 
+ *
+ * Returns: 0 on success; non-zero error code on failure.
+ * 
+ */
+
+int gretl_invert_diagonal_matrix (gretl_matrix *a)
+{
+    double x;
+    int i;
+
+    if (a->cols != a->rows) {
+	fputs("gretl_invert_diagonal_matrix: input is not square\n",
+	      stderr);
+	return GRETL_MATRIX_NON_CONFORM;
+    }
+
+    for (i=0; i<a->rows; i++) {
+	x = a->val[mdx(a,i,i)];
+	a->val[mdx(a,i,i)] = 1.0 / x;
+    }
+
+    return 0;
+}
+
 /**
  * gretl_invert_symmetric_matrix:
  * @a: matrix to invert.
@@ -1434,27 +1636,80 @@ int gretl_matrix_rows (const gretl_matrix *m)
 /* ....................................................... */
 
 static int
-get_ols_vcv (const gretl_vector *y, const gretl_matrix *X,
-	     const gretl_vector *b, gretl_matrix *vcv)
+get_svd_ols_vcv (const gretl_matrix *A, const gretl_matrix *B,
+		 const double *s, gretl_matrix *vcv, double *s2)
 {
-    double u, sigma2 = 0.0;
-    int k = X->cols;
-    int n = X->rows;
-    int i, j;
+    double aik, ajk, vij;
+    int m = A->cols;
+    int i, j, k;
 
-    if (gretl_invert_symmetric_matrix(vcv)) return 1;
+    /* Get X'X{-1}, based on the work done by the SV decomp:
+       reciprocals of the squares of the (positive) singular values,
+       premultiplied by V and postmultiplied by V-transpose
+    */
 
-    for (i=0; i<n; i++) {
-	u = y->val[i];
-	for (j=0; j<k; j++) {
-	    u -= X->val[mdx(X, i, j)] * b->val[j];
+    for (i=0; i<m; i++) {
+	for (j=i; j<m; j++) {
+	    vij = 0.0;
+	    for (k=0; k<m; k++) {
+		if (s[k] > 0.0) {
+		    aik = A->val[mdx(A, k, i)];
+		    ajk = A->val[mdx(A, k, j)];
+		    vij += aik * ajk / (s[k] * s[k]);
+		}
+	    }
+	    vcv->val[mdx(vcv, i, j)] = vij;
+	    if (j != i) {
+		vcv->val[mdx(vcv, j, i)] = vij;
+	    }
 	}
-	sigma2 += u * u;
     }
 
-    sigma2 /= (n - k);
+    if (s2 != NULL) {
+	double sigma2 = 0.0;
+	int T = A->rows;
 
-    gretl_matrix_multiply_by_scalar(vcv, sigma2);  
+	for (i=m; i<T; i++) {
+	    sigma2 += B->val[i] * B->val[i];
+	}
+	sigma2 /= T - m;
+	gretl_matrix_multiply_by_scalar(vcv, sigma2);  
+	*s2 = sigma2;
+    }
+
+    return 0;
+}
+
+static int
+get_ols_vcv (const gretl_vector *y, const gretl_matrix *X,
+	     const gretl_vector *b, gretl_matrix *vcv,
+	     double *s2)
+{
+    if (gretl_invert_general_matrix(vcv)) {
+	gretl_matrix_print(vcv, "vcv: inversion failed", NULL);
+	return 1;
+    }
+
+    if (s2 != NULL) {
+	double u, sigma2 = 0.0;
+	int k = X->cols;
+	int n = X->rows;
+	int i, j;
+
+	for (i=0; i<n; i++) {
+	    u = y->val[i];
+	    for (j=0; j<k; j++) {
+		u -= X->val[mdx(X, i, j)] * b->val[j];
+	    }
+	    sigma2 += u * u;
+	}
+
+	sigma2 /= (n - k);
+
+	gretl_matrix_multiply_by_scalar(vcv, sigma2);  
+
+	*s2 = sigma2;
+    }
 
     return 0;
 }
@@ -1477,6 +1732,125 @@ get_ols_uhat (const gretl_vector *y, const gretl_matrix *X,
     }
 }
 
+/* OLS for data represented as gretl matrices, using Singular
+   Value decomposition: if vcv is non-NULL compute covariance
+   matrix -- but if s2 is NULL, make this just (X'X)^{-1}.
+   if uhat is non-NULL, compute residuals in uhat
+*/
+
+int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
+			  gretl_vector *b, gretl_matrix *vcv,
+			  gretl_vector *uhat, double *s2)
+{
+    gretl_vector *A = NULL;
+    gretl_matrix *B = NULL;
+
+    int T = X->rows;
+    int k = X->cols;
+
+    integer m = T;
+    integer n = k;
+    integer nrhs = 1;
+    integer lda = T;
+    integer ldb = T;
+    integer lwork = -1;
+    integer rank;
+    integer info;
+
+    double rcond = -1.0;
+    double *work = NULL;
+    double *work2 = NULL;
+    double *s = NULL;
+
+    int err = GRETL_MATRIX_OK;
+
+    if (gretl_vector_get_length(b) != k) {
+	err = GRETL_MATRIX_NON_CONFORM;
+	goto bailout;
+    }
+
+    A = gretl_matrix_copy(X);
+    if (A == NULL) {
+	err = GRETL_MATRIX_NOMEM;
+	goto bailout;
+    }
+
+    B = gretl_matrix_copy(y);
+    if (B == NULL) {
+	err = GRETL_MATRIX_NOMEM;
+	goto bailout;
+    }
+
+    /* for singular values of A */
+    s = malloc(k * sizeof *s);
+    if (s == NULL) {
+	err = GRETL_MATRIX_NOMEM; 
+	goto bailout;
+    }
+
+    work = malloc(sizeof *work);
+    if (work == NULL) {
+	err = GRETL_MATRIX_NOMEM; 
+	goto bailout;
+    } 
+
+    /* workspace query */
+    dgelss_(&m, &n, &nrhs, A->val, &lda, B->val, &ldb, s, &rcond,
+	    &rank, work, &lwork, &info);
+
+    if (info != 0 || work[0] <= 0.0) {
+	fputs(wspace_fail, stderr);
+	goto bailout;
+    }	
+
+    lwork = (integer) work[0];
+
+    work2 = realloc(work, lwork * sizeof *work);
+    if (work2 == NULL) {
+	err = GRETL_MATRIX_NOMEM; 
+	goto bailout;
+    } 
+
+    work = work2;
+
+    /* get actual solution */
+    dgelss_(&m, &n, &nrhs, A->val, &lda, B->val, &ldb, s, &rcond,
+	    &rank, work, &lwork, &info);
+
+    if (info != 0) {
+	err = GRETL_MATRIX_ERR;
+    }
+
+    if (rank < k) {
+	fprintf(stderr, "gretl_matrix_svd_ols:\n"
+		" dgelss: rank of data matrix X = %d (rows = %d, cols = %d)\n", 
+		(int) rank, X->rows, X->cols);
+    }
+
+    if (!err) {
+	int i;
+
+	for (i=0; i<k; i++) {
+	    b->val[i] = B->val[i];
+	}
+	if (vcv != NULL) {
+	    err = get_svd_ols_vcv(A, B, s, vcv, s2);
+	}
+	if (uhat != NULL) {
+	    get_ols_uhat(y, X, b, uhat);
+	}
+    }
+
+    bailout:
+
+    gretl_matrix_free(A);
+    gretl_matrix_free(B);
+    free(work);
+    free(s);
+
+    return err;
+}
+
 /**
  * gretl_matrix_ols:
  * @y: dependent variable vector.
@@ -1486,6 +1860,8 @@ get_ols_uhat (const gretl_vector *y, const gretl_matrix *X,
  * or %NULL if this is not needed.
  * @uhat: vector to hold the regression residuals, or %NULL if 
  * these are not needed.
+ * @s2: pointer to receive residual variance, or %NULL.  Note:
+ * if s2 is NULL, the vcv estimate will be plain (X'X)^{-1}.
  *
  * Computes OLS estimates using LU factorization, and puts the
  * coefficient estimates in @b.  Optionally, calculates the
@@ -1497,7 +1873,7 @@ get_ols_uhat (const gretl_vector *y, const gretl_matrix *X,
 
 int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
 		      gretl_vector *b, gretl_matrix *vcv,
-		      gretl_vector *uhat)
+		      gretl_vector *uhat, double *s2)
 {
     gretl_vector *XTy = NULL;
     gretl_matrix *XTX = NULL;
@@ -1545,7 +1921,7 @@ int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
 	    b->val[i] = XTy->val[i];
 	}
 	if (vcv != NULL) {
-	    err = get_ols_vcv(y, X, b, vcv);
+	    err = get_ols_vcv(y, X, b, vcv, s2);
 	}
 	if (uhat != NULL) {
 	    get_ols_uhat(y, X, b, uhat);
@@ -1558,6 +1934,129 @@ int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
     return err;
 }
 
+/**
+ * gretl_matrix_restricted_ols:
+ * @y: dependent variable vector.
+ * @X: matrix of independent variables.
+ * @R: left-hand restriction matrix, as in Rb = q.
+ * @q: right-hand restriction vector.
+ * @b: vector to hold coefficient estimates.
+ * @vcv: matrix to hold the covariance matrix of the coefficients,
+ * or %NULL if this is not needed.
+ * @s2: pointer ro receive residual variance, or NULL.  If vcv is non-NULL
+ * and s2 is NULL, the vcv estimate is just W^{-1}.
+ *
+ * Computes OLS estimates restricted by R and q, using LU factorization, 
+ * and puts the coefficient estimates in @b.  Optionally, calculates the
+ * covariance matrix in @vcv.
+ * 
+ * Returns: 0 on success, non-zero error code on failure.
+ * 
+ */
+
+int 
+gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
+			     const gretl_matrix *R, const gretl_vector *q,
+			     gretl_vector *b, gretl_matrix *vcv,
+			     double *s2)
+{
+    gretl_matrix *XTX = NULL;
+    gretl_vector *V = NULL;
+    gretl_matrix *W = NULL;
+    int k = X->cols;
+    int nr = R->rows;
+    int ldW = k + nr;
+    int err = GRETL_MATRIX_OK;
+    int i, j;
+
+    if (gretl_vector_get_length(b) != k) {
+	err = GRETL_MATRIX_NON_CONFORM;
+    }
+
+    if (!err) {
+	XTX = gretl_matrix_alloc(k, k);
+	V = gretl_column_vector_alloc(ldW);
+	W = gretl_matrix_alloc(ldW, ldW);
+	if (XTX == NULL || V == NULL || W == NULL) {
+	    err = GRETL_MATRIX_NOMEM;
+	}
+    }
+
+    /* construct V matrix: X'y augmented by q (or by a 0
+       matrix if q is NULL)
+    */
+
+    if (!err) {
+	V->rows = k;
+	err = gretl_matrix_multiply_mod(X, GRETL_MOD_TRANSPOSE,
+					y, GRETL_MOD_NONE,
+					V);
+	V->rows = ldW;
+    }
+
+    if (!err) {
+	for (i=k; i<ldW; i++) {
+	    if (q != NULL) {
+		V->val[i] = q->val[i-k];
+	    } else {
+		V->val[i] = 0.0;
+	    }
+	}
+    }
+
+    /* construct W matrix: X'X augmented by R and R' */
+
+    if (!err) {
+	gretl_matrix_zero(W);
+	err = gretl_matrix_multiply_mod(X, GRETL_MOD_TRANSPOSE,
+					X, GRETL_MOD_NONE,
+					XTX);
+    }
+
+    if (!err) {
+	for (i=0; i<XTX->rows; i++) {
+	    for (j=0; j<XTX->cols; j++) {
+		W->val[mdx(W,i,j)] = XTX->val[mdx(XTX,i,j)];
+	    }
+	}
+	for (i=0; i<R->rows; i++) {
+	    for (j=0; j<R->cols; j++) {
+		W->val[mdx(W,i+k,j)] = R->val[mdx(R,i,j)];
+	    }
+	}
+	for (i=0; i<R->cols; i++) {
+	    for (j=0; j<R->rows; j++) {
+		W->val[mdx(W,i,j+k)] = R->val[mdx(R,j,i)];
+	    }
+	}
+    }
+
+    if (!err && vcv != NULL) {
+	err = gretl_matrix_copy_values(vcv, W);
+    }
+
+    if (!err) {
+	err = gretl_LU_solve(W, V);
+    }
+
+    if (!err) {
+	int i;
+	
+	for (i=0; i<k; i++) {
+	    b->val[i] = V->val[i];
+	}
+	if (vcv != NULL) {
+	    err = get_ols_vcv(y, X, b, vcv, s2);
+	}
+    }
+
+    if (XTX != NULL) gretl_matrix_free(XTX);
+    if (V != NULL) gretl_vector_free(V);
+    if (W != NULL) gretl_matrix_free(W);
+
+    return err;
+}
+
 /**
  * gretl_scalar_b_prime_X_b:
  * @b: column k-vector.
@@ -1572,38 +2071,38 @@ int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
  */
 
 double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
-				 int *err)
+				 int *errp)
 {
-    gretl_matrix *tmp;
+    gretl_matrix *tmp = NULL;
     double ret = NADBL;
 
-    *err = 0;
+    *errp = 0;
 
     if (b->rows != X->rows ||
 	X->rows != X->cols ||
 	b->cols != 1) {
-	*err = GRETL_MATRIX_NON_CONFORM;
-	return ret;
+	*errp = GRETL_MATRIX_NON_CONFORM;
     }
 
-    tmp = gretl_matrix_alloc(1, b->rows);
-    if (tmp == NULL) {
-	*err = GRETL_MATRIX_NOMEM;
-	return ret;
+    if (*errp == 0) {
+	tmp = gretl_matrix_alloc(1, b->rows);
+	if (tmp == NULL) {
+	    *errp = GRETL_MATRIX_NOMEM;
+	}
     }
 
-    *err = gretl_matrix_multiply_mod(b, GRETL_MOD_TRANSPOSE,
-				     X, GRETL_MOD_NONE,
-				     tmp);
+    if (*errp == 0) {
+	*errp = gretl_matrix_multiply_mod(b, GRETL_MOD_TRANSPOSE,
+					  X, GRETL_MOD_NONE,
+					  tmp);
+    }
 
-    if (!*err) {
+    if (*errp == 0) {
 	ret = gretl_matrix_dot_product(tmp, GRETL_MOD_NONE,
 				       b, GRETL_MOD_NONE,
-				       err);
+				       errp);
     }
 
-    if (*err) ret = NADBL;
-
     gretl_matrix_free(tmp);
 
     return ret;
@@ -1824,6 +2323,100 @@ int gretl_is_zero_vector (const gretl_vector *v)
     return 1;
 }
 
+/**
+ * gretl_covariance_matrix_from_varlist:
+ * @list: list of variables by ID number.
+ * @Z: data array.
+ * @pdinfo: pointer to data information struct.
+ * @means: pointer to pick up vector of means, or %NULL to discard.
+ * @err: pointer to pass back error code.
+ *
+ * Returns: the variance-covariance matrix of the listed variables
+ * (over the currently defined data sample), or %NULL in case of
+ * failure.
+ */
+
+gretl_matrix *
+gretl_covariance_matrix_from_varlist (const int *list, const double **Z, 
+				      const DATAINFO *pdinfo, 
+				      gretl_matrix **means,
+				      int *err)
+{
+    gretl_matrix *vcv;
+    gretl_vector *xbar;
+
+    int k = list[0];
+    int t, i, j;
+
+    double vv, x, y;
+    double xbi, xbj;
+    int nv;
+
+    *err = 0;
+
+    vcv = gretl_matrix_alloc(k, k);
+    if (vcv == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    xbar = gretl_vector_alloc(k);
+    if (xbar == NULL) {
+	*err = E_ALLOC;
+	gretl_matrix_free(vcv);
+	return NULL;
+    }
+    
+    for (i=0; i<k && !*err; i++) {
+	xbi = gretl_mean(pdinfo->t1, pdinfo->t2, Z[list[i+1]]);
+	if (na(xbi)) {
+	    *err = E_DATA;
+	} else {
+	    gretl_vector_set(xbar, i, xbi);
+	}
+    }
+
+    for (i=0; i<k && !*err; i++) {
+	xbi = gretl_vector_get(xbar, i);
+	for (j=i; j<k; j++) {
+	    xbj = gretl_vector_get(xbar, j);
+	    vv = 0.0;
+	    nv = 0;
+	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		x = Z[list[i+1]][t];
+		y = Z[list[j+1]][t];
+		if (na(x) || na(y)) {
+		    continue;
+		}
+		vv += (x - xbi) * (y - xbj);
+		nv++;
+	    }
+	    if (nv < 2) {
+		*err = E_DATA;
+		vv = NADBL;
+	    } else {
+		vv /= (nv - 1); /* plain nv? */
+	    }
+	    gretl_matrix_set(vcv, i, j, vv);
+	    gretl_matrix_set(vcv, j, i, vv);
+	}
+    }
+
+    if (means != NULL && !*err) {
+	*means = xbar;
+    } else {
+	gretl_vector_free(xbar);
+    }
+
+    if (*err) {
+	gretl_matrix_free(vcv);
+	vcv = NULL;
+    }
+
+    return vcv;
+}
+
+
 #if 0
 
 /* Some of the following may be useful later. */
