@@ -53,6 +53,8 @@ gretl_matrix *gretl_packed_matrix_alloc (int rows);
 
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
+gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m);
+
 gretl_vector *gretl_column_vector_from_array (const double *x, 
 					      int n, int mod);
 
@@ -106,7 +108,13 @@ double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
 
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
-double gretl_LU_determinant (gretl_matrix *a);
+double gretl_matrix_determinant (gretl_matrix *a);
+
+double gretl_matrix_log_determinant (gretl_matrix *a);
+
+double gretl_matrix_log_abs_determinant (gretl_matrix *a);
+
+double gretl_vcv_log_determinant (gretl_matrix *a);
 
 int gretl_LU_solve (gretl_matrix *a, gretl_vector *b);
 
@@ -114,6 +122,8 @@ int gretl_invert_general_matrix (gretl_matrix *a);
 
 int gretl_invert_symmetric_matrix (gretl_matrix *a);
 
+int gretl_invert_diagonal_matrix (gretl_matrix *a);
+
 double *gretl_general_matrix_eigenvals (gretl_matrix *m, gretl_matrix *ev);
 
 double *gretl_symmetric_matrix_eigenvals (gretl_matrix *m,
@@ -123,7 +133,17 @@ int gretl_matrix_cholesky_decomp (gretl_matrix *a);
 
 int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
 		      gretl_vector *b, gretl_matrix *vcv,
-		      gretl_vector *uhat);
+		      gretl_vector *uhat, double *s2);
+
+int 
+gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
+			     const gretl_matrix *R, const gretl_vector *q,
+			     gretl_vector *b, gretl_matrix *vcv,
+			     double *s2);
+
+int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
+			  gretl_vector *b, gretl_matrix *vcv,
+			  gretl_vector *uhat, double *s2);
 
 double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
 				 int *err);
@@ -154,4 +174,9 @@ int gretl_is_identity_matrix (const gretl_matrix *m);
 
 int gretl_is_zero_vector (const gretl_vector *v);
 
+gretl_matrix *
+gretl_covariance_matrix_from_varlist (const int *list, const double **Z, 
+				      const DATAINFO *pdinfo, 
+				      gretl_matrix **means, int *err);
+
 #endif /* GRETL_MATRIX_H */
