@@ -161,26 +161,44 @@ int gretl_model_set_double (MODEL *pmod, const char *key, double val)
 }
 
 /**
- * gretl_model_get_data:
+ * gretl_model_get_data_and_size:
  * @pmod: pointer to #MODEL.
  * @key: key string.
+ * @sz: pointer to receive the size of the data
  *
  * Returns the data pointer identified by @key, or %NULL on failure.
  */
 
-void *gretl_model_get_data (const MODEL *pmod, const char *key)
+void *gretl_model_get_data_and_size (const MODEL *pmod, const char *key,
+				     size_t *sz)
 {
     int i;
 
     for (i=0; i<pmod->n_data_items; i++) {
-	if (!strcmp(key, (pmod->data_items[i])->key)) {
-	    return (pmod->data_items[i])->ptr;
+	if (!strcmp(key, pmod->data_items[i]->key)) {
+	    if (sz != NULL) {
+		*sz = pmod->data_items[i]->size;
+	    }
+	    return pmod->data_items[i]->ptr;
 	}
     }
 
     return NULL;
 }
 
+/**
+ * gretl_model_get_data:
+ * @pmod: pointer to #MODEL.
+ * @key: key string.
+ *
+ * Returns: the data pointer identified by @key, or %NULL on failure.
+ */
+
+void *gretl_model_get_data (const MODEL *pmod, const char *key)
+{
+    return gretl_model_get_data_and_size(pmod, key, NULL);
+}
+
 /**
  * gretl_model_get_int:
  * @pmod: pointer to #MODEL.
@@ -262,6 +280,44 @@ static void destroy_all_data_items (MODEL *pmod)
     pmod->data_items = NULL;
 }
 
+int gretl_model_destroy_data_item (MODEL *pmod, const char *key)
+{
+    model_data_item *junk = NULL;
+    int i, targ = 0;
+    int err = 0;
+
+    for (i=0; i<pmod->n_data_items; i++) {
+	if (!strcmp(key, pmod->data_items[i]->key)) {
+	    junk = pmod->data_items[i];
+	    targ = i;
+	    break;
+	}
+    }
+
+    if (junk == NULL) {
+	err = 1;
+    } else {
+	model_data_item **items;
+	int n_items = pmod->n_data_items - 1;
+
+	for (i=targ; i<n_items; i++) {
+	    pmod->data_items[i] = pmod->data_items[i+1];
+	}
+
+	items = realloc(pmod->data_items, n_items * sizeof *items);
+	if (items != NULL) {
+	    pmod->data_items = items;
+	}
+
+	pmod->n_data_items -= 1;
+
+	free(junk->key);
+	free(junk);
+    }
+
+    return err;
+}
+
 /* .......................................................... */
 
 void gretl_model_set_auxiliary (MODEL *pmod, int aux)
@@ -425,6 +481,13 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
 
 #endif /* MODEL_DEBUG */
 
+static void gretl_test_free (GRETLTEST *test)
+{
+    if (test->param != NULL) {
+	free(test->param);
+    }
+}
+
 /**
  * clear_model:
  * @pmod: pointer to #MODEL.
@@ -437,6 +500,8 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
 void clear_model (MODEL *pmod)
 {
     if (pmod != NULL) {
+	int i;
+
 #ifdef MODEL_DEBUG
 	debug_print_model_info(pmod, "Doing clear_model");
 #endif
@@ -453,10 +518,13 @@ void clear_model (MODEL *pmod)
 	if (pmod->arinfo) {
 	    clear_ar_info(pmod);
 	}
-	if (pmod->ntests) free(pmod->tests);
+	if (pmod->ntests) {
+	    for (i=0; i<pmod->ntests; i++) {
+		gretl_test_free(&pmod->tests[i]);
+	    }
+	    free(pmod->tests);
+	}
 	if (pmod->params) {
-	    int i;
-
 	    for (i=0; i<pmod->nparams; i++) {
 		free(pmod->params[i]);
 	    }
@@ -476,9 +544,8 @@ void clear_model (MODEL *pmod)
 
 static void copy_test (GRETLTEST *targ, const GRETLTEST *src)
 {
-    strcpy(targ->type, src->type);
-    strcpy(targ->h_0, src->h_0);
-    strcpy(targ->param, src->param);
+    targ->type = src->type;
+    targ->param = gretl_strdup(src->param);
     targ->teststat = src->teststat;
     targ->dfn = src->dfn;
     targ->dfd = src->dfd;
@@ -502,6 +569,305 @@ static int copy_model_tests (MODEL *targ, const MODEL *src)
     return 0;
 }
 
+void gretl_test_init (GRETLTEST *test, int which)
+{
+    test->type = which;
+    test->order = 0;
+    test->param = NULL;
+    test->teststat = 0;
+    test->dfn = test->dfd = 0;
+    test->value = test->pvalue = NADBL;
+}
+
+int add_test_to_model (MODEL *pmod, GRETLTEST *test)
+{
+    GRETLTEST *tests;
+    int i, nt = pmod->ntests;
+
+    for (i=0; i<nt; i++) {
+	if (test->type == pmod->tests[i].type) {
+	    /* already done */
+	    return -1;
+	}
+    }
+
+    tests = realloc(pmod->tests, (nt + 1) * sizeof *tests);
+    if (tests == NULL) {
+	return 1;
+    }
+
+    pmod->tests = tests;
+
+    pmod->tests[nt].type = test->type;
+
+    pmod->tests[nt].param = test->param;
+    test->param = NULL;
+
+    pmod->tests[nt].teststat = test->teststat;
+    pmod->tests[nt].value = test->value;
+    pmod->tests[nt].dfn = test->dfn;
+    pmod->tests[nt].dfd = test->dfd;
+    pmod->tests[nt].pvalue = test->pvalue;
+
+    pmod->ntests += 1;
+
+    return 0;
+}
+
+static int gretl_test_print_string (GRETLTEST *test, PRN *prn)
+{
+    const char *test_strs[] = {
+	N_("Test for addition of variables"),
+	N_("Test for ARCH of order %s"),
+	N_("LM test for autocorrelation up to order %s"),
+	N_("Chow test for structural break at observation %s"),
+	N_("CUSUM test for parameter stability"),
+	N_("Likelihood ratio test for groupwise heteroskedasticity"),
+	N_("Non-linearity test (logs)"),
+	N_("Test for normality of residual"),
+	N_("Test for omission of variables"),
+	N_("RESET test for specification"),
+	N_("Non-linearity test (squares)"),
+	N_("White's test for heteroskedasticity")
+    };
+    char ord[16];
+    char *param = NULL;
+
+    if (test->type >= GRETL_TEST_MAX) {
+	return 1;
+    }
+
+    if (test->order > 0) {
+	sprintf(ord, "%d", test->order);
+	param = ord;
+    } else if (test->type == GRETL_TEST_CHOW) {
+	param = test->param;
+    }
+
+    if (param != NULL) {
+	if (plain_format(prn)) {
+	    pprintf(prn, _(test_strs[test->type]), param);
+	} else {
+	    pprintf(prn, I_(test_strs[test->type]), param);
+	}
+    } else {
+	if (plain_format(prn)) {
+	    pputs(prn, _(test_strs[test->type]));
+	} else {
+	    pputs(prn, I_(test_strs[test->type]));
+	}
+    }
+
+    return 0;
+}
+
+static int print_add_omit_varnames (const char *s, PRN *prn)
+{
+    const char *endings[] = {
+	"\n    ",
+	"\\\\\n \\qquad ",
+	"\\par\n    "
+    };
+    const char *ending = NULL;
+
+    if (s == NULL || *s == '\0') {
+	return 1;
+    }
+
+    if (plain_format(prn)) {
+	ending = endings[0];
+    } else if (tex_format(prn)) {
+	ending = endings[1];
+    } else if (rtf_format(prn)) {
+	ending = endings[2];
+    } else {
+	return 1;
+    }
+
+    pputs(prn, ending);
+
+    while (*s) {
+	if (*s == ' ') {
+	    pputs(prn, ending);
+	} else {
+	    pputc(prn, *s);
+	}
+	s++;
+    }
+
+    return 0;
+}
+
+static int gretl_test_print_h_0 (GRETLTEST *test, PRN *prn)
+{
+    const char *h_0_strs[] = {
+	N_("parameters are zero for the variables"),
+	N_("no ARCH effect is present"),
+	N_("no autocorrelation"),
+	N_("no structural break"),
+	N_("no change in parameters"),
+	N_("the units have a common error variance"),
+	N_("relationship is linear"),
+	N_("error is normally distributed"),
+	N_("parameters are zero for the variables"),
+	N_("specification is adequate"),
+	N_("relationship is linear"),
+	N_("heteroskedasticity not present")
+    };
+
+    if (test->type >= GRETL_TEST_MAX) {
+	return 1;
+    }  
+
+    if (plain_format(prn)) {
+	pputs(prn, _(h_0_strs[test->type]));
+    } else {
+	pputs(prn, I_(h_0_strs[test->type]));
+    }  
+
+    if (test->type == GRETL_TEST_ADD || test->type == GRETL_TEST_OMIT) {
+	print_add_omit_varnames(test->param, prn);
+    } 
+
+    return 0;
+}
+
+static void 
+get_test_stat_string (const GRETLTEST *test, char *str, int format)
+{
+    int tex = is_tex(format);
+
+    switch (test->teststat) {
+    case GRETL_STAT_TR2:
+	if (tex) {
+	    sprintf(str, "$TR^2$ = %g", test->value);
+	} else if (is_rtf(format)) {
+	    sprintf(str, "TR{\\super 2} = %g", test->value);
+	} else {
+	    sprintf(str, "TR^2 = %g", test->value);
+	}
+	break;
+    case GRETL_STAT_F:
+    case GRETL_STAT_RESET:
+	if (tex) {
+	    sprintf(str, "$F(%d, %d)$ = %g", test->dfn, test->dfd, test->value);
+	} else {
+	    sprintf(str, "F(%d, %d) = %g", test->dfn, test->dfd, test->value);
+	}
+	break;
+    case GRETL_STAT_LMF:
+	sprintf(str, "LMF = %g", test->value);
+	break;
+    case GRETL_STAT_HARVEY_COLLIER:
+	if (tex) {
+	    sprintf(str, "Harvey--Collier $t(%d)$ = %g", test->dfn, test->value);
+	} else {
+	    sprintf(str, "Harvey-Collier t(%d) = %g", test->dfn, test->value);
+	}
+	break;
+    case GRETL_STAT_NORMAL_CHISQ:
+	if (tex) {
+	    sprintf(str, "$\\chi^2_2$ = %g", test->value); 
+	} else {
+	    sprintf(str, "%s(2) = %g", _("Chi-square"), test->value);
+	}
+	break;
+    case GRETL_STAT_LR:
+	if (tex) {
+	    sprintf(str, "$\\chi^2_%d$ = %g", test->dfn, test->value); 
+	} else {
+	    sprintf(str, "%s(%d) = %g", _("Chi-square"), test->dfn, test->value);
+	}
+	break;
+    default:
+	*str = 0;
+    }
+}
+
+static void 
+get_test_pval_string (const GRETLTEST *test, char *str, int format)
+{
+    int tex = is_tex(format);
+
+    switch (test->teststat) {
+    case GRETL_STAT_TR2:
+	if (tex) sprintf(str, "$P$($\\chi^2_{%d} >$ %g) = %g", 
+			 test->dfn, test->value, test->pvalue);
+	else sprintf(str, "P(Chi-Square(%d) > %g) = %g", 
+		     test->dfn, test->value, test->pvalue);
+	break;
+    case GRETL_STAT_F:
+    case GRETL_STAT_RESET:
+	if (tex) {
+	    sprintf(str, "$P$($F(%d, %d) >$ %g) = %g", 
+		    test->dfn, test->dfd, test->value, test->pvalue);
+	} else {
+	    sprintf(str, "P(F(%d, %d) > %g) = %g", 
+		    test->dfn, test->dfd, test->value, test->pvalue);
+	}
+	break;
+    case GRETL_STAT_LMF:
+	if (tex) {
+	    sprintf(str, "$P$($F(%d, %d) >$ %g) = %g", 
+		    test->dfn, test->dfd, test->value, test->pvalue);
+	} else {
+	    sprintf(str, "P(F(%d,%d) > %g) = %g", 
+		    test->dfn, test->dfd, test->value, test->pvalue);
+	}
+	break;
+    case GRETL_STAT_HARVEY_COLLIER:
+	if (tex) {
+	    sprintf(str, "$P$($t_{%d} >$ %g)  = %g", 
+		    test->dfn, test->value, test->pvalue);
+	} else {
+	    sprintf(str, "P(t(%d) > %g) = %g", 
+		    test->dfn, test->value, test->pvalue);
+	}
+	break;
+    case GRETL_STAT_NORMAL_CHISQ:
+    case GRETL_STAT_LR:
+	sprintf(str, "%g", test->pvalue);
+	break;
+    default:
+	*str = 0;
+    }
+}
+
+void gretl_model_test_print (GRETLTEST *test, PRN *prn)
+{
+    char test_str[128], pval_str[128];
+
+    get_test_stat_string(test, test_str, prn->format);
+    get_test_pval_string(test, pval_str, prn->format);
+
+    if (plain_format(prn)) {
+	gretl_test_print_string(test, prn);
+	pprintf(prn, " -\n  %s: ", _("Null hypothesis"));
+	gretl_test_print_h_0(test, prn);
+	pprintf(prn, "\n  %s: %s\n"
+		"  %s = %s\n\n",
+		_("Test statistic"), test_str, 
+		_("with p-value"), pval_str);
+    } else if (tex_format(prn)) {
+	gretl_test_print_string(test, prn);
+	pprintf(prn, " --\\\\\n\\quad %s: ", I_("Null hypothesis"));
+	gretl_test_print_h_0(test, prn);
+	pprintf(prn, "\\\\\n\\quad %s: %s\\\\\n"
+		"\\quad %s = %s\\\\\n",
+		I_("Test statistic"), test_str, 
+		I_("with p-value"), pval_str);
+    } else if (rtf_format(prn)) {
+	pputs(prn, "\\par \\ql ");
+	gretl_test_print_string(test, prn);
+	pprintf(prn, " -\\par\n %s: ", I_("Null hypothesis"));
+	gretl_test_print_h_0(test, prn);
+	pprintf(prn, "\\par\n %s: %s\\par\n"
+		" %s = %s\\par\n\n",
+		I_("Test statistic"), test_str, 
+		I_("with p-value"), pval_str);
+    }
+}
+
 static ARINFO *copy_ar_info (const ARINFO *src)
 {
     ARINFO *targ;
@@ -770,10 +1136,13 @@ int command_ok_for_model (int test_ci, int model_ci)
 	if (model_ci != OLS) ok = 0;
 	break;
 
+    case LMTEST:
+	if (model_ci != OLS && model_ci != POOLED) ok = 0;
+	break;
+
     case ARCH:
     case CHOW:
     case CUSUM:
-    case LMTEST:
     case LEVERAGE:
     case RESET:
 	if (model_ci != OLS) ok = 0;
@@ -876,5 +1245,36 @@ int highest_numbered_var_in_model (const MODEL *pmod,
     return vmax;
 }
 
+int mle_aic_bic (MODEL *pmod, int addk)
+{
+    int err = 0;
+
+    if (na(pmod->lnL)) {
+	pmod->criterion[C_AIC] = NADBL;
+	pmod->criterion[C_BIC] = NADBL;
+	err = 1;
+    } else {
+	int k = pmod->ncoeff + addk;
+
+	pmod->criterion[C_AIC] = -2.0 * pmod->lnL + 2.0 * k;
+	pmod->criterion[C_BIC] = -2.0 * pmod->lnL + k * log(pmod->nobs);
+    }
+
+    return err;
+}
+
+/* As of 2005-02-19, not ready for this yet: need to adjust
+   the text in the model printout.
+*/
+
+double coeff_pval (const MODEL *pmod, double x, int df)
+{
+    if (0 && ML_ESTIMATOR(pmod->ci)) {
+	return gaussprob(x);
+    } else {
+	return tprob(x, df);
+    }
+}
+
 
 
