@@ -34,6 +34,26 @@
 
 #define AR_MODEL(c) (c == AR || c == CORC || c == PWE || c == HILU)
 
+#define ML_ESTIMATOR(c) (c ==  ARMA || c == GARCH || c == POISSON || \
+                         c == LOGIT || c == PROBIT || c == TOBIT)
+
+#define LIMDEP(c) (c == LOGIT || c == PROBIT || c == TOBIT)
+
+enum gretl_test_types {
+    GRETL_TEST_ADD,
+    GRETL_TEST_ARCH,
+    GRETL_TEST_AUTOCORR,
+    GRETL_TEST_CHOW,
+    GRETL_TEST_CUSUM,
+    GRETL_TEST_GROUPWISE,
+    GRETL_TEST_LOGS,
+    GRETL_TEST_NORMAL,
+    GRETL_TEST_OMIT,
+    GRETL_TEST_RESET,
+    GRETL_TEST_SQUARES,
+    GRETL_TEST_WHITES,
+    GRETL_TEST_MAX
+};
 
 MODEL *gretl_model_new (void);
 
@@ -49,12 +69,17 @@ void clear_model (MODEL *pmod);
 
 int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size);
 
+int gretl_model_destroy_data_item (MODEL *pmod, const char *key);
+
 int gretl_model_set_int (MODEL *pmod, const char *key, int val);
 
 int gretl_model_set_double (MODEL *pmod, const char *key, double val);
 
 void *gretl_model_get_data (const MODEL *pmod, const char *key);
 
+void *gretl_model_get_data_and_size (const MODEL *pmod, const char *key,
+				     size_t *sz);
+
 int gretl_model_get_int (const MODEL *pmod, const char *key);
 
 double gretl_model_get_double (const MODEL *pmod, const char *key);
@@ -77,9 +102,19 @@ void model_count_minus (void);
 
 void set_model_id (MODEL *pmod);
 
+void gretl_test_init (GRETLTEST *test, int which);
+
+int add_test_to_model (MODEL *pmod, GRETLTEST *test);
+
+void gretl_model_test_print (GRETLTEST *test, PRN *prn);
+
 void model_list_to_string (int *list, char *buf);
 
 int highest_numbered_var_in_model (const MODEL *pmod, 
 				   const DATAINFO *pdinfo);
 
+int mle_aic_bic (MODEL *pmod, int addk);
+
+double coeff_pval (const MODEL *pmod, double x, int df);
+
 #endif /* GRETL_MODEL_H */
