@@ -20,6 +20,8 @@
 /* gretl_paths.c for gretl  */
 
 #include "libgretl.h"
+#include "libset.h"
+
 #include <unistd.h>
 
 #ifdef WIN32
@@ -35,6 +37,12 @@
 # endif /* GLIB_CHECK_VERSION */
 #endif /* ! WIN32 */
 
+#if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
+# ifdef WIN32
+#  define USE_G_FOPEN
+# endif
+#endif
+
 enum {
     CURRENT_DIR,
     DATA_SEARCH,
@@ -60,6 +68,33 @@ static int add_gdt_suffix (char *fname)
 
 /* .......................................................... */
 
+FILE *gretl_fopen (const char *filename, const char *mode)
+{
+#ifdef USE_G_FOPEN
+    return g_fopen((const gchar *) filename, (const gchar *) mode);
+#else
+    return fopen(filename, mode);
+#endif
+}
+
+gzFile gretl_gzopen (const char *filename, const char *mode)
+{
+#ifdef USE_G_FOPEN
+    gchar *cp_filename = g_locale_from_utf8(filename, -1, NULL, NULL, NULL);
+    gzFile fz = gzopen(cp_filename, mode);
+    int save_errno = errno;
+
+    g_free(cp_filename);
+
+    errno = save_errno;
+    return fz;
+#else
+    return gzopen(filename, mode);
+#endif
+}
+
+/* .......................................................... */
+
 int path_append (char *file, const char *path)
 {
     char temp[MAXLEN];
@@ -97,10 +132,10 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, "\\");
     strcat(tmp, targ);
 
-    fp = fopen(tmp, "r");
+    fp = gretl_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
 	if (add_gdt_suffix(tmp)) {
-	    fp = fopen(tmp, "r");
+	    fp = gretl_fopen(tmp, "r");
 	}
     }
 
@@ -180,10 +215,10 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, "/");
     strcat(tmp, targ);
 
-    fp = fopen(tmp, "r");
+    fp = gretl_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
 	if (add_gdt_suffix(tmp)) {
-	    fp = fopen(tmp, "r");
+	    fp = gretl_fopen(tmp, "r");
 	}
     }
 
@@ -262,13 +297,13 @@ static char *search_dir (char *fname, const char *topdir, int code)
     strcpy(orig, fname);
 
     if (path_append(fname, topdir) == 0) {
-	test = fopen(fname, "r");
+	test = gretl_fopen(fname, "r");
 	if (test != NULL) {
 	    fclose(test);
 	    return fname;
 	}
 	if (code == DATA_SEARCH && add_gdt_suffix(fname)) {
-	    test = fopen(fname, "r");
+	    test = gretl_fopen(fname, "r");
 	    if (test != NULL) {
 		fclose(test);
 		return fname;
@@ -345,7 +380,7 @@ char *addpath (char *fname, PATHS *ppaths, int script)
     strcpy(orig, fname);
 
     /* try opening filename as given */
-    test = fopen(fname, "r");
+    test = gretl_fopen(fname, "r");
     if (test != NULL) { 
 	fclose(test); 
 	if (!path_is_absolute(fname)) {
@@ -498,8 +533,7 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
 /* .......................................................... */
 
 enum paths_status_flags {
-    GRETL_USING_GUI      = 1 << 0,
-    STRING_TABLE_WRITTEN = 1 << 1
+    STRING_TABLE_WRITTEN = 1 << 0
 };
 
 struct INTERNAL_PATHS {
@@ -609,11 +643,6 @@ int gretl_string_table_written (void)
     return ret;
 }
 
-int gretl_using_gui (void)
-{
-    return (gretl_paths.status & GRETL_USING_GUI);
-}
-
 static void ensure_slash (char *str)
 {
     if (str[strlen(str) - 1] != SLASH) {
@@ -679,21 +708,17 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 
     if (opt & OPT_X) {
 	/* gui program */
+	gretl_set_gui_mode(1);
 	if (opt & OPT_N) {
 	    /* force english */
 	    sprintf(ppaths->helpfile, "%sgretl_hlp.txt", ppaths->gretldir);
 	    sprintf(ppaths->cmd_helpfile, "%sgretlcli_hlp.txt", ppaths->gretldir);
 	} else {
-	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir,
-		    _("gretl_hlp.txt"));
-	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir,
-		_("gretlcli_hlp.txt"));
+	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretl_hlp.txt"));
+	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
 	}
-	gretl_paths.status |= GRETL_USING_GUI;
     } else { 
-	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir,
-		_("gretlcli_hlp.txt"));
-	gretl_paths.status = 0;
+	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
     }
 
     sprintf(envstr, "GTKSOURCEVIEW_LANGUAGE_DIR=%sshare\\gtksourceview-1.0"
@@ -762,21 +787,17 @@ int set_paths (PATHS *ppaths, gretlopt opt)
     sprintf(ppaths->scriptdir, "%sscripts/", ppaths->gretldir);
 
     if (opt & OPT_X) {
+	gretl_set_gui_mode(1);
 	if (opt & OPT_N) {
 	    /* force english */
 	    sprintf(ppaths->helpfile, "%sgretl.hlp", ppaths->gretldir);
 	    sprintf(ppaths->cmd_helpfile, "%sgretlcli.hlp", ppaths->gretldir);
 	} else {
-	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir,
-		    _("gretl.hlp"));
-	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir,
-		    _("gretlcli.hlp"));
+	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretl.hlp"));
+	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
 	}
-	gretl_paths.status |= GRETL_USING_GUI;
     } else {
-	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir,
-		_("gretlcli.hlp"));
-	gretl_paths.status = 0;
+	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
     }
 
     ensure_slash(ppaths->userdir);
