@@ -26,6 +26,10 @@ int gretl_string_table_written (void);
 
 int gretl_using_gui (void);
 
+FILE *gretl_fopen (const char *filename, const char *mode);
+
+gzFile gretl_gzopen (const char *filename, const char *mode);
+
 char *addpath (char *fname, PATHS *ppaths, int script);
 
 int getopenfile (const char *line, char *fname, PATHS *ppaths,
