@@ -51,8 +51,6 @@ int gretl_iszero (int t1, int t2, const double *x);
 
 int gretl_isconst (int t1, int t2, const double *x);
 
-double gretl_mean (int t1, int t2, const double *x);
-
 void gretl_minmax (int t1, int t2, const double *x, 
 		   double *min, double *max);
 
@@ -60,13 +58,16 @@ int gretl_hasconst (const int *list);
 
 int gretl_compare_doubles (const void *a, const void *b);
 
-void gretl_aic_etc (MODEL *pmod);
+int gretl_criteria (double ess, int nobs, int ncoeff, PRN *prn);
 
-void gretl_criteria (double ess, int nobs, int ncoeff, PRN *prn);
+int calculate_criteria (double *x, double ess, int nobs, int ncoeff);
 
 int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2, 
 		 const double **Z, int *misst);
 
+int get_t_from_obs_string (char *s, const double **Z, 
+			   const DATAINFO *pdinfo);
+
 int undo_daily_repack (MODEL *pmod, double **Z, 
 		       const DATAINFO *pdinfo);
 
@@ -88,12 +89,9 @@ int gretl_forecast (int t1, int t2, int nv,
 
 int gretl_is_reserved (const char *str);
 
-void gretl_test_init (GRETLTEST *test);
-
-int vars_identical (const double *x, const double *y, int n);
-
 int real_list_laggenr (const int *list, double ***pZ, 
-		       DATAINFO *pdinfo, int maxlag);
+		       DATAINFO *pdinfo, int maxlag, 
+		       int **lagnums);
 
 int lagvarnum (int v, int l, const DATAINFO *pdinfo);
 
@@ -125,9 +123,13 @@ char *get_month_name (char *mname, int m);
 
 int gretl_function_stack_depth (void);
 
-/* init and cleanup functions */
+void record_test_result (double teststat, double pval, char *blurb);
 
-int gretl_cmd_init (CMD *cmd);
+double get_last_test_statistic (char *blurb);
+
+double get_last_pvalue (char *blurb);
+
+/* init and cleanup functions */
 
 void gretl_cmd_free (CMD *cmd);
 
