@@ -17,15 +17,23 @@
  *
  */
 
+#ifndef GRETL_RESTRICT_H
+#define GRETL_RESTRICT_H
+
 typedef struct restriction_set_ gretl_restriction_set;
 
 gretl_restriction_set *
 restriction_set_start (const char *line, MODEL *pmod, const DATAINFO *pdinfo);
 
+gretl_restriction_set *
+cross_restriction_set_start (const char *line, gretl_equation_system *sys);
+
 int 
 restriction_set_parse_line (gretl_restriction_set *rset, const char *line);
 
 int
 gretl_restriction_set_finalize (gretl_restriction_set *rset, PRN *prn);
 
+#endif
+
 
