@@ -21,6 +21,7 @@
 
 #include "libgretl.h"
 #include "gretl_private.h"
+#include "system.h"
 
 #include <errno.h>
 
@@ -292,12 +293,11 @@ int isdummy (const double *x, int t1, int t2)
 int gretl_iszero (int t1, int t2, const double *x)
 /*  checks whether all obs are zero for variable x from t1 to t2 */
 {
+    double sum = 0.0;
     int t;
-    double xx, sum = 0.0;
 
     for (t=t1; t<=t2; t++) {
-        xx = x[t];
-        sum = sum + xx * xx;
+        sum += x[t] * x[t];
     }
 
     return floateq(sum, 0.0);
@@ -412,8 +412,11 @@ int gretl_hasconst (const int *list)
 {
     int i;
 
-    for (i=2; i<=list[0]; i++) 
-        if (list[i] == 0) return i;
+    for (i=2; i<=list[0]; i++) {
+        if (list[i] == 0) {
+	    return 1;
+	}
+    }
 
     return 0;
 }
@@ -518,8 +521,12 @@ void printlist (const int *list, const char *msg)
 	fprintf(stderr, "list: ");
     }
 
-    for (i=0; i<=list[0]; i++) {
-	fprintf(stderr, "%d ", list[i]);
+    if (list == NULL) {
+	fputs( "list is NULL", stderr);
+    } else {
+	for (i=0; i<=list[0]; i++) {
+	    fprintf(stderr, "%d ", list[i]);
+	}
     }
 
     fputc('\n', stderr);
@@ -560,63 +567,58 @@ int print_list_to_buffer (const int *list, char *buf, size_t len)
 
 /* ....................................................... */
 
-static void calculate_criteria (double *criterion, double ess, 
-				int nobs, int ncoeff)
+int calculate_criteria (double *x, double ess, int nobs, int ncoeff)
 {
-    double sig2 = ess / nobs;
-    double nx = nobs;
-    double dfx = nobs - ncoeff;
-    double cobs = (double) ncoeff / nobs;
-    double cobs1 = 1.0 - cobs;
-    double cobs2 = 2.0 * cobs;
+    if (na(ess) || ess <= 0.0 || ncoeff < 1 || nobs <= ncoeff) {
+	x[C_AIC] = NADBL;
+	x[C_BIC] = NADBL;
 
-    criterion[C_SGMASQ] = ess / dfx;
+	return 1;
+    } else {
+	const double ln2pi1 = 2.837877066409345;
+	double ll;
 
-    criterion[C_FPE] = sig2 * (nobs + ncoeff) / dfx;
+	errno = 0;
+	ll = -.5 * nobs * log(ess);
 
-    criterion[C_AIC] = sig2 * exp(cobs2);
+	if (errno == EDOM || errno == ERANGE) {
+	    x[C_AIC] = NADBL;
+	    x[C_BIC] = NADBL;
+	} else {
+	    ll += -.5 * nobs * (ln2pi1 - log((double) nobs));
+	    x[C_AIC] = -2.0 * ll + 2 * ncoeff;
+	    x[C_BIC] = -2.0 * ll + ncoeff * log(nobs);
+	}
 
-    criterion[C_SHIBATA] = sig2 * (1.0 + cobs2);
-
-    criterion[C_RICE] = (cobs2 < 1.0)? (sig2 / (1.0 - cobs2)) : NADBL;
-
-    criterion[C_HQ] = sig2 * pow(log(nx), cobs2);
-
-    criterion[C_BIC] = sig2 * pow((double) nobs, cobs);
-
-    criterion[C_GCV] = sig2 / (cobs1 * cobs1);
+	return 0;
+    }
 }
 
 /* Compute model selection criteria */
 
-void gretl_aic_etc (MODEL *pmod)
+int ls_aic_bic (MODEL *pmod)
 {
-    calculate_criteria(pmod->criterion, pmod->ess, pmod->nobs,
-		       pmod->ncoeff);
+    return calculate_criteria(pmod->criterion, 
+			      pmod->ess, pmod->nobs,
+			      pmod->ncoeff);
 }
 
-void gretl_criteria (const double ess, int nobs, int ncoeff, 
-		     PRN *prn)
+int gretl_criteria (double ess, int nobs, int ncoeff, PRN *prn)
 {
-    double criterion[8];
+    double x[2];
+    int err;
 
-    calculate_criteria(criterion, ess, nobs, ncoeff);
-    
-    pprintf(prn, _("Using ess = %f, %d observations, %d coefficients\n"), 
-	    ess, nobs, ncoeff);
-    pputs(prn, _("\nMODEL SELECTION STATISTICS\n\n"));	
-    pprintf(prn, "SGMASQ    %13g     AIC       %13g     FPE       %12g\n"
-	    "HQ        %13g     SCHWARZ   %13g     SHIBATA   %12g\n"
-	    "GCV       %13g",
-	    criterion[C_SGMASQ], criterion[C_AIC], 
-	    criterion[C_FPE], criterion[C_HQ], 
-	    criterion[C_BIC], criterion[C_SHIBATA], criterion[C_GCV]);
-    if (!na(criterion[C_RICE])) {
-	pprintf(prn, "     RICE      %13g\n", criterion[C_RICE]);
+    err = calculate_criteria(x, ess, nobs, ncoeff);
+
+    if (err) {
+	pputs(prn, _("Error calculating model selection criteria\n"));
     } else {
-	pputs(prn, "     RICE            undefined\n");
+	pprintf(prn, _("Using ess = %g, %d observations, %d coefficients\n"), 
+		ess, nobs, ncoeff);
+	pprintf(prn, "\nAIC = %g\nBIC = %g\n\n", x[C_AIC], x[C_BIC]);
     }
-    pputc(prn, '\n');
+
+    return err;
 }
 
 char *real_format_obs (char *obs, int maj, int min, int pd, char sep)
@@ -640,22 +642,68 @@ char *format_obs (char *obs, int maj, int min, int pd)
     return real_format_obs(obs, maj, min, pd, ':');
 }
 
+static int get_stobs_maj_min (char *stobs, int *maj, int *min)
+{
+    int dotc = 0;
+    char *p = stobs;
+    int err = 0;
+
+    while (*p) {
+	if (*p == ':') {
+	    *p = '.';
+	    dotc++;
+	} else if (*p == '.') {
+	    dotc++;
+	} else if (!isdigit((unsigned char) *p)) {
+	    err = 1;
+	    break;
+	}
+	p++;
+    }
+
+    if (!err) {
+	if (dotc > 1 || *stobs == '.' || 
+	    stobs[strlen(stobs) - 1] == '.') {
+	    err = 1;
+	}
+    }
+
+    if (!err) {
+	if (dotc > 0) {
+	    sscanf(stobs, "%d.%d", maj, min);
+	    if (*maj <= 0 || *min <= 0) {
+		err = 1;
+	    }
+	} else {
+	    sscanf(stobs, "%d", maj);
+	    if (*maj <= 0) {
+		err = 1;
+	    }
+	}
+    }
+
+    return err;
+}
+
+#define recognized_ts_frequency(f) (f == 4 || f == 12 || f == 24)
+
 /**
  * set_obs:
  * @line: command line.
  * @pdinfo: data information struct.
- * @opt: OPT_S for stacked time-series, OPT_C for stacked cross-section.
+ * @opt: OPT_S for stacked time-series, OPT_C for stacked cross-section,
+ * OPT_T for time series, OPT_X for cross section.
  * 
- * Impose a time-series or panel interpretation on a data set.
+ * Set the frequency and initial observation for a dataset.
  *
  * Returns: 0 on successful completion, 1 on error.
  */
 
 int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
 {
-    int pd, i, len, bad = 0;
-    char stobs[OBSLEN], endobs[OBSLEN], endbit[7], *p;
-    long ed0 = 0L;
+    char stobs[OBSLEN];
+    int structure = STRUCTURE_UNKNOWN;
+    int pd, dated = 0;
 
     *gretl_errmsg = '\0';
 
@@ -665,112 +713,146 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
     }
 
     /* truncate stobs if not a date */
-    if (strchr(stobs, '/') == NULL) {
+    if (strchr(stobs, '/') != NULL) {
+	dated = 1;
+    } else {
 	stobs[8] = '\0';
     }
 
     /* does frequency make sense? */
-    if (pd < 1 || (pdinfo->n > 0 && pd > pdinfo->n)) {
+    if (pd < 1 || (pdinfo->n > 0 && pd > pdinfo->n && opt != OPT_T)) {
 	sprintf(gretl_errmsg, 
 		_("frequency (%d) does not make seem to make sense"), pd);
 	return 1;
     }
 
-    p = stobs;
-    while (*p) {
-	if (*p == ':') *p = '.';
-	p++;
+    /* if an explicit structure option was passed in, respect it */
+    if (opt == OPT_X) {
+	structure = CROSS_SECTION;
+    } else if (opt == OPT_T) {
+	structure = TIME_SERIES;
+    } else if (opt == OPT_S) {
+	structure = STACKED_TIME_SERIES;
+    } else if (opt == OPT_C) {
+	structure = STACKED_CROSS_SECTION;
+    } else if (opt == OPT_N) {
+	structure = SPECIAL_TIME_SERIES;
     }
 
-    /* special case: daily data (dated or undated) */
-    if ((pd >= 5 && pd <= 7) && (strchr(stobs, '/') || !strcmp(stobs, "1")) &&
-	opt != OPT_S && opt != OPT_C) {
-	if (strcmp(stobs, "1")) {
-	    /* dated */
-	    ed0 = get_epoch_day(stobs);
+    if (dated) {
+	if (opt == OPT_X || opt == OPT_S || opt == OPT_C) {
+	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
+	    return 1;
+	}
+
+	if (pd == 5 || pd == 6 || pd == 7 || pd == 52) {
+	    /* calendar-dated data, daily or weekly */
+	    double ed0 = get_epoch_day(stobs);
+
 	    if (ed0 < 0) {
 		sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
 		return 1;
 	    }
-	    pdinfo->sd0 = (double) ed0;
+
+	    pdinfo->sd0 = ed0;
+	    structure = TIME_SERIES;
+
 	    /* replace any existing markers with date strings */
 	    destroy_dataset_markers(pdinfo);
 	} else {
-	    /* undated */
-	    pdinfo->sd0 = 1.0;
-	}
-	pdinfo->time_series = TIME_SERIES;
-    } else {
-	int maj = 0, min = 0, dc = 0, pos;
-
-	len = pos = strlen(stobs);
-	for (i=0; i<len; i++) {
-	    if (stobs[i] != '.' && !isdigit((unsigned char) stobs[i])) {
-		bad = 1;
-		break;
-	    }
-	    if (stobs[i] == '.') {
-		if (dc == 0) pos = i;
-		dc++;
-	    }
-	}
-
-	if (bad || dc > 1) {
 	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
-	if (pd > 1 && pos == len) {
-	    strcpy(gretl_errmsg, _("starting obs must contain a ':' with "
-		   "frequency > 1"));
+    } else {
+	int maj = 0, min = 0;
+
+	if (get_stobs_maj_min(stobs, &maj, &min)) {
+	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
-	if (pd == 1 && pos < len) {
-	    strcpy(gretl_errmsg, _("no ':' allowed in starting obs with "
-		   "frequency 1"));
-	    return 1;
-	}  
 
-	if (pd > 1) {
-	    maj = atoi(stobs);
-	    strcpy(endbit, stobs + pos + 1);
-	    min = atoi(endbit);
-	    if (min < 0 || min > pd) {
-		sprintf(gretl_errmsg, 
-			_("starting obs '%s' is incompatible with frequency"), 
-			stobs);
+	/* catch undated daily or weekly data */
+	if ((pd == 5 || pd == 6 || pd == 7 || pd == 52)  
+	    && min == 0 && opt != OPT_X && opt != OPT_S && opt != OPT_C) {
+	    pdinfo->structure = TIME_SERIES;
+	} else {
+	    int balanced = 1;
+	    int err = 0;
+
+	    /* various pathologies */
+
+	    if (pd == 1) {
+		if (min > 0) {
+		    strcpy(gretl_errmsg, _("no ':' allowed in starting obs with "
+					   "frequency 1"));
+		    err = 1;
+		} else if (opt == OPT_S || opt == OPT_C) {
+		    strcpy(gretl_errmsg, _("panel data must have frequency > 1"));
+		    err = 1;
+		}
+	    } else {
+		if (min == 0) {
+		    strcpy(gretl_errmsg, _("starting obs must contain a ':' with "
+					   "frequency > 1"));
+		    err = 1;
+		} else if (min > pd) {
+		    sprintf(gretl_errmsg, 
+			    _("starting obs '%s' is incompatible with frequency"), 
+			    stobs);
+		    err = 1;
+		} else if (opt == OPT_X) {
+		    strcpy(gretl_errmsg, _("cross-sectional data: frequency must be 1"));
+		    err = 1;
+		} else if (pdinfo->n % pd != 0) {
+		    balanced = 0;
+		    if (opt == OPT_S || opt == OPT_C) {
+			sprintf(gretl_errmsg, _("Panel datasets must be balanced.\n"
+						"The number of observations (%d) is not a multiple\n"
+						"of the number of %s (%d)."), 
+				pdinfo->n, ((opt == OPT_S)? _("periods") : _("units")), pd);
+			err = 1;
+		    }
+		}
+	    }
+
+	    if (err) {
 		return 1;
 	    }
-	    real_format_obs(stobs, maj, min, pd, '.');
+
+	    /* OK? */
+		
+	    if (pd == 1) {
+		sprintf(stobs, "%d", maj);
+		if (structure == STRUCTURE_UNKNOWN) {
+		    if (maj > 1) {
+			structure = TIME_SERIES; /* annual? */
+		    } else {
+			structure = CROSS_SECTION;
+		    }
+		}
+	    } else {
+		real_format_obs(stobs, maj, min, pd, '.');
+		if (structure == STRUCTURE_UNKNOWN) {
+		    if (maj > 1500 && recognized_ts_frequency(pd)) {
+			structure = TIME_SERIES;
+		    } else if (balanced) {
+			structure = STACKED_TIME_SERIES; /* panel? */
+		    } else {
+			structure = TIME_SERIES; /* ?? */
+		    }
+		}
+	    }
 	}
-    }
 
-    /* adjust data info struct */
-    pdinfo->pd = pd;
-
-    if (ed0 == 0L) {
+	/* for non-calendar data */
 	pdinfo->sd0 = dot_atof(stobs);
-    } 
-
-    if (opt == OPT_S) {
-	pdinfo->time_series = STACKED_TIME_SERIES;
-    } else if (opt == OPT_C) {
-	pdinfo->time_series = STACKED_CROSS_SECTION;
-    } else if (pdinfo->sd0 >= 1.0) {
-        pdinfo->time_series = TIME_SERIES; /* but might be panel? */
-    } else {
-	pdinfo->time_series = 0;
     }
 
-    /* try to catch panels with frequency 5, 6 or 7 */
-    if ((pdinfo->pd >= 5 && pdinfo->pd <= 7) &&
-	pdinfo->sd0 > 1.0 && pdinfo->sd0 < 2.0 &&
-	opt != OPT_S && opt != OPT_C) {
-	pdinfo->time_series = 0;
-    }
+    pdinfo->pd = pd;
+    pdinfo->structure = structure;
 
     ntodate_full(pdinfo->stobs, 0, pdinfo); 
-    ntodate_full(endobs, pdinfo->n - 1, pdinfo);
-    strcpy(pdinfo->endobs, endobs);
+    ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
     return 0;
 }
@@ -796,6 +878,7 @@ int is_model_cmd (const char *s)
 	!strncmp(s, "logit", 5)  ||
 	!strncmp(s, "probit", 6) ||
 	!strncmp(s, "tobit", 5) ||
+	!strncmp(s, "poisson", 7) ||
 	!strncmp(s, "garch", 5) ||
 	!strncmp(s, "logistic", 8) ||
 	!strncmp(s, "end nls", 7) ||
@@ -872,20 +955,18 @@ int list_dups (const int *list, int ci)
 int *copylist (const int *src)
 {
     int *targ;
-    int i, n;
+    int i;
 
     if (src == NULL) {
 	return NULL;
     }
 
-    n = src[0];
-
-    targ = malloc((n + 1) * sizeof *targ);
+    targ = malloc((src[0] + 1) * sizeof *targ);
     if (targ == NULL) {
 	return NULL;
     }
 
-    for (i=0; i<=n; i++) {
+    for (i=0; i<=src[0]; i++) {
 	targ[i] = src[i];
     }
 
@@ -950,12 +1031,15 @@ int grow_nobs (int newobs, double ***pZ, DATAINFO *pdinfo)
 	    return E_ALLOC;
 	}
     }
+    
+    if (pdinfo->t2 == pdinfo->n - 1) {
+	pdinfo->t2 = bign - 1;
+    }
 
-    pdinfo->n += newobs;
-    pdinfo->t2 = pdinfo->n - 1;
+    pdinfo->n = bign;
 
     /* does daily data need special handling? */
-    ntodate(pdinfo->endobs, pdinfo->t2, pdinfo);
+    ntodate(pdinfo->endobs, bign - 1, pdinfo);
 
     return 0;
 }
@@ -978,7 +1062,6 @@ static int real_dataset_add_vars (int newvars, double *x,
 	/* new var is pre-allocated */
 	newZ[v] = x;
     } else {
-	/* need to allocate for new vars(s) */
 	for (i=0; i<newvars; i++) {
 	    newZ[v+i] = malloc(n * sizeof **newZ);
 	    if (newZ[v+i] == NULL) return E_ALLOC;
@@ -988,7 +1071,10 @@ static int real_dataset_add_vars (int newvars, double *x,
     *pZ = newZ;
 
     varname = realloc(pdinfo->varname, (v + newvars) * sizeof *varname);
-    if (varname == NULL) return E_ALLOC;
+    if (varname == NULL) {
+	return E_ALLOC;
+    }
+
     pdinfo->varname = varname;
 
     for (i=0; i<newvars; i++) {
@@ -1016,7 +1102,10 @@ static int real_dataset_add_vars (int newvars, double *x,
     }
 
     vector = realloc(pdinfo->vector, (v + newvars));
-    if (vector == NULL) return E_ALLOC;
+    if (vector == NULL) {
+	return E_ALLOC;
+    }
+
     pdinfo->vector = vector;
 
     for (i=0; i<newvars; i++) {
@@ -1146,6 +1235,8 @@ static int shrink_dataset_to_size (double ***pZ,
     return 0;
 }
 
+#undef DROPDBG
+
 int dataset_drop_listed_vars (const int *list, double ***pZ, 
 			      DATAINFO *pdinfo, int *renumber)
 {
@@ -1156,6 +1247,12 @@ int dataset_drop_listed_vars (const int *list, double ***pZ,
 	*renumber = 0;
     }
 
+#if DROPDBG
+    printlist(list, "vars to be deleted");
+#endif
+
+    /* free and set to NULL all the vars to be deleted */
+
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 	if (v > 0 && v < oldv) {
@@ -1169,6 +1266,8 @@ int dataset_drop_listed_vars (const int *list, double ***pZ,
 	}
     }
 
+    /* rearrange pointers if necessary */
+
     for (v=1; v<vmax; v++) {
 	if ((*pZ)[v] == NULL) {
 	    int gap = 1;
@@ -1180,12 +1279,13 @@ int dataset_drop_listed_vars (const int *list, double ***pZ,
 		    break;
 		}
 	    }
+
 	    if (i < vmax) {
-		if (renumber != NULL) {
-		    *renumber = 1;
-		}
 		vmax -= gap;
 		for (i=v; i<vmax; i++) {
+		    if (renumber != NULL && !hidden_var(i + gap, pdinfo)) {
+			*renumber = 1;
+		    }
 		    pdinfo->varname[i] = pdinfo->varname[i + gap];
 		    pdinfo->varinfo[i] = pdinfo->varinfo[i + gap];
 		    pdinfo->vector[i] = pdinfo->vector[i + gap];
@@ -1598,17 +1698,23 @@ int hidden_var (int i, const DATAINFO *pdinfo)
 
 double *copyvec (const double *src, int n)
 {
+    double *targ;
     int i;
-    double *xx;
 
-    if (n == 0 || src == NULL) return NULL;
+    if (n == 0 || src == NULL) {
+	return NULL;
+    }
 
-    xx = malloc(n * sizeof *xx);
-    if (xx == NULL) return NULL;
+    targ = malloc(n * sizeof *targ);
+    if (targ == NULL) {
+	return NULL;
+    }
 
-    for (i=0; i<n; i++) xx[i] = src[i];
+    for (i=0; i<n; i++) {
+	targ[i] = src[i];
+    }
 
-    return xx;
+    return targ;
 }
 
 /* ........................................................... */
@@ -1696,7 +1802,7 @@ FITRESID *get_fit_resid (const MODEL *pmod, double ***pZ,
     int depvar, t, ft;
     FITRESID *fr;
 
-    if (pmod->ci == ARMA) {
+    if (pmod->ci == ARMA || pmod->ci == GARCH) {
 	depvar = pmod->list[4];
     } else {
 	depvar = pmod->list[1];
@@ -1708,8 +1814,10 @@ FITRESID *get_fit_resid (const MODEL *pmod, double ***pZ,
     }
 
     fr->sigma = pmod->sigma;
+
     fr->t1 = pmod->t1;
     fr->t2 = pmod->t2;
+    fr->real_nobs = pmod->nobs;
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	ft = t - fr->t1;
@@ -1767,6 +1875,18 @@ static void fcast_adjust_t1_t2 (const MODEL *pmod,
     *t2 = my_t2;
 }
 
+static int 
+fcast_x_missing (const int *list, const double **Z, int t)
+{
+    int i;
+
+    for (i=2; i<=list[0]; i++) {
+	if (na(Z[list[i]][t])) return 1;
+    }
+
+    return 0;
+}
+
 /* 
    Below: the method for generating forecasts and prediction errors
    that is presented in Wooldridge's Introductory Econometrics,
@@ -1781,9 +1901,8 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
     DATAINFO *finfo = NULL;
     MODEL fmod; 
     FITRESID *fr;
-    int *list = NULL;
-    int ft1, ft2;
-    int i, j, k, t, nfcast, n_est, nv;
+    int *flist = NULL;
+    int i, k, t, n_est, nv;
     int yno = pmod->list[1];
     char t1str[OBSLEN], t2str[OBSLEN];
 
@@ -1809,27 +1928,29 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
 	return fr;
     }
 
-    ft1 = dateton(t1str, pdinfo);
-    ft2 = dateton(t2str, pdinfo);
-    if (ft1 < 0 || ft2 < 0 || ft2 <= ft1) {
+    fr->t1 = dateton(t1str, pdinfo);
+    fr->t2 = dateton(t2str, pdinfo);
+
+    if (fr->t1 < 0 || fr->t2 < 0 || fr->t2 <= fr->t1) {
 	fr->err = E_OBS;
 	return fr;
     }
 
     /* move endpoints if there are missing vals for the
        independent variables */
-    fcast_adjust_t1_t2(pmod, (const double **) *pZ, &ft1, &ft2);
+    fcast_adjust_t1_t2(pmod, (const double **) *pZ, &fr->t1, &fr->t2);
 
     /* number of obs for which forecasts will be generated */
-    nfcast = ft2 - ft1 + 1;
+    fr->nobs = fr->t2 - fr->t1 + 1;
 
-    if (allocate_fit_resid_arrays(fr, nfcast, 1)) {
+    if (allocate_fit_resid_arrays(fr, fr->nobs, 1)) {
 	fr->err = E_ALLOC;
 	return fr;
     }
 
     nv = pmod->list[0];
     if (!pmod->ifc) nv++;
+
     n_est = pmod->t2 - pmod->t1 + 1;
 
     finfo = create_new_dataset(&fZ, nv, n_est, 0);
@@ -1844,16 +1965,17 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
     }
 
     /* create new list */
-    list = malloc((finfo->v + 1) * sizeof *list);
-    if (list == NULL) {
+    flist = malloc((finfo->v + 1) * sizeof *flist);
+    if (flist == NULL) {
 	fr->err = E_ALLOC;
 	goto fcast_bailout;
     }
-    list[0] = finfo->v;
-    list[1] = 1;
-    list[2] = 0;
-    for (i=3; i<=list[0]; i++) {
-	list[i] = i - 1;
+
+    flist[0] = finfo->v;
+    flist[1] = 1;
+    flist[2] = 0;
+    for (i=3; i<=flist[0]; i++) {
+	flist[i] = i - 1;
     }
 
     gretl_model_init(&fmod);
@@ -1863,36 +1985,47 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
 
 #ifdef FCAST_DEBUG
     printf("get_fcast_with_errs: ft1=%d, ft2=%d, pmod->t1=%d, pmod->t2=%d\n",
-	   ft1, ft2, pmod->t1, pmod->t2);
+	   fr->t1, fr->t2, pmod->t1, pmod->t2);
 #endif
 
-    /* FIXME: case of missing obs within model range */
+    for (k=0; k<fr->nobs; k++) {
+	int tk = k + fr->t1;
 
-    for (k=0; k<nfcast; k++) {
+	fr->actual[k] = (*pZ)[yno][tk];
 
-	if (model_missing(pmod, k + ft1)) {
+	if (fcast_x_missing(pmod->list, (const double **) *pZ, tk)) {
 	    fr->sderr[k] = fr->fitted[k] = NADBL;
-	    fr->actual[k] = (*pZ)[pmod->list[1]][k + ft1];
 	    continue;
 	}
 
 	/* form modified indep vars: original data minus the values
-	   to be used for the forecast */
-	for (i=3; i<=list[0]; i++) {
-	    j = (pmod->ifc)? pmod->list[i] : pmod->list[i-1];
+	   to be used for the forecast 
+	*/
+	for (i=3; i<=flist[0]; i++) {
+	    int v = (pmod->ifc)? pmod->list[i] : pmod->list[i-1];
+	    const double *xv = (*pZ)[v];
+
 	    for (t=0; t<finfo->n; t++) {
-		fZ[i-1][t] = (*pZ)[j][t + pmod->t1] 
-		    - (*pZ)[j][k + ft1];
+		int tp = t + pmod->t1;
+
+		if (na(xv[tp])) {
+		    fZ[i-1][t] = NADBL;
+		} else {
+		    fZ[i-1][t] = xv[tp] - xv[tk];
+		}
 	    }
 	}
-	clear_model(&fmod);
-	fmod = lsq(list, &fZ, finfo, OLS, OPT_A, 0.0);
+
+	fmod = lsq(flist, &fZ, finfo, OLS, OPT_A, 0.0);
+
 	if (fmod.errcode) {
 	    fr->err = fmod.errcode;
 	    clear_model(&fmod);
 	    goto fcast_bailout;
 	}
+
 	fr->fitted[k] = fmod.coeff[0];
+
 	/* what exactly do we want here? */
 #ifdef GIVE_SDERR_OF_EXPECTED_Y
 	fr->sderr[k] = fmod.sderr[0];
@@ -1900,22 +2033,17 @@ FITRESID *get_fcast_with_errs (const char *str, const MODEL *pmod,
 	fr->sderr[k] = sqrt(fmod.sderr[0] * fmod.sderr[0] + 
 			    fmod.sigma * fmod.sigma);
 #endif
-	fr->actual[k] = (*pZ)[pmod->list[1]][k + ft1];
+	clear_model(&fmod);
     }
 
-    clear_model(&fmod);
-
     fr->tval = tcrit95(pmod->dfd);
     strcpy(fr->depvar, pdinfo->varname[yno]);
-
-    fr->t1 = ft1;
-    fr->t2 = ft2;
-    fr->nobs = ft2 - ft1 + 1;
     fr->df = pmod->dfd;
 
  fcast_bailout:
+
     free_Z(fZ, finfo);
-    free(list);
+    free(flist);
     clear_datainfo(finfo, CLEAR_FULL);
     free(finfo);
 
@@ -1932,16 +2060,14 @@ int fcast_with_errs (const char *str, const MODEL *pmod,
     int err;
 
     fr = get_fcast_with_errs(str, pmod, pZ, pdinfo, prn);
+
     if (fr == NULL) {
 	return E_ALLOC;
-    } 
-    else if (fr->err) {
-	err = fr->err;
-	free_fit_resid(fr);
-	return err;
     }
 
-    err = text_print_fcast_with_errs(fr, pZ, pdinfo, prn, plot);
+    if ((err = fr->err) == 0) {
+	err = text_print_fcast_with_errs(fr, pZ, pdinfo, prn, plot);
+    }
 
     free_fit_resid(fr);
     
@@ -1963,8 +2089,10 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 
     cmd.list = malloc(sizeof *cmd.list);
     cmd.param = malloc(1);
-    if (cmd.list == NULL || cmd.param == NULL) 
+
+    if (cmd.list == NULL || cmd.param == NULL) {
 	return 1;
+    }
 
     getcmd(model_spec, pdinfo, &cmd, &ignore, pZ, NULL);
 
@@ -1984,9 +2112,6 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 	    *tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, 0, rho);
 	}
 	break;
-    case HCCM:
-	*tmpmod = hccm_func(cmd.list, pZ, pdinfo);
-	break;
     case HSK:
 	*tmpmod = hsk_func(cmd.list, pZ, pdinfo);
 	break;
@@ -1997,8 +2122,12 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     case TOBIT:
 	*tmpmod = tobit_model(cmd.list, pZ, pdinfo, NULL);
 	break;
+    case POISSON:
+	*tmpmod = poisson_model(cmd.list, pZ, pdinfo, NULL);
+	break;
     case OLS:
     case WLS:
+    case HCCM:
     case POOLED:
 	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, 0.0);
 	break;
@@ -2013,8 +2142,8 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 	clear_model(tmpmod);
     }
 
-    if (cmd.list) free(cmd.list);
-    if (cmd.param) free(cmd.param);
+    free(cmd.list);
+    free(cmd.param);
 
     return err;
 }
@@ -2026,17 +2155,19 @@ int guess_panel_structure (double **Z, DATAINFO *pdinfo)
     int v, panel;
 
     v = varindex(pdinfo, "year");
+
     if (v == pdinfo->v) {
 	v = varindex(pdinfo, "Year");
     }
+
     if (v == pdinfo->v) {
 	panel = 0; /* can't guess */
     } else {
 	if (floateq(Z[v][0], Z[v][1])) { /* "year" is same for first two obs */
-	    pdinfo->time_series = STACKED_CROSS_SECTION; 
+	    pdinfo->structure = STACKED_CROSS_SECTION; 
 	    panel = STACKED_CROSS_SECTION;
 	} else {
-	    pdinfo->time_series = STACKED_TIME_SERIES; 
+	    pdinfo->structure = STACKED_TIME_SERIES; 
 	    panel = STACKED_TIME_SERIES;
 	}
     }
@@ -2049,14 +2180,14 @@ int guess_panel_structure (double **Z, DATAINFO *pdinfo)
    T = number pf time-periods per cross-sectional unit
 */
 
-int get_panel_structure (DATAINFO *pdinfo, int *nunits, int *T)
+int get_panel_structure (const DATAINFO *pdinfo, int *nunits, int *T)
 {
     int err = 0;
 
-    if (pdinfo->time_series == STACKED_TIME_SERIES) {
+    if (pdinfo->structure == STACKED_TIME_SERIES) {
         *nunits = pdinfo->n / pdinfo->pd;
         *T = pdinfo->pd;
-    } else if (pdinfo->time_series == STACKED_CROSS_SECTION) {
+    } else if (pdinfo->structure == STACKED_CROSS_SECTION) {
 	*nunits = pdinfo->pd;
 	*T = pdinfo->n / pdinfo->pd;
     } else {
@@ -2071,7 +2202,7 @@ int get_panel_structure (DATAINFO *pdinfo, int *nunits, int *T)
 int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn)
 {
     int nunits, T;
-    int old_ts = pdinfo->time_series;
+    int old_ts = pdinfo->structure;
 
     if (pdinfo->pd == 1) {
 	pputs(prn, _("The current data frequency, 1, is not "
@@ -2081,18 +2212,18 @@ int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn)
     }
 
     if (opt == OPT_C) {
-	pdinfo->time_series = STACKED_CROSS_SECTION;
+	pdinfo->structure = STACKED_CROSS_SECTION;
     } else {
-	pdinfo->time_series = STACKED_TIME_SERIES;
+	pdinfo->structure = STACKED_TIME_SERIES;
     }
 
     if (get_panel_structure(pdinfo, &nunits, &T)) {
 	pputs(prn, _("Failed to set panel structure\n"));
-	pdinfo->time_series = old_ts;
+	pdinfo->structure = old_ts;
 	return 1;
     } else {
 	pprintf(prn, _("Panel structure set to %s\n"),
-		(pdinfo->time_series == STACKED_CROSS_SECTION)? 
+		(pdinfo->structure == STACKED_CROSS_SECTION)? 
 		_("stacked cross sections") : _("stacked time series"));
 	pprintf(prn, _("(%d units observed in each of %d periods)\n"),
 		nunits, T);
@@ -2149,19 +2280,26 @@ static void free_mp_varnames (mp_results *mpvals)
 
 int allocate_mp_varnames (mp_results *mpvals)
 {
-    int i, n = mpvals->ncoeff + 1;
+    int i, j, n = mpvals->ncoeff + 1;
 
     mpvals->varnames = malloc(n * sizeof *mpvals->varnames);
-    if (mpvals->varnames == NULL) return 1;
+
+    if (mpvals->varnames == NULL) {
+	return 1;
+    }
 
     for (i=0; i<n; i++) {
 	mpvals->varnames[i] = malloc(12);
 	if (mpvals->varnames[i] == NULL) {
-	    free_mp_varnames(mpvals);
+	    for (j=0; j<i; j++) {
+		free(mpvals->varnames[j]);
+	    }
+	    free(mpvals->varnames);
 	    return 1;
 	}
 	mpvals->varnames[i][0] = 0;
     }
+
     return 0;
 }
 
@@ -2265,6 +2403,7 @@ FITRESID *fit_resid_new (int n, int errs)
     fr->t1 = 0;
     fr->t2 = 0;
     fr->nobs = 0;
+    fr->real_nobs = 0;
 
     if (n == 0) {
 	fr->actual = NULL;
@@ -2302,11 +2441,16 @@ static int copy_main_list (int **targ, const int *src)
     for (i=1; i<=src[0] && src[i]!=LISTSEP; i++) n++;
 
     if (*targ != NULL) free(*targ);
+
     *targ = malloc((n + 2) * sizeof *targ);
-    if (*targ == NULL) return 1;
+    if (*targ == NULL) {
+	return 1;
+    }
     
     (*targ)[0] = n;
-    for (i=1; i<=n; i++) (*targ)[i] = src[i];
+    for (i=1; i<=n; i++) {
+	(*targ)[i] = src[i];
+    }
 
     return 0;
 }
@@ -2389,7 +2533,7 @@ static int font_not_found (const char *s)
     }
 }
 
-static int real_gretl_spawn (const char *cmdline, int verbose)
+int gretl_spawn (const char *cmdline)
 {
     GError *error = NULL;
     gchar *errout = NULL, *sout = NULL;
@@ -2408,25 +2552,24 @@ static int real_gretl_spawn (const char *cmdline, int verbose)
 
     if (!ok) {
 	strcpy(gretl_errmsg, error->message);
-	if (verbose) {
-	    fprintf(stderr, "gretl_spawn: '%s'\n", error->message);
-	}
+	fprintf(stderr, "gretl_spawn: '%s'\n", error->message);
 	g_error_free(error);
 	ret = 1;
     } else if (errout && *errout) {
 	strcpy(gretl_errmsg, errout);
-	if (verbose) {
-	    fprintf(stderr, "stderr: '%s'\n", errout);
-	}
+	fprintf(stderr, "stderr: '%s'\n", errout);
 	if (!font_not_found(errout)) {
 	    ret = 1;
 	}
     } else if (status != 0) {
-	sprintf(gretl_errmsg, "%s\n%s", 
-		_("Command failed"),
-		sout);
-	if (verbose) {
+	if (sout != NULL) {
+	    sprintf(gretl_errmsg, "%s\n%s", 
+		    _("Command failed"),
+		    sout);
 	    fprintf(stderr, "status=%d: '%s'\n", status, sout);
+	} else {
+	    strcpy(gretl_errmsg, _("Command failed"));
+	    fprintf(stderr, "status=%d\n", status);
 	}
 	ret = 1;
     }
@@ -2434,7 +2577,7 @@ static int real_gretl_spawn (const char *cmdline, int verbose)
     if (errout != NULL) g_free(errout);
     if (sout != NULL) g_free(sout);
 
-    if (ret && verbose) {
+    if (ret) {
 	fprintf(stderr, "Failed command: '%s'\n", cmdline);
     } 
 
@@ -2445,7 +2588,7 @@ static int real_gretl_spawn (const char *cmdline, int verbose)
 
 #if !defined(WIN32) && !defined(GRETL_GLIB)
 
-static int real_gretl_spawn (const char *cmdline, int verbose)
+int gretl_spawn (const char *cmdline)
 {
     int err;
 
@@ -2454,7 +2597,7 @@ static int real_gretl_spawn (const char *cmdline, int verbose)
     signal(SIGCHLD, SIG_DFL);
 
     err = system(cmdline);
-    if (err && verbose) {
+    if (err) {
 	fprintf(stderr, "Failed command: '%s'\n", cmdline);
 	perror(NULL);
     }
@@ -2464,20 +2607,6 @@ static int real_gretl_spawn (const char *cmdline, int verbose)
 
 #endif
 
-#ifndef WIN32
-
-int gretl_spawn (const char *cmdline)
-{
-    return real_gretl_spawn(cmdline, 1);
-}
-
-int gretl_spawn_quiet (const char *cmdline)
-{
-    return real_gretl_spawn(cmdline, 0);
-}
-
-#endif
-
 /* library init and cleanup functions */
 
 void libgretl_init (CMD *cmd)
@@ -2501,6 +2630,7 @@ void libgretl_cleanup (CMD *cmd)
 
     gretl_rand_free();
     gretl_functions_cleanup();
+    gretl_equation_systems_cleanup();
     testvec(0);
 
     p = strstr(gretl_plotfile(), "gpttmp");
@@ -2512,3 +2642,54 @@ void libgretl_cleanup (CMD *cmd)
 	}
     }
 }
+
+/* record and retrieve hypothesis test results */
+
+enum {
+    SET_TEST_STAT,
+    GET_TEST_STAT,
+    GET_TEST_PVAL
+};
+
+static double
+record_or_get_test_result (double teststat, double pval, char *blurb,
+			   int code)
+{
+    static double val = NADBL;
+    static double pv = NADBL;
+    static char info[128] = {0};
+
+    double ret = NADBL;
+
+    if (code == SET_TEST_STAT) {
+	val = teststat;
+	pv = pval;
+	if (blurb != NULL) {
+	    strcpy(info, blurb);
+	} else {
+	    *info = '\0';
+	}
+    } else if (code == GET_TEST_STAT || code == GET_TEST_PVAL) {
+	if (blurb != NULL) {
+	    strcpy(blurb, info);
+	}
+	ret = (code == GET_TEST_STAT)? val : pv;
+    } 
+	
+    return ret;
+}
+
+void record_test_result (double teststat, double pval, char *blurb)
+{
+    record_or_get_test_result(teststat, pval, blurb, SET_TEST_STAT);
+}
+
+double get_last_test_statistic (char *blurb)
+{
+    return record_or_get_test_result(0, 0, blurb, GET_TEST_STAT);
+}
+
+double get_last_pvalue (char *blurb)
+{
+    return record_or_get_test_result(0, 0, blurb, GET_TEST_PVAL);
+}
