@@ -23,14 +23,8 @@
 #include "libgretl.h"
 
 enum model_selection_criteria {
-    C_SGMASQ = 0,
     C_AIC,
-    C_FPE,
-    C_HQ,
     C_BIC,
-    C_SHIBATA,
-    C_GCV,
-    C_RICE,
     C_MAX
 };
 
@@ -41,7 +35,17 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains time series
  * data (1) or not (0).
  */
-#define dataset_is_time_series(p) ((p)->time_series == TIME_SERIES)
+#define dataset_is_time_series(p) ((p)->structure == TIME_SERIES || \
+				   (p)->structure == SPECIAL_TIME_SERIES)
+
+/**
+ * custom_time_series:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains time series
+ * data with custom (non-standard) frequency (1) or not (0).
+ */
+#define custom_time_series(p) ((p)->structure == SPECIAL_TIME_SERIES)
 
 /**
  * dataset_is_daily:
@@ -50,7 +54,7 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains daily time series
  * data (1) or not (0).
  */
-#define dataset_is_daily(p) (p->time_series == TIME_SERIES \
+#define dataset_is_daily(p) (p->structure == TIME_SERIES \
                              && (p->pd == 5 || p->pd == 6 || p->pd == 7))
 
 /**
@@ -60,7 +64,7 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains weekly time series
  * data (1) or not (0).
  */
-#define dataset_is_weekly(p) (p->time_series == TIME_SERIES \
+#define dataset_is_weekly(p) (p->structure == TIME_SERIES \
                               && p->pd == 52)
 
 /**
@@ -70,7 +74,7 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains hourly time series
  * data (1) or not (0).
  */
-#define dataset_is_hourly(p) (p->time_series == TIME_SERIES \
+#define dataset_is_hourly(p) (p->structure == TIME_SERIES \
                               && p->pd == 24)
 
 /**
@@ -80,7 +84,7 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains dated daily time series
  * data (1) or not (0).
  */
-#define dated_daily_data(p) (p->time_series == TIME_SERIES \
+#define dated_daily_data(p) (p->structure == TIME_SERIES \
                              && (p->pd == 5 || p->pd == 6 || p->pd == 7) \
                              && p->sd0 > 10000.0)
 
@@ -91,10 +95,32 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains dated daily 
  * (seven-day) time series data (1) or not (0).
  */
-#define dated_seven_day_data(p) (p->time_series == TIME_SERIES \
+#define dated_seven_day_data(p) (p->structure == TIME_SERIES \
                                  && p->pd == 7 && \
                                  p->sd0 > 10000.0)
 
+/**
+ * dated_weekly_data:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains dated weekly 
+ * time series data (1) or not (0).
+ */
+#define dated_weekly_data(p) (p->structure == TIME_SERIES \
+                              && p->pd == 52 && \
+                              p->sd0 > 10000.0)
+
+/**
+ * calendar_data:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set uses calendar
+ * dates for observation strings (1) or not (0).
+ */
+#define calendar_data(p) (p->structure == TIME_SERIES && \
+                          (p->pd == 5 || p->pd == 6 || p->pd == 7 \
+                           || p->pd == 52) && p->sd0 > 10000.0) 
+                          
 /**
  * dataset_is_panel:
  * @p: pointer to data information struct.
@@ -102,8 +128,8 @@ enum model_selection_criteria {
  * Attempt to determine whether a data set contains panel
  * data (1) or not (0).
  */
-#define dataset_is_panel(p) ((p)->time_series == STACKED_TIME_SERIES \
-                            || (p)->time_series == STACKED_CROSS_SECTION)
+#define dataset_is_panel(p) ((p)->structure == STACKED_TIME_SERIES || \
+                             (p)->structure == STACKED_CROSS_SECTION)
 
 #include <float.h>
 
@@ -179,7 +205,7 @@ int ijton (int i, int j, int nrows);
 
 int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo);
 
-int get_panel_structure (DATAINFO *pdinfo, int *nunits, int *T);
+int get_panel_structure (const DATAINFO *pdinfo, int *nunits, int *T);
 
 int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn); 
 
@@ -201,6 +227,10 @@ CONFINT *get_model_confints (const MODEL *pmod);
 
 void free_confint (CONFINT *cf);
 
+int ls_aic_bic (MODEL *pmod);
+
+double gretl_mean (int t1, int t2, const double *x);
+
 double gretl_stddev (int t1, int t2, const double *x);
 
 double gretl_variance (int t1, int t2, const double *x);
@@ -209,7 +239,6 @@ double gretl_sst (int t1, int t2, const double *x);
 
 #ifndef WIN32
 int gretl_spawn (const char *cmdline);
-int gretl_spawn_quiet (const char *cmdline);
 #endif
 
 #endif /* GRETL_UTILS_H */
