@@ -26,6 +26,8 @@
 static int use_qr = -1;
 static int halt_on_error = -1;
 static double hp_lambda;
+static int bkbp_k = 8;
+static int bkbp_periods[2] = { 8, 32 };
 
 enum {
     AUTO_LAG_STOCK_WATSON,
@@ -48,6 +50,17 @@ double get_hp_lambda (void)
     return hp_lambda;
 }
 
+int get_bkbp_k (void)
+{
+    return bkbp_k;
+}
+
+void get_bkbp_periods (int *periods)
+{
+    periods[0] = bkbp_periods[0];
+    periods[1] = bkbp_periods[1];
+}
+
 static int get_or_set_force_hc (int f)
 {
     static int force;
@@ -64,17 +77,27 @@ static int get_or_set_garch_vcv (int v)
     return variant;
 }
 
-static void set_garch_vcv_variant (const char *s)
+static int get_or_set_garch_robust_vcv (int v)
+{
+    static int variant;
+
+    if (v >= 0) variant = v;
+    return variant;
+}
+
+static int set_garch_vcv_variant (const char *s)
 {
     int vopt = VCV_UNSET;
 
-    if (!strcmp(s, "hessian"))    vopt = VCV_HESSIAN;
-    else if (!strcmp(s, "im"))    vopt = VCV_IM;
-    else if (!strcmp(s, "op"))    vopt = VCV_OP;
-    else if (!strcmp(s, "qml"))   vopt = VCV_QML;
-    else if (!strcmp(s, "bw"))    vopt = VCV_BW;
+    if (!strcmp(s, "hessian"))  vopt = VCV_HESSIAN;
+    else if (!strcmp(s, "im"))  vopt = VCV_IM;
+    else if (!strcmp(s, "op"))  vopt = VCV_OP;
+    else if (!strcmp(s, "qml")) vopt = VCV_QML;
+    else if (!strcmp(s, "bw"))  vopt = VCV_BW;
 
     get_or_set_garch_vcv(vopt);
+    
+    return (vopt == VCV_UNSET);
 }
 
 int get_garch_vcv_version (void)
@@ -82,6 +105,11 @@ int get_garch_vcv_version (void)
     return get_or_set_garch_vcv(-1);
 }
 
+int get_garch_robust_vcv_version (void)
+{
+    return get_or_set_garch_robust_vcv(-1);
+}
+
 static void set_force_hc (int f)
 {
     get_or_set_force_hc(f);
@@ -111,32 +139,104 @@ int get_hac_lag (int m)
     return 0.75 * pow(m, 1.0 / 3.0);
 }
 
+static int parse_hc_variant (const char *s)
+{
+    int err = 1;
+
+    if (!strcmp(s, "0") || !strcmp(s, "1") ||
+	!strcmp(s, "2") || !strcmp(s, "3")) {
+	robust_opts.hc_version = atoi(s);
+	err = 0;
+    } else if (!strcmp(s, "3a")) {
+	robust_opts.hc_version = 4;
+	err = 0;
+    }
+
+    if (err) {
+	int hcv;
+
+	if (!strcmp(s, "hc3a")) {
+	    robust_opts.hc_version = 4;
+	    err = 0;
+	} else if (sscanf(s, "hc%d", &hcv)) {
+	    if (hcv >= 0 && hcv <= 4) {
+		robust_opts.hc_version = hcv;
+		err = 0;
+	    }
+	}
+    }
+
+    return err;
+}
+
+void set_xsect_hccme (const char *s)
+{
+    char *scpy = gretl_strdup(s);
+
+    if (scpy == NULL) return;
+
+    lower(scpy);
+    parse_hc_variant(scpy);
+    free(scpy);
+}
+
+void set_tseries_hccme (const char *s)
+{
+    char *scpy = gretl_strdup(s);
+
+    if (scpy == NULL) return;
+
+    lower(scpy);
+    if (parse_hc_variant(scpy) == 0) {
+	set_force_hc(1);
+    } else {
+	set_force_hc(0);
+    }
+    free(scpy);
+}
+
+void set_garch_robust_vcv (const char *s)
+{
+    char *scpy = gretl_strdup(s);
+
+    if (scpy == NULL) return;
+
+    lower(scpy);
+    if (!strcmp(s, "qml")) {
+	get_or_set_garch_robust_vcv(VCV_QML);
+    } else if (!strcmp(s, "bw")) {
+	get_or_set_garch_robust_vcv(VCV_BW);
+    }
+    free(scpy);
+}
+
 int parse_set_line (const char *line, int *echo_off, PRN *prn)
 {
-    char setobj[16], setarg[16];
+    char setobj[16], setarg[16], setarg2[16];
     int nw, err = E_PARSE;
 
-    *setobj = *setarg = '\0';
+    *setobj = *setarg = *setarg2 = '\0';
 
-    nw = sscanf(line, "%*s %15s %15s", setobj, setarg);
+    nw = sscanf(line, "%*s %15s %15s %15s", setobj, setarg, setarg2);
     
     if (nw == 1) {
 	if (!strcmp(setobj, "echo")) {
 	    *echo_off = 0;
 	    err = 0;
 	}
-    }
-	    
-    else if (nw == 2) {
+    } else if (nw == 2) {
+	lower(setarg);
+
 	/* set echo on/off */
 	if (!strcmp(setobj, "echo")) {
-	    if (!strcmp(setarg, "off")) {
-		*echo_off = 1;
-		err = 0;
-	    } 
-	    else if (!strcmp(setarg, "on")) {
-		*echo_off = 0;
-		err = 0;
+	    if (echo_off != NULL) {
+		if (!strcmp(setarg, "off")) {
+		    *echo_off = 1;
+		    err = 0;
+		} else if (!strcmp(setarg, "on")) {
+		    *echo_off = 0;
+		    err = 0;
+		}
 	    } 
 	} else if (!strcmp(setobj, "hac_lag")) {
 	    /* set max lag for HAC estimation */
@@ -144,48 +244,35 @@ int parse_set_line (const char *line, int *echo_off, PRN *prn)
 		robust_opts.auto_lag = AUTO_LAG_STOCK_WATSON;
 		robust_opts.user_lag = 0;
 		err = 0;
-	    }
-	    else if (!strcmp(setarg, "nw2")) {
+	    } else if (!strcmp(setarg, "nw2")) {
 		robust_opts.auto_lag = AUTO_LAG_WOOLDRIDGE;
 		robust_opts.user_lag = 0;
 		err = 0;
-	    }
-	    else if (isdigit(*setarg)) {
+	    } else if (isdigit(*setarg)) {
 		robust_opts.user_lag = atoi(setarg);
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "hc_version")) {
 	    /* set HCCM variant */
-	    if (!strcmp(setarg, "0") || !strcmp(setarg, "1") ||
-		!strcmp(setarg, "2") || !strcmp(setarg, "3")) {
-		robust_opts.hc_version = atoi(setarg);
-		err = 0;
-	    }
+	    err = parse_hc_variant(setarg);
 	} else if (!strcmp(setobj, "force_hc")) {
 	    /* use HCCM, not HAC, even for time series */
 	    if (!strcmp(setarg, "on")) { 
 		set_force_hc(1);
 		err = 0;
-	    }
-	    else if (!strcmp(setarg, "off")) { 
+	    } else if (!strcmp(setarg, "off")) { 
 		set_force_hc(0);
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "garch_vcv")) {
 	    /* set GARCH VCV variant */
-	    if (!strcmp(setarg, "hessian") || !strcmp(setarg, "im") ||
-		!strcmp(setarg, "op") || !strcmp(setarg, "qml") ||
-		!strcmp(setarg, "bw") || !strcmp(setarg, "unset")) {
-		set_garch_vcv_variant(setarg);
-		err = 0;
-	    }
+	    err = set_garch_vcv_variant(setarg);
 	} else if (!strcmp(setobj, "qr")) {
 	    /* switch QR vs Cholesky decomposition */
 	    if (!strcmp(setarg, "on")) {
 		use_qr = 1;
 		err = 0;
-	    }
-	    else if (!strcmp(setarg, "off")) {
+	    } else if (!strcmp(setarg, "off")) {
 		use_qr = 0;
 		err = 0;
 	    }
@@ -193,8 +280,7 @@ int parse_set_line (const char *line, int *echo_off, PRN *prn)
 	    if (!strcmp(setarg, "on")) {
 		halt_on_error = 1;
 		err = 0;
-	    }
-	    else if (!strcmp(setarg, "off")) {
+	    } else if (!strcmp(setarg, "off")) {
 		halt_on_error = 0;
 		err = 0;
 	    }
@@ -218,7 +304,36 @@ int parse_set_line (const char *line, int *echo_off, PRN *prn)
 		hp_lambda = atof(setarg);
 		err = 0;
 	    }
+	} else if (!strcmp(setobj, "bkbp_k")) {
+	    /* Baxter-King approximation order */
+	    if (isdigit(*setarg)) {
+		bkbp_k = atoi(setarg);
+		pprintf(prn, 
+			_("Baxter-King approximation = %d\n"), bkbp_k);
+		err = 0;
+	    }
 	}	
+    } else if (nw == 3) {
+	if (!strcmp(setobj, "bkbp_limits")) {
+	    /* Baxter-King threshold periodicities */
+	    if (isdigit(*setarg)) {
+		bkbp_periods[0] = atoi(setarg);
+		if (isdigit(*setarg2)) {
+		    bkbp_periods[1] = atoi(setarg2);
+		    if (bkbp_periods[1] < bkbp_periods[0]) {
+			/* 2nd entry should be bigger than 1st one */
+			int tmp = bkbp_periods[1];
+
+			bkbp_periods[1] = bkbp_periods[0];
+			bkbp_periods[0] = tmp;
+		    }
+		    pprintf(prn, 
+			    _("Baxter-King band = %d-%d periods\n"), 
+			    bkbp_periods[0], bkbp_periods[1]);
+		    err = 0;
+		}
+	    }
+	}
     }
 		    
     return err;
@@ -263,18 +378,59 @@ int get_halt_on_error (void)
     return halt_on_error;
 }
 
-/* pause between screens of output? (cli operation, not in
-   batch mode) */
+/* switches for looping, batch mode, and pausing between
+   screens of output */
 
 static int gretl_text_pause;
-
-void gretl_set_text_pause (int p)
-{
-    gretl_text_pause = p;
-}
+static int loop_on;
+static int batch_mode;
+static int gui_mode;
 
 int gretl_get_text_pause (void)
 {
     return gretl_text_pause;
 }
 
+void set_loop_on (void)
+{
+    loop_on = 1;
+    gretl_text_pause = 0;
+}
+
+void set_loop_off (void)
+{
+    loop_on = 0;
+    if (!batch_mode && !gui_mode) {
+	gretl_text_pause = 1;
+    }
+}
+
+int gretl_looping (void)
+{
+    return loop_on;
+}
+
+void gretl_set_batch_mode (int b)
+{
+    batch_mode = b;
+    if (batch_mode) {
+	gretl_text_pause = 0;
+    }	
+}
+
+int gretl_in_batch_mode (void)
+{
+    return batch_mode;
+}
+
+void gretl_set_gui_mode (int g)
+{
+    gui_mode = g;
+}
+
+int gretl_in_gui_mode (void)
+{
+    return gui_mode;
+}
+
+
