@@ -22,35 +22,22 @@
 #ifndef MONTE_CARLO_H
 #define MONTE_CARLO_H
 
-enum ifcodes {
-    SET_FALSE,
-    SET_TRUE,
-    SET_ELSE,
-    SET_ENDIF,
-    IS_FALSE,
-    RELAX
-};
-
 typedef struct LOOPSET_ LOOPSET;
 
 /* functions follow */
 
 int ok_in_loop (int ci, const LOOPSET *loop);
 
-LOOPSET *add_to_loop (char *line, int ci, gretlopt oflags,
+LOOPSET *add_to_loop (char *line, int ci, gretlopt opt,
 		      DATAINFO *pdinfo, double ***pZ,
 		      LOOPSET *loop, int *loopstack, int *looprun);
 
 void gretl_loop_destroy (LOOPSET *loop);
 
-void get_cmd_ci (const char *line, CMD *command);
+void get_cmd_ci (const char *line, CMD *cmd);
 
 int loop_exec (LOOPSET *loop, char *line,
 	       double ***pZ, DATAINFO **ppdinfo, 
 	       MODEL **models, int *echo_off, PRN *prn);
 
-int if_eval (const char *line, double ***pZ, DATAINFO *pdinfo);
-
-int ifstate (int code);
-
 #endif /* MONTE_CARLO_H */
