@@ -26,7 +26,7 @@
                         c == OMIT || c == TSLS || c == LOGIT || \
                         c == PROBIT || c == TOBIT || c == ARMA || \
                         c == AR || c == LOGISTIC || c == NLS || \
-                        c == GARCH)
+                        c == GARCH || c == POISSON)
 
 struct gretl_option {
     int ci;              /* command index (context) */
@@ -52,13 +52,18 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_R, "ctt" },     
     { ADF,      OPT_T, "ct" }, 
     { ADF,      OPT_V, "verbose" },
+    { ADF,      OPT_Q, "quiet" },
     { ARMA,     OPT_N, "native" },
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
+    { COINT,    OPT_N, "nc" },
     { COINT2,   OPT_O, "verbose" },
     { EQNPRINT, OPT_O, "complete" },
     { TABPRINT, OPT_O, "complete" },
+    { ESTIMATE, OPT_M, "geomean" },
+    { ESTIMATE, OPT_N, "no-df-corr" },
+    { ESTIMATE, OPT_T, "iterate" },
     { FCASTERR, OPT_O, "plot" },
     { FREQ,     OPT_O, "gamma" },
     { FREQ,     OPT_Q, "quiet" },
@@ -70,23 +75,30 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_S, "suppress-fitted" },
     { GNUPLOT,  OPT_Z, "dummy" },
     { GRAPH,    OPT_O, "tall" },
+    { HAUSMAN,  OPT_T, "time-effects" },
     { IMPORT,   OPT_O, "box1" },
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
-    { LEVERAGE, OPT_O, "save" },
+    { KPSS,     OPT_Q, "quiet" },
+    { LEVERAGE, OPT_S, "save" },
     { LMTEST,   OPT_L, "logs" },
     { LMTEST,   OPT_M, "autocorr" },
     { LMTEST,   OPT_O, "autocorr" },
-    { LMTEST,   OPT_S, "squares" },    
+    { LMTEST,   OPT_S, "squares" }, 
+    { LMTEST,   OPT_P, "panel" },
     { LMTEST,   OPT_W, "white" },
     { LOOP,     OPT_P, "progressive" },
+    { LOOP,     OPT_Q, "quiet" },
     { LOOP,     OPT_V, "verbose" },
+    { MAHAL,    OPT_S, "save" },
+    { MAHAL,    OPT_V, "vcv" },
     { MEANTEST, OPT_O, "unequal-vars" },
     { OLS,      OPT_N, "no-df-corr" },
     { OLS,      OPT_O, "vcv" }, 
     { OLS,      OPT_P, "print-final" },
     { OLS,      OPT_R, "robust" },
     { OLS,      OPT_Q, "quiet" },
+    { OLS,      OPT_S, "simple-print" },
     { OMIT,     OPT_Q, "quiet" },
     { OMITFROM, OPT_Q, "quiet" },
     { OUTFILE,  OPT_A, "append" },
@@ -94,6 +106,9 @@ struct gretl_option gretl_opts[] = {
     { OUTFILE,  OPT_W, "write" },
     { PANEL,    OPT_C, "cross-section" },
     { PANEL,    OPT_S, "time-series" },
+    { POISSON,  OPT_V, "verbose" },
+    { POOLED,   OPT_T, "iterate" },
+    { POOLED,   OPT_V, "verbose" },
     { POOLED,   OPT_W, "unit-weights" },
     { PCA,      OPT_A, "save-all" },
     { PCA,      OPT_O, "save" },
@@ -103,6 +118,9 @@ struct gretl_option gretl_opts[] = {
     { PRINT,    OPT_T, "ten" },
     { SETOBS,   OPT_C, "stacked-cross-section" },
     { SETOBS,   OPT_S, "stacked-time-series" },
+    { SETOBS,   OPT_T, "time-series" },
+    { SETOBS,   OPT_X, "cross-section" },
+    { SETOBS,   OPT_N, "special-time-series" },
     { SMPL,     OPT_C, "replace" },    
     { SMPL,     OPT_O, "dummy" },
     { SMPL,     OPT_M, "no-missing" },
@@ -119,27 +137,32 @@ struct gretl_option gretl_opts[] = {
     { TOBIT,    OPT_V, "verbose" },
     { TSLS,     OPT_R, "robust" },  
     { TSLS,     OPT_S, "save" },
+    { VAR,      OPT_V, "impulse-responses" },
     { VAR,      OPT_Q, "quiet" },  
-    { VAR,      OPT_R, "robust" },  
+    { VAR,      OPT_R, "robust" }, 
+    { WLS,      OPT_Q, "quiet" },
     { 0,        0L,    NULL }
 };
 
-const char **get_opts_for_command (int ci)
+const char **get_opts_for_command (int ci, int *nopt)
 {
-    int i, j, nopt = 0;
+    int i, j, n = 0;
     const char **ret = NULL;
 
     if (is_model_ci(ci) && ci != OLS && ci != LAD) {
-	nopt++; /* vcv */
+	n++; /* vcv */
     }
 
     for (i=0; gretl_opts[i].ci != 0; i++) {
-	if (gretl_opts[i].ci == ci) nopt++;
+	if (gretl_opts[i].ci == ci) n++;
     }
 
-    if (nopt == 0) return NULL;
+    if (n == 0) {
+	*nopt = 0;
+	return NULL;
+    }
 
-    ret = malloc((nopt + 1) * sizeof *ret);
+    ret = malloc(n * sizeof *ret);
     if (ret == NULL) return NULL;
 
     j = 0;
@@ -153,7 +176,7 @@ const char **get_opts_for_command (int ci)
 	ret[j++] = "vcv";
     }
 
-    ret[j] = NULL;
+    *nopt = n;
 
     return ret;
 }
@@ -266,21 +289,39 @@ static int is_long_opt (const char *lopt)
 
 static int valid_long_opt (int ci, const char *lopt)
 {
+    int opt = OPT_NONE;
     int i;
-    int opt = 0L;
 
     if (is_model_ci(ci) && ci != LAD && !strcmp(lopt, "vcv")) {
 	/* VCV is available for all but LAD models */
 	return OPT_O;
     }
 
+    /* start by looking for an exact match */
     for (i=0; gretl_opts[i].o != 0; i++) {
-	if (ci == gretl_opts[i].ci && !strcmp(lopt, gretl_opts[i].longopt)) {
+	if (ci == gretl_opts[i].ci && 
+	    !strcmp(lopt, gretl_opts[i].longopt)) {
 	    opt = gretl_opts[i].o;
 	    break;
 	}
     }
 
+    /* if this failed, try for an abbreviation or extension */
+    if (opt == OPT_NONE) {
+	int len, len1, len2;
+
+	len1 = strlen(lopt);
+	for (i=0; gretl_opts[i].o != 0; i++) {
+	    len2 = strlen(gretl_opts[i].longopt);
+	    len = (len2 > len1)? len1 : len2;
+	    if (ci == gretl_opts[i].ci && 
+		!strncmp(lopt, gretl_opts[i].longopt, len)) {
+		opt = gretl_opts[i].o;
+		break;
+	    }
+	} 
+    }   
+
     return opt;
 }
   
@@ -294,6 +335,7 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 
 	sscanf(p + 2, "%31s", longopt);
 	match = valid_long_opt(ci, longopt);
+
 	if (match > 0) {
 	    ret |= match;
 	    gretl_delete(p, 0, 2 + strlen(longopt));
