@@ -50,7 +50,10 @@ enum {
     P_TOBIT,
     P_GARCH,
     P_AUDIO,
-    P_URCDIST
+    P_URCDIST,
+    P_KERNEL,
+    P_FRACTAL,
+    P_POISSON
 } plugin_codes;
 
 struct plugin_info {
@@ -89,7 +92,10 @@ struct plugin_info plugins[] = {
     { P_TOBIT,           "tobit" },
     { P_GARCH,           "garch" },
     { P_AUDIO,           "audio" },
-    { P_URCDIST,         "urcdist" }
+    { P_URCDIST,         "urcdist" },
+    { P_KERNEL,          "kernel" },
+    { P_FRACTAL,         "fractals" },
+    { P_POISSON,         "poisson" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -157,6 +163,7 @@ struct plugin_function plugin_functions[] = {
     { "logistic_estimate", P_LOGISTIC },
     { "tobit_estimate",    P_TOBIT },
     { "garch_model",       P_GARCH },
+    { "poisson_estimate",  P_POISSON },
 
     /* audio graphs etc */
     { "midi_play_graph",   P_AUDIO },
@@ -165,6 +172,12 @@ struct plugin_function plugin_functions[] = {
     /* MacKinnon Dickey-Fuller p-values */
     { "mackinnon_pvalue",  P_URCDIST },
 
+    /* kernel density estimation */
+    { "kernel_density",    P_KERNEL },
+
+    /* Hurst exponent estimation */
+    { "hurst_exponent",    P_FRACTAL },
+
     /* sentinel */
     { NULL, 0 }
 };
