@@ -118,8 +118,9 @@ void lib_logo (void)
 
 /**
  * gui_script_logo:
+ * @prn: gretl printing struct.
  *
- * header for script output in gui program
+ * Print to @prn a header for script output in gui program.
  */
 
 void gui_script_logo (PRN *prn)
@@ -227,19 +228,28 @@ void printcorr (const CORRMAT *corrmat, const DATAINFO *pdinfo,
 
 static void print_freq_test (const FREQDIST *freq, PRN *prn)
 {
+    double pval = NADBL;
+
     if (freq->dist == NORMAL) {
+	pval = chisq(freq->test, 2);
 	pprintf(prn, "\n%s:\n", 
 		_("Test for null hypothesis of normal distribution"));
 	pprintf(prn, "%s(2) = %.3f %s %.5f\n", 
 		_("Chi-square"), freq->test, 
-		_("with p-value"), chisq(freq->test, 2));
+		_("with p-value"), pval);
     } else if (freq->dist == GAMMA) {
+	pval = 2.0 * normal(fabs(freq->test));
 	pprintf(prn, "\n%s:\n", 
 		_("Test for null hypothesis of gamma distribution"));
-	pprintf(prn, "z = %.3f %s %.5f\n", 
-		freq->test, 
-		_("with p-value"), 2.0 * normal(fabs(freq->test)));
-    }	    
+	pprintf(prn, "z = %.3f %s %.5f\n", freq->test, 
+		_("with p-value"), pval);
+    }	
+
+    if (!na(pval)) {
+	record_test_result(freq->test, pval, 
+			   (freq->dist == NORMAL)? 
+			   "normality" : "gamma");
+    }
 }
 
 /**
@@ -497,22 +507,28 @@ int outcovmx (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 
     if (pmod->ci == TSLS) {
 	k = 2;
-	while (pmod->list[k++] != LISTSEP) nbeta++;
+	while (pmod->list[k++] != LISTSEP) {
+	    nbeta++;
+	}
     } else if (pmod->ci == ARMA || pmod->ci == GARCH) {
-	nbeta = 1 + pmod->list[1] + pmod->list[2] + pmod->list[0] - 4;
+	nbeta = pmod->ifc + pmod->list[1] + pmod->list[2] + pmod->list[0] - 4;
     } else {
 	nbeta = pmod->list[0] - 1;
     }
 
     tmplist = gretl_list_new(nbeta);
-    if (tmplist == NULL) return E_ALLOC;
+    if (tmplist == NULL) {
+	return E_ALLOC;
+    }
 
     for (k=1; k<=tmplist[0]; k++) {
 	tmplist[k] = pmod->list[k+1];
     }
 
     if (pmod->vcv == NULL) {
-	if (makevcv(pmod)) return E_ALLOC;
+	if (makevcv(pmod)) {
+	    return E_ALLOC;
+	}
     }
 
     text_print_matrix(pmod->vcv, tmplist, pmod, pdinfo, prn);  
@@ -692,9 +708,11 @@ static void fit_resid_head (const FITRESID *fr,
     ntodate(mdate2, fr->t2, pdinfo);
 
     pprintf(prn, _("Model estimation range: %s - %s"), mdate1, mdate2);
-    pprintf(prn, " (n = %d)\n", fr->nobs);
+    pprintf(prn, " (n = %d)\n", fr->real_nobs);
 
-    pprintf(prn, _("Standard error of residuals = %f\n"), fr->sigma);
+    if (!na(fr->sigma)) {
+	pprintf(prn, _("Standard error of residuals = %f\n"), fr->sigma);
+    }
     
     pprintf(prn, "\n     %s ", _("Obs"));
 
@@ -734,10 +752,14 @@ void gretl_printxs (double xx, int n, int ci, PRN *prn)
     int ls;
     char s[32];
 
-    printxx(xx, s, ci);
+    if (na(xx)) {
+	*s = '\0';
+    } else {
+	printxx(xx, s, ci);
+    }
     ls = strlen(s);
     pputc(prn, ' ');
-    bufspace(n-3-ls, prn);
+    bufspace(n - 3 - ls, prn);
     pputs(prn, s);
 }
 
@@ -1011,7 +1033,7 @@ void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 /**
  * printdata:
  * @list: list of variables to print.
- * @pZ: pointer to data matrix.
+ * @Z: data matrix.
  * @pdinfo: data information struct.
  * @oflag: if = OPT_O, print the data by observation (series in columns);
  *          if = OPT_T, print the data to 10 significant digits.
@@ -1239,10 +1261,9 @@ int text_print_fcast_with_errs (const FITRESID *fr,
 				double ***pZ, DATAINFO *pdinfo, PRN *prn,
 				int plot)
 {
-    int pv, err = 0;
-    int t;
+    int t, pv, err = 0;
     double *maxerr;
-    int time_series = (pdinfo->time_series == TIME_SERIES);
+    int time_series = (pdinfo->structure == TIME_SERIES);
 
     maxerr = malloc(fr->nobs * sizeof *maxerr);
     if (maxerr == NULL) {
@@ -1260,11 +1281,11 @@ int text_print_fcast_with_errs (const FITRESID *fr,
 
     for (t=0; t<fr->nobs; t++) {
 	print_obs_marker(t + fr->t1, pdinfo, prn);
-	if (na(fr->actual[t]) || na(fr->fitted[t])) {
+	gretl_printxs(fr->actual[t], 15, PRINT, prn);
+	if (na(fr->fitted[t])) {
 	    pputc(prn, '\n');
 	    continue;
 	}
-	gretl_printxs(fr->actual[t], 15, PRINT, prn);
 	gretl_printxs(fr->fitted[t], 15, PRINT, prn);
 	gretl_printxs(fr->sderr[t], 15, PRINT, prn);
 	maxerr[t] = fr->tval * fr->sderr[t];
@@ -1407,7 +1428,7 @@ PRN *gretl_print_new (int prncode, const char *fname)
 	
     else if (prncode == GRETL_PRINT_FILE) {
 	prn->buf = NULL;
-	prn->fp = fopen(fname, "w");
+	prn->fp = gretl_fopen(fname, "w");
 	if (prn->fp == NULL) {
 	    fprintf(stderr, _("gretl_prn_new: couldn't open %s\n"), fname);
 	    free(prn);
