@@ -36,6 +36,36 @@ static void getdf (const char *str);
 
 const char negval[] = N_("\nEnter x value (value < 0 will exit menu): "); 
 
+/**
+ * x_factorial:
+ * @x: input value.
+ * 
+ * Returns: the factorial of int(x), cast to a double, or
+ * NADBL on failure.
+ *
+ */
+
+double x_factorial (double x)
+{
+    double fact;
+    int n = x;
+
+    if (x < 0.0) {
+	fact = NADBL;
+    } else if (x > 12.0) {
+	fact = cephes_gamma(1.0 + x);
+    } else if (n == 0) {
+	fact = 1.0;
+    } else {
+	fact = n;
+	while (--n > 1) {
+	    fact *= n;
+	}
+    }
+
+    return fact;
+}
+
 /**
  * tcrit95:
  * @df: degrees of freedom.
@@ -66,23 +96,43 @@ double rhocrit95 (int n)
     return sqrt(x*x / (x*x - 2 + n));
 }
 
+/**
+ * gaussprob:
+ * @x: the cutoff point in the distribution.
+ * 
+ * Returns: the probability that z is greater than @x
+ * (two-sided, using absolute values).
+ */
+
+double gaussprob (double x)
+{
+    return 2.0 * (1.0 - ndtr(fabs(x)));
+}
+
 /**
  * tprob:
  * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
  * 
- * Returns: the probability that t(@df) is greater than @x.
- *
+ * Returns: the probability that t(@df) is greater than @x
+ * (two-sided, using absolute values).
  */
 
 double tprob (double x, int df)
 {
-    double xx;
+    if (df <= 0) {
+	return -1.0;
+    } else {
+#if 0
+	return fdist(x * x, 1, df);
+#else
+	double s = stdtr(df, fabs(x));
+	double ret = 2.0 * (1.0 - s);
 
-    if (df <= 0) return -1.0;
-    xx = x*x;
-    return fdist(xx, 1, df);
-    /* return 1.0 - stdtr(df, x); */
+	if (ret < 0.0) ret = 0.0;
+	return ret;
+#endif
+    }
 }
 
 /**
@@ -100,10 +150,15 @@ double fdist (double x, int dfn, int dfd)
 {
     double ret = 1.0;
 
-    if (dfn < 0 || dfd < 0) {
-	ret = -1.0;
+    if (dfn < 1 || dfd < 1) {
+        ret = -1.0;
     } else if (x > 0.0) {
-	ret = fdtrc(dfn, dfd, x);
+	if (0 && dfd > 300) {
+	    /* FIXME igamc underflow ?? */
+	    ret = chisq(x * dfn, dfn);
+	} else {
+	    ret = fdtrc(dfn, dfd, x);
+	}
     }
 
     return ret;
@@ -125,7 +180,7 @@ double chisq (double x, int df)
     if (df < 0) {
 	ret = -1.0;
     } else if (x > 0.0) {
-	ret = 1.0 - chdtr(df, x);
+	ret = chdtrc(df, x);
     }
 
     return ret;
@@ -588,40 +643,20 @@ static void getdf (const char *str)
 
 double f_crit_a (double a, int df1, int df2)
 {
-    double add = 0.5, x = 1.0;
-
-    if (fdist(x, df1, df2) < a) return NADBL;
-
-    while (add > 1.0e-7) {
-	while (fdist(x, df1, df2) > a) x += add; 
-	if (x > add) x -= add;
-	if (add == 0.5) {
-	    add /= 5.0;
-	} else {
-	    add /= 10.0;
-	}
+    if (df1 < 1 || df2 < 1 || a < 0.0) {
+	return NADBL;
+    } else {
+	return fdtri(df1, df2, a);
     }
-
-    return x;
 }
 
 static double chi_crit_a (double a, int df)
 {
-    double add = 0.5, x = 1.0;
-
-    if (chisq(x, df) < a) return NADBL;
-
-    while (add > 1.0e-7) {
-	while (chisq(x, df) > a) x += add; 
-	if (x > add) x -= add;
-	if (add == 0.5) {
-	    add /= 5.0;
-	} else {
-	    add /= 10.0;
-	}
+    if (df < 1 || a < 0.0) {
+	return NADBL;
+    } else {
+	return chdtri(df, a);
     }
-
-    return x;
 }
 
 /* ........................................................ */
