@@ -49,12 +49,16 @@ int slashpos (const char *str);
 
 void delchar (int c, char *str);
 
-int haschar (char c, const char *str);
+int haschar (char c, const char *s);
 
 int lastchar (char c, const char *s);
 
 int ends_with_backslash (const char *s);
 
+char *gretl_strdup (const char *src);
+
+char *gretl_strndup (const char *src, size_t n);
+
 char *charsub (char *str, char find, char repl);
 
 void lower (char *str);
@@ -69,7 +73,9 @@ int get_base (char *targ, const char *src, char c);
 
 int top_n_tail (char *str);
 
-void compress_spaces (char *str);
+char *tailstrip (char *str);
+
+char *compress_spaces (char *s);
 
 char *space_to_score (char *s);
 
@@ -91,13 +97,32 @@ char *colonize_obs (char *obs);
 
 #ifdef ENABLE_NLS
 char *iso_gettext (const char *msgid);
+
 char *maybe_iso_gettext (const char *msgid);
+
+void set_gretl_charset (const char *s);
+
+const char *get_gretl_charset (void);
+
+const char *get_gnuplot_charset (void);
+
+int use_latin_2 (void);
+
+char *sprint_l2_to_html (char *targ, const char *s, size_t len);
+
+char *sprint_html_to_l2 (char *targ, const char *s);
+
+int print_as_html (const char *s, FILE *fp);
+
+int print_as_locale (const char *s, FILE *fp);
+
 int get_utf_width (const char *str, int width);
+
 # define UTF_WIDTH(s, w) get_utf_width(s, w) 
 int get_utf_width (const char *str, int width);
 #else
 # define UTF_WIDTH(s, w)    w
-#endif  
+#endif  /* ENABLE_NLS */
 
 const char *print_time (const time_t *timep);
 
@@ -109,6 +134,8 @@ char *iso_to_ascii (char *s);
 
 char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo);
 
+int fix_varname_duplicates (DATAINFO *pdinfo);
+
 char *append_dir (char *fname, const char *dir);
 
 int build_path (const char *dir, const char *fname, char *path, 
