@@ -570,7 +570,7 @@ maybe_reconstitute_panel (const DATAINFO *pdinfo,
 	gretlopt opt = OPT_C;
 
 	sprintf(line, "setobs %d 1.1", pd);
-	if (pdinfo->time_series == STACKED_TIME_SERIES) {
+	if (pdinfo->structure == STACKED_TIME_SERIES) {
 	    opt = OPT_S;
 	} 
 	set_obs(line, subinfo, opt);
@@ -824,18 +824,38 @@ int restrict_sample (const char *line,
 
 /* .......................................................... */
 
-static int get_sample_increment (const char *str)
+static int 
+get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo)
 {
-    if ((*str == '-' || *str == '+') && isdigit((unsigned char) str[1])) {
-	return atoi(str);
+    int v, ret = -1;
+
+    if (*s == '-' || *s == '+') {
+	/* incremental form */
+	int incr = 0;
+
+	if (isdigit((unsigned char) s[1])) {
+	    incr = atoi(s);
+	} else {
+	    v = varindex(pdinfo, s + 1);
+	    if (v < pdinfo->v) {
+		incr = (int) Z[v][0];
+		if (*s == '-') {
+		    incr = -incr;
+		}
+	    }
+	}
+	ret = pdinfo->t1 + incr;
+    } else {
+	/* absolute form */
+	ret = get_t_from_obs_string(s, Z, pdinfo);
     }
 
-    return 0;
+    return ret;
 }
 
 /* .......................................................... */
 
-int set_sample (const char *line, DATAINFO *pdinfo)
+int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 {
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
     char cmd[5], newstart[OBSLEN], newstop[OBSLEN];
@@ -844,6 +864,15 @@ int set_sample (const char *line, DATAINFO *pdinfo)
 
     nf = count_fields(line);
 
+#ifdef SUBDEBUG
+    fprintf(stderr, "set_sample: line='%s', nf=%d, pdinfo=%p\n", 
+	    line, nf, (void *) pdinfo);
+    if (pdinfo != NULL) {
+	fprintf(stderr, "pdinfo->v = %d, pdinfo->n = %d\n",
+		pdinfo->v, pdinfo->n);
+    }
+#endif
+
     if (nf == 3 && pdinfo->n == 0) {
 	/* database special */
 	return db_set_sample(line, pdinfo);
@@ -856,17 +885,8 @@ int set_sample (const char *line, DATAINFO *pdinfo)
 	    strcpy(gretl_errmsg, _("error reading smpl line"));
 	    return 1;
 	} else {
-	    new_t1 = get_sample_increment(newstart);
-	    if (new_t1) {
-		new_t1 = pdinfo->t1 + new_t1;
-		if (new_t1 < 0) {
-		   strcpy(gretl_errmsg, _("Observation number out of bounds"));
-		} 
-	    } else {
-		new_t1 = dateton(newstart, pdinfo);
-	    }
-	    if (*gretl_errmsg) return 1;
-	    if (new_t1 < 0 || new_t1 > pdinfo->n) {
+	    new_t1 = get_sample_limit(newstart, Z, pdinfo);
+	    if (new_t1 < 0 || new_t1 >= pdinfo->n) {
 		strcpy(gretl_errmsg, _("error in new starting obs"));
 		return 1;
 	    }
@@ -875,34 +895,23 @@ int set_sample (const char *line, DATAINFO *pdinfo)
 	}
     }
 
+    /* now we're looking at nf = 3 case */
+
     if (sscanf(line, "%4s %10s %10s", cmd, newstart, newstop) != 3) {
 	strcpy(gretl_errmsg, _("error reading smpl line"));
 	return 1;
     }
 
     if (strcmp(newstart, ";")) {
-	new_t1 = get_sample_increment(newstart);
-	if (new_t1) {
-	    new_t1 = pdinfo->t1 + new_t1;
-	    if (new_t1 < 0) {
-		strcpy(gretl_errmsg, _("Observation number out of bounds"));
-	    }	
-	} else {
-	    new_t1 = dateton(newstart, pdinfo);
-	}
-	if (*gretl_errmsg) {
+	new_t1 = get_sample_limit(newstart, Z, pdinfo);
+	if (new_t1 < 0 || new_t1 >= pdinfo->n) {
+	    strcpy(gretl_errmsg, _("error in new starting obs"));
 	    return 1;
-	}
+	}	
     }
 
     if (strcmp(newstop, ";")) {
-	new_t2 = get_sample_increment(newstop);
-	if (new_t2) {
-	    new_t2 = pdinfo->t2 + new_t2;
-	} else {
-	    new_t2 = dateton(newstop, pdinfo);
-	}
-	if (*gretl_errmsg) return 1;
+	new_t2 = get_sample_limit(newstop, Z, pdinfo);
 	if (new_t2 < 0 || new_t2 >= pdinfo->n) {
 	    strcpy(gretl_errmsg, _("error in new ending obs"));
 	    return 1;
@@ -1021,7 +1030,7 @@ static int make_smpl_mask (double ***pZ, DATAINFO *pdinfo)
 
    If the call is a prelude to _replacing_ an existing
    sample restriction, the "opt" parameter should equal
-   OPT_C (think "replaCe").
+   OPT_C (think "replaCe"!).
 */
 
 int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
@@ -1303,7 +1312,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	modinfo->sd0 = fullinfo->sd0;
 	strcpy(modinfo->stobs, fullinfo->stobs);
 	strcpy(modinfo->endobs, fullinfo->endobs);
-	modinfo->time_series = fullinfo->time_series;
+	modinfo->structure = fullinfo->structure;
     }
 
     pmod->dataset->Z = modZ;
