@@ -260,8 +260,8 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     } else {
 	tex_dcolumn_double(pmod->sderr[v], sderr);
 	sprintf(tratio, "%.4f", pmod->coeff[v] / pmod->sderr[v]);
-	sprintf(pval, "%.4f", tprob(pmod->coeff[v] / pmod->sderr[v], 
-				    pmod->dfd));
+	sprintf(pval, "%.4f", coeff_pval(pmod, pmod->coeff[v] / pmod->sderr[v], 
+					 pmod->dfd));
     }    
 
     *tmp = 0;
@@ -326,7 +326,7 @@ static int make_texfile (int ID, int equation,
 		(equation)? "equation" : "model", ID);
     }
 
-    fp = fopen(texfile, "w");
+    fp = gretl_fopen(texfile, "w");
     if (fp == NULL) return 1;
 
     gretl_print_attach_file(prn, fp);
@@ -363,7 +363,7 @@ void set_gretl_tex_preamble (void)
 
     /* first choice: localized preamble file */
     sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), localtex);
-    fp = fopen(tex_preamble_file, "r");
+    fp = gretl_fopen(tex_preamble_file, "r");
     if (fp == NULL) {
 	tex_preamble_file[0] = '\0';
     } else {
@@ -374,7 +374,7 @@ void set_gretl_tex_preamble (void)
 
     /* preamble file on disk */
     sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), gretltex);
-    fp = fopen(tex_preamble_file, "r");
+    fp = gretl_fopen(tex_preamble_file, "r");
     if (fp == NULL) {
 	tex_preamble_file[0] = '\0';
     } else {
@@ -388,7 +388,7 @@ void gretl_tex_preamble (PRN *prn, int ams)
     int userfile = 0;
 
     if (tex_preamble_file[0] != '\0') {
-	fp = fopen(tex_preamble_file, "r");
+	fp = gretl_fopen(tex_preamble_file, "r");
 	if (fp != NULL) {
 	    char line[128];
 
@@ -439,9 +439,18 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     double tstat;
     char tmp[48];
     int i, nc = pmod->ncoeff;
-    int split = (nc > MAXCOEFF);
+    int split = 0, offvar = 0;
     int cchars = 0, ccount = 0;
 
+    if (pmod->ci == POISSON) {
+	offvar = gretl_model_get_int(pmod, "offset_var");
+	if (offvar > 0) {
+	    nc++;
+	}
+    }
+
+    split = (nc > MAXCOEFF);
+
     if (standalone) {
 	gretl_tex_preamble(prn, 1);
     } else{
@@ -456,24 +465,36 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 
     /* dependent variable */
     *tmp = '\0';
-    if (pmod->ci == ARMA || pmod->ci == GARCH) {
+    if (pmod->ci == POISSON) {
+	char vname[24];
+
+	tex_escape(vname, pdinfo->varname[pmod->list[1]]);
+	sprintf(tmp, "log(%s)", vname);
+    } else if (pmod->ci == ARMA || pmod->ci == GARCH) {
 	tex_escape(tmp, pdinfo->varname[pmod->list[4]]);
     } else {
 	tex_escape(tmp, pdinfo->varname[pmod->list[1]]);
     }
+
     pprintf(prn, "\\widehat{\\rm %s} %s= \n", tmp, (split? "&" : ""));
 
-    if (pmod->ci == GARCH) 
+    if (pmod->ci == GARCH) {
 	nc -= (1 + pmod->list[1] + pmod->list[2]);
+    }
 
     /* coefficients times indep vars */
     for (i=0; i<nc; i++) {
-	tstat = pmod->coeff[i] / pmod->sderr[i];
-	pprintf(prn, "%s\\underset{(%.3f)}{", 
-		(pmod->coeff[i] < 0.0)? "-" :
-		(i > 0)? "+" : "", tstat);
-	tex_print_float(pmod->coeff[i], prn);
-	pputc(prn, '}');
+	if (offvar > 0 && i == nc - 1) {
+	    pputc(prn, '+');
+	    tex_print_float(1.0, prn);
+	} else {
+	    tstat = pmod->coeff[i] / pmod->sderr[i];
+	    pprintf(prn, "%s\\underset{(%.3f)}{", 
+		    (pmod->coeff[i] < 0.0)? "-" :
+		    (i > 0)? "+" : "", tstat);
+	    tex_print_float(pmod->coeff[i], prn);
+	    pputc(prn, '}');
+	}
 	if (i > 0 || pmod->ifc == 0) {
 	    pputs(prn, "\\,");
 	    if (pmod->ci == ARMA) {
@@ -484,6 +505,10 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 		cchars += strlen(pmod->params[i+1]);
 		tex_garch_coeff_name(tmp, pmod->params[i+1], 1);
 		pputs(prn, tmp);
+	    } else if (offvar > 0 && i == nc - 1) {
+		cchars += strlen(pdinfo->varname[offvar]);
+		tex_escape(tmp, pdinfo->varname[offvar]);
+		pprintf(prn, "\\mbox{%s}", tmp);
 	    } else {
 		cchars += strlen(pdinfo->varname[pmod->list[i+2]]);
 		tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
