@@ -20,22 +20,50 @@
 #include "libgretl.h"
 #include "gretl_private.h"
 
+#undef TRDEBUG
+
 /* newlag is a library global, used when auto-generating a lag in the
    context of reading a regression list (interact.c)
 */
 int newlag;
 
-int vars_identical (const double *x, const double *y, int n)
+enum {
+    VARS_IDENTICAL,
+    X_HAS_MISSING,
+    Y_HAS_MISSING,
+    VARS_DIFFER
+} varcomp_codes;
+
+static int check_vals (const double *x, const double *y, int n)
 {
+    int ret = VARS_IDENTICAL;
     int t;
 
     for (t=0; t<n; t++) {
 	if (floatneq(x[t], y[t])) {
-	    return 0;
+	    if (na(x[t]) && !na(y[t])) {
+		if (ret == VARS_IDENTICAL || ret == X_HAS_MISSING) {
+		    ret = X_HAS_MISSING;
+		} else {
+		    ret = VARS_DIFFER;
+		}
+	    } else if (!na(x[t]) && na(y[t])) {
+		if (ret == VARS_IDENTICAL || ret == Y_HAS_MISSING) {
+		    ret = Y_HAS_MISSING;
+		} else {
+		    ret = VARS_DIFFER;
+		}
+	    } else {
+		ret = VARS_DIFFER;
+	    }
+	    
+	}
+	if (ret == VARS_DIFFER) {
+	    break;
 	}
     }
 
-    return 1;
+    return ret;
 }
 
 static int 
@@ -67,10 +95,12 @@ make_transform_varname (char *vname, const char *orig, int cmd,
     return 0;
 }
 
-static void
+static int
 make_transform_label (char *label, const char *parent,
 		      int cmd, int lag)
 {
+    int err = 0;
+
     if (cmd == DIFF) {
 	sprintf(label, _("= first difference of %s"), parent);
     } else if (cmd == LDIFF) {
@@ -81,7 +111,11 @@ make_transform_label (char *label, const char *parent,
 	sprintf(label, _("= %s squared"), parent);
     } else if (cmd == LAGS) {
 	sprintf(label, "= %s(t - %d)", parent, lag);
-    } 
+    } else {
+	err = 1;
+    }
+
+    return err;
 }
 
 static void make_xp_varname (char *vname, const char *v1, 
@@ -150,7 +184,7 @@ static int get_lag (int v, int lag, double *lagvec,
 	lagvec[t] = NADBL;
     }
 
-    if (pdinfo->time_series == STACKED_CROSS_SECTION) {
+    if (pdinfo->structure == STACKED_CROSS_SECTION) {
 	/* needs rather special handling */
 	for (t=t1; t<=pdinfo->t2; t++) { 
 	    lt = t - lag * pdinfo->pd;
@@ -179,7 +213,7 @@ static int get_lag (int v, int lag, double *lagvec,
     }
 
     /* post-process missing panel values */
-    if (pdinfo->time_series == STACKED_TIME_SERIES) {
+    if (pdinfo->structure == STACKED_TIME_SERIES) {
 	char *p, obs[OBSLEN];
 	int j;
 
@@ -233,12 +267,12 @@ static int get_diff (int v, double *diffvec, int ci,
     t1 = (pdinfo->t1 > 1)? pdinfo->t1 : 1;
 
     for (t=t1; t<=pdinfo->t2; t++) {
-	if (pdinfo->time_series == STACKED_TIME_SERIES &&
+	if (pdinfo->structure == STACKED_TIME_SERIES &&
 	    panel_unit_first_obs(t, pdinfo)) {
 	    continue;
 	}
 	x0 = Z[v][t];
-	if (pdinfo->time_series == STACKED_CROSS_SECTION) {
+	if (pdinfo->structure == STACKED_CROSS_SECTION) {
 	    x1 = (t - pdinfo->pd >= 0)? Z[v][t-pdinfo->pd] : NADBL;
 	} else {
 	    x1 = Z[v][t-1];
@@ -294,8 +328,25 @@ check_add_transform (int vnum, const double *x,
 
     if (vnum < pdinfo->v) {
 	/* a variable of this name already exists */
-	if (vars_identical(x, (*pZ)[vnum], pdinfo->n)) {
-	    /* and it is just what we want */
+	int chk = check_vals(x, (*pZ)[vnum], pdinfo->n);
+
+	/* heuristic: we'll assume that if the variables have the same
+	   name, and differ only in respect of one being more complete
+	   than the other (having valid values where the other has
+	   missing values), then we should not create a new variable,
+	   but rather use the more complete series.
+
+	   Watch out for cases where this is not the desired behavior!
+	*/
+
+	if (chk == VARS_IDENTICAL) {
+	    ret = VAR_EXISTS_OK;
+	} else if (chk == X_HAS_MISSING) { /* is this right? */
+	    ret = VAR_EXISTS_OK;
+	} else if (chk == Y_HAS_MISSING) {
+	    for (t=0; t<pdinfo->n; t++) {
+		(*pZ)[vnum][t] = x[t];
+	    }
 	    ret = VAR_EXISTS_OK;
 	} else {
 	    if (!strcmp(label, VARLABEL(pdinfo, vnum))) {
@@ -303,6 +354,7 @@ check_add_transform (int vnum, const double *x,
 		for (t=0; t<pdinfo->n; t++) {
 		    (*pZ)[vnum][t] = x[t];
 		}
+		ret = VAR_EXISTS_OK;
 	    } else {
 		/* labels do not match: problem */
 		ret = VARNAME_DUPLICATE;
@@ -559,24 +611,38 @@ int list_loggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
 
 int 
 real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
-		   int maxlag)
+		   int maxlag, int **lagnums)
 {
     int lagnum, l, i, v;
     int startlen;
+    int *record = NULL;
 
-    startlen = get_starting_length(list, pdinfo, 
-				   (maxlag > 9)? 3 : 2);
+    startlen = get_starting_length(list, pdinfo, (maxlag > 9)? 3 : 2);
     
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 	if (v == 0 || !pdinfo->vector[v]) {
 	    continue;
 	}
+	if (lagnums != NULL) {
+	    record = lagnums[i-1];
+	}
 	for (l=1; l<=maxlag; l++) {
 	    lagnum = get_transform(LAGS, v, l, pZ, pdinfo, startlen);
+#if TRDEBUG
+	    fprintf(stderr, "base var '%s', lag %d: lagnum = %d\n",
+		    pdinfo->varname[v], l, lagnum);
+#endif
 	    if (lagnum < 0) {
 		return 1;
 	    }
+#if TRDEBUG
+	    fprintf(stderr, "lag var name '%s', label '%s'\n",
+		    pdinfo->varname[lagnum], VARLABEL(pdinfo, lagnum));
+#endif
+	    if (record != NULL) {
+		record[l] = lagnum;
+	    }
 	}
     }
 
@@ -597,14 +663,20 @@ real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 int list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
 {
-    int maxlag = pdinfo->pd;
+    int maxlag;
+
+    if (pdinfo->pd < 52) {
+	maxlag = pdinfo->pd;
+    } else {
+	maxlag = 14;
+    } 
 
     /* play safe with panel data */
     if (dataset_is_panel(pdinfo)) {
 	maxlag = 1;
     }
 
-    return real_list_laggenr(list, pZ, pdinfo, maxlag);  
+    return real_list_laggenr(list, pZ, pdinfo, maxlag, NULL);  
 }
 
 /**
@@ -724,11 +796,27 @@ int lagvarnum (int v, int l, const DATAINFO *pdinfo)
 
     make_transform_label(label, pdinfo->varname[v], LAGS, l);
 
+#if TRDEBUG
+    fprintf(stderr, "Looking for lag var with label '%s'...\n", label);
+#endif
+
     for (i=1; i<pdinfo->v; i++) {
 	if (!strcmp(label, VARLABEL(pdinfo, i))) {
 	    return i;
 	}
     }
 
+    /* second attempt? */
+    sprintf(label, "= %s(-%d)", pdinfo->varname[v], l);
+    for (i=1; i<pdinfo->v; i++) {
+	if (strstr(VARLABEL(pdinfo, i), label)) {
+	    return i;
+	}
+    }
+
+#if TRDEBUG
+    fputs("*** not found\n", stderr);
+#endif
+
     return -1;
 }
