@@ -23,7 +23,7 @@
 #include "var.h"  
 #include "gretl_private.h"
 
-#undef VAR_DEBUG 
+#define VAR_DEBUG 0
 
 struct GRETL_VAR_ {
     int neqns;         /* number of equations in system */
@@ -45,12 +45,19 @@ struct var_resids {
     int t1, t2;
 };
 
+struct var_lists {
+    int *detvars;
+    int *stochvars;
+    int *reglist;
+    int *testlist;
+    int **lagvlist;
+};
+
 enum {
     VAR_PRINT_MODELS =      1 << 0,
     VAR_DO_FTESTS    =      1 << 1,
     VAR_PRINT_PAUSE  =      1 << 2,
-    VAR_IMPULSE_RESPONSES = 1 << 3,
-    VAR_SAVE =              1 << 4
+    VAR_IMPULSE_RESPONSES = 1 << 3
 } var_flags;
 
 #define TREND_FAILED 9999
@@ -75,10 +82,10 @@ static void pad_var_coeff_matrix (GRETL_VAR *var)
 
 static int 
 gretl_var_init (GRETL_VAR *var, int neqns, int order, const DATAINFO *pdinfo,
-		char flags)
+		int save)
 {
-    int i, j, err = 0;
     int rows = neqns * order;
+    int i, j, err = 0;
 
     var->neqns = neqns;
     var->order = order;
@@ -130,14 +137,16 @@ gretl_var_init (GRETL_VAR *var, int neqns, int order, const DATAINFO *pdinfo,
 	}
     } 
 
-    if (!err && (flags & VAR_SAVE)) {
-	int m = neqns * neqns;
+    if (!err && save) {
+	int m = neqns * neqns + neqns;
 	
-	if (order > 1) m += neqns;
-#ifdef VAR_DEBUG
+#if VAR_DEBUG
 	fprintf(stderr, "var->Fvals: allocating %d terms\n", m);
 #endif
 	var->Fvals = malloc(m  * sizeof *var->Fvals);
+	if (var->Fvals == NULL) {
+	    err = 1;
+	}
     }
 
     return err;
@@ -177,14 +186,14 @@ void gretl_var_free_unnamed (GRETL_VAR *var)
 }
 
 GRETL_VAR *gretl_var_new (int neqns, int order, const DATAINFO *pdinfo,
-			  char flags)
+			  int save)
 {
     GRETL_VAR *var;
 
     var = malloc(sizeof *var);
     if (var == NULL) return NULL;
 
-    if (gretl_var_init(var, neqns, order, pdinfo, flags)) {
+    if (gretl_var_init(var, neqns, order, pdinfo, save)) {
 	free(var);
 	return NULL;
     } 
@@ -200,7 +209,9 @@ static int gretl_var_do_error_decomp (GRETL_VAR *var)
     int i, j, err = 0;
 
     tmp = gretl_matrix_alloc(var->neqns, var->neqns);
-    if (tmp == NULL) err = E_ALLOC;
+    if (tmp == NULL) {
+	err = E_ALLOC;
+    }
 
     /* form e'e */
     if (!err && gretl_matrix_multiply_mod (var->E, GRETL_MOD_TRANSPOSE,
@@ -212,10 +223,12 @@ static int gretl_var_do_error_decomp (GRETL_VAR *var)
     /* divide by T (or use df correction?) to get sigma-hat.
        Note: RATS 4 uses straight T.
     */
-    gretl_matrix_divide_by_scalar(tmp, (double) var->n);
+    if (!err) {
+	gretl_matrix_divide_by_scalar(tmp, (double) var->n);
+    }
 
-#ifdef VAR_DEBUG
-    if (1) {
+#if VAR_DEBUG
+    if (!err) {
 	PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
 
 	gretl_matrix_print(tmp, "Sigma-hat from VAR system", prn);
@@ -244,7 +257,9 @@ static int gretl_var_do_error_decomp (GRETL_VAR *var)
 	}
     }
 
-    if (tmp != NULL) gretl_matrix_free(tmp);
+    if (tmp != NULL) {
+	gretl_matrix_free(tmp);
+    }
 
     /* we're done with the full-length residual series */
     gretl_matrix_free(var->E);
@@ -417,8 +432,11 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 		    pprintf(prn, "%#12.5g ", r);
 		}
 	    }
-	    if (TEX_PRN(prn)) pputs(prn, "\\\\\n");
-	    else pputc(prn, '\n');
+	    if (TEX_PRN(prn)) {
+		pputs(prn, "\\\\\n");
+	    } else {
+		pputc(prn, '\n');
+	    }
 	}
 
 	if (TEX_PRN(prn)) {
@@ -464,7 +482,9 @@ gretl_var_get_impulse_responses (GRETL_VAR *var, int targ, int shock,
     }
 
     resp = malloc(periods * sizeof *resp);
-    if (resp == NULL) return NULL;
+    if (resp == NULL) {
+	return NULL;
+    }
 
     rtmp = gretl_matrix_alloc(rows, var->neqns);
     if (rtmp == NULL) {
@@ -489,9 +509,9 @@ gretl_var_get_impulse_responses (GRETL_VAR *var, int targ, int shock,
 	    gretl_matrix_copy_values(rtmp, ctmp);
 	}
 
-	if (err) break;
-
-	resp[t] = gretl_matrix_get(rtmp, targ, shock);
+	if (!err) {
+	    resp[t] = gretl_matrix_get(rtmp, targ, shock);
+	}
     }
 
     gretl_matrix_free(rtmp);
@@ -773,58 +793,250 @@ static int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
     return index;
 }
 
-/* ...................................................................  */
-
-static void reset_list (int *list1, int *list2)
+static void var_lists_free (struct var_lists *vl)
 {
-    int i;
-    
-    for (i=2; i<=list1[0]; i++) {
-	list1[i] = list2[i];
+    if (vl->lagvlist != NULL && vl->stochvars != NULL) {
+	int i, ns = vl->stochvars[0];
+
+	for (i=0; i<ns; i++) {
+	    free(vl->lagvlist[i]);
+	}
+	free(vl->lagvlist);
     }
+
+    free(vl->detvars);
+    free(vl->stochvars);
+    free(vl->reglist);
+    free(vl->testlist);
 }
 
-/* parse varlist (for a VAR) and determine how long the augmented list
-   will be, once all the appropriate lag terms are inserted
+static int **lagvlist_construct (int nstoch, int order)
+{
+    int **lvlist;
+    int i, j;
+
+    lvlist = malloc(nstoch * sizeof *lvlist);
+    if (lvlist == NULL) {
+	return NULL;
+    }
+
+    for (i=0; i<nstoch; i++) {
+	lvlist[i] = malloc((order + 1) * sizeof **lvlist);
+	if (lvlist[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(lvlist[j]);
+	    }
+	    free(lvlist);
+	    lvlist = NULL;
+	}
+	lvlist[i][0] = order;
+    }
+
+    return lvlist;
+}
+
+static int var_lists_init (struct var_lists *vl,
+			   int ndet, int nstoch, 
+			   int order)
+{
+    int nreg = 1 + ndet + nstoch * order;
+    int ntest = nstoch * 2 + ndet;
+
+    vl->detvars = NULL;
+    vl->stochvars = NULL;
+    vl->reglist = NULL;
+    vl->testlist = NULL;
+    vl->lagvlist = NULL;
+
+    vl->detvars = malloc((ndet + 1) * sizeof *vl->detvars);
+    vl->stochvars = malloc((nstoch + 1) * sizeof *vl->stochvars);
+    vl->reglist = malloc((nreg + 1) * sizeof *vl->reglist);
+    vl->testlist = malloc((ntest + 1) * sizeof *vl->testlist);
+
+    if (vl->detvars == NULL || vl->stochvars == NULL ||
+	vl->reglist == NULL || vl->testlist == NULL) {
+	goto bailout;
+    }
+
+    vl->detvars[0] = ndet;
+    vl->stochvars[0] = nstoch;
+    vl->reglist[0] = nreg;
+    vl->testlist[0] = ntest;
+
+    vl->lagvlist = lagvlist_construct(nstoch, order);
+    if (vl->lagvlist == NULL) {
+	goto bailout;
+    }
+
+    return 0;
+    
+ bailout:
+
+    var_lists_free(vl);
+
+    return E_ALLOC;
+}
+
+int var_max_order (const int *list, const DATAINFO *pdinfo)
+{
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int nstoch = 0, ndet = 0;
+    int gotsep = 0;
+    int order = 1;
+    int i;
+
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] == LISTSEP) {
+	    gotsep = 1;
+	    continue;
+	}
+	if (!gotsep) {
+	    nstoch++;
+	} else {
+	    ndet++;
+	}
+    }
+
+    order = (T - ndet) / nstoch;
+
+    while (order > 0) {
+	int t1 = (order > pdinfo->t1)? order : pdinfo->t1;
+
+	T = pdinfo->t2 - t1 + 1;
+	if (nstoch * order + ndet > T) {
+	    order--;
+	} else {
+	    break;
+	}
+    }
+
+    return order - 1;
+}
+
+/* Given an incoming regression list, separate it into deterministic
+   components (constant, trend, dummy variables) and stochastic
+   components, and construct a list for each sort of variable.  Also
+   allocate a VAR regression list that is long enough to hold the
+   deterministic vars plus order lags of each stochastic var.
 */
 
-static int 
-get_var_listlen (int *varlist, char *detlist, 
-		 int order, int *depvars,
-		 double **Z, const DATAINFO *pdinfo)
+static int organize_var_lists (const int *list, const double **Z,
+			       const DATAINFO *pdinfo, int order,
+			       struct var_lists *vlists)
 {
-    int i, j = 1, v = 1;
+    int ndet = 0, nstoch = 0;
     int gotsep = 0;
+    char *d;
+    int i, j, k, li;
+    
+    d = calloc(list[0] + 1, 1);
+    if (d == NULL) {
+	return E_ALLOC;
+    }
 
-    depvars[0] = 0;
-
-    for (i=1; i<=varlist[0]; i++) {
-	if (varlist[i] == LISTSEP) {
+    /* figure out the lengths of the lists */
+    for (i=1; i<=list[0]; i++) {
+	li = list[i];
+	if (li == LISTSEP) {
 	    gotsep = 1;
 	    continue;
 	}
 	if (gotsep || 
-	    strcmp(pdinfo->varname[varlist[i]], "time") == 0 ||
-	    strcmp(pdinfo->varname[varlist[i]], "const") == 0 ||
-	    isdummy(Z[varlist[i]], pdinfo->t1, pdinfo->t2)) {
-	    v++;
-	    detlist[j] = 1;
+	    !strcmp(pdinfo->varname[li], "const") ||	   
+	    !strcmp(pdinfo->varname[li], "time") ||
+	    isdummy(Z[li], pdinfo->t1, pdinfo->t2)) {
+	    d[i] = 1;
+	    ndet++;
 	} else {
-	    v += order;
-	    detlist[j] = 0;
-	    depvars[0] += 1;
-	    depvars[depvars[0]] = varlist[i];
+	    nstoch++;
 	}
-	varlist[j++] = varlist[i];
     }
 
-    if (gotsep) {
-	varlist[0] -= 1;
+    /* check for degrees of freedom */
+    if (nstoch * order + ndet > pdinfo->t2 - pdinfo->t1 + 1) {
+	free(d);
+	return E_DF;
     }
 
-    detlist[0] = varlist[0];
+    /* allocate the lists */
+    if (var_lists_init(vlists, ndet, nstoch, order)) {
+	free(d);
+	return E_ALLOC;
+    }
 
-    return v;
+    /* fill out the detvars and stochvars lists */
+    j = k = 1;
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] != LISTSEP) {
+	    if (d[i]) {
+		vlists->detvars[j++] = list[i];
+	    } else {
+		vlists->stochvars[k++] = list[i];
+	    }
+	}
+    }
+
+    free(d);
+
+#if VAR_DEBUG
+    printlist(vlists->detvars, "deterministic vars");
+    printlist(vlists->stochvars, "stochastic vars");
+#endif
+
+    return 0;
+}
+
+/* compose a VAR regression list: it may be complete, or one variable
+   may be omitted (to run an F-test), or the order may be one less
+   than the full VAR order (again, for an F-test)
+*/
+
+static int
+compose_varlist (struct var_lists *vl, int depvar, int order, int omit, 
+		 const DATAINFO *pdinfo)
+{
+    int l0 = 1 + vl->detvars[0] + order * vl->stochvars[0];
+    int i, j, pos;
+    int err = 0;
+
+    if (omit) {
+	l0 -= order;
+    } 
+
+    vl->reglist[0] = l0;
+    vl->reglist[1] = depvar;
+
+    pos = 2;
+    for (i=1; i<=vl->stochvars[0]; i++) {
+	if (i != omit) {
+	    /* insert order lags of the given var */
+	    for (j=1; j<=order; j++) {
+		vl->reglist[pos++] = vl->lagvlist[i-1][j];
+	    }
+	}
+    }
+
+    /* append the deterministic vars */
+    for (i=1; i<=vl->detvars[0]; i++) {
+	vl->reglist[pos++] = vl->detvars[i];
+    }
+
+    /* now build the test list (to screen missing values) */
+    pos = 1;
+    for (i=1; i<=vl->stochvars[0]; i++) {
+	vl->testlist[pos++] = vl->stochvars[i];
+	vl->testlist[pos++] = vl->lagvlist[i-1][order];
+    }
+    for (i=1; i<=vl->detvars[0]; i++) {
+	vl->testlist[pos++] = vl->detvars[i];
+    }    
+
+#if VAR_DEBUG
+    printlist(vl->reglist, "composed VAR list");
+    printlist(vl->testlist, "composed test list");
+#endif
+
+    return err;
 }
 
 /* ...................................................................  */
@@ -865,55 +1077,35 @@ static int add_model_data_to_var (GRETL_VAR *var, const MODEL *pmod, int k)
 }
 
 static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
-			int *varlist, int *depvars,
+			struct var_lists *vl,
 			double ***pZ, DATAINFO *pdinfo,
-			int order, int neqns, int idx, int *k, 
+			int order, int neqns, int i, int *k, 
 			int pause, PRN *prn)
 {
     MODEL testmod;
     double F = NADBL;
     int robust = gretl_model_get_int(varmod, "robust");
-    int *shortlist = NULL, *outlist = NULL;
-    int end, err = 0;
-    int j, l;
+    int depvar = vl->stochvars[i + 1];
+    int *outlist = NULL;
+    int j, err = 0;
 
     if (robust) {
 	outlist = malloc(varmod->list[0] * sizeof *outlist);
 	if (outlist == NULL) return E_ALLOC;
     }
 
-    shortlist = malloc((varmod->list[0] + 1) * sizeof *shortlist);
-    if (shortlist == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-    
-    shortlist[0] = varlist[0] - order;
-    shortlist[1] = varlist[1];
-
     pputs(prn, _("\nF-tests of zero restrictions:\n\n"));
 
+    /* restrictions for all lags of specific variables */
     for (j=0; j<neqns; j++) {
-	reset_list(shortlist, varlist);
-	for (l=1; l<=order; l++) {
-	    idx = l + 1 + j * order;
-	    if (idx > shortlist[0]) break;
-	    shortlist[idx] = varlist[idx + order];
-	}
-	end = 0;
-	for (l=shortlist[0]; l>idx; l--) {
-	    shortlist[l] = varlist[varlist[0] - end];
-	    end++;
-	}
-	pprintf(prn, _("All lags of %-8s "), 
-		pdinfo->varname[depvars[j + 1]]);
+	compose_varlist(vl, depvar, order, j + 1, pdinfo);	
 
 	if (robust) {
-	    gretl_list_diff(outlist, varmod->list, shortlist);
+	    gretl_list_diff(outlist, varmod->list, vl->reglist);
 	    F = robust_omit_F(outlist, varmod);
 	    if (na(F)) err = 1;
 	} else {
-	    testmod = lsq(shortlist, pZ, pdinfo, VAR, OPT_A, 0.0);
+	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
 	    err = testmod.errcode;
 	    if (!err) {
 		F = ((testmod.ess - varmod->ess) / order) / 
@@ -921,38 +1113,33 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 	    }
 	    clear_model(&testmod);
 	}
+
 	if (err) break;
 
-	pprintf(prn, "F(%d, %d) = %f, ", order, varmod->dfd, F);
-	pprintf(prn, _("p-value %f\n"), fdist(F, order, varmod->dfd));
+	pprintf(prn, _("All lags of %-8s "), 
+		pdinfo->varname[vl->stochvars[j + 1]]);
+	pprintf(prn, "F(%d, %d) = %10g  ", order, varmod->dfd, F);
+	pprintf(prn, "%s %g\n", _("p-value"), fdist(F, order, varmod->dfd));
+
 	if (var != NULL) {
 	    var->Fvals[*k] = F;
 	    *k += 1;
 	}
     }
-
-    if (order > 1 && !err) {
-	pprintf(prn, _("All vars, lag %-6d "), order);
-	reset_list(shortlist, varlist);
-	idx = 2;
-	for (j=1; j<=neqns*(order); j++) {
-	    if (j % order) {
-		shortlist[idx] = varlist[j+1];
-		idx++;
-	    }
-	}
-	end = 0;
-	for (l=shortlist[0]; l>=idx; l--) {
-	    shortlist[l] = varlist[varlist[0]-end];
-	    end++;
-	}
+    
+    /* restrictions for last lag, all variables */
+    if (!err) {
+	compose_varlist(vl, depvar, order - 1, 0, pdinfo);	
 
 	if (robust) {
-	    gretl_list_diff(outlist, varmod->list, shortlist);
+	    gretl_list_diff(outlist, varmod->list, vl->reglist);
 	    F = robust_omit_F(outlist, varmod);
 	    if (na(F)) err = 1;
 	} else {
-	    testmod = lsq(shortlist, pZ, pdinfo, VAR, OPT_A, 0.0);
+	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
+#if VAR_DEBUG
+	    printmodel(&testmod, pdinfo, OPT_NONE, prn);
+#endif
 	    err = testmod.errcode;
 	    if (!err) {
 		F = ((testmod.ess - varmod->ess) / neqns) / 
@@ -962,8 +1149,9 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 	}
 
 	if (!err) {
-	    pprintf(prn, "F(%d, %d) = %f, ", neqns, varmod->dfd, F);
-	    pprintf(prn, _("p-value %f\n"), fdist(F, neqns, varmod->dfd)); 
+	    pprintf(prn, _("All vars, lag %-6d "), order); 
+	    pprintf(prn, "F(%d, %d) = %10g  ", neqns, varmod->dfd, F);
+	    pprintf(prn, "%s %g\n", _("p-value"), fdist(F, neqns, varmod->dfd)); 
 	    if (var != NULL) {
 		var->Fvals[*k] = F;
 		*k += 1;
@@ -972,10 +1160,11 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
     }
 
     pputc(prn, '\n');
-    if (!err && pause) takenotes(0);
 
- bailout:
-    free(shortlist);
+    if (!err && pause) {
+	takenotes(0);
+    }
+
     if (outlist != NULL) {
 	free(outlist);
     }   
@@ -1003,19 +1192,23 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 static int real_var (int order, const int *inlist, 
 		     double ***pZ, DATAINFO *pdinfo,
 		     GRETL_VAR **pvar, struct var_resids *resids, 
-		     PRN *prn, gretlopt opts, char flags)
+		     PRN *prn, gretlopt opt, char flags)
 {
-    int i, idx, k, l, listlen, end, neqns;
-    int *list = NULL, *varlist = NULL, *depvars = NULL;
-    char *detlist = NULL;
+    int i, k, neqns;
     int t1, t2, oldt1, oldt2;
     int missv = 0, misst = 0;
+    struct var_lists vlists;
     MODEL var_model;
     GRETL_VAR *var = NULL;
-    int save = (flags & VAR_SAVE);
+    int save = (pvar != NULL);
     int pause = (flags & VAR_PRINT_PAUSE);
+    int impulses = (flags & VAR_IMPULSE_RESPONSES);
     int err = 0;
 
+    if (save) {
+	*pvar = NULL;
+    }
+
     oldt1 = pdinfo->t1;
     oldt2 = pdinfo->t2;
 
@@ -1024,77 +1217,35 @@ static int real_var (int order, const int *inlist,
 	return 1;
     }
 
-    listlen = inlist[0] + 1;
-    detlist = malloc(listlen * sizeof *detlist);
-    if (detlist == NULL) {
-	return E_ALLOC;
+    err = organize_var_lists(inlist, (const double **) *pZ, pdinfo, 
+			     order, &vlists);
+    if (err) {
+	return err;
     }
 
-    list = copylist(inlist);
-    if (list == NULL) {
-	free(detlist);
-	return E_ALLOC;
-    }
-
-    depvars = malloc(listlen * sizeof *depvars);
-    if (depvars == NULL) {
-	free(detlist);
-	free(list);
-	return E_ALLOC;
-    }
-
-    /* how long will our list have to be? */
-    listlen = get_var_listlen(list, detlist, order, depvars,
-			      *pZ, pdinfo);
-
-    varlist = gretl_list_new(listlen);
-    if (varlist == NULL) {
-	err = E_ALLOC;
-	goto var_bailout;
-    }
-
-    neqns = depvars[0];
-
     /* generate the required lags */
-    if (real_list_laggenr(depvars, pZ, pdinfo, order)) {
+    if (real_list_laggenr(vlists.stochvars, pZ, pdinfo, 
+			  order, vlists.lagvlist)) {
 	err = E_ALLOC;
 	goto var_bailout;
     }
 
-    /* initialize */
-    idx = 2; /* skip beyond the counter and the dep var */
-    end = listlen;
+    neqns = vlists.stochvars[0];    
 
-    /* and fill out the list */
-    for (i=1; i<=list[0]; i++) {
-	if (detlist[i]) {
-	    /* deterministic var: put at end of list */
-	    varlist[end] = list[i];
-	    end--;
-	    continue;	    
-	}
-	/* otherwise it's a "real" variable and we replace it with
-	   <order> lags of itself */
-	if (varindex(pdinfo, pdinfo->varname[list[i]]) < pdinfo->v) {
-	    for (l=1; l<=order; l++) {
-		int lnum = lagvarnum(list[i], l, pdinfo);
-
-		if (lnum > 0) {
-		    varlist[idx++] = lnum; 
-		} else {
-		    err = E_ALLOC;
-		    goto var_bailout;
-		}
-	    }
-	}
+    /* compose base VAR list (entry 1 will vary across equations);
+       assemble test list for t1 and t2 while we're at it */
+    err = compose_varlist(&vlists, vlists.stochvars[1], 
+			  order, 0, pdinfo);
+    if (err) {
+	err = E_DATA;
+	goto var_bailout;
     }
 
     /* sort out sample range */
     t1 = pdinfo->t1;
     t2 = pdinfo->t2;
-    varlist[1] = depvars[1];
-
-    if ((missv = adjust_t1t2(NULL, varlist, &t1, &t2, 
+    
+    if ((missv = adjust_t1t2(NULL, vlists.testlist, &t1, &t2, 
 			     (const double **) *pZ, &misst))) {
 	err = 1;
 	goto var_bailout;
@@ -1103,8 +1254,8 @@ static int real_var (int order, const int *inlist,
     pdinfo->t1 = t1;
     pdinfo->t2 = t2;
 
-    if (flags & VAR_IMPULSE_RESPONSES) {
-	var = gretl_var_new(neqns, order, pdinfo, flags);
+    if (save || impulses) {
+	var = gretl_var_new(neqns, order, pdinfo, save);
 	if (var == NULL) {
 	    err = E_ALLOC;
 	    goto var_bailout;
@@ -1114,7 +1265,7 @@ static int real_var (int order, const int *inlist,
 	}
     } 
 
-    /* even in case of VAR_IMPULSE_RESPONSES this may be used */
+    /* play safe by initializing this model */
     gretl_model_init(&var_model);
 
     if (flags & VAR_PRINT_MODELS) {
@@ -1136,17 +1287,25 @@ static int real_var (int order, const int *inlist,
     for (i=0; i<neqns; i++) {
 	MODEL *pmod;
 
-	if (flags & VAR_IMPULSE_RESPONSES) {
+	if (save || impulses) {
 	    pmod = var->models[i];
 	} else {
 	    pmod = &var_model;
 	}
 
-	varlist[1] = depvars[i + 1];
-
 	/* run an OLS regression for the current dependent var */
-	*pmod = lsq(varlist, pZ, pdinfo, VAR, (opts | OPT_A), 0.0);
-	pmod->aux = VAR;
+
+	compose_varlist(&vlists, vlists.stochvars[i + 1], 
+			order, 0, pdinfo);
+
+	*pmod = lsq(vlists.reglist, pZ, pdinfo, VAR, (opt | OPT_A), 0.0);
+
+	if (pmod->errcode) {
+	    err = pmod->errcode;
+	    goto var_bailout;
+	}
+
+	pmod->aux = AUX_VAR;
 	pmod->ID = i + 1;
 
 	/* save the residuals if required */
@@ -1161,7 +1320,7 @@ static int real_var (int order, const int *inlist,
 	    printmodel(pmod, pdinfo, OPT_NONE, prn);
 	}
 
-	if (flags & VAR_IMPULSE_RESPONSES) {
+	if (save || impulses) {
 	    /* store info in var structure */
 	    err = add_model_data_to_var(var, pmod, i);
 	    if (err) goto var_bailout;
@@ -1171,10 +1330,11 @@ static int real_var (int order, const int *inlist,
 	    /* estimate equations for Johansen test */
 	    MODEL jmod;
 
-	    varlist[1] = resids->levels_list[i + 1]; 
-	    jmod = lsq(varlist, pZ, pdinfo, VAR, (opts | OPT_A), 0.0);
+	    vlists.reglist[1] = resids->levels_list[i + 1]; 
+	    jmod = lsq(vlists.reglist, pZ, pdinfo, VAR, (opt | OPT_A), 0.0);
 	    if (flags & VAR_PRINT_MODELS) {
-		jmod.aux = VAR;
+		jmod.aux = AUX_JOHANSEN;
+		jmod.ID = -1;
 		printmodel(&jmod, pdinfo, OPT_NONE, prn);
 	    }
 	    resids->uhat[i + neqns] = jmod.uhat;
@@ -1183,56 +1343,59 @@ static int real_var (int order, const int *inlist,
 	}
 
 	if (flags & VAR_DO_FTESTS) {
-	    var_F_tests(pmod, (save)? var : NULL,
-			varlist, depvars,
+	    var_F_tests(pmod, (save)? var : NULL, &vlists,
 			pZ, pdinfo, order, neqns, 
-			idx, &k, pause, prn);
+			i, &k, pause, prn);
 	}
 
-	if (1) { /* FIXME? */
+	if (pmod == &var_model) {
 	    clear_model(&var_model);
 	}
-
     }
     pputc(prn, '\n');
 
  var_bailout:
 
-    free(list);
-    free(varlist);
-    free(depvars);
-    free(detlist);
+    var_lists_free(&vlists);
 
     /* reset sample range to what it was before */
     pdinfo->t1 = oldt1;
     pdinfo->t2 = oldt2;
 
-    if (flags & VAR_IMPULSE_RESPONSES) {
-	if (!err) {
-#ifdef VAR_DEBUG
-	    gretl_matrix_print(var->A, "var->A", prn);
+    if ((save || impulses) && !err) {
+#if VAR_DEBUG
+	gretl_matrix_print(var->A, "var->A", prn);
 #endif
-	    err = gretl_var_do_error_decomp(var);
-	    if (!err) {
-#ifdef VAR_DEBUG
-		gretl_matrix_print(var->C, "var->C", prn);
+	err = gretl_var_do_error_decomp(var);
+	if (err) {
+	    fprintf(stderr, "failed: gretl_var_do_error_decomp\n");
+	}
+    }
+
+    if (impulses && !err) {
+#if VAR_DEBUG
+	gretl_matrix_print(var->C, "var->C", prn);
 #endif
-		for (i=0; i<var->neqns; i++) {
-		    /* FIXME: make horizon configurable */
-		    gretl_var_print_impulse_response(var, i, 0, pdinfo, 
-						     pause, prn);
-		    gretl_var_print_fcast_decomp(var, i, 0, pdinfo, 
-						 pause, prn);
-		}
-	    } else {
-		fprintf(stderr, "failed: gretl_var_do_error_decomp\n");
-	    }
+	for (i=0; i<var->neqns; i++) {
+	    /* FIXME: make horizon configurable */
+	    gretl_var_print_impulse_response(var, i, 0, pdinfo, 
+					     pause, prn);
+	    gretl_var_print_fcast_decomp(var, i, 0, pdinfo, 
+					 pause, prn);
 	}
-	if ((flags & VAR_SAVE) && pvar != NULL) {
-	    *pvar = var;
-	} else {
-	    gretl_var_free(var);
+    } 
+
+    if (var != NULL) {
+	if (err) {
+	   gretl_var_free(var);
+	   var = NULL;
 	}
+    } 
+
+    if (save) {
+	*pvar = var;
+    } else if (var != NULL) {
+	gretl_var_free(var);
     }
 
     return err;
@@ -1245,6 +1408,8 @@ static int real_var (int order, const int *inlist,
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  * @pause: if = 1, pause after showing each model.
+ * @opts: if OPT_Q, don't print results; if OPT_R, use robust VCV,
+ *        if OPT_V, print impulse responses
  * @prn: gretl printing struct.
  *
  * Estimate a vector auto-regression (VAR) and print the results.
@@ -1259,37 +1424,39 @@ int simple_var (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
     char flags = VAR_PRINT_MODELS | VAR_DO_FTESTS;
     PRN *myprn;
 
-#if 1
-    flags |= VAR_IMPULSE_RESPONSES;
-#endif
-
-    if (pause) {
-	flags |= VAR_PRINT_PAUSE;
+    if (opts & OPT_V) {
+	flags |= VAR_IMPULSE_RESPONSES;
     }
 
-    if (opts & OPT_Q) myprn = NULL;
-    else myprn = prn;
+    if (opts & OPT_Q) {
+	myprn = NULL;
+    } else {
+	myprn = prn;
+	if (pause) {
+	    flags |= VAR_PRINT_PAUSE;
+	}	
+    }
 
     return real_var(order, list, pZ, pdinfo, NULL, NULL, myprn, opts, flags);
 }
 
-/* "full" version returns pointer to VAR struct */
+/* "full" version returns pointer to VAR struct -- invoked by gui */
 
 GRETL_VAR *full_var (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
 		     gretlopt opts, PRN *prn)
 {
+    char flags = VAR_PRINT_MODELS | VAR_DO_FTESTS;
     GRETL_VAR *var = NULL;
     int err;
 
-    err = real_var(order, list, pZ, pdinfo, &var, NULL, prn, opts, 
-		   VAR_PRINT_MODELS | VAR_DO_FTESTS | 
-		   VAR_IMPULSE_RESPONSES | VAR_SAVE);
-    if (err) {
-	gretl_var_free(var);
-	return NULL;
-    } else {
-	return var;
+    if (opts & OPT_V) {
+	flags |= VAR_IMPULSE_RESPONSES;
     }
+
+    err = real_var(order, list, pZ, pdinfo, &var, NULL, prn, 
+		   opts, flags);
+
+    return var;
 }
 
 static double df_pvalue_from_plugin (double tau, int n, int niv, int itv)
@@ -1332,6 +1499,8 @@ static double df_pvalue_from_plugin (double tau, int n, int niv, int itv)
  * @list: specifies the variables to use.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
+ * @opt: if OPT_N, do not an include a constant in the
+ *       cointegrating regression.
  * @prn: gretl printing struct.
  *
  * Test for cointegration.  
@@ -1341,13 +1510,20 @@ static double df_pvalue_from_plugin (double tau, int n, int niv, int itv)
  */
 
 int coint (int order, const int *list, double ***pZ, 
-	   DATAINFO *pdinfo, PRN *prn)
+	   DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
     int i, t, n, nv, l0 = list[0];
+    int hasconst = gretl_hasconst(list);
     MODEL cmod;
     int *cointlist = NULL;
     int adfcode = UR_CONST;
 
+    if (order <= 0 || list[0] - hasconst < 2) {
+	strcpy(gretl_errmsg, "coint: needs a positive lag order "
+	       "and at least two variables");
+	return 1;
+    }
+
     gretl_model_init(&cmod);
 
     /* step 1: test all the vars for unit root */
@@ -1361,7 +1537,8 @@ int coint (int order, const int *list, double ***pZ,
     }
 
     /* step 2: carry out the cointegrating regression */
-    if (gretl_hasconst(list) == 0) {
+    if (!hasconst && !(opt & OPT_N)) {
+	/* add const to coint regression list */
 	cointlist = malloc((l0 + 2) * sizeof *cointlist);
 	if (cointlist == NULL) {
 	    return E_ALLOC;
@@ -1476,6 +1653,73 @@ static int *adf_prepare_vars (int order, int varno,
     return list;
 }
 
+#define ADF_DEBUG 0
+
+static int auto_adjust_order (int *list, int order_max,
+			      double ***pZ, DATAINFO *pdinfo,
+			      PRN *prn)
+{
+    MODEL kmod;
+    double tstat, pval = 1.0;
+    int i, k = order_max;
+
+    for (k=order_max; k>0; k--) {
+	int j = k;
+
+	if (list[list[0]] == 0) j++;
+
+	kmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
+
+	if (kmod.errcode) {
+	    clear_model(&kmod);
+	    fprintf(stderr, "adf: model failed in auto_adjust_order()\n");
+	    k = -1;
+	    break;
+	}
+
+#if ADF_DEBUG
+	printmodel(&kmod, pdinfo, OPT_NONE, prn);
+#endif
+
+	tstat = kmod.coeff[j] / kmod.sderr[j];
+	clear_model(&kmod);
+	pval = 2.0 * normal(tstat);
+
+	if (pval > 0.10) {
+#if ADF_DEBUG
+	    fprintf(stderr, "auto_adjust_order: lagged difference not "
+		    "significant at order %d (t = %g)\n", k, tstat);
+#endif
+	    if (k == 1) {
+		k = 0;
+		break;
+	    } else {
+		for (i=k+2; i<list[0]; i++) {
+		    list[i] = list[i+1];
+		}
+		list[0] -= 1;
+	    }
+	} else {
+#if ADF_DEBUG
+	    fprintf(stderr, "auto_adjust_order: lagged difference is "
+		    "significant at order %d (t = %g)\n", k, tstat);
+#endif
+	    break;
+	}
+    }
+
+    return k;
+}
+
+static void copy_list_values (int *targ, const int *src)
+{
+    int i;
+
+    for (i=0; i<=src[0]; i++) {
+	targ[i] = src[i];
+    }
+}
+
 static int real_adf_test (int varno, int order, int niv,
 			  double ***pZ, DATAINFO *pdinfo, 
 			  gretlopt opt, int cointcode,
@@ -1499,20 +1743,49 @@ static int real_adf_test (int varno, int order, int niv,
 	N_("with constant and trend"),
 	N_("with constant and quadratic trend")
     };
+
+    MODEL dfmod;
+
     int orig_nvars = pdinfo->v;
     int blurb_done = 0;
-    MODEL dfmod;
+    int auto_order = 0;
+    int order_max = 0;
     int *list;
-    char pvstr[48];
-    double DFt, pv;
+    int *biglist = NULL;
+    double DFt = NADBL;
+    double pv = NADBL;
     char mask[4] = {0};
     int i, itv;
+    int err = 0;
+
+#if ADF_DEBUG
+    fprintf(stderr, "real_adf_test: got order = %d\n", order);
+#endif
+
+    if (order < 0) {
+	auto_order = 1;
+	order = -order;
+    }
+
+    order_max = order;
 
     list = adf_prepare_vars(order, varno, pZ, pdinfo);
     if (list == NULL) {
 	return E_ALLOC;
     }
 
+    if (auto_order) {
+	int tmp = list[0];
+
+	list[0] = order + 5;
+	biglist = copylist(list);
+	if (biglist == NULL) {
+	    free(list);
+	    return E_ALLOC;
+	}
+	list[0] = tmp;
+    }
+
     gretl_model_init(&dfmod);
 
     if (opt == 0L || opt == OPT_V) {
@@ -1544,6 +1817,11 @@ static int real_adf_test (int varno, int order, int niv,
 	    continue;
 	}
 
+	if (auto_order) {
+	    order = order_max;
+	    copy_list_values(list, biglist);
+	}
+
 	list[0] = 2 + order + i;
 
 	if (i > 0) {
@@ -1553,19 +1831,35 @@ static int real_adf_test (int varno, int order, int niv,
 	if (i >= 2) {
 	    list[3 + order] = gettrend(pZ, pdinfo, 0);
 	    if (list[3 + order] == TREND_FAILED) {
-		return 1;
+		err = E_ALLOC;
+		goto bailout;
 	    }
 	}
+
 	if (i > 2) {
 	    list[4 + order] = gettrend(pZ, pdinfo, 1);
 	    if (list[4 + order] == TREND_FAILED) {
-		return 1;
+		err = E_ALLOC;
+		goto bailout;
 	    }
-	}	    
+	}
+
+	if (auto_order) {
+	    order = auto_adjust_order(list, order_max, pZ, pdinfo, prn);
+	    if (order < 0) {
+		err = 1;
+		clear_model(&dfmod);
+		goto bailout;
+	    }
+	}
 
 	dfmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
 	if (dfmod.errcode) {
-	    return dfmod.errcode;
+	    fprintf(stderr, "adf_test: dfmod.errcode = %d\n", 
+		    dfmod.errcode);
+	    err = dfmod.errcode;
+	    clear_model(&dfmod);
+	    goto bailout;
 	}
 
 	DFt = dfmod.coeff[dfnum] / dfmod.sderr[dfnum];
@@ -1584,39 +1878,45 @@ static int real_adf_test (int varno, int order, int niv,
 				   (order > 0)? 0 : dfmod.nobs, 
 				   niv, itv);
 
-	if (na(pv)) {
-	    sprintf(pvstr, "%s %s", _("p-value"), _("unknown"));
-	} else {
-	    sprintf(pvstr, "%s %.4g", 
-		    (order > 0)? _("asymptotic p-value") : _("p-value"), 
-		    pv);
-	} 
+	if (!(opt & OPT_Q)) {
+	    char pvstr[48];
 
-	if (!blurb_done) {
-	    if (order > 0) {
-		pprintf(prn, _("\nAugmented Dickey-Fuller tests, order %d, for %s\n"),
-			order, pdinfo->varname[varno]);
+	    if (na(pv)) {
+		sprintf(pvstr, "%s %s", _("p-value"), _("unknown"));
 	    } else {
-		pprintf(prn, _("\nDickey-Fuller tests for %s\n"),
-			pdinfo->varname[varno]);
-	    }
-	    pprintf(prn, _("sample size %d\n"), dfmod.nobs);
-	    pputs(prn, _("unit-root null hypothesis: a = 1"));
-	    pputs(prn, "\n\n");
-	    blurb_done = 1;
-	}
+		sprintf(pvstr, "%s %.4g", 
+			(order > 0)? _("asymptotic p-value") : _("p-value"), 
+			pv);
+	    } 
 
-	pprintf(prn, "   %s\n", _(teststrs[i]));
-	if (cointcode == 0) {
-	    pprintf(prn, "   %s: %s\n", _("model"), 
-		    (order > 0)? aug_models[i] : models[i]);
+	    if (!blurb_done) {
+		if (order > 0) {
+		    pprintf(prn, _("\nAugmented Dickey-Fuller tests, order %d, for %s\n"),
+			    order, pdinfo->varname[varno]);
+		} else {
+		    pprintf(prn, _("\nDickey-Fuller tests for %s\n"),
+			    pdinfo->varname[varno]);
+		}
+		pprintf(prn, _("sample size %d\n"), dfmod.nobs);
+		pputs(prn, _("unit-root null hypothesis: a = 1"));
+		pputs(prn, "\n\n");
+		blurb_done = 1;
+	    }
+
+	    pprintf(prn, "   %s\n", _(teststrs[i]));
+
+	    if (cointcode == 0) {
+		pprintf(prn, "   %s: %s\n", _("model"), 
+			(order > 0)? aug_models[i] : models[i]);
+	    }
+
+	    pprintf(prn, "   %s: %g\n"
+		    "   %s: t = %g\n"
+		    "   %s\n",
+		    _("estimated value of (a - 1)"), dfmod.coeff[dfnum],
+		    _("test statistic"), DFt,
+		    pvstr);	
 	}
-	pprintf(prn, "   %s: %g\n"
-		"   %s: t = %g\n"
-		"   %s\n",
-		_("estimated value of (a - 1)"), dfmod.coeff[dfnum],
-		_("test statistic"), DFt,
-		pvstr);	
 
 	if (opt & OPT_V) {
 	    /* verbose */
@@ -1626,21 +1926,33 @@ static int real_adf_test (int varno, int order, int niv,
 		gretl_model_set_double(&dfmod, "dfpval", pv);
 	    }
 	    printmodel(&dfmod, pdinfo, OPT_NONE, prn);
-	} else {
+	} else if (!(opt & OPT_Q)) {
 	    pputc(prn, '\n');
 	}
 
 	clear_model(&dfmod);
     }
 
-    if (cointcode >= 0) {
-	pputs(prn, _("P-values based on MacKinnon (JAE, 1996)\n"));
+    if (!err) {
+	if (cointcode == 0) {
+	    record_test_result(DFt, pv, "Dickey-Fuller");
+	}
+	if (cointcode >= 0 && !(opt & OPT_Q)) {
+	    pputs(prn, _("P-values based on MacKinnon (JAE, 1996)\n"));
+	}	
     }
 
+ bailout:
+
     free(list);
+
+    if (biglist != NULL) {
+	free(biglist);
+    }
+
     dataset_drop_vars(pdinfo->v - orig_nvars, pZ, pdinfo);
 
-    return 0;
+    return err;
 }
 
 /**
@@ -1689,6 +2001,7 @@ int kpss_test (int order, int varno, double ***pZ,
     int hastrend = 0;
     double s2 = 0.0;
     double cumsum = 0.0, cumsum2 = 0.0;
+    double teststat;
     double *autocov;
     double et;
 
@@ -1764,6 +2077,7 @@ int kpss_test (int order, int varno, double ***pZ,
     }
 
     s2 /= T;
+    teststat = cumsum2 / (s2 * T * T);
 
     if (opt & OPT_V) {
 	pprintf(prn, "  %s: %g\n", _("Robust estimate of variance"), s2);
@@ -1771,20 +2085,25 @@ int kpss_test (int order, int varno, double ***pZ,
 		cumsum2);
     }
 
-    pprintf(prn, _("\nKPSS test for %s %s\n\n"), pdinfo->varname[varno],
-	    (hastrend)? _("(including trend)") : _("(without trend)"));
+    if (!(opt & OPT_Q)) {
+	pprintf(prn, _("\nKPSS test for %s %s\n\n"), pdinfo->varname[varno],
+		(hastrend)? _("(including trend)") : _("(without trend)"));
 
-    pprintf(prn, _("Lag truncation parameter = %d\n"), order);
-    pprintf(prn, "%s = %g\n\n", _("Test statistic"), cumsum2 / (s2 * T * T));
+	pprintf(prn, _("Lag truncation parameter = %d\n"), order);
 
-    pprintf(prn, "		    10%%\t   5%%\t 2.5%%\t   1%%\n");
+	pprintf(prn, "%s = %g\n\n", _("Test statistic"), teststat);
 
-    if (hastrend) {
-	pprintf(prn, "%s: 0.119\t0.146\t0.176\t0.216\n\n", _("Critical values"));
-    } else {
-	pprintf(prn, "%s: 0.347\t0.463\t0.574\t0.739\n\n", _("Critical values"));
+	pprintf(prn, "		    10%%\t   5%%\t 2.5%%\t   1%%\n");
+
+	if (hastrend) {
+	    pprintf(prn, "%s: 0.119\t0.146\t0.176\t0.216\n\n", _("Critical values"));
+	} else {
+	    pprintf(prn, "%s: 0.347\t0.463\t0.574\t0.739\n\n", _("Critical values"));
+	}
     }
 
+    record_test_result(teststat, NADBL, "KPSS");
+
     clear_model(&KPSSmod);
 
     free(autocov);
@@ -1984,29 +2303,38 @@ int johansen_test (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
     int *varlist;
     int verbose = (opt & OPT_O);
     int hasconst = 0;
+    int l0 = list[0];
     int trends = 0;
 
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] == 0) hasconst = 1;
+    }
+
+    if (order <= 0 || list[0] - hasconst < 2) {
+	strcpy(gretl_errmsg, "coint2: needs a positive lag order "
+	       "and at least two variables");
+	return 1;
+    }
+
     /* we're assuming that the list we are fed is in levels */
-    resids.levels_list = malloc((1 + list[0]) * sizeof *list);
+    resids.levels_list = malloc((1 + l0) * sizeof *list);
     if (resids.levels_list == NULL) {
 	return E_ALLOC;
     }
 
-    varlist = malloc((2 + list[0]) * sizeof *list);
+    varlist = malloc((2 + l0) * sizeof *list);
     if (varlist == NULL) {
 	free(resids.levels_list);
 	return E_ALLOC;
     }
 
-    resids.levels_list[0] = varlist[0] = list[0];
+    varlist[0] = resids.levels_list[0] = l0 - hasconst;
 
     j = 1;
     for (i=1; i<=list[0]; i++) {
 	int lnum;
 
 	if (list[i] == 0) {
-	    resids.levels_list[0] -= 1;
-	    hasconst = 1;
 	    continue;
 	}
 	lnum = laggenr(list[i], 1, pZ, pdinfo);
@@ -2018,23 +2346,24 @@ int johansen_test (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
 	resids.levels_list[j++] = lnum;
     }
 
-    /* now get differences and put them into list */
+    /* now get first differences and put them into list */
+    j = 1;
     for (i=1; i<=list[0]; i++) {
 	if (list[i] == 0) {
 	    continue;
 	}
-	varlist[i] = diffgenr(list[i], pZ, pdinfo, 0);
-	if (varlist[i] < 0) {
+	varlist[j] = diffgenr(list[i], pZ, pdinfo, 0);
+	if (varlist[j] < 0) {
 	    free(varlist);
 	    free(resids.levels_list);
 	    return E_DATA;
-	}
+	} 
+	j++;
     }
 
-    if (!hasconst) {
-	varlist[0] += 1;
-	varlist[varlist[0]] = 0;
-    }
+    /* add the constant to the VAR list */
+    varlist[0] += 1;
+    varlist[varlist[0]] = 0;
 
     if (verbose) {
 	flags = VAR_PRINT_MODELS;
