@@ -29,6 +29,8 @@ enum ur_codes {
     UR_TREND_SQUARED
 };
 
+int var_max_order (const int *list, const DATAINFO *pdinfo);
+
 int simple_var (int order, const int *list, 
 		double ***pZ, DATAINFO *pdinfo,
 		int pause, gretlopt opts, PRN *prn);
@@ -43,7 +45,7 @@ void gretl_var_free_unnamed (GRETL_VAR *var);
 
 int coint (int order, const int *list, 
 	   double ***pZ, DATAINFO *pdinfo, 
-	   PRN *prn);
+	   gretlopt opt, PRN *prn);
 
 int adf_test (int order, int varno, 
 	      double ***pZ, DATAINFO *pdinfo, 
