@@ -30,12 +30,23 @@
 
 #undef ARMA_DEBUG
 
-static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo)
+#define MAX_ARMA_ORDER 6
+
+struct arma_info {
+    int p;      /* AR order */
+    int q;      /* MA order */
+    int maxlag; /* longest lag in model */
+    int r;      /* number of other regressors (ARMAX) */
+    int ifc;    /* 1 for intercept included, otherwise 0 */
+    int t1;     /* starting observation */
+    int t2;     /* ending observation */
+};
+
+static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
+			       struct arma_info *ainfo)
 {
-    int p = pmod->list[1];
-    int q = pmod->list[2];
-    int r = pmod->list[0] - 4;
-    int i, j, np = 2 + p + q + r;
+    int np = ainfo->p + ainfo->q + ainfo->r + 1 + ainfo->ifc;
+    int i, j;
 
     pmod->params = malloc(np * sizeof pmod->params);
     if (pmod->params == NULL) {
@@ -48,7 +59,9 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo)
     for (i=0; i<np; i++) {
 	pmod->params[i] = malloc(VNAMELEN);
 	if (pmod->params[i] == NULL) {
-	    for (j=0; j<i; j++) free(pmod->params[j]);
+	    for (j=0; j<i; j++) {
+		free(pmod->params[j]);
+	    }
 	    free(pmod->params);
 	    pmod->params = NULL;
 	    pmod->nparams = 0;
@@ -58,10 +71,15 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo)
     }
 
     strcpy(pmod->params[0], pdinfo->varname[pmod->list[4]]);
-    strcpy(pmod->params[1], pdinfo->varname[0]);
 
-    j = 2;
-    for (i=0; i<p; i++) {
+    if (ainfo->ifc) {
+	strcpy(pmod->params[1], pdinfo->varname[0]);
+	j = 2;
+    } else {
+	j = 1;
+    }
+
+    for (i=0; i<ainfo->p; i++) {
 	const char *depvar = pmod->params[0];
 	size_t n = strlen(depvar);
 	
@@ -71,14 +89,19 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo)
 	    sprintf(pmod->params[j++], "y(-%d)", i + 1);
 	}
     }
-    for (i=0; i<q; i++) {
+
+    for (i=0; i<ainfo->q; i++) {
 	sprintf(pmod->params[j++], "e(-%d)", i + 1);
     }
-    for (i=0; i<r; i++) {
-	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]);
+
+    for (i=0; i<ainfo->r; i++) {
+	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]); /* 5? */
     }    
 }
 
+/* check whether the MA estimates have gone out of bounds in the
+   course of BHHH iterations */
+
 static int ma_out_of_bounds (int q, const double *ma_coeff)
 {
     double *temp = NULL, *tmp2 = NULL;
@@ -86,18 +109,20 @@ static int ma_out_of_bounds (int q, const double *ma_coeff)
     cmplx *roots = NULL;
     int i, err = 0, allzero = 1;
 
-    for (i=1; i<=q; i++){
+    for (i=0; i<q; i++){
 	if (ma_coeff[i] != 0.0) {
 	    allzero = 0;
 	}    
     }    
     
-    if (allzero) return 0;
+    if (allzero) {
+	return 0;
+    }
 
     /* we'll use a budget version of the "arma_roots" function here */
 
-    temp  = malloc((q+1) * sizeof *temp);
-    tmp2  = malloc((q+1) * sizeof *tmp2);
+    temp  = malloc((q + 1) * sizeof *temp);
+    tmp2  = malloc((q + 1) * sizeof *tmp2);
     roots = malloc(q * sizeof *roots);
 
     if (temp == NULL || tmp2 == NULL || roots == NULL) {
@@ -108,9 +133,8 @@ static int ma_out_of_bounds (int q, const double *ma_coeff)
     }
 
     temp[0] = 1.0;
-    /* ma_coeff is 1-based */
-    for (i=1; i<=q; i++){
-	temp[i] = ma_coeff[i];
+    for (i=0; i<q; i++){
+	temp[i+1] = ma_coeff[i];
     }
     polrt(temp, tmp2, q, roots);
 
@@ -132,8 +156,11 @@ static int ma_out_of_bounds (int q, const double *ma_coeff)
     return err;
 }
 
+/* Calculate ARMA log-likelihood.  This function is passed to the
+   bhhh_max() routine as a "callback". */
+
 static int arma_ll (double *coeff, 
-		    const double **X, double **Z, 
+		    const double **bhX, double **Z, 
 		    model_info *arma,
 		    int do_score)
 {
@@ -141,20 +168,39 @@ static int arma_ll (double *coeff,
     int t1 = model_info_get_t1(arma);
     int t2 = model_info_get_t2(arma);
     int n = t2 - t1 + 1;
-    int p, q, r;
+    int p, q, r, ifc;
+
     const double K = 1.41893853320467274178; /* ln(sqrt(2*pi)) + 0.5 */
-    const double *y = X[0];
+    const double *y = bhX[0];
+    const double **X = bhX + 1;
     double **series = model_info_get_series(arma);
     double *e = series[0];
     double **de = series + 1;
+    double **de_a, **de_m, **de_r;
     const double *ar_coeff, *ma_coeff, *reg_coeff;
+
+    struct arma_info *ainfo;
+
     double ll, s2 = 0.0;
     int err = 0;
 
-    model_info_get_pqr(arma, &p, &q, &r);
-    ar_coeff = coeff;
-    ma_coeff = coeff + p;
-    reg_coeff = coeff + p + q;
+    /* retrieve ARMA-specific information */
+    ainfo = model_info_get_extra_info(arma);
+
+    p = ainfo->p;
+    q = ainfo->q;
+    r = ainfo->r;
+    ifc = ainfo->ifc;
+
+    /* pointers to blocks of coefficients */
+    ar_coeff = coeff + ifc;
+    ma_coeff = ar_coeff + p;
+    reg_coeff = ma_coeff + q;
+
+    /* pointers to blocks of derivatives */
+    de_a = de + ifc;
+    de_m = de_a + p;
+    de_r = de_m + q;
 
     if (ma_out_of_bounds(q, ma_coeff)) {
 	fputs("arma: MA estimate(s) out of bounds\n", stderr);
@@ -165,19 +211,23 @@ static int arma_ll (double *coeff,
 
     for (t=t1; t<=t2; t++) {
 
-	e[t] = y[t] - coeff[0];
+	e[t] = y[t];
 
-	for (i=1; i<=p; i++) {
-	    e[t] -= ar_coeff[i] * y[t-i];
+	if (ifc) {
+	    e[t] -= coeff[0];
+	} 
+
+	for (i=0; i<p; i++) {
+	    e[t] -= ar_coeff[i] * y[t-i-1];
 	}
 
-	for (i=1; i<=q; i++) {
-	    if (t - i >= t1) {
-		e[t] -= ma_coeff[i] * e[t-i];
+	for (i=0; i<q; i++) {
+	    if (t - i - 1 >= t1) {
+		e[t] -= ma_coeff[i] * e[t-i-1];
 	    }
 	}
 
-	for (i=1; i<=r; i++) {
+	for (i=0; i<r; i++) {
 	    e[t] -= reg_coeff[i] * X[i][t];
 	}
 
@@ -192,48 +242,46 @@ static int arma_ll (double *coeff,
     model_info_set_ll(arma, ll, do_score);
 
     if (do_score) {
-	int col, nc = p + q + r + 1;
+	int lag, nc = p + q + r + ifc;
 	double x;
 
 	for (t=t1; t<=t2; t++) {
 
-	    col = 0;
-
-	    /* the constant term (de_c) */
-	    de[col][t] = -1.0;
-	    for (i=1; i<=q; i++) {
-		de[col][t] -= ma_coeff[i] * de[col][t-i];
+	    /* the constant term (de_0) */
+	    if (ifc) {
+		de[0][t] = -1.0;
+		for (i=0; i<q; i++) {
+		    de[0][t] -= ma_coeff[i] * de[0][t-i-1];
+		}
 	    }
 
 	    /* AR terms (de_a) */
 	    for (j=0; j<p; j++) {
-		col++;
-		if (t >= col) {
-		    de[col][t] = -y[t-col];
-		    for (i=1; i<=q; i++) {
-			de[col][t] -= ma_coeff[i] * de[col][t-i];
+		lag = j + 1;
+		if (t >= lag) {
+		    de_a[j][t] = -y[t-lag];
+		    for (i=0; i<q; i++) {
+			de_a[j][t] -= ma_coeff[i] * de_a[j][t-i-1];
 		    }
 		}
 	    }
 
 	    /* MA terms (de_m) */
 	    for (j=0; j<q; j++) {
-		int lag = ++col - p;
-
+		lag = j + 1;
 		if (t >= lag) {
-		    de[col][t] = -e[t-lag];
-		    for (i=1; i<=q; i++) {
-			de[col][t] -= ma_coeff[i] * de[col][t-i];
+		    de_m[j][t] = -e[t-lag];
+		    for (i=0; i<q; i++) {
+			de_m[j][t] -= ma_coeff[i] * de_m[j][t-i-1];
 		    }
 		}
 	    }
 
-	    /* ordinary regressors */
+	    /* ordinary regressors (de_r) */
 	    for (j=0; j<r; j++) {
-		col++;
-		de[col][t] = -X[j+1][t]; 
-		for (i=1; i<=q; i++) {
-		    de[col][t] -= ma_coeff[i] * de[col][t-i];
+		de_r[j][t] = -X[j][t]; 
+		for (i=0; i<q; i++) {
+		    de_r[j][t] -= ma_coeff[i] * de_r[j][t-i-1];
 		}
 	    }
 
@@ -245,34 +293,36 @@ static int arma_ll (double *coeff,
 	}
     }
 
-    if (isnan(ll)) err = 1;
+    if (isnan(ll)) {
+	err = 1;
+    }
 
     return err;
 }
 
-static cmplx *arma_roots (int p, int q, const double *coeff) 
-     /*
-       Given an ARMA process $A(L) y_t = C(L) \epsilon_t$, returns the 
-       roots of the two polynomials;
+/*
+  Given an ARMA process $A(L) y_t = C(L) \epsilon_t$, returns the 
+  roots of the two polynomials;
 
-       Syntax:
-       p: order of A(L)
-       q: order of C(L)
-       coeff: p+q+1 vector of coefficients (element 0 is the constant
-       and is ignored)
-       returns: the p + q roots (AR part first)
-     */
+  Syntax:
+  p: order of A(L)
+  q: order of C(L)
+  coeff: p+q+ifc vector of coefficients (if an intercept is present
+         it is element 0 and is ignored)
+  returns: the p + q roots (AR part first)
+*/
+
+static cmplx *arma_roots (struct arma_info *ainfo, const double *coeff) 
 {
-    int maxlag = (p > q)? p : q;
-    const double *ar_coeff = coeff;
-    const double *ma_coeff = coeff + p;
+    const double *ar_coeff = coeff + ainfo->ifc;
+    const double *ma_coeff = coeff + ainfo->ifc + ainfo->p;
     double *temp = NULL, *temp2 = NULL;
     cmplx *roots = NULL;
     int j;
 
-    temp  = malloc((maxlag+1) * sizeof *temp);
-    temp2 = malloc((maxlag+1) * sizeof *temp2);
-    roots = malloc((p + q) * sizeof *roots);
+    temp  = malloc((ainfo->maxlag + 1) * sizeof *temp);
+    temp2 = malloc((ainfo->maxlag + 1) * sizeof *temp2);
+    roots = malloc((ainfo->p + ainfo->q) * sizeof *roots);
 
     if (temp == NULL || temp2 == NULL || roots == NULL) {
 	free(temp);
@@ -284,16 +334,16 @@ static cmplx *arma_roots (int p, int q, const double *coeff)
     temp[0] = 1.0;
 
     /* A(L) */
-    for (j=1; j<=p; j++){
-	temp[j] = -ar_coeff[j];
+    for (j=0; j<ainfo->p; j++){
+	temp[j+1] = -ar_coeff[j];
     }
-    polrt(temp, temp2, p, roots);
+    polrt(temp, temp2, ainfo->p, roots);
 
     /* C(L) */
-    for (j=1; j<=q; j++){
-	temp[j] = ma_coeff[j];
+    for (j=0; j<ainfo->q; j++){
+	temp[j+1] = ma_coeff[j];
     }
-    polrt(temp, temp2, q, roots + p);
+    polrt(temp, temp2, ainfo->q, roots + ainfo->p);
 
     free(temp);
     free(temp2);
@@ -301,8 +351,11 @@ static cmplx *arma_roots (int p, int q, const double *coeff)
     return roots;
 }
 
+/* package the various statistics from ARMA estimation in the
+   form of a gretl MODEL struct */
+
 static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
-				      const int *list, const double *y, 
+				      const int *list, int ifc, const double *y, 
 				      const double *theta, int nc)
 {
     int i, t;
@@ -312,7 +365,7 @@ static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
     double mean_error;
 
     pmod->ci = ARMA;
-    pmod->ifc = 1;
+    pmod->ifc = ifc;
 
     pmod->lnL = model_info_get_ll(arma);
 
@@ -349,8 +402,7 @@ static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
     }
 
     if (pmod->tss > pmod->ess) {
-	pmod->fstt = pmod->dfd * (pmod->tss - pmod->ess) / 
-	    (pmod->dfn * pmod->ess);
+	pmod->fstt = pmod->dfd * (pmod->tss - pmod->ess) / (pmod->dfn * pmod->ess);
     } else {
 	pmod->fstt = NADBL;
     }
@@ -366,15 +418,14 @@ static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
 	}
     }
 
-    /* AIC, as per X-12-ARIMA */
-    pmod->criterion[C_AIC] = -2.0 * pmod->lnL + 2.0 * (pmod->ncoeff + 1);
-
-    /* BIC, as per X-12-ARIMA */
-    pmod->criterion[C_BIC] = -2.0 * pmod->lnL + (pmod->ncoeff + 1) * log(pmod->nobs);
+    mle_aic_bic(pmod, 1);
 }
 
-static void remove_const (int *list)
+/* remove the constant or intercept from a list of regressors */
+
+static int remove_const (int *list)
 {
+    int ret = 0;
     int i, j;
 
     for (i=5; i<=list[0]; i++) {
@@ -383,35 +434,50 @@ static void remove_const (int *list)
 		list[j] = list[j+1];
 	    }
 	    list[0] -= 1;
+	    ret = 1;
 	    break;
 	}
     }
+
+    return ret;
 }
 
-static int check_arma_list (int *list)
+static int check_arma_list (int *list, struct arma_info *ainfo)
 {
     int err = 0;
 
-    if (list[0] < 4) err = 1;
+    if (list[0] < 4) {
+	err = 1;
+    } else if (list[1] < 0 || list[1] > MAX_ARMA_ORDER) {
+	err = 1;
+    } else if (list[2] < 0 || list[2] > MAX_ARMA_ORDER) {
+	err = 1;
+    } else if (list[1] + list[2] == 0) {
+	err = 1;
+    }
 
-    /* for now we'll accept ARMA (4,4) at max */
-    else if (list[1] < 0 || list[1] > 4) err = 1;
-    else if (list[2] < 0 || list[2] > 4) err = 1;
-    else if (list[1] + list[2] == 0) err = 1;
-
-    /* remove const from list of regressors (ARMAX), since it
-       is added automatically */
-    if (list[0] > 4) {
-	remove_const(list);
+    /* signal for adding a constant to the regression list, unless it
+       is deliberately omitted, in an ARMAX model 
+    */
+    if (list[0] == 4 || remove_const(list)) {
+	ainfo->ifc = 1;
+    } else {
+	ainfo->ifc = 0;
     }
 
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
     }
-    
+
     return err;
 }
 
+/* construct a "virtual dataset" in the form of a set of pointers into
+   the main dataset: this will be passed to the bhhh_max function.
+   The dependent variable is put in position 0; following this are the
+   independent variables.
+*/
+
 static const double **make_armax_X (int *list, const double **Z)
 {
     const double **X;
@@ -419,10 +485,14 @@ static const double **make_armax_X (int *list, const double **Z)
     int v, i;
 
     X = malloc((nv + 1) * sizeof *X);
-    if (X == NULL) return NULL;
+    if (X == NULL) {
+	return NULL;
+    }
 
+    /* the dependent variable */
     X[0] = Z[list[4]];
 
+    /* the independent variables */
     for (i=1; i<=nv; i++) {
 	v = list[i + 4];
 	X[i] = Z[v];
@@ -431,77 +501,98 @@ static const double **make_armax_X (int *list, const double **Z)
     return X;
 }
 
-/* below: run an initial OLS to get starting values for the
-   AR coefficients */
+/* Run an initial OLS to get initial values for the AR coefficients */
 
 static int ar_init_by_ols (const int *list, double *coeff,
 			   const double **Z, const DATAINFO *pdinfo,
-			   int arma_t1)
+			   struct arma_info *ainfo)
 {
-    int an = pdinfo->t2 - arma_t1 + 1;
-    int p = list[1];
-    int q = list[2];
-    int r = list[0] - 4;
+    int an = pdinfo->t2 - ainfo->t1 + 1;
     int ynum = list[4];
-    int av = p + r + 2;
+    int av = ainfo->p + ainfo->r + 2;
     double **aZ = NULL;
-    DATAINFO *ainfo = NULL;
+    DATAINFO *adinfo = NULL;
     int *alist = NULL;
     MODEL armod;
+    int offset;
     int i, j, t, err = 0;
 
     gretl_model_init(&armod);  
 
     alist = gretl_list_new(av);
-    if (alist == NULL) return 1;
+    if (alist == NULL) {
+	return 1;
+    }
 
     alist[1] = 1;
-    alist[2] = 0;
-    for (i=0; i<p; i++) {
-	alist[i + 3] = i + 2;
-    }
-    for (i=0; i<r; i++) {
-	alist[i + 3 + p] = i + p + 2;
+
+    if (ainfo->ifc) {
+	alist[2] = 0;
+	offset = 3;
+    } else {
+	alist[0] -= 1;
+	offset = 2;
     }
 
-    ainfo = create_new_dataset(&aZ, av, an, 0);
-    if (ainfo == NULL) {
+    for (i=0; i<ainfo->p; i++) {
+	alist[i + offset] = i + 2;
+    }
+
+    for (i=0; i<ainfo->r; i++) {
+	alist[i + offset + ainfo->p] = i + ainfo->p + 2;
+    }
+
+    adinfo = create_new_dataset(&aZ, av, an, 0);
+    if (adinfo == NULL) {
 	free(alist);
 	return 1;
     }
-    
+
+    /* build temporary dataset containing lagged vars */
     for (t=0; t<an; t++) {
 	int j, s;
 
-	for (i=0; i<=p; i++) {
-	    s = t + arma_t1 - i;
+	for (i=0; i<=ainfo->p; i++) {
+	    s = t + ainfo->t1 - i;
 	    aZ[i+1][t] = Z[ynum][s];
 	}
-	for (i=0; i<r; i++) {
+
+	for (i=0; i<ainfo->r; i++) {
 	    j = list[i+5];
-	    s = t + arma_t1;
-	    aZ[i+p+2][t] = Z[j][s];
+	    s = t + ainfo->t1;
+	    aZ[i + ainfo->p + 2][t] = Z[j][s];
 	}
     }
 
-    armod = lsq(alist, &aZ, ainfo, OLS, OPT_A, 0.0);
+    /* run the OLS */
+    armod = lsq(alist, &aZ, adinfo, OLS, OPT_A, 0.0);
     err = armod.errcode;
     if (!err) {
 	j = 0;
 	for (i=0; i<armod.ncoeff; i++) {
-	    if (i == p + 1) j += q; /* leave space for MA coeffs */
+	    if (i == ainfo->p + ainfo->ifc) {
+		j += ainfo->q; /* leave space for MA coeffs */
+	    }
 	    coeff[j++] = armod.coeff[i];
 	}
-	for (i=0; i<q; i++) {
-	    /* squeeze in some zeros */
-	    coeff[i+p+1] = 0.0;
+	for (i=0; i<ainfo->q; i++) {
+	    /* squeeze in some zeros for MA coeffs */
+	    coeff[i + ainfo->p + ainfo->ifc] = 0.0;
 	} 
     }
 
+#if ARMA_DEBUG > 1
+    fprintf(stderr, "OLS init: armod.ncoeff = %d\n", armod.ncoeff);
+    for (i=0; i<armod.ncoeff; i++) {
+	fprintf(stderr, " coeff[%d] = %g\n", i, armod.coeff[i]);
+    }
+#endif
+
+    /* clear everything up */
     free(alist);
-    free_Z(aZ, ainfo);
-    clear_datainfo(ainfo, CLEAR_FULL);
-    free(ainfo);
+    free_Z(aZ, adinfo);
+    clear_datainfo(adinfo, CLEAR_FULL);
+    free(adinfo);
 
     clear_model(&armod);
 
@@ -509,13 +600,9 @@ static int ar_init_by_ols (const int *list, double *coeff,
 }
 
 static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
-			  int *arma_t1, int *arma_t2)
+			  struct arma_info *ainfo)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int p = list[1];
-    int q = list[2];
-    int r = list[0] - 4;
-    int maxlag = (p > q)? p : q;
     int an, i, v, t, t1min = 0;
     int anymiss;
 
@@ -528,12 +615,18 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
 		break;
 	    }
 	}
-	if (anymiss) t1min++;
-        else break;
+	if (anymiss) {
+	    t1min++;
+        } else {
+	    break;
+	}
     }
 
-    t1min += maxlag;
-    if (t1 < t1min) t1 = t1min;
+    t1min += ainfo->maxlag;
+
+    if (t1 < t1min) {
+	t1 = t1min;
+    }
 
     for (t=pdinfo->t2; t>=t1; t--) {
 	anymiss = 0;
@@ -544,19 +637,24 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
 		break;
 	    }
 	}
-	if (anymiss) t2--;
-        else break;
+	if (anymiss) {
+	    t2--;
+        } else {
+	    break;
+	}
     }
 
-    for (t=t1-p; t<t2; t++) {
+    for (t=t1-ainfo->p; t<t2; t++) {
 	for (i=4; i<=list[0]; i++) {
-	    if (t < t1 && i > 4) continue;
+	    if (t < t1 && i > 4) {
+		continue;
+	    }
 	    v = list[i];
 	    if (na(Z[v][t])) {
 		char msg[64];
 
 		sprintf(msg, _("Missing value encountered for "
-                           "variable %d, obs %d"), v, t + 1);
+			       "variable %d, obs %d"), v, t + 1);
 		gretl_errmsg_set(msg);
 		return 1;
 	    }
@@ -564,26 +662,37 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
     }
 
     an = t2 - t1 + 1;
-    if (an <= p + q + r + 1) return 1; 
+    if (an <= ainfo->p + ainfo->q + ainfo->r + 1) {
+	return 1; 
+    }
 
-    *arma_t1 = t1;
-    *arma_t2 = t2;
+    ainfo->t1 = t1;
+    ainfo->t2 = t2;
 
     return 0;
 }
 
+/* set up a model_info struct for passing to bhhh_max */
+
 static model_info *
-set_up_arma_info (int t1, int t2, int p, int q, int r)
+set_up_arma_model_info (struct arma_info *ainfo)
 {
     model_info *arma = model_info_new();
+    int m;
 
     if (arma == NULL) return NULL;
 
     model_info_set_opts(arma, PRESERVE_OPG_MODEL);
     model_info_set_tol(arma, 1.0e-6);
-    model_info_set_pqr(arma, p, q, r);
-    model_info_set_n_series(arma, 2 + p + q + r);
-    model_info_set_t1_t2(arma, t1, t2);
+
+    m = ainfo->p + ainfo->q + ainfo->r + ainfo->ifc;
+    model_info_set_k(arma, m);
+
+    model_info_set_n_series(arma, m + 1);
+    model_info_set_t1_t2(arma, ainfo->t1, ainfo->t2);
+
+    /* add pointer to ARMA-specific details */
+    model_info_set_extra_info(arma, ainfo);
 
     return arma;
 }
@@ -591,46 +700,48 @@ set_up_arma_info (int t1, int t2, int p, int q, int r)
 MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo, 
 		  PRN *prn)
 {
-    int nc, v, p, q, r, maxlag;
-    int arma_t1, arma_t2;
-    int err = 0;
-    double *coeff;
+    int nc, yno;
+    double *coeff = NULL;
     const double **X = NULL;
     MODEL armod;
     model_info *arma;
+    struct arma_info ainfo;
+    int err = 0;
 
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
-    if (check_arma_list(list)) {
+    if (check_arma_list(list, &ainfo)) {
 	armod.errcode = E_UNSPEC;
 	return armod;
     }
 
-    v = list[4]; /* dependent var */
-    p = list[1]; /* AR order */
-    q = list[2]; /* MA order */
-    maxlag = (p > q)? p : q; /* maximum lag in the model */
+    yno = list[4];         /* dependent variable */
+    ainfo.p = list[1];     /* AR order */
+    ainfo.q = list[2];     /* MA order */
+    ainfo.maxlag = (ainfo.p > ainfo.q)? 
+	ainfo.p : ainfo.q; /* maximum lag in the model */
 
-    r = list[0] - 4; /* number of ordinary regressors */
+    ainfo.r = list[0] - 4; /* number of ordinary regressors */
 
-    /* adjust sample? */
-    if (adjust_sample(pdinfo, Z, list, &arma_t1, &arma_t2)) {
+    /* adjust sample range if need be */
+    if (adjust_sample(pdinfo, Z, list, &ainfo)) {
         armod.errcode = E_DATA;
         return armod;
     }
 
-    /* number of coefficients */
-    nc = 1 + p + q + r;
+    /* tally of coefficients */
+    nc = ainfo.p + ainfo.q + ainfo.r + ainfo.ifc;
 
-    /* coefficient vector (plus error variance) */
+    /* initial coefficient vector */
     coeff = malloc(nc * sizeof *coeff);
     if (coeff == NULL) {
 	armod.errcode = E_ALLOC;
 	return armod;
     }
 
-    arma = set_up_arma_info(arma_t1, arma_t2, p, q, r);
+    /* create model_info struct to feed to bhhh_max() */
+    arma = set_up_arma_model_info(&ainfo);
     if (arma == NULL) {
 	free(coeff);
 	armod.errcode = E_ALLOC;
@@ -639,7 +750,7 @@ MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo,
 
     /* initialize the coefficients: AR and regression part by OLS, 
        MA at 0 */
-    err = ar_init_by_ols(list, coeff, Z, pdinfo, arma_t1);
+    err = ar_init_by_ols(list, coeff, Z, pdinfo, &ainfo);
     if (err) {
 	free(coeff);
 	model_info_free(arma);
@@ -655,6 +766,7 @@ MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo,
 	return armod;
     }
 
+    /* call BHHH conditional ML function (OPG regression) */
     err = bhhh_max(arma_ll, X, coeff, arma, prn);
 
     if (err) {
@@ -665,16 +777,17 @@ MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo,
 	double *theta = model_info_get_theta(arma);
 	cmplx *roots;
 
-	rewrite_arma_model_stats(pmod, arma, list, Z[v], theta, nc);
+	rewrite_arma_model_stats(pmod, arma, list, ainfo.ifc, Z[yno], 
+				 theta, nc);
 
 	/* compute and save polynomial roots */
-	roots = arma_roots(p, q, theta);
+	roots = arma_roots(&ainfo, theta);
 	if (roots != NULL) {
 	    gretl_model_set_data(pmod, "roots", roots,
-				 (p + q) * sizeof *roots);
+				 (ainfo.p + ainfo.q) * sizeof *roots);
 	}
 
-	add_arma_varnames(pmod, pdinfo);
+	add_arma_varnames(pmod, pdinfo, &ainfo);
 
 	armod = *pmod;
     }
