@@ -369,7 +369,7 @@ static int make_axis_label_comment (const char *line, dataset *dset)
     char tmp[16];
     int ret = 0;
 
-    if (sscanf(line, "set ylabel '%15[^\']", tmp) == 1) {
+    if (sscanf(line, "set ylabel '%15[^']", tmp) == 1) {
 	if (dset->pd > 0) {
 	    dset->comments[YLABEL_COMMENT] = g_strdup(tmp);
 	} else {
@@ -377,7 +377,7 @@ static int make_axis_label_comment (const char *line, dataset *dset)
 		g_strdup_printf("y variable %s", tmp);
 	}
 	ret = 1;
-    } else if (sscanf(line, "set xlabel '%15[^\']", tmp) == 1) {
+    } else if (sscanf(line, "set xlabel '%15[^']", tmp) == 1) {
 	if (dset->pd > 0) {
 	    dset->comments[XLABEL_COMMENT] = g_strdup(tmp);
 	} else {
@@ -490,7 +490,7 @@ static int read_datafile (const char *fname, dataset *dset)
 
     dataset_init(dset);
 
-    fdat = fopen(fname, "r");
+    fdat = gretl_fopen(fname, "r");
     if (fdat == NULL) {
 	fprintf(stderr, "Couldn't open '%s'\n", fname);
 	return 1;
@@ -1062,7 +1062,7 @@ int midi_play_graph (const char *fname, const char *userdir,
 
     sprintf(outname, "%sgretl.mid", userdir);
 
-    spec.fp = fopen(outname, "wb");
+    spec.fp = gretl_fopen(outname, "wb");
     if (spec.fp == NULL) {
 	fprintf(stderr, "Couldn't write to '%s'\n", outname);
 	return 1;
