@@ -6,7 +6,7 @@ int garch_estimate (int t1, int t2, int nobs,
 		    double *coeff, int nc, double *vc, 
 		    double *res2, double *res, double *h,
 		    const double *ystoc, double *amax, double *b, 
-		    int *iters, PRN *prn, int vopt);
+		    double scale, int *iters, PRN *prn, int vopt);
 
 #endif /* FCP_H */
 
