@@ -157,10 +157,10 @@ static int write_garch_stats (MODEL *pmod, const double **Z,
     pmod->adjrsq = NADBL; 
     pmod->fstt = NADBL;
 
-    pmod->criterion[C_AIC] = -2.0 * pmod->lnL + 2.0 * (pmod->ncoeff + 1);
-    pmod->criterion[C_BIC] = -2.0 * pmod->lnL + (pmod->ncoeff + 1) * log(pmod->nobs);
+    mle_aic_bic(pmod, 1);
 
     pmod->ci = GARCH;
+    pmod->ifc = 1;
     
     add_garch_varnames(pmod, pdinfo, list);
 
@@ -260,12 +260,17 @@ static int make_garch_dataset (const int *list, double **Z,
 static int get_vopt (int robust)
 {
     int vopt = get_garch_vcv_version();
+    int ropt = get_garch_robust_vcv_version();
 
     /* The defaults: QML if "robust" option is in force,
        otherwise negative Hessian */
     if (vopt == VCV_UNSET) {
 	if (robust) {
-	    vopt = VCV_QML;
+	    if (ropt == VCV_UNSET) {
+		vopt = VCV_QML;
+	    } else {
+		vopt = ropt;
+	    }
 	} else {
 	    vopt = VCV_HESSIAN;
 	}
