@@ -26,6 +26,8 @@
 
 #undef TDEBUG 
 
+#define TOBIT_TOL 1.0e-10 /* was 1.0e-9: do more checking? */
+
 /* Below: we are buying ourselves a considerable simplification when it comes
    to the tobit_ll function.  That function needs access to the original y
    and X data.  But the sample used for estimation may be at an offset into
@@ -188,10 +190,8 @@ static int add_norm_test_to_model (MODEL *pmod, double chi2)
     pmod->tests = malloc(sizeof *pmod->tests);
     if (pmod->tests == NULL) return 1;
 
-    strcpy(pmod->tests[0].type, N_("Test for normality of residual"));
-    strcpy(pmod->tests[0].h_0, N_("error is normally distributed"));
-    pmod->tests[0].param[0] = '\0';
-    pmod->tests[0].teststat = GRETL_TEST_NORMAL_CHISQ;
+    gretl_test_init(&pmod->tests[0], GRETL_TEST_NORMAL);
+    pmod->tests[0].teststat = GRETL_STAT_NORMAL_CHISQ;
     pmod->tests[0].value = chi2;
     pmod->tests[0].dfn = 2;
     pmod->tests[0].dfd = 0;
@@ -293,6 +293,8 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
 
     pmod->fstt = pmod->rsq = pmod->adjrsq = NADBL;
 
+    mle_aic_bic(pmod, 1);
+
     make_vcv(pmod, VCV, scale);
 
     pmod->ci = TOBIT;
@@ -311,7 +313,7 @@ tobit_model_info_init (int nobs, int k, int n_series)
     if (tobit == NULL) return NULL;
 
     model_info_set_opts(tobit, FULL_VCV_MATRIX);
-    model_info_set_tol(tobit, 1.0e-09);
+    model_info_set_tol(tobit, TOBIT_TOL);
     model_info_set_k(tobit, k);
     model_info_set_n_series(tobit, n_series);
     model_info_set_t1_t2(tobit, 0, nobs - 1);
