@@ -284,20 +284,15 @@ static void get_seats_command (char *seats, const char *tramo)
     else strcpy(seats, "seats");
 }
 
-static void truncate (char *str, int n)
-{
-    int len = strlen(str);
-
-    if (len > n) str[n] = 0;
-}
-
 static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 {
     FILE *fp = NULL;
     char title[32];
     int t;
 
-    if (gnuplot_init(PLOT_TRI_GRAPH, &fp)) return E_FOPEN;
+    if (gnuplot_init(PLOT_TRI_GRAPH, &fp)) {
+	return E_FOPEN;
+    }
 
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
@@ -429,20 +424,22 @@ static int add_series_from_file (const char *fname, int code,
 	if (p != NULL) strcpy(p + 1, x12a_series_strings[code]);
     }
 
-    fp = fopen(sfname, "r");
+    fp = gretl_fopen(sfname, "r");
     if (fp == NULL) {
 	int gotit = 0;
 
 	/* this is a bit of a pest: under some configurations, tramo/seats
-	   outputs a series "irfin", but sometimes that is not created, but
+	   outputs a series "irfin"; sometimes that is not created, but
 	   we do get an "irreg".  So if we can't find the one, try looking
 	   for the other.
 	*/
-	if (opt == TRAMO_SEATS && code == D13) { /* irregular, try the alternate */
+	if (opt == TRAMO_SEATS && code == D13) { 
 	    sprintf(sfname, "%s%cgraph%cseries%c%s", fname, SLASH, SLASH, SLASH,
 		    tramo_series_strings[code + 1]);
-	    fp = fopen(sfname, "r");
-	    if (fp != NULL) gotit = 1;
+	    fp = gretl_fopen(sfname, "r");
+	    if (fp != NULL) {
+		gotit = 1;
+	    }
 	    tramo_got_irfin = 0;
 	}
 	if (!gotit) {
@@ -452,15 +449,12 @@ static int add_series_from_file (const char *fname, int code,
     }
 
     /* formulate name of new variable to add */
-    strcpy(varname, pdinfo->varname[0]);
     if (opt == TRAMO_SEATS) {
-	truncate(varname, 5);
-	strcat(varname, "_");
-	strncat(varname, tramo_series_strings[code], 2);
+	sprintf(varname, "%.5s_%.2s", pdinfo->varname[0], 
+		tramo_series_strings[code]);
     } else {
-	truncate(varname, 4);
-	strcat(varname, "_");
-	strcat(varname, x12a_series_strings[code]);
+	sprintf(varname, "%.4s_%s", pdinfo->varname[0], 
+		x12a_series_strings[code]);
     }
 
     /* copy varname and label into place */
@@ -472,7 +466,9 @@ static int add_series_from_file (const char *fname, int code,
 	strcat(VARLABEL(pdinfo, v), " (X-12-ARIMA)");
     }	
 
-    for (t=0; t<pdinfo->n; t++) Z[v][t] = NADBL;
+    for (t=0; t<pdinfo->n; t++) {
+	Z[v][t] = NADBL;
+    }
 
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
@@ -496,7 +492,9 @@ static int add_series_from_file (const char *fname, int code,
     } else {
 	/* grab the data from the x12arima file */
 	while (fgets(line, 127, fp)) {
-	    if (*line == 'd' || *line == '-') continue;
+	    if (*line == 'd' || *line == '-') {
+		continue;
+	    }
 	    if (sscanf(line, "%d %lf", &d, &x) != 2) {
 		err = 1; 
 		break;
@@ -560,7 +558,7 @@ static int write_tramo_file (const char *fname,
     int t;
     char *p, tmp[8];
 
-    fp = fopen(fname, "w");
+    fp = gretl_fopen(fname, "w");
     if (fp == NULL) return 1;
 
 #ifdef ENABLE_NLS
@@ -610,7 +608,7 @@ static int write_spc_file (const char *fname,
     int startyr, startper;
     char *p, tmp[8];
 
-    fp = fopen(fname, "w");
+    fp = gretl_fopen(fname, "w");
     if (fp == NULL) return 1;    
 
 #ifdef ENABLE_NLS
@@ -688,7 +686,7 @@ static void copy_basic_data_info (DATAINFO *targ, DATAINFO *src)
     targ->t1 = src->t1;
     targ->t2 = src->t2;
     targ->pd = src->pd;
-    targ->time_series = src->time_series;
+    targ->structure = src->structure;
 }
 
 static int save_vars_to_dataset (double ***pZ, DATAINFO *pdinfo,
@@ -805,9 +803,9 @@ int write_tx_data (char *fname, int varnum,
     if (request.code == X12A) { 
 	/* make a default x12a.mdl file if it doesn't already exist */
 	sprintf(fname, "%s%cx12a.mdl", workdir, SLASH);
-	fp = fopen(fname, "r");
+	fp = gretl_fopen(fname, "r");
 	if (fp == NULL) {
-	    fp = fopen(fname, "w");
+	    fp = gretl_fopen(fname, "w");
 	    if (fp == NULL) return 1;
 	    fprintf(fp, "%s", default_mdl);
 	    fclose(fp);
