@@ -597,7 +597,7 @@ static int set_tolerance (char *line)
     if (toler == 0.0) return 0;
 
     sprintf(line, "genr toler = %g", toler);
-    return generate(&Z, datainfo, line, 0, NULL, 0);
+    return generate(&Z, datainfo, line, NULL);
 }
 
 static int generate_params (char *line, int round, PRN *prn)
@@ -611,7 +611,7 @@ static int generate_params (char *line, int round, PRN *prn)
 	x = (round == 1)? tester.coeffs[i].s1 : tester.coeffs[i].s2;
 	sprintf(line, "genr %s = %g", tester.coeffs[i].name, x);
 	if (verbose) printf("%s\n", line);
-	err = generate(&Z, datainfo, line, 0, NULL, 0);
+	err = generate(&Z, datainfo, line, NULL);
 	if (err) {
 	    fprintf(stderr, "%s: ERROR: genr failed in round %d\n '%s'\n", 
 		    tester.datname, round, line);
@@ -792,7 +792,7 @@ static int real_run_check (int round, PRN *prn)
     char line[256];
     MODEL *pmod = NULL;
 
-    pmod = gretl_model_new(datainfo);
+    pmod = gretl_model_new();
     if (pmod == NULL) {
 	fputs("Out of memory\n", stderr);
 	return 1;
@@ -825,18 +825,22 @@ static int real_run_check (int round, PRN *prn)
     if (!err) {
 	*pmod = nls(&Z, datainfo, prn);
 	if (pmod->errcode) {
+	    char errtext[128];
+
 	    err = pmod->errcode;
-	    fprintf(stderr, "%s: ERROR: model error %d\n", tester.datname, err);
+	    get_errmsg(err, errtext, NULL);
+	    fprintf(stderr, "%s: ERROR: model error %d (%s)\n", tester.datname, 
+		    err, errtext);
 	    errmsg(err, prn);
 	} else {
 	    if (verbose) {
 		pmod->ID = 0;
-		printmodel(pmod, datainfo, prn);
+		printmodel(pmod, datainfo, OPT_NONE, prn);
 	    }
-	    /* bodge */
-	    print_tol = pmod->chisq;
-	    total_iters += pmod->correct;
-	    /* end bodge */
+	    
+	    print_tol = gretl_model_get_double(pmod, "tol");
+	    total_iters += gretl_model_get_int(pmod, "iters");
+
 	    get_accuracy(pmod, &coeff_acc, &sderr_acc);
 	    if (coeff_acc < worst_coeff_acc) {
 		worst_coeff_acc = coeff_acc;
